@@ -29,7 +29,10 @@ echo ""
 ./abi-extract.pl -N bg-BG
 ./abi-extract.pl -N tr-TR
 ./abi-extract.pl -N de-CH
+./abi-extract.pl -N ta-IN
 ./abi-extract.pl -N qu-BO
+./abi-extract.pl -N ast-ES
+./abi-extract.pl -N ku
 
 ./update.pl gl-ES
 ./update.pl nb-NO
@@ -58,4 +61,8 @@ echo ""
 ./update.pl bg-BG 
 ./update.pl tr-TR
 ./update.pl de-CH
+./update.pl ta-IN
 ./update.pl qu-BO
+./update.pl ast-ES
+./update.pl ku
+
