@@ -40,8 +40,6 @@ public:
 	void mouseScroll(AV_View* pView, PtCallbackInfo_t *event);
 
 protected:
-	UT_uint32				m_clickState;	/* {NoClick,SingleClick,DoubleClick} */
-	EV_EditMouseContext		m_contextState;	/* mouse context of click */
 };
 
 #endif // EV_QNXMOUSE_H
