@@ -202,7 +202,7 @@ static const char ** _ev_GetLabelName(XAP_UnixApp * pUnixApp,
 	const char * szLabelName;
 	
 	if (pAction->hasDynamicLabel())
-		szLabelName = pAction->getDynamicLabel(pFrame,pLabel);
+		szLabelName = pAction->getDynamicLabel(pLabel);
 	else
 		szLabelName = pLabel->getMenuLabel();
 
@@ -222,7 +222,7 @@ static const char ** _ev_GetLabelName(XAP_UnixApp * pUnixApp,
 			EV_EditMethod * pEM = pEMC->findEditMethodByName(szMethodName);
 			UT_ASSERT(pEM);						// make sure it's bound to something
 
-			const EV_EditEventMapper * pEEM = pFrame->getEditEventMapper();
+			const EV_EditEventMapper * pEEM = pUnixApp->getEditEventMapper();
 			UT_ASSERT(pEEM);
 
 			const char * string = pEEM->getShortcutFor(pEM);
@@ -431,22 +431,20 @@ bool EV_UnixMenu::menuEvent(XAP_Menu_Id id)
 	return true;
 }
 
-static char _ev_get_underlined_char(const char * szString)
+static guint _ev_get_underlined_char(const char * szString)
 {
 
 	UT_ASSERT(szString);
 	
-	// return the char right after the underline
-	const char * p = szString;
-	while (*p && *(p+1))
+	// return the keycode right after the underline
+	const UT_UCS4String str(szString);
+	for (int i = 0; i < str.length() - 1; )
 	{
-		if (*p == '_')
-			return *++p;
-		else
-			p++;
+		if (str[i++] == '_')
+			return gdk_unicode_to_keyval(str[i]);
 	}
 
-	return 0;
+	return GDK_VoidSymbol;
 }
 
 static void _ev_strip_underline(char * bufResult,
@@ -670,7 +668,7 @@ bool EV_UnixMenu::synthesizeMenu(GtkWidget * wMenuRoot)
 		{
 		case EV_MLF_Normal:
 		{
-			const char ** data = _ev_GetLabelName(m_pUnixApp, m_pFrame, pAction, pLabel);
+			const char ** data = getLabelName(m_pUnixApp, pAction, pLabel);
 			szLabelName = data[0];
 			szMnemonicName = data[1];
 			GtkWidget * w;
@@ -749,12 +747,12 @@ bool EV_UnixMenu::synthesizeMenu(GtkWidget * wMenuRoot)
 				// version of the underlined char, since all the menus ignore upper
 				// case (SHIFT-MOD1-[char]) invokations of accelerators.
 
-				if (bAltOnMod1)
+				if (keyCode != GDK_VoidSymbol && bAltOnMod1)
 				{
-					EV_EditEventMapper * pEEM = m_pFrame->getEditEventMapper();
+					EV_EditEventMapper * pEEM = XAP_App::getApp()->getEditEventMapper();
 					UT_ASSERT(pEEM);
 					EV_EditMethod * pEM = NULL;
-					pEEM->Keystroke(EV_EKP_PRESS|EV_EMS_ALT|tolower(keyCode),&pEM);
+					pEEM->Keystroke(EV_EKP_PRESS|EV_EMS_ALT|keyCode,&pEM);
 
 					// if the pointer is valid, there is a conflict
 					bConflict = (pEM != NULL);
@@ -962,9 +960,9 @@ bool EV_UnixMenu::_refreshMenu(AV_View * pView, GtkWidget * wMenuRoot)
 					
 					// we do NOT add a new item, we point the existing index at our new widget
 					// (update the pointers)
-					void ** old = NULL;
+					GtkWidget* old = NULL;
 					GtkWidget *oldItem = GTK_WIDGET(m_vecMenuWidgets.getNthItem(k));
-					if (m_vecMenuWidgets.setNthItem(k, w, old))
+					if (m_vecMenuWidgets.setNthItem(k, w, &old))
 					{
 						UT_DEBUGMSG(("Could not update dynamic menu widget vector item %s.\n", k));
 						UT_ASSERT(0);
@@ -992,7 +990,7 @@ bool EV_UnixMenu::_refreshMenu(AV_View * pView, GtkWidget * wMenuRoot)
 				// if no dynamic label, all we need to do
 				// is enable/disable and/or check/uncheck it.
 
-				GtkWidget * item = static_cast<GtkWidget *>(m_vecMenuWidgets.getNthItem(k));
+				GtkWidget * item = m_vecMenuWidgets.getNthItem(k);
 				UT_ASSERT(item);
 
 				// check boxes 
@@ -1009,7 +1007,7 @@ bool EV_UnixMenu::_refreshMenu(AV_View * pView, GtkWidget * wMenuRoot)
 			}
 
 			// Get the item
-			GtkWidget * item = static_cast<GtkWidget *>(m_vecMenuWidgets.getNthItem(k));
+			GtkWidget * item = m_vecMenuWidgets.getNthItem(k);
 
 			// if item is null, there is no widget for it, so ignore its attributes for
 			// this pass
@@ -1031,8 +1029,8 @@ bool EV_UnixMenu::_refreshMenu(AV_View * pView, GtkWidget * wMenuRoot)
 				// have no children
 				GtkWidget * w = gtk_menu_item_new();
 				UT_ASSERT(w);
-				void ** blah = NULL;
-				if(m_vecMenuWidgets.setNthItem(k, w, blah))
+				GtkWidget* blah = NULL;
+				if(m_vecMenuWidgets.setNthItem(k, w, &blah))
 				{
 					UT_DEBUGMSG(("Could not update dynamic menu widget vector item %s.\n", k));
 					UT_ASSERT(0);
@@ -1077,7 +1075,7 @@ bool EV_UnixMenu::_refreshMenu(AV_View * pView, GtkWidget * wMenuRoot)
 
 			// we need to nest sub menus to have some sort of context so
 			// we can parent menu items
-			GtkWidget * item = static_cast<GtkWidget *>(m_vecMenuWidgets.getNthItem(k));
+			GtkWidget * item = m_vecMenuWidgets.getNthItem(k);
 			UT_ASSERT(item);
 
 			bool bEnable = true;
@@ -1087,7 +1085,7 @@ bool EV_UnixMenu::_refreshMenu(AV_View * pView, GtkWidget * wMenuRoot)
 				if (mis & EV_MIS_Gray)
 					bEnable = false;
 			}
-			gtk_widget_set_sensitive(static_cast<GtkWidget *>(item), bEnable);
+			gtk_widget_set_sensitive(item, bEnable);
 
 			// must have an entry for each and every layout item in the vector
 			stack.push(item);
