@@ -20,6 +20,8 @@
 #include <windows.h>
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
+#include <stdlib.h>
 #include "ut_types.h"
 #include "ut_stack.h"
 #include "ut_string.h"
@@ -83,24 +85,7 @@ static const EV_Menu_Bitmap s_bitmaps[] =
 
 	// Format
 	{AP_MENU_ID_FMT_FONT, "Menu_AbiWord_Font"},
-
-	/*{AP_MENU_ID_FMT_BOLD, "Menu_AbiWord_Bold"},
-	{AP_MENU_ID_FMT_ITALIC, "Menu_AbiWord_Italic"},
-	{AP_MENU_ID_FMT_UNDERLINE, "Menu_AbiWord_Underline"},
-	{AP_MENU_ID_FMT_STRIKE, "Menu_AbiWord_Strike"},		
-	{AP_MENU_ID_FMT_TOPLINE, "Menu_AbiWord_Topline"},
-	{AP_MENU_ID_FMT_BOTTOMLINE, "Menu_AbiWord_Bottomline"},
-	{AP_MENU_ID_SUPERSCRIPT, "Menu_AbiWord_Superscript"},
-	{AP_MENU_ID_SUBSCRIPT, "Menu_AbiWord_Subscript"},
-	{AP_MENU_ID_OVERLINE, "Menu_AbiWord_Overline"},*/
-		
-
-	//{AP_MENU_ID_ALIGN_LEFT, "Menu_AbiWord_Align_Left"},
-	//{AP_MENU_ID_ALIGN_RIGHT, "Menu_AbiWord_Align_Right"},
-	//{AP_MENU_ID_ALIGN_CENTER, "Menu_AbiWord_Align_Center"},
-	//{AP_MENU_ID_ALIGN_JUSTIFY, "Menu_AbiWord_Align_Justify"},	
 	{AP_MENU_ID_TABLE_INSERT_TABLE, "Menu_AbiWord_Insert_Table"},
-	//{AP_MENU_ID_VIEW_SHOWPARA, "Menu_AbiWord_View_Showpara"},
 	{AP_MENU_ID_TOOLS_SPELL, "Menu_AbiWord_Spellcheck"},			
 	{AP_MENU_ID_TOOLS_OPTIONS, "Menu_AbiWord_Preferences"},
 	{AP_MENU_ID_TOOLS_SCRIPTS, "Menu_AbiWord_Execute"},
@@ -124,8 +109,6 @@ static const EV_Menu_Bitmap s_bitmaps[] =
 	
 
 	// Help
-	{AP_MENU_ID_HELP_CONTENTS,      "Menu_AbiWord_Help"},    
-	//{AP_MENU_ID_HELP_INDEX, GTK_STOCK_INDEX},
 	{AP_MENU_ID_HELP_SEARCH, "Menu_AbiWord_Search"},
 	{AP_MENU_ID_HELP_ABOUT, "Menu_AbiWord_About"},
 	{AP_MENU_ID_HELP_CREDITS, "Menu_AbiWord_Credits"},
@@ -149,14 +132,14 @@ static const char * _ev_GetLabelName(XAP_Win32App * pWin32App,
 	const char * szLabelName;
 	
 	if (pAction->hasDynamicLabel())
-		szLabelName = pAction->getDynamicLabel(pFrame,pLabel);
+		szLabelName = pAction->getDynamicLabel(pLabel);
 	else
 		szLabelName = pLabel->getMenuLabel();
 
 	if (!szLabelName || !*szLabelName)
 		return NULL;
 
-	UT_String str = AP_Win32App::s_fromUTF8ToAnsi(szLabelName);
+	UT_String str = AP_Win32App::s_fromUTF8ToWinLocale(szLabelName);
 	szLabelName = str.c_str();
 
 	
@@ -518,8 +501,9 @@ bool EV_Win32Menu::onInitMenu(XAP_Frame * pFrame, AV_View * pView, HWND hWnd, HM
 					// is enable/disable and/or check/uncheck it.
 					pos++;
 
-					EnableMenuItem(hMenuBar,cmd,uEnable);
+					// need to check before disabling ...
 					CheckMenuItem(hMenuBar,cmd,uCheck);
+					EnableMenuItem(hMenuBar,cmd,uEnable);
 					break;
 				}
 
@@ -557,9 +541,9 @@ bool EV_Win32Menu::onInitMenu(XAP_Frame * pFrame, AV_View * pView, HWND hWnd, HM
 					// dynamic label has changed
 					XAP_Menu_Id id = MenuIdFromWmCommand(cmd);
 					EV_Menu_Item*	item;
-					for(UT_sint32 i = 0; i< m_vecItems.getItemCount(); i++)
+					for(UT_uint32 i = 0; i< m_vecItems.getItemCount(); i++)
 					{
-						item = reinterpret_cast<EV_Menu_Item*>(m_vecItems.getNthItem(i));											
+						item = (EV_Menu_Item*)m_vecItems.getNthItem(i);
 						if (id==item->id)
 						{
 							strcpy (item->szText, szLabelName);					
@@ -709,15 +693,12 @@ void EV_Win32Menu::onMeasureItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 
 	// Retrieve the width and height of the item's string 
 	GetTextExtentPoint32(hdc, item->szText, lstrlen(item->szText), &size); 
-
-	UT_DEBUGMSG(("EV_Win32Menu::onMeasureItem->%s\n", item->szText));
-	
-	if (lpmis->itemHeight<item->pMenu->m_nBitmapCY)
+		
+	if (size.cy < item->pMenu->m_nBitmapCY)
 		lpmis->itemHeight = item->pMenu->m_nBitmapCY;
 	else
 		lpmis->itemHeight = size.cy;
 
-
 	lpmis->itemWidth =  size.cx + item->pMenu->m_nBitmapCX + SPACE_ICONTEXT;
 
 	SelectObject(hdc, hfontOld); 
@@ -757,7 +738,7 @@ LPARAM EV_Win32Menu::onMenuChar(HWND hwnd, WPARAM wParam, LPARAM lParam)
 
 				pHotKeyPos++;
 
-				if (*pHotKeyPos==n)			
+				if (toupper(*pHotKeyPos)==toupper(n))			
 					return MAKELRESULT(i, MNC_EXECUTE);														 
 				
 			}
@@ -777,12 +758,12 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 	EV_Menu_Item*	item = (EV_Menu_Item *) lpdis->itemData;            			           	
     COLORREF crText;    
     COLORREF crBkgnd;   
-	DWORD dwColor = GetSysColor(COLOR_MENU);
+	DWORD dwColor;
 	int x = 255, y=255;									
 	RECT rect;
 	UT_String sTextRight, sTextLeft;
-	UINT nFormat;
 	HBITMAP hBitmap;	
+	int colorID;
 	UT_ASSERT(lpdis->CtlType==ODT_MENU); 						
 
 	/* Rect to draw the text */	
@@ -799,14 +780,16 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 		else
 			crText = SetTextColor(lpdis->hDC, GetSysColor(COLOR_MENUTEXT));			
 
-	//if (lpdis->itemState & ODS_SELECTED)     
-	//	dwColor = GetSysColor(COLOR_HIGHLIGHT);
-
+	if (lpdis->itemState & ODS_SELECTED)     
+		colorID = COLOR_HIGHLIGHT;
+	else
+		colorID = COLOR_MENU;		
+	
+	dwColor = GetSysColor(colorID);
+	
 	UT_RGBColor Color(GetRValue(dwColor),GetGValue(dwColor),GetBValue(dwColor));
 	hBitmap =  EV_Win32Menu::_loadBitmap(item->id,  x, y, Color);			
 
-	UT_DEBUGMSG(("EV_Win32Menu::onDrawItem->%s\n", item->szText));
-
 	if (hBitmap)	
 	{
 		BITMAP bitmap;		
@@ -814,18 +797,7 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 	}	
 
 	/*Draw the background of the item*/
-	if (lpdis->itemState & ODS_SELECTED) 
-    { 
-		dwColor = GetSysColor(COLOR_HIGHLIGHT);
-
-		/* We only highlight the text part of the item, not the bitmap*/
-		if (lpdis->itemState & ODS_CHECKED || hBitmap)
-			FillRect(lpdis->hDC, &rect, GetSysColorBrush(COLOR_HIGHLIGHT));															  		
-		else
-			FillRect(lpdis->hDC, &lpdis->rcItem, GetSysColorBrush(COLOR_HIGHLIGHT));															  		
-    }  			
-	else
-		FillRect(lpdis->hDC, &lpdis->rcItem, GetSysColorBrush(COLOR_MENU));
+	FillRect(lpdis->hDC, &lpdis->rcItem, GetSysColorBrush(colorID));
 
 	rect.left = rect.left + SPACE_ICONTEXT;  
 	
@@ -871,28 +843,12 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 	/* Draw bitmap*/
 	if (hBitmap)
 	{	
-		/*if (lpdis->itemState & ODS_GRAYED) 
-		{
-			HBRUSH hBGBrush = CreateSolidBrush(GetSysColor(COLOR_MENU));
-			crBkgnd = SetBkColor(lpdis->hDC, GetSysColor(COLOR_MENU));
-			SelectObject(lpdis->hDC, hBGBrush);
-
-			SetBkMode(lpdis->hDC, OPAQUE);
-			PatBlt(lpdis->hDC, lpdis->rcItem.left, lpdis->rcItem.top, m_nBitmapCX, m_nBitmapCY, PATCOPY);
-
-			DrawState(lpdis->hDC, hBrush, NULL, (LPARAM) hBitmap, 0, 
-				lpdis->rcItem.left, lpdis->rcItem.top, m_nBitmapCX, m_nBitmapCY, DST_BITMAP|DSS_DISABLED);		
-
-			SetBkColor(lpdis->hDC, crBkgnd); 			
-		}	*/	
-				  		
+					  		
 		HDC hdcMem = CreateCompatibleDC(lpdis->hDC);
 		SelectObject(hdcMem,(void *)hBitmap);				
-		BitBlt(lpdis->hDC, lpdis->rcItem.left+1, lpdis->rcItem.top+1, 16, 16, /*m_nBitmapCX, m_nBitmapCY,*/	hdcMem, 0, 0, SRCCOPY );
-
-		//StretchBlt(lpdis->hDC, lpdis->rcItem.left, lpdis->rcItem.top, 20, 20, /*m_nBitmapCX, m_nBitmapCY,*/	hdcMem, 
-		//	0,0, m_nBitmapCX, m_nBitmapCY, SRCCOPY);
+		BitBlt(lpdis->hDC, lpdis->rcItem.left+1, lpdis->rcItem.top+1, 16, 16, hdcMem, 0, 0, SRCCOPY );
 
+		
 		DeleteDC(hdcMem);					
 				
 	}
@@ -904,10 +860,11 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 			RECT r;
 			HBITMAP bm = CreateBitmap(nWidth, nHeight, 1, 1, NULL );
 			HDC hdcMem = CreateCompatibleDC(lpdis->hDC);
-			SetTextColor(lpdis->hDC, GetSysColor(COLOR_MENUTEXT));
+
+			SetBkColor(lpdis->hDC, GetSysColor(colorID));
 			SelectObject(hdcMem, bm);
 			SetRect(&r, 0, 0, nWidth, nHeight);
-			DrawFrameControl(hdcMem, &r, DFC_MENU,DFCS_MENUCHECK);
+			DrawFrameControl(hdcMem, &r, DFC_MENU,DFCS_MENUCHECK);			
         
 			BitBlt(lpdis->hDC, lpdis->rcItem.left+2, lpdis->rcItem.top+2, nWidth, nHeight,
 				hdcMem, 0, 0, SRCCOPY);
@@ -917,8 +874,7 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 		}	
 
     SelectObject(lpdis->hDC, hfontOld); 
-    SetTextColor(lpdis->hDC, crText); 
-	
+    SetTextColor(lpdis->hDC, crText); 	
 	if (hBitmap)
 		DeleteObject(hBitmap);	
 }
@@ -964,8 +920,8 @@ bool EV_Win32Menu::onMenuSelect(XAP_Frame * pFrame, AV_View * pView,
 	if (!szMsg || !*szMsg)
 		szMsg = "TODO This menu item doesn't have a StatusMessage defined.";
 	
-	//UT_DEBUGMSG(("SetMessage [%s]\n",szMsg));
-	pFrame->setStatusMessage(szMsg);
+	UT_String str = AP_Win32App::s_fromUTF8ToWinLocale(szMsg);
+	pFrame->setStatusMessage(str.c_str());
 	return true;
 }
 
