@@ -35,54 +35,74 @@ class QNXFont : public GR_Font {
   public:
 		QNXFont(FontID *aFont); 
 		~QNXFont();
+		const char *getFont(void); 
+		const int getSize(void);
+
+
 		const int getDisplayFontSize(void);
 		const char *getDisplayFont(void);
-		void    createDisplayFont(UT_uint32 size);
-		void deleteDisplayFont();	const char *getFont(void); 
+		void	createDisplayFont(UT_uint32 size); 
+		void deleteDisplayFont();
 	
-		const int getSize(void);
 		virtual UT_sint32 measureUnremappedCharForCache(UT_UCSChar cChar) const;
 
 	private:
 		FontID	*m_fontID;
-		FontID  *m_displayFontID;
-		FontID  *m_120ptFontID;
+		FontID	*m_displayFontID;
+		FontID	*m_120ptFontID;
+};
+
+class GR_QNXAllocInfo : public GR_AllocInfo
+{
+  public:
+	GR_QNXAllocInfo(PtWidget_t * win, PtWidget_t * draw, XAP_App * app) 
+	: m_win(win),m_draw(draw) {};
+
+	virtual GR_GraphicsId getType() const {return GRID_QNX;}
+	virtual bool isPrinterGraphics() const { return false; }
+	PtWidget_t *m_win,*m_draw;
+
 };
 
 class GR_QNXGraphics : public GR_Graphics
 {
+	// all constructors are protected; instances must be created via
+	// GR_GraphicsFactory
  public:
-	GR_QNXGraphics(PtWidget_t * win, PtWidget_t * draw, XAP_App * app);
 	~GR_QNXGraphics();
 
+	static UT_uint32 s_getClassId() {return GRID_QNX;}
+	virtual UT_uint32 getClassId() {return s_getClassId();}
+
+	virtual GR_Capability getCapability() {UT_ASSERT(UT_NOT_IMPLEMENTED); return GRCAP_UNKNOWN;}
+	
+	static const char *    graphicsDescriptor(void){return "QNX Default";}
+	static GR_Graphics *   graphicsAllocator(GR_AllocInfo&);
+	
 	virtual void		drawGlyph(UT_uint32 glyph_idx,UT_sint32 xoff,UT_sint32 yoff);
 	virtual void 		drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff);
 	virtual void 		drawChars(const UT_UCSChar* pChars, int iCharOffset,
 						   		  int iLength, UT_sint32 xoff, UT_sint32 yoff,
 								  int * pCharWidths = NULL);
 	virtual void 		setFont(GR_Font* pFont);
+	virtual void        clearFont(void) {m_pFont = NULL; }
 //	virtual UT_uint32		measureString(const UT_UCSChar *s,int iOffset,int num,UT_GrowBufElement *pWidths);
-	virtual UT_uint32 	measureUnRemappedChar(const UT_UCSChar c);
+	virtual UT_sint32 	measureUnRemappedChar(const UT_UCSChar c);
 	virtual void 		getColor(UT_RGBColor& clr);
 	virtual void 		setColor(const UT_RGBColor& clr);
 
 	virtual GR_Font* 	getGUIFont();
-	virtual GR_Font* 	findFont(const char* pszFontFamily, 
-								 const char* pszFontStyle, 
-								 const char* pszFontVariant, 
-								 const char* pszFontWeight, 
-								 const char* pszFontStretch, 
-								 const char* pszFontSize);
 	virtual UT_uint32 	getFontAscent();
 	virtual UT_uint32 	getFontDescent();
 	virtual UT_uint32 	getFontHeight();
-
 	virtual void getCoverage(UT_NumberVector &coverage);
+
+	virtual void		_beginPaint();
+	virtual void		_endPaint();
 	virtual void 		drawLine(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
 	virtual void 		setLineWidth(UT_sint32);
 	virtual void 		xorLine(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
 	virtual void		polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints);
-	
 	virtual void 		polyLine(UT_Point * pts, UT_uint32 nPoints);
 	virtual void 		fillRect(const UT_RGBColor& c, UT_sint32 x, UT_sint32 y, UT_sint32 w, UT_sint32 h);
 	virtual void 		fillRect(const UT_RGBColor& c, UT_Rect &r);
@@ -132,21 +152,32 @@ class GR_QNXGraphics : public GR_Graphics
 	virtual UT_uint32 	getFontAscent(GR_Font *);
 	virtual UT_uint32 	getFontDescent(GR_Font *);
 	virtual UT_uint32 	getFontHeight(GR_Font *);
+	virtual void		setZoomPercentage(UT_uint32 iZoom);
 	//Need this in the textdrawing cb.
 	PgColor_t getCurrentPgColor() { return m_currentColor; };
 	QNXFont *getCurrentQNXFont() { return m_pFont; };
 	PhGC_t	*getCurrentGC() { return m_pGC; };
- protected:
+
+protected:
+	// all instances have to be created via GR_GraphicsFactory; see gr_Graphics.h
+	GR_QNXGraphics(PtWidget_t * win, PtWidget_t * draw, XAP_App * app);
+	virtual GR_Font* 	_findFont(const char* pszFontFamily, 
+								  const char* pszFontStyle, 
+								  const char* pszFontVariant, 
+								  const char* pszFontWeight, 
+								  const char* pszFontStretch, 
+								  const char* pszFontSize);
+
 	virtual UT_uint32 	getDeviceResolution(void) const;
 	inline int 				DrawSetup();
 	inline int 				DrawTeardown();
 
 	PtWidget_t *  	m_pWin;
 	PtWidget_t *  	m_pDraw;
-	PdOffscreenContext_t *m_pOSC;
-	PhDrawContext_t				*m_pOldDC;
+//	PdOffscreenContext_t *m_pOSC;
+//	PhDrawContext_t				*m_pOldDC;
 	PhPoint_t		m_OffsetPoint;		
-	PhTile_t *		m_pClipList;		
+	PhTile_t		*m_pClipList;
 	struct _Pf_ctrl * m_pFontCx;	
 
 	// our currently requested font by handle
@@ -170,12 +201,20 @@ class GR_QNXGraphics : public GR_Graphics
 	UT_uint32			m_iShadowZoomPercentage;
  	bool             m_bPrintNextPage;    
 	PpPrintContext_t *  m_pPrintContext;
+
+	virtual GR_Image * genImageFromRectangle(const UT_Rect & r);
 private:
+
 	virtual void saveRectangle(UT_Rect &r, UT_uint32 iIndx);
 	virtual void restoreRectangle(UT_uint32 iIndx);
-
-	UT_Vector				m_vSaveRect;
-	UT_Vector 				m_vSaveRectBuf;
+	bool	OSCIsValid();
+	void	blitScreen();
+	void	setDamage(int x,int y,int h,int w);
+	
+	UT_GenericVector<UT_Rect*>				m_vSaveRect;
+	UT_GenericVector<PhImage_t *>			m_vSaveRectBuf;
+	PdOffscreenContext_t *m_pOSC;
+	PhArea_t				m_DamagedArea;
 };
 
 #endif /* GR_QNXGRAPHICS_H */
