@@ -109,8 +109,6 @@ bool UT_Xpm2Bitmap(const char ** pIconData,
 		UT_uint32 nf;
 		nf = sscanf(&pIconDataPalette[k][charsPerPixel+1],
 						" %s %s",bufKey,bufColorValue);
-		UT_ASSERT(nf == 2);
-		UT_ASSERT(bufKey[0] == 'c');
 
 		// make the ".." a hash key and store our color index as the data.
 		// we add k+1 because the hash code does not like null pointers...
