@@ -26,6 +26,7 @@
 
 void _UT_OutputMessage(const char *s, ...)
 {
+#ifdef UT_DEBUG
 	char sBuf[1024];
 	va_list marker;
 
@@ -34,4 +35,5 @@ void _UT_OutputMessage(const char *s, ...)
 	vsprintf(sBuf, s, marker);
 
 	fprintf(stderr,"DEBUG: %s\n",sBuf);
+#endif
 }
