@@ -724,8 +724,9 @@ bool UT_GenericUTF8Hash::grow ()
 	return true;
 }
 
-UT_UTF8Hash::UT_UTF8Hash () :
-	UT_GenericUTF8Hash(32)
+UT_UTF8Hash::UT_UTF8Hash (bool bStripEmptyValues) :
+	UT_GenericUTF8Hash(32),
+	m_bStripEmptyValues(bStripEmptyValues)
 {
 }
 
@@ -746,6 +747,13 @@ bool UT_UTF8Hash::pair (UT_uint32 index, const UT_UTF8String *& key, const UT_UT
 
 bool UT_UTF8Hash::ins (const UT_UTF8String & key, const UT_UTF8String & value)
 {
+	if (m_bStripEmptyValues)
+		if (value.byteLength () == 0)
+			{
+				del (key);
+				return true;
+			}
+
 	UT_UTF8String * utf8_value = 0;
 	UT_TRY
 		{
@@ -765,6 +773,13 @@ bool UT_UTF8Hash::ins (const char * key, const char * value)
 	if (value == 0)
 		return false;
 
+	if (m_bStripEmptyValues)
+		if (*value == 0)
+			{
+				del (key);
+				return true;
+			}
+
 	UT_UTF8String utf8_key(key);
 
 	UT_UTF8String * utf8_value = 0;
@@ -795,6 +810,13 @@ bool UT_UTF8Hash::ins (const char ** attrs)
 
 			if (value == 0) continue;
 
+			if (m_bStripEmptyValues)
+				if (*value == 0)
+					{
+						del (utf8_key);
+						continue;
+					}
+
 			UT_UTF8String * utf8_value = 0;
 			UT_TRY
 				{
@@ -1080,8 +1102,6 @@ void UT_UTF8Hash::parse_attributes (const char * attributes)
 	UT_UTF8String name;
 	UT_UTF8String value;
 
-	bool bSkip = false;
-
 	while (*atstr)
 		{
 			s_pass_whitespace (atstr);
@@ -1110,3 +1130,31 @@ void UT_UTF8Hash::parse_attributes (const char * attributes)
 			ins (name, value);
 		}
 }
+
+bool operator== (const UT_UTF8Hash & lhs, const UT_UTF8Hash & rhs)
+{
+	if (lhs.count () != rhs.count ())
+		return false;
+
+	const UT_UTF8String * lhs_key   = 0;
+	const UT_UTF8String * lhs_value = 0;
+	const UT_UTF8String * rhs_key   = 0;
+	const UT_UTF8String * rhs_value = 0;
+
+	bool equal = true;
+
+	UT_uint32 count = lhs.count ();
+
+	for (UT_uint32 i = 0; i < count; i++)
+		{
+			lhs.pair (i, lhs_key, lhs_value);
+			rhs.pair (i, rhs_key, rhs_value);
+
+			if ((*lhs_key != *rhs_key) || (*lhs_value != *rhs_value))
+				{
+					equal = false;
+					break;
+				}
+		}
+	return equal;
+}
