@@ -22,13 +22,3 @@
 #include <string.h>
 #include "ut_pair.h"
 
-UT_Pair::UT_Pair(const pair_type first, const pair_type second)
-:	m_first(first),
-	m_second(second)
-{
-}
-
-UT_Pair::~UT_Pair()
-{
-}
-
