@@ -1,6 +1,8 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiSource Application Framework
  * Copyright (C) 1998 AbiSource, Inc.
- * Copyright (C) 2001, 2003 Hubert Figuiere
+ * Copyright (C) 2001, 2003-2004 Hubert Figuiere
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -51,9 +53,10 @@ public:
 
 	virtual const char * 					getDefaultEncoding () const;
 
-	virtual bool							initialize();
+	virtual bool							initialize(const char * szKeyBindingsKey, const char * szKeyBindingsDefaultValue);
 	virtual XAP_Frame * 					newFrame() = 0;
 	virtual void							reallyExit();
+	virtual void							notifyFrameCountChange ();
 
 	virtual XAP_DialogFactory *				getDialogFactory();
 	virtual XAP_Toolbar_ControlFactory *	getControlFactory();
@@ -63,6 +66,8 @@ public:
 	virtual void							pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipboard, bool bHonorFormatting = true) = 0;
 	virtual bool							canPasteFromClipboard() = 0;
 	virtual const char *					getUserPrivateDirectory();
+	virtual bool							findAbiSuiteLibFile(UT_String & path, const char * filename, const char * subdir = 0);
+	virtual bool							findAbiSuiteAppFile(UT_String & path, const char * filename, const char * subdir = 0); // doesn't check user-dir
 
 	virtual void							setSelectionStatus(AV_View * pView) = 0;
 	virtual void							clearSelection() = 0;
@@ -92,8 +97,10 @@ public:
 	void							setTimeOfLastEvent(NSTimeInterval eventTime);
 	NSTimeInterval					getTimeOfLastEvent() const { return m_eventTime; };
 	virtual UT_sint32				makeDirectory(const char * szPath, const UT_sint32 mode ) const;
+    virtual BidiSupportType         theOSHasBidiSupport() const;
 	EV_CocoaMenuBar*				getCocoaMenuBar(void) const { return m_pCocoaMenu; };
 	XAP_Frame * 					_getFrontFrame(void);
+	AP_CocoaToolbar_Icons *			getToolbarIcons () const { return m_pCocoaToolbarIcons; }
 protected:
 	bool							_loadFonts();
 	void							_setAbiSuiteLibDir();
@@ -106,6 +113,7 @@ private:
 	NSTimeInterval			m_eventTime; // e->time field of a recent X event
 										 // (we use this to sync clipboard
 										 // operations with the server).
+
 protected:				// TODO move that to private
 	EV_CocoaMenuBar*			m_pCocoaMenu;
 	const char*			m_szMenuLayoutName;
