@@ -28,6 +28,7 @@
 
 #include "xap_CocoaApp.h"
 #include "xap_CocoaAppController.h"
+#include "xap_CocoaFont.h"
 #include "xap_CocoaModule.h"
 #include "xap_CocoaPlugin.h"
 #include "xap_CocoaToolPalette.h"
@@ -308,7 +309,8 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 		{
 			XAP_AppController_Instance = self;
 
-			m_bFileOpenedDuringLaunch = NO;
+			m_FilesRequestedDuringLaunch = [[NSMutableArray alloc] initWithCapacity:8];
+
 			m_bApplicationLaunching   = YES;
 
 			m_bAutoLoadPluginsAfterLaunch = NO;
@@ -316,7 +318,12 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 			m_PanelMenu   = [[NSMenu alloc] initWithTitle:@"Panels"];
 			m_ContextMenu = [[NSMenu alloc] initWithTitle:@"Context Menu"];
 
-			m_FontDictionary = 0;
+			m_FontReferenceDictionary = [[NSMutableDictionary alloc] initWithCapacity:128];
+			m_FontFamilyDictionary    = [XAP_CocoaFontFamilyHelper fontFamilyHelperDictionary:m_FontReferenceDictionary];
+
+			[m_FontFamilyDictionary retain];
+
+			m_MenuIDRefDictionary = [[NSMutableDictionary alloc] initWithCapacity:16];
 
 			m_Plugins      = [[NSMutableArray alloc] initWithCapacity:16];
 			m_PluginsTools = [[NSMutableArray alloc] initWithCapacity:16];
@@ -329,6 +336,11 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 
 - (void)dealloc
 {
+	if (m_FilesRequestedDuringLaunch)
+		{
+			[m_FilesRequestedDuringLaunch release];
+			m_FilesRequestedDuringLaunch = 0;
+		}
 	if (m_PanelMenu)
 		{
 			[m_PanelMenu release];
@@ -339,10 +351,20 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 			[m_ContextMenu release];
 			m_ContextMenu = 0;
 		}
-	if (m_FontDictionary)
+	if (m_FontReferenceDictionary)
 		{
-			[m_FontDictionary release];
-			m_FontDictionary = 0;
+			[m_FontReferenceDictionary release];
+			m_FontReferenceDictionary = 0;
+		}
+	if (m_FontFamilyDictionary)
+		{
+			[m_FontFamilyDictionary release];
+			m_FontFamilyDictionary = 0;
+		}
+	if (m_MenuIDRefDictionary)
+		{
+			[m_MenuIDRefDictionary release];
+			m_MenuIDRefDictionary = 0;
 		}
 	if (m_Plugins)
 		{
@@ -428,7 +450,35 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 			UT_DEBUGMSG(("[...FinishLaunching] Auto-Loading plug-ins:\n"));
 			XAP_CocoaModule::loadAllPlugins();
 		}
-	if (m_bFileOpenedDuringLaunch == NO)
+
+	BOOL bFileOpenedDuringLaunch = NO;
+
+	unsigned count = [m_FilesRequestedDuringLaunch count];
+
+	for (unsigned i = 0; i < count; i++) // filter out plugins from list and open those first
+		{
+			NSString * filename = (NSString *) [m_FilesRequestedDuringLaunch objectAtIndex:i];
+
+			UT_UTF8String path([filename UTF8String]);
+
+			if (XAP_CocoaModule::hasPluginExtension(path))
+				{
+					[self application:NSApp openFile:filename];
+				}
+		}
+	for (unsigned i = 0; i < count; i++) // open the rest
+		{
+			NSString * filename = (NSString *) [m_FilesRequestedDuringLaunch objectAtIndex:i];
+
+			UT_UTF8String path([filename UTF8String]);
+
+			if (!XAP_CocoaModule::hasPluginExtension(path))
+				if ([self application:NSApp openFile:filename])
+					{
+						bFileOpenedDuringLaunch = YES;
+					}
+		}
+	if (bFileOpenedDuringLaunch == NO)
 		{
 			UT_DEBUGMSG(("[...FinishLaunching] No file opened during launch, so opening untitled document:\n"));
 			[self applicationOpenUntitledFile:NSApp];
@@ -453,7 +503,21 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 
 - (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
 {
+	if (m_bApplicationLaunching == YES)
+		{
+			[m_FilesRequestedDuringLaunch addObject:filename];
+			return YES;
+		}
+
 	UT_DEBUGMSG(("Requested to open %s\n", [filename UTF8String]));
+
+	UT_UTF8String path([filename UTF8String]);
+
+	if (XAP_CocoaModule::hasPluginExtension(path))
+		{
+			return (XAP_CocoaModule::loadPlugin(path) ? YES : NO);
+		}
+
 	XAP_App * pApp = XAP_App::getApp();
 	XAP_Frame * pNewFrame = pApp->newFrame();
 
@@ -467,9 +531,6 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 	if (result)
 	{
 		pNewFrame->show();
-
-		if (m_bApplicationLaunching == YES)
-			m_bFileOpenedDuringLaunch = YES;
 	}
 	return (result ? YES : NO);
 }
@@ -494,6 +555,11 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 
 - (BOOL)applicationOpenUntitledFile:(NSApplication *)theApplication
 {
+	if (m_bApplicationLaunching == YES)
+		{
+			return YES;
+		}
+
 	UT_DEBUGMSG(("Requested to open untitled file...\n"));
 
 	EV_EditMethodContainer * pEMC = XAP_App::getApp()->getEditMethodContainer();
@@ -504,13 +570,7 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 	if (!pEM)
 		return NO;
 
-	bool result = pEM->Fn(0,0);
-	if (result)
-	{
-		if (m_bApplicationLaunching == YES)
-			m_bFileOpenedDuringLaunch = YES;
-	}
-	return (result ? YES : NO);
+	return (pEM->Fn(0,0) ? YES : NO);
 }
 
 - (BOOL)applicationOpenFile:(NSApplication *)theApplication
@@ -717,39 +777,35 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 {
 	NSString * familyName = nil;
 
-	if (!m_FontDictionary)
+	if (XAP_CocoaFontReference * fontRef = [m_FontReferenceDictionary objectForKey:fontName])
 		{
-			NSArray * pAvailableFontFamilies = [[NSFontManager sharedFontManager] availableFontFamilies];
-
-			unsigned family_count = [pAvailableFontFamilies count];
-
-			m_FontDictionary = [[NSMutableDictionary alloc] initWithCapacity:family_count];
-
-			for (unsigned ff = 0; ff < family_count; ff++)
-				{
-					NSString * family_name = (NSString *) [pAvailableFontFamilies objectAtIndex:ff];
-
-					NSArray * font_members = [[NSFontManager sharedFontManager] availableMembersOfFontFamily:family_name];
-
-					unsigned member_count = [font_members count];
-
-					for (unsigned fm = 0; fm < member_count; fm++)
-						{
-							NSArray * font = (NSArray *) [font_members objectAtIndex:fm];
-
-							NSString * name = (NSString *) [font objectAtIndex:0];
-
-							[m_FontDictionary setObject:family_name forKey:name];
-						}
-				}
-		}
-	if (m_FontDictionary && fontName)
-		{
-			familyName = (NSString *) [m_FontDictionary objectForKey:fontName];
+			familyName = [fontRef fontFamily];
 		}
 	return familyName;
 }
 
+- (XAP_CocoaFontReference *)helperReferenceForFont:(NSString *)fontName
+{
+	return [m_FontReferenceDictionary objectForKey:fontName];
+}
+
+- (XAP_CocoaFontFamilyHelper *)helperForFontFamily:(NSString *)fontFamilyName
+{
+	return [m_FontFamilyDictionary objectForKey:fontFamilyName];
+}
+
+- (XAP_CocoaFontFamilyHelper *)helperForUnknownFontFamily:(NSString *)fontFamilyName
+{
+	XAP_CocoaFontFamilyHelper * helper = [[XAP_CocoaFontFamilyHelper alloc] initWithFontFamilyName:fontFamilyName known:NO];
+
+	[m_FontFamilyDictionary setObject:helper forKey:fontFamilyName];
+
+	[helper addFontReferences:m_FontReferenceDictionary];
+	[helper release];
+
+	return helper;
+}
+
 - (void)appendPluginMenuItem:(NSMenuItem *)menuItem
 {
 	if (![m_PluginsTools containsObject:menuItem])
@@ -940,4 +996,28 @@ static XAP_CocoaAppController * XAP_AppController_Instance = nil;
 	return YES;
 }
 
+/* This provides a mechanism for associating XAP_CocoaPlugin_MenuItem objects
+ * with a given menu ID.
+ */
+- (void)addRef:(AP_CocoaPlugin_MenuIDRef *)ref forMenuID:(NSNumber *)menuid
+{
+	[m_MenuIDRefDictionary setObject:ref forKey:menuid];
+}
+
+/* This provides a mechanism for finding XAP_CocoaPlugin_MenuItem objects associated
+ * with a given menu ID.
+ */
+- (AP_CocoaPlugin_MenuIDRef *)refForMenuID:(NSNumber *)menuid
+{
+	return (AP_CocoaPlugin_MenuIDRef *) [m_MenuIDRefDictionary objectForKey:menuid];
+}
+
+/* This provides a mechanism for removing XAP_CocoaPlugin_MenuItem objects associated
+ * with a given menu ID.
+ */
+- (void)removeRefForMenuID:(NSNumber *)menuid
+{
+	[m_MenuIDRefDictionary removeObjectForKey:menuid];
+}
+
 @end
