@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiSource Application Framework
  * Copyright (C) 1998 AbiSource, Inc.
  * Copyright (C) 2003 Hubert Figuiere
@@ -39,19 +41,20 @@
 
 
 /*****************************************************************/
-XAP_Dialog * XAP_CocoaDialog_FontChooser::static_constructor(XAP_DialogFactory * pFactory,
-														 XAP_Dialog_Id dlgid)
+XAP_Dialog * XAP_CocoaDialog_FontChooser::static_constructor(XAP_DialogFactory * pFactory, XAP_Dialog_Id dlgid)
 {
 	XAP_CocoaDialog_FontChooser * p = new XAP_CocoaDialog_FontChooser(pFactory,dlgid);
 	return p;
 }
 
-XAP_CocoaDialog_FontChooser::XAP_CocoaDialog_FontChooser(XAP_DialogFactory * pDlgFactory,
-												   XAP_Dialog_Id dlgid)
-	: XAP_Dialog_FontChooser(pDlgFactory,dlgid),
-		m_currentFamily(NULL),
-		m_dlg(nil)
+XAP_CocoaDialog_FontChooser::XAP_CocoaDialog_FontChooser(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id dlgid) :
+	XAP_Dialog_FontChooser(pDlgFactory,dlgid),
+	m_bSuperScriptInitialValue(false),
+	m_bSubScriptInitialValue(false),
+	m_currentFamily(NULL),
+	m_dlg(nil)
 {
+	// 
 }
 
 XAP_CocoaDialog_FontChooser::~XAP_CocoaDialog_FontChooser(void)
@@ -63,15 +66,6 @@ XAP_CocoaDialog_FontChooser::~XAP_CocoaDialog_FontChooser(void)
 
 /*****************************************************************/
 
-void XAP_CocoaDialog_FontChooser::underlineChanged(bool value)
-{
-	m_bUnderline = value;
-	m_bChangedUnderline = !m_bChangedUnderline;
-	setFontDecoration(m_bUnderline,m_bOverline,m_bStrikeout,m_bTopline,m_bBottomline);
-	updatePreview();
-}
-
-
 void XAP_CocoaDialog_FontChooser::strikeoutChanged(bool value)
 {
 	m_bStrikeout = value;
@@ -80,6 +74,13 @@ void XAP_CocoaDialog_FontChooser::strikeoutChanged(bool value)
 	updatePreview();
 }
 
+void XAP_CocoaDialog_FontChooser::underlineChanged(bool value)
+{
+	m_bUnderline = value;
+	m_bChangedUnderline = !m_bChangedUnderline;
+	setFontDecoration(m_bUnderline,m_bOverline,m_bStrikeout,m_bTopline,m_bBottomline);
+	updatePreview();
+}
 
 void XAP_CocoaDialog_FontChooser::overlineChanged(bool value)
 {
@@ -89,6 +90,37 @@ void XAP_CocoaDialog_FontChooser::overlineChanged(bool value)
 	updatePreview();
 }
 
+void XAP_CocoaDialog_FontChooser::superscriptChanged(bool value)
+{
+	m_bSuperScript = value;
+	m_bSubScript   = false;
+
+	[m_dlg setSubscript:m_bSubScript];
+
+	m_bChangedSuperScript = (m_bSuperScript != m_bSuperScriptInitialValue);
+	m_bChangedSubScript   = (m_bSubScript   != m_bSubScriptInitialValue  );
+
+	setSuperScript(m_bSuperScript);
+	setSubScript  (m_bSubScript);
+
+	updatePreview();
+}
+
+void XAP_CocoaDialog_FontChooser::subscriptChanged(bool value)
+{
+	m_bSuperScript = false;
+	m_bSubScript   = value;
+
+	[m_dlg setSuperscript:m_bSuperScript];
+
+	m_bChangedSuperScript = (m_bSuperScript != m_bSuperScriptInitialValue);
+	m_bChangedSubScript   = (m_bSubScript   != m_bSubScriptInitialValue  );
+
+	setSuperScript(m_bSuperScript);
+	setSubScript  (m_bSubScript);
+
+	updatePreview();
+}
 
 void XAP_CocoaDialog_FontChooser::transparencyChanged(bool value)
 {
@@ -142,7 +174,6 @@ void XAP_CocoaDialog_FontChooser::styleRowChanged(int rowNumber)
 	updatePreview();
 }
 
-
 void XAP_CocoaDialog_FontChooser::sizeRowChanged(void)
 {
 	NSString* fontSize;
@@ -164,8 +195,7 @@ void XAP_CocoaDialog_FontChooser::sizeRowChanged(void)
 	\param attr the CSS attribute
 	\param buf the buf to store the color (static in the caller)
  */
-void XAP_CocoaDialog_FontChooser::_colorChanged(NSColor* color, const XML_Char* attr,
-					char* buf)
+void XAP_CocoaDialog_FontChooser::_colorChanged(NSColor * color, const XML_Char * attr, char * buf)
 {
 	float r,g,b,a;
 	color = [color colorUsingColorSpaceName:NSDeviceRGBColorSpace];
@@ -187,7 +217,6 @@ void XAP_CocoaDialog_FontChooser::fgColorChanged(void)
 	_colorChanged([m_dlg textColor], "color", buf_color);
 }
 
-
 /*!
 	Called when text background color has been changed in the widget
  */
@@ -197,8 +226,6 @@ void XAP_CocoaDialog_FontChooser::bgColorChanged(void)
 	_colorChanged([m_dlg bgColor], "bgcolor", buf_color);
 }
 
-
-
 void XAP_CocoaDialog_FontChooser::runModal(XAP_Frame * pFrame)
 {
 	UT_ASSERT(m_pApp);
@@ -213,6 +240,8 @@ void XAP_CocoaDialog_FontChooser::runModal(XAP_Frame * pFrame)
 	UT_ASSERT(window);
 
 	// freeze updates of the preview
+	_createGC([m_dlg preview]);
+	event_previewClear();
 
 
 	// Set the defaults in the list boxes according to dialog data
@@ -252,6 +281,12 @@ void XAP_CocoaDialog_FontChooser::runModal(XAP_Frame * pFrame)
 	[m_dlg setStrikeout:m_bStrikeout];
 	[m_dlg setUnderline:m_bUnderline];
 	[m_dlg setOverline:m_bOverline];
+
+	m_bSuperScriptInitialValue = m_bSuperScript;
+	m_bSubScriptInitialValue   = m_bSubScript;
+
+	[m_dlg setSuperscript:m_bSuperScript];
+	[m_dlg setSubscript:m_bSubScript];
 	
 	updatePreview();
 	// Run the dialog
@@ -307,7 +342,7 @@ void XAP_CocoaDialog_FontChooser::updatePreview(void)
 
 void XAP_CocoaDialog_FontChooser::_okAction(void)
 {
-	m_answer = XAP_Dialog_FontChooser::a_CANCEL;
+	m_answer = XAP_Dialog_FontChooser::a_OK;
 	[NSApp stopModal];
 }
 
@@ -322,7 +357,9 @@ void XAP_CocoaDialog_FontChooser::_cancelAction(void)
 void	XAP_CocoaDialog_FontChooser::_createGC(XAP_CocoaNSView* owner)
 {
 	NSSize  size;
-	m_pGraphics = new GR_CocoaGraphics(owner, m_pApp);
+	GR_CocoaAllocInfo ai(owner, m_pApp);
+	m_pGraphics = static_cast<GR_CocoaGraphics*>(XAP_App::getApp()->newGraphics(ai));
+
 	size = [owner bounds].size;
 	_createFontPreviewFromGC(m_pGraphics, lrintf(size.width), lrintf(size.height));
 }
@@ -353,6 +390,7 @@ void XAP_CocoaDialog_FontChooser::_deleteGC(void)
 {
 	[m_sizeDataSource release];
 	[m_stylesDataSource release];
+	[m_fontDataSource release];
 	[super dealloc];
 }
 
@@ -364,25 +402,26 @@ void XAP_CocoaDialog_FontChooser::_deleteGC(void)
 -(void)windowDidLoad
 {
 	if (_xap) {
-		_xap->_createGC(_preview);
-		_xap->event_previewClear();
-		/* localize */
 		const XAP_StringSet * pSS = _xap->getApp()->getStringSet();
-		[[self window] setTitle:[NSString stringWithUTF8String:pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_FontTitle).utf8_str()]];
-		LocalizeControl(_okBtn, pSS, XAP_STRING_ID_DLG_OK);
-		LocalizeControl(_cancelBtn, pSS, XAP_STRING_ID_DLG_Cancel);
-		LocalizeControl(_fontLabel, pSS, XAP_STRING_ID_DLG_UFS_FontLabel);
-		LocalizeControl(_styleLabel, pSS, XAP_STRING_ID_DLG_UFS_StyleLabel);
-		LocalizeControl(_sizeLabel, pSS, XAP_STRING_ID_DLG_UFS_SizeLabel);
-		LocalizeControl(_effectLabel, pSS, XAP_STRING_ID_DLG_UFS_EffectsFrameLabel);
-		LocalizeControl(_strikeButton, pSS, XAP_STRING_ID_DLG_UFS_StrikeoutCheck);
-		LocalizeControl(_underlineButton, pSS, XAP_STRING_ID_DLG_UFS_UnderlineCheck);
-		LocalizeControl(_overlineButton, pSS, XAP_STRING_ID_DLG_UFS_OverlineCheck);
-		LocalizeControl(_strikeButton, pSS, XAP_STRING_ID_DLG_UFS_StrikeoutCheck);
-	//	LocalizeControl(_strikeButton, pSS, XAP_STRING_ID_DLG_UFS_EncodingLabel);
-		LocalizeControl(_textColorLabel, pSS, XAP_STRING_ID_DLG_UFS_ColorTab);
-		LocalizeControl(_textHighlightColorLabel, pSS, XAP_STRING_ID_DLG_UFS_BGColorTab);
-		LocalizeControl(_noHighlightColorButton, pSS, XAP_STRING_ID_DLG_UFS_TransparencyCheck);
+
+		LocalizeControl([self window],				pSS, XAP_STRING_ID_DLG_UFS_FontTitle);
+
+		LocalizeControl(_okBtn,						pSS, XAP_STRING_ID_DLG_OK);
+		LocalizeControl(_cancelBtn,					pSS, XAP_STRING_ID_DLG_Cancel);
+		LocalizeControl(_fontLabel,					pSS, XAP_STRING_ID_DLG_UFS_FontLabel);
+		LocalizeControl(_styleLabel,				pSS, XAP_STRING_ID_DLG_UFS_StyleLabel);
+		LocalizeControl(_sizeLabel,					pSS, XAP_STRING_ID_DLG_UFS_SizeLabel);
+		LocalizeControl(_effectLabel,				pSS, XAP_STRING_ID_DLG_UFS_EffectsFrameLabel);
+		LocalizeControl(_strikeButton,				pSS, XAP_STRING_ID_DLG_UFS_StrikeoutCheck);
+		LocalizeControl(_underlineButton,			pSS, XAP_STRING_ID_DLG_UFS_UnderlineCheck);
+		LocalizeControl(_overlineButton,			pSS, XAP_STRING_ID_DLG_UFS_OverlineCheck);
+		LocalizeControl(_strikeButton,				pSS, XAP_STRING_ID_DLG_UFS_StrikeoutCheck);
+	//	LocalizeControl(_strikeButton,				pSS, XAP_STRING_ID_DLG_UFS_EncodingLabel);
+		LocalizeControl(_superscriptButton,			pSS, XAP_STRING_ID_DLG_UFS_SuperScript);
+		LocalizeControl(_subscriptButton,			pSS, XAP_STRING_ID_DLG_UFS_SubScript);
+		LocalizeControl(_textColorLabel,			pSS, XAP_STRING_ID_DLG_UFS_ColorTab);
+		LocalizeControl(_textHighlightColorLabel,	pSS, XAP_STRING_ID_DLG_UFS_BGColorTab);
+		LocalizeControl(_noHighlightColorButton,	pSS, XAP_STRING_ID_DLG_UFS_TransparencyCheck);
 		
 		m_fontDataSource = [[XAP_StringListDataSource alloc] init];
 		[m_fontDataSource loadFontList];
@@ -390,10 +429,15 @@ void XAP_CocoaDialog_FontChooser::_deleteGC(void)
 		[_fontList setDelegate:self];
 	
 		m_stylesDataSource = [[XAP_StringListDataSource alloc] init];
-		[m_stylesDataSource addUT_UTF8String:pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_StyleRegular)];
-		[m_stylesDataSource addUT_UTF8String:pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_StyleItalic)];
-		[m_stylesDataSource addUT_UTF8String:pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_StyleBold)];
-		[m_stylesDataSource addUT_UTF8String:pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_StyleBoldItalic)];
+		UT_UTF8String label;
+		pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_StyleRegular, label);
+		[m_stylesDataSource addUT_UTF8String:label];
+		pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_StyleItalic, label);
+		[m_stylesDataSource addUT_UTF8String:label];
+		pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_StyleBold, label);
+		[m_stylesDataSource addUT_UTF8String:label];
+		pSS->getValueUTF8(XAP_STRING_ID_DLG_UFS_StyleBoldItalic, label);
+		[m_stylesDataSource addUT_UTF8String:label];
 		[_styleList setDataSource:m_stylesDataSource];
 		[_styleList setDelegate:self];
 	
@@ -417,32 +461,35 @@ void XAP_CocoaDialog_FontChooser::_deleteGC(void)
 	else {
 		NSLog(@"Loaded Windows without XAP (%s:%d)", __FILE__, __LINE__);
 	}
+
+	if ([[NSColorPanel sharedColorPanel] isVisible])
+		{
+			[_textColorWell activate:YES];
+		}
 }
 
 -(IBAction)okAction:(id)sender
 {
-	static_cast<XAP_CocoaDialog_FontChooser*>(_xap)->_okAction();
-}
+	[[NSColorPanel sharedColorPanel] orderOut:self];
 
+	static_cast<XAP_CocoaDialog_FontChooser *>(_xap)->_okAction();
+}
 
 -(IBAction)cancelAction:(id)sender
 {
-	static_cast<XAP_CocoaDialog_FontChooser*>(_xap)->_cancelAction();
-}
+	[[NSColorPanel sharedColorPanel] orderOut:self];
 
+	static_cast<XAP_CocoaDialog_FontChooser *>(_xap)->_cancelAction();
+}
 
 -(IBAction)colorWellAction:(id)sender
 {
-//	NSColorPanel*	panel = [NSColorPanel sharedColorPanel];
-//	[panel setTarget:sender];
-//	[panel setColor:[sender color]];
 	if (sender == _textColorWell) {
 		_xap->fgColorChanged();
 	}
 	else if (sender == _textHighlightColorWell) {
 		_xap->bgColorChanged();
 	}
-
 }
 
 -(IBAction)underlineAction:(id)sender
@@ -451,27 +498,35 @@ void XAP_CocoaDialog_FontChooser::_deleteGC(void)
 				([(NSControl*)sender intValue] == NSOffState ? 0 : 1);
 }
 
-
 -(IBAction)overlineAction:(id)sender
 {
 	static_cast<XAP_CocoaDialog_FontChooser*>(_xap)->overlineChanged
 			([(NSControl*)sender intValue] == NSOffState ? 0 : 1);
 }
 
-
 -(IBAction)strikeoutAction:(id)sender
 {
 	static_cast<XAP_CocoaDialog_FontChooser*>(_xap)->strikeoutChanged
 			([(NSControl*)sender intValue] == NSOffState ? 0 : 1);
 }
 
-
--(IBAction)transparentAction:(id)sender
+-(IBAction)superscriptAction:(id)sender
 {
-	static_cast<XAP_CocoaDialog_FontChooser*>(_xap)->strikeoutChanged
+	static_cast<XAP_CocoaDialog_FontChooser*>(_xap)->superscriptChanged
 			([(NSControl*)sender intValue] == NSOffState ? 0 : 1);
 }
 
+-(IBAction)subscriptAction:(id)sender
+{
+	static_cast<XAP_CocoaDialog_FontChooser*>(_xap)->subscriptChanged
+			([(NSControl*)sender intValue] == NSOffState ? 0 : 1);
+}
+
+-(IBAction)transparentAction:(id)sender
+{
+	static_cast<XAP_CocoaDialog_FontChooser*>(_xap)->transparencyChanged
+			([(NSControl*)sender intValue] == NSOffState ? 0 : 1);
+}
 
 - (void)tableViewSelectionDidChange:(NSNotification *)aNotification
 {
@@ -489,23 +544,35 @@ void XAP_CocoaDialog_FontChooser::_deleteGC(void)
 	}
 }
 
--(void)setStrikeout:(bool)value
+- (XAP_CocoaNSView*)preview;
 {
-	[_strikeButton setIntValue:value];
+	return _preview;
 }
 
+-(void)setStrikeout:(bool)value
+{
+	[_strikeButton setState:(value ? NSOnState : NSOffState)];
+}
 
 -(void)setUnderline:(bool)value
 {
-	[_underlineButton setIntValue:value];
+	[_underlineButton setState:(value ? NSOnState : NSOffState)];
 }
 
-
 -(void)setOverline:(bool)value
 {
-	[_overlineButton setIntValue:value];
+	[_overlineButton setState:(value ? NSOnState : NSOffState)];
 }
 
+-(void)setSuperscript:(bool)value
+{
+	[_superscriptButton setState:(value ? NSOnState : NSOffState)];
+}
+
+-(void)setSubscript:(bool)value
+{
+	[_subscriptButton setState:(value ? NSOnState : NSOffState)];
+}
 
 -(void)selectFont:(char*)value
 {
