@@ -22,7 +22,7 @@
 #ifndef XAP_COCOAPLUGIN_H
 #define XAP_COCOAPLUGIN_H
 
-#define XAP_COCOAPLUGIN_INTERFACE 20050302 /** The current version of the CocoaPlugin API. */
+#define XAP_COCOAPLUGIN_INTERFACE 20050304 /** The current version of the CocoaPlugin API. */
 
 #import <Cocoa/Cocoa.h>
 
@@ -152,6 +152,27 @@
  */
 - (NSString *)title;
 
+/**
+ * Selects and returns the current word, if any.
+ * 
+ * \return The current word, or nil if none could be selected.
+ */
+- (NSString *)selectWord;
+
+/**
+ * Returns the current selection, if any.
+ * 
+ * \return The currently selected text, or nil if none is selected.
+ */
+- (NSString *)selectedText;
+
+/**
+ * Insert a text string into the document.
+ * 
+ * \param text The string to insert.
+ */
+- (void)insertText:(NSString *)text;
+
 /* Mail Merge interface
  */
 - (NSString *)documentMailMergeSource; // may return nil
