@@ -71,7 +71,6 @@ XAP_QNXDialog_PluginManager::~XAP_QNXDialog_PluginManager(void)
 
 /*****************************************************************/
 /*****************************************************************/
-
 void XAP_QNXDialog_PluginManager::event_DeactivateAll ()
 {
 	deactivateAllPlugins ();
@@ -89,7 +88,7 @@ void XAP_QNXDialog_PluginManager::event_Close () {
 void XAP_QNXDialog_PluginManager::event_Deactivate ()
 {
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
-
+	UT_UTF8String s;
 	XAP_Module * pModule = 0;
 
 	unsigned short * selectedRow = NULL;
@@ -101,7 +100,8 @@ void XAP_QNXDialog_PluginManager::event_Deactivate ()
 	else 
 	{
 		// error message box - didn't select a plugin
-		_errorMessage (m_pFrame, pSS->getValueUTF8(XAP_STRING_ID_DLG_PLUGIN_MANAGER_NONE_SELECTED ).utf8_str());
+		pSS->getValueUTF8(XAP_STRING_ID_DLG_PLUGIN_MANAGER_NONE_SELECTED,s);
+		_errorMessage (m_pFrame, s.utf8_str());
 		return;
 	}
 
@@ -115,21 +115,23 @@ void XAP_QNXDialog_PluginManager::event_Deactivate ()
 		else
 		{
 			// error message box
-			_errorMessage (m_pFrame, 
-pSS->getValueUTF8(XAP_STRING_ID_DLG_PLUGIN_MANAGER_COULDNT_UNLOAD ).utf8_str());
+			pSS->getValueUTF8(XAP_STRING_ID_DLG_PLUGIN_MANAGER_COULDNT_UNLOAD,s);
+			_errorMessage (m_pFrame,s.utf8_str());
 		}
 	}
 	else
 	{
 		// error message box
-		_errorMessage (m_pFrame, 
-pSS->getValueUTF8(XAP_STRING_ID_DLG_PLUGIN_MANAGER_COULDNT_UNLOAD ).utf8_str());
+		pSS->getValueUTF8(XAP_STRING_ID_DLG_PLUGIN_MANAGER_COULDNT_UNLOAD,s);
+		_errorMessage (m_pFrame,s.utf8_str());
 	}
 }
 
 void XAP_QNXDialog_PluginManager::event_Load ()
 {
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
+	UT_UTF8String s;
+	
 
 	XAP_DialogFactory * pDialogFactory
 		= (XAP_DialogFactory *) m_pFrame->getDialogFactory();
@@ -180,7 +182,8 @@ void XAP_QNXDialog_PluginManager::event_Load ()
 			else
 			{
 				// error message
-				_errorMessage (m_pFrame, pSS->getValueUTF8(XAP_STRING_ID_DLG_PLUGIN_MANAGER_COULDNT_LOAD ).utf8_str());
+				pSS->getValueUTF8(XAP_STRING_ID_DLG_PLUGIN_MANAGER_COULDNT_LOAD,s);
+				_errorMessage (m_pFrame, s.utf8_str());
 			}
 		}
 	}
@@ -217,10 +220,10 @@ void XAP_QNXDialog_PluginManager::_refreshTab1 ()
 	// first, refresh the CList
 	PtListDeleteAllItems(m_clist);
 	
-	const UT_Vector * pVec = XAP_ModuleManager::instance().enumModules ();
+	const UT_GenericVector<XAP_Module *> * pVec = XAP_ModuleManager::instance().enumModules ();
 
 	for (UT_uint32 i = 0; i < pVec->size(); i++) {
-		pModule = (XAP_Module *)pVec->getNthItem (i);
+		pModule = pVec->getNthItem (i);
 		text [0] = pModule->getModuleInfo()->name;
 		PtListAddItems(m_clist, text, 1, 0);
 	}
@@ -303,7 +306,6 @@ static int s_load_clicked (PtWidget_t *w, void *data, PtCallbackInfo_t *info)
 	dlg->event_Load ();
 	return Pt_CONTINUE;
 }
-
 /*****************************************************************/
 /*****************************************************************/
 
@@ -344,6 +346,7 @@ void XAP_QNXDialog_PluginManager::runModal(XAP_Frame * pFrame)
 	PtDestroyWidget(mainWindow);
 }
 
+
 PtWidget_t * XAP_QNXDialog_PluginManager::_constructWindow ()
 {
 	PtWidget_t *windowPlugins;
@@ -365,8 +368,10 @@ PtWidget_t * XAP_QNXDialog_PluginManager::_constructWindow ()
 
 	PtArg_t args[10];
 	int n=0;
+	UT_UTF8String s;
 
-PtSetArg(&args[n++], Pt_ARG_WINDOW_TITLE, _(XAP,DLG_PLUGIN_MANAGER_TITLE) , 0);
+	pSS->getValueUTF8(XAP_STRING_ID_DLG_PLUGIN_MANAGER_TITLE,s);
+PtSetArg(&args[n++], Pt_ARG_WINDOW_TITLE, s.utf8_str(), 0);
 	PtSetArg(&args[n++], Pt_ARG_WINDOW_RENDER_FLAGS, 0, ABI_MODAL_WINDOW_RENDER_FLAGS);
 	PtSetArg(&args[n++], Pt_ARG_WINDOW_MANAGED_FLAGS, 0, ABI_MODAL_WINDOW_MANAGE_FLAGS);
 	windowPlugins = PtCreateWidget(PtWindow, NULL, n, args);
@@ -473,13 +478,15 @@ PtSetArg(&args[n++], Pt_ARG_WINDOW_TITLE, _(XAP,DLG_PLUGIN_MANAGER_TITLE) , 0);
 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_WIDTH, ABI_DEFAULT_BUTTON_WIDTH, 0);
-PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, _(XAP,DLG_OK), 0);
+	pSS->getValueUTF8(XAP_STRING_ID_DLG_OK,s);
+	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, s.utf8_str(), 0);
 	PtWidget_t *buttonOK = PtCreateWidget(PtButton, hgroup, n, args);
 	PtAddCallback(buttonOK, Pt_CB_ACTIVATE, s_close_clicked, this);
 
 	n = 0;
+	pSS->getValueUTF8(XAP_STRING_ID_DLG_Close,s);
 	PtSetArg(&args[n++], Pt_ARG_WIDTH, ABI_DEFAULT_BUTTON_WIDTH, 0);
-PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, _(XAP,DLG_Close), 0);
+PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, s.utf8_str(), 0);
 	PtWidget_t *buttonClose = PtCreateWidget(PtButton, hgroup, n, args);
 	PtAddCallback(buttonClose, Pt_CB_ACTIVATE, s_close_clicked, this);
 	
