@@ -33,6 +33,7 @@
 #include "ut_debugmsg.h"
 #include "xap_UnixDialogHelper.h"
 #include "ut_string.h"
+#include "ut_uuid.h"
 
 #include "xap_Strings.h"
 #include "xap_Args.h"
@@ -43,12 +44,12 @@
 #include "xap_Unix_TB_CFactory.h"
 #include "xap_Prefs.h"
 #include "xap_UnixEncodingManager.h"
-
-#ifndef WITH_PANGO
 #include "xap_UnixFontManager.h"
-#endif
 
 #include "xap_UnixNullGraphics.h"
+#include "xap_UnixPSGraphics.h"
+#include "gr_UnixGraphics.h"
+
 UnixNull_Graphics * abi_unixnullgraphics_instance = 0;
 
 /*****************************************************************/
@@ -65,6 +66,37 @@ XAP_UnixApp::XAP_UnixApp(XAP_Args * pArgs, const char * szAppName)
 
 	memset(&m_geometry, 0, sizeof(m_geometry));
 
+	// create an instance of UT_UUIDGenerator or appropriate derrived class
+	_setUUIDGenerator(new UT_UUIDGenerator());
+
+	// register graphics allocator
+	GR_GraphicsFactory * pGF = getGraphicsFactory();
+	UT_ASSERT( pGF );
+
+	if(pGF)
+	{
+		bool bSuccess = pGF->registerClass(GR_UnixGraphics::graphicsAllocator,
+										   GR_UnixGraphics::graphicsDescriptor,
+										   GR_UnixGraphics::s_getClassId());
+
+		// we are in deep trouble if this did not succeed
+		UT_ASSERT( bSuccess );
+		pGF->registerAsDefault(GR_UnixGraphics::s_getClassId(), true);
+
+		bSuccess = pGF->registerClass(PS_Graphics::graphicsAllocator,
+									  PS_Graphics::graphicsDescriptor,
+									  PS_Graphics::s_getClassId());
+
+		UT_ASSERT( bSuccess );
+		pGF->registerAsDefault(PS_Graphics::s_getClassId(), false);
+		
+
+		bSuccess = pGF->registerClass(UnixNull_Graphics::graphicsAllocator,
+									  UnixNull_Graphics::graphicsDescriptor,
+									  UnixNull_Graphics::s_getClassId());
+
+		UT_ASSERT( bSuccess );
+	}
 
 	/* We need to link UnixNull_Graphics because the AbiCommand
 	 * plugin uses it.
@@ -72,36 +104,33 @@ XAP_UnixApp::XAP_UnixApp(XAP_Args * pArgs, const char * szAppName)
 	if (abi_unixnullgraphics_instance)
 	  {
 	    delete abi_unixnullgraphics_instance;
-	    abi_unixnullgraphics_instance = new UnixNull_Graphics(0,0);
+	    //abi_unixnullgraphics_instance = new UnixNull_Graphics(0,0);
+		XAP_UnixNullGraphicsAllocInfo ai(NULL, NULL);
+		abi_unixnullgraphics_instance = (UnixNull_Graphics*) XAP_App::getApp()->newGraphics(GRID_UNIX_NULL, ai);
 	  }
 }
 
 XAP_UnixApp::~XAP_UnixApp()
 {
 	DELETEP(m_pUnixToolbarIcons);
-
-#ifndef WITH_PANGO
 	delete m_fontManager;
-#endif	
 }
 
-bool XAP_UnixApp::initialize(bool have_display)
+bool XAP_UnixApp::initialize(const char * szKeyBindingsKey, const char * szKeyBindingsDefaultValue)
 {
 	if (!g_thread_supported ()) g_thread_init (NULL);
 
 	// let our base class do it's thing.
 	
-	XAP_App::initialize();
+	XAP_App::initialize(szKeyBindingsKey, szKeyBindingsDefaultValue);
 
 	/*******************************/
 
-#ifndef WITH_PANGO	
 	// load the font stuff from the font directory
 	UT_DEBUGMSG(("Loading Fonts\n"));
 	if (!_loadFonts())
 		return false;
 	UT_DEBUGMSG(("Fonts Loaded \n"));
-#endif
 	
 	/*******************************/
 	
@@ -134,12 +163,10 @@ XAP_Toolbar_ControlFactory * XAP_UnixApp::getControlFactory()
 	return &m_controlFactory;
 }
 
-#ifndef WITH_PANGO
 XAP_UnixFontManager * XAP_UnixApp::getFontManager()
 {
 	return m_fontManager;
 }
-#endif
 
 void XAP_UnixApp::setWinGeometry(int x, int y, UT_uint32 width, UT_uint32 height,
 								 UT_uint32 flags)
@@ -190,7 +217,6 @@ const char * XAP_UnixApp::getUserPrivateDirectory()
 	return buf;
 }
 
-#ifndef WITH_PANGO
 bool XAP_UnixApp::_loadFonts()
 {
 	// create a font manager for our app to use
@@ -206,7 +232,6 @@ bool XAP_UnixApp::_loadFonts()
 	UT_DEBUGMSG(("Scavange Fonts finished \n"));
 	return true;
 }
-#endif //#ifndef WITH_PANGO
 
 void XAP_UnixApp::_setAbiSuiteLibDir()
 {
