@@ -151,11 +151,9 @@ void XAP_UnixDialog_Zoom::runModal(XAP_Frame * pFrame)
     UT_ASSERT(m_previewArea && m_previewArea->window);
     
     // make a new Unix GC
-#ifndef WITH_PANGO		
-    m_unixGraphics = new GR_UnixGraphics(m_previewArea->window, unixapp->getFontManager(), m_pApp);
-#else
-    m_unixGraphics = new GR_UnixGraphics(m_previewArea->window,m_pApp);
-#endif		
+    //m_unixGraphics = new GR_UnixGraphics(m_previewArea->window, unixapp->getFontManager(), m_pApp);
+	GR_UnixAllocInfo ai(m_previewArea->window, unixapp->getFontManager(), m_pApp);
+	m_unixGraphics = (GR_UnixGraphics*) XAP_App::getApp()->newGraphics(ai);
     
     // let the widget materialize
     _createPreviewFromGC(m_unixGraphics,
@@ -271,7 +269,9 @@ GtkWidget * XAP_UnixDialog_Zoom::_constructWindow(void)
 	m_spinAdj = gtk_spin_button_get_adjustment( GTK_SPIN_BUTTON(m_spinPercent) );
 
 	// set the dialog title
-	abiDialogSetTitle(window, pSS->getValueUTF8(XAP_STRING_ID_DLG_Zoom_ZoomTitle).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(XAP_STRING_ID_DLG_Zoom_ZoomTitle,s);
+	abiDialogSetTitle(window, s.utf8_str());
 
 	// disable double buffering on our preview
 	gtk_widget_set_double_buffered(m_previewArea, FALSE);  
