@@ -47,6 +47,7 @@
 #include "gr_Graphics.h"
 #include "xap_UnixDialogHelper.h"
 #include "xap_Strings.h"
+#include "xap_Prefs.h"
 
 #include "fv_View.h"
 
@@ -71,12 +72,18 @@ enum {
 	TARGET_UNKNOWN
 } DragDropTypes;
 
+// FIXME Rob: this should be split into a static and a dynamic part.
+// At least the image formats could be added dynamically based on gdk_pixbuf_get_formats ()
+// but we should be able to also dynamically determine document formats by looking at the
+// loaded importers.
 static const GtkTargetEntry drag_types[] =
 	{
 		{"application/rtf", 0, TARGET_URI_LIST},
 		{"text/richtext", 0, TARGET_URI_LIST},
 		{"text/rtf", 0, TARGET_URI_LIST},
 		{"application/msword", 0, TARGET_URI_LIST},
+		{"application/vnd.ms-word", 0, TARGET_URI_LIST},
+		{"application/vnd.sun.xml.writer", 0, TARGET_URI_LIST},
 		{"application/x-applix-word", 0, TARGET_URI_LIST},
 		{"application/x-palm-database", 0, TARGET_URI_LIST},
 		{"application/vnd.palm", 0, TARGET_URI_LIST},
@@ -86,7 +93,7 @@ static const GtkTargetEntry drag_types[] =
 		{"text/xml", 0, TARGET_URI_LIST},
 		{"text/vnd.wap.wml", 0, TARGET_URI_LIST},
 		{"image/png", 0, TARGET_IMAGE},
-		{"image/bmp", 0, TARGET_IMAGE},
+		{"image/jpeg", 0, TARGET_IMAGE},
 		{"image/gif", 0, TARGET_IMAGE},
 		{"image/x-xpixmap", 0, TARGET_IMAGE},
 		{"image/bmp", 0, TARGET_IMAGE},
@@ -122,14 +129,13 @@ s_mapMimeToUriType (const char * uri)
 }
 
 static bool
-s_ensure_uri_on_disk (const gchar * uri, UT_String & outName)
+s_ensure_uri_on_disk (const gchar * uri, UT_UTF8String & outName)
 {
 	GnomeVFSResult    result = GNOME_VFS_OK;
 	GnomeVFSHandle   *handle = NULL;
 	gchar             buffer[1024];
 	GnomeVFSFileSize  bytes_read;
 	GnomeVFSURI 	 *hndl = NULL;	
-
 	outName = "";
 	hndl = gnome_vfs_uri_new (uri);
 	if (hndl == NULL) 
@@ -142,7 +148,30 @@ s_ensure_uri_on_disk (const gchar * uri, UT_String & outName)
 		char * short_name = gnome_vfs_uri_to_string (hndl, static_cast<GnomeVFSURIHideOptions>(GNOME_VFS_URI_HIDE_USER_NAME | GNOME_VFS_URI_HIDE_PASSWORD |
 													 GNOME_VFS_URI_HIDE_HOST_NAME | GNOME_VFS_URI_HIDE_HOST_PORT |
 													 GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD | GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER));
-		outName = short_name;
+		UT_uint32 i = 0;
+		UT_uint32 len = strlen(short_name);
+		UT_UTF8String left = "";
+		while(i < len)
+		{
+			if(short_name[i] != '%')
+			{
+				left += short_name[i];
+				i += 1;
+			}
+			else if(i+2 < len)
+			{
+				char c = 16*(short_name[i+1] - '0') +  short_name[i+2] - '0';
+				left += c;
+				i += 3;
+			}
+			else
+			{
+				left += short_name[i];
+				i +=1;
+			}
+		}
+		outName = left;
+		UT_DEBUGMSG(("short_name %s szTmp %s \n",short_name,outName.utf8_str()));
 		g_free (short_name);
 		gnome_vfs_uri_unref (hndl);
 		return true;
@@ -155,7 +184,30 @@ s_ensure_uri_on_disk (const gchar * uri, UT_String & outName)
 		gnome_vfs_uri_unref (hndl);
 		return false;
 	}
-	outName = outCName;
+	UT_uint32 i = 0;
+	UT_uint32 len = strlen(outCName);
+	UT_UTF8String left = "";
+	while(i < len)
+	{
+		if(outCName[i] != '%')
+		{
+			left += outCName[i];
+			i += 1;
+		}
+		else if(i+2 < len)
+		{
+			char c = 16*(outCName[i+1] - '0') +  outCName[i+2] - '0';
+			left += c;
+			i += 3;
+		}
+		else
+		{
+			left += outCName[i];
+			i +=1;
+		}
+	}
+	outName = left;
+	UT_DEBUGMSG(("outCName %s szTmp %s \n",outCName,outName.utf8_str()));
 	g_free (outCName);
 
 	FILE * onDisk = fdopen (fd, "r");
@@ -185,51 +237,49 @@ s_ensure_uri_on_disk (const gchar * uri, UT_String & outName)
 }
 
 static void
-s_load_image (const UT_String & file, XAP_Frame * pFrame, FV_View * pView)
+s_load_image (const UT_UTF8String & file, XAP_Frame * pFrame, FV_View * pView)
 {
 	IE_ImpGraphic * pIEG = 0;
 	FG_Graphic    * pFG  = 0;
-
-	UT_Error error = IE_ImpGraphic::constructImporter(file.c_str(), 0, &pIEG);
+	UT_Error error = IE_ImpGraphic::constructImporter(file.utf8_str(), 0, &pIEG);
 	if (error != UT_OK || !pIEG)
 		{
-			xxx_UT_DEBUGMSG(("Couldn't construct importer for %s\n", file.c_str()));
+			UT_DEBUGMSG(("Couldn't construct importer for %s\n", file.utf8_str()));
 			return;
 		}
 
-	error = pIEG->importGraphic(file.c_str(), &pFG);
+	error = pIEG->importGraphic(file.utf8_str(), &pFG);
 
 	DELETEP(pIEG);
 
 	if (error != UT_OK || !pFG)
 		{
-			xxx_UT_DEBUGMSG(("Dom: could not import graphic (%s)\n", file.c_str()));
+			UT_DEBUGMSG(("Dom: could not import graphic (%s)\n", file.utf8_str()));
 			return;
 		}
 
-	pView->cmdInsertGraphic(pFG, file.c_str());
+	pView->cmdInsertGraphic(pFG);
 	DELETEP(pFG);
 }
 
 static void
-s_load_document (const UT_String & file, XAP_Frame * pFrame)
+s_load_document (const UT_UTF8String & file, XAP_Frame * pFrame)
 {
 	XAP_Frame * pNewFrame = 0;
-	
 	if (pFrame->isDirty() || pFrame->getFilename() || 
 		(pFrame->getViewNumber() > 0))
 		pNewFrame = XAP_App::getApp()->newFrame ();
 	else
 		pNewFrame = pFrame;
 
-	UT_Error error = pNewFrame->loadDocument(file.c_str(), 0 /* IEFT_Unknown */);
+	UT_Error error = pNewFrame->loadDocument(file.utf8_str(), 0 /* IEFT_Unknown */);
 	if (error)
 		{
 			// TODO: warn user that we couldn't open that file		 
 			// TODO: we crash if we just delete this without putting something
 			// TODO: in it, so let's go ahead and open an untitled document
 			// TODO: for now.
-			xxx_UT_DEBUGMSG(("DOM: couldn't load document %s\n", file.c_str()));
+			xxx_UT_DEBUGMSG(("DOM: couldn't load document %s\n", file.utf8_str()));
 			pNewFrame->loadDocument(NULL, 0 /* IEFT_Unknown */);
 		}
 }
@@ -252,14 +302,14 @@ s_load_uri (XAP_Frame * pFrame, const char * uri)
 			return;
 		}
 
-	UT_String onDisk;
+	UT_UTF8String onDisk;
 	if (!s_ensure_uri_on_disk (uri, onDisk))
 		{
 			xxx_UT_DEBUGMSG(("DOM: couldn't ensure %s on disk\n", uri));
 			return;
 		}
 
-	xxx_UT_DEBUGMSG(("DOM: %s on disk\n", onDisk.c_str()));
+	xxx_UT_DEBUGMSG(("DOM: %s on disk\n", onDisk.utf8_str()));
 
 	if (type == TARGET_IMAGE)
 		{
@@ -340,47 +390,6 @@ static void s_gtkMenuPositionFunc(GtkMenu * /* menu */, gint * x, gint * y, gboo
 	*push_in = TRUE ;
 }
 
-static void wmspec_change_state(bool add, GdkWindow *w, GdkAtom atom1, GdkAtom atom2)
-{
-   XEvent xev;
-#define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
-#define _NET_WM_STATE_ADD           1    /* add/set property */
-#define _NET_WM_STATE_TOGGLE        2    /* toggle property  */  
-
-   xev.xclient.type = ClientMessage;
-   xev.xclient.serial = 0;
-   xev.xclient.send_event = True;
-   xev.xclient.display = gdk_display;
-   xev.xclient.window = GDK_WINDOW_XID (w);
-   xev.xclient.message_type = gdk_x11_get_xatom_by_name ("_NET_WM_STATE");
-   xev.xclient.format = 32;
-   xev.xclient.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE;
-   xev.xclient.data.l[1] = gdk_x11_atom_to_xatom (atom1);
-   xev.xclient.data.l[2] = gdk_x11_atom_to_xatom (atom2);
-   XSendEvent(gdk_display, GDK_WINDOW_XID (gdk_get_default_root_window ()),
-              False, SubstructureRedirectMask | SubstructureNotifyMask,
-              &xev);
-}
-
-static void wmspec_change_layer(bool fullscreen, GdkWindow *window)
-{
-   XEvent xev;
-#define _WIN_LAYER_TOP        -1    /* remove/unset property */
-#define _WIN_LAYER_NORMAL      4    /* add/set property */
-
-   xev.xclient.type = ClientMessage;
-   xev.xclient.serial = 0;
-   xev.xclient.send_event = True;
-   xev.xclient.display = gdk_display;
-   xev.xclient.window = GDK_WINDOW_XID (window);
-   xev.xclient.message_type = gdk_x11_get_xatom_by_name ("_WIN_LAYER");
-   xev.xclient.format = 32;
-   xev.xclient.data.l[0] = fullscreen ? _WIN_LAYER_TOP : _WIN_LAYER_NORMAL ;
-   XSendEvent(gdk_display, GDK_WINDOW_XID (gdk_get_default_root_window ()),
-              False, SubstructureRedirectMask | SubstructureNotifyMask,
-              &xev);
-}
-
 /****************************************************************/
 XAP_UnixFrameImpl::XAP_UnixFrameImpl(XAP_Frame *pFrame, XAP_UnixApp * pApp) : 
 	XAP_FrameImpl(pFrame),
@@ -571,9 +580,12 @@ gint XAP_UnixFrameImpl::_fe::do_ZoomUpdate(gpointer /* XAP_UnixFrameImpl * */ p)
 		iNewWidth = pUnixFrameImpl->m_iNewWidth;
 		iNewHeight = pUnixFrameImpl->m_iNewHeight;
 
+		// don't need this for quickZoom
+#if 0
 		pUnixFrameImpl->_startViewAutoUpdater(); 
+#endif
 		pView->setWindowSize(iNewWidth, iNewHeight);
-		pFrame->updateZoom();
+		pFrame->quickZoom(); // was update zoom
 
 	}
 	while((iNewWidth != pUnixFrameImpl->m_iNewWidth) || (iNewHeight != pUnixFrameImpl->m_iNewHeight));
@@ -590,7 +602,7 @@ gint XAP_UnixFrameImpl::_fe::configure_event(GtkWidget* w, GdkEventConfigure *e)
 	XAP_UnixFrameImpl * pUnixFrameImpl = static_cast<XAP_UnixFrameImpl *>(g_object_get_data(G_OBJECT(w), "user_data"));
 	XAP_Frame* pFrame = pUnixFrameImpl->getFrame();
 	AV_View * pView = pFrame->getCurrentView();
-
+	XAP_Frame::tZoomType zType = pFrame->getZoomType();
 	if (pView)
 	{
 		pUnixFrameImpl->m_iNewWidth = e->width;
@@ -598,7 +610,6 @@ gint XAP_UnixFrameImpl::_fe::configure_event(GtkWidget* w, GdkEventConfigure *e)
 		XAP_App * pApp = pFrame->getApp();
 		UT_sint32 x,y;
 		UT_uint32 width,height,flags;
-
 		pApp->getGeometry(&x,&y,&width,&height,&flags);
 //
 // Who ever wants to change this code in the future. The height and widths you
@@ -618,8 +629,11 @@ gint XAP_UnixFrameImpl::_fe::configure_event(GtkWidget* w, GdkEventConfigure *e)
         pApp->setGeometry(e->x,e->y,gwidth,gheight,flags);
 
 		// Dynamic Zoom Implementation
-        if(!pUnixFrameImpl->m_bDoZoomUpdate && (pUnixFrameImpl->m_iZoomUpdateID == 0))
-            pUnixFrameImpl->m_iZoomUpdateID = g_idle_add(static_cast<GSourceFunc>(do_ZoomUpdate), static_cast<gpointer>(pUnixFrameImpl));
+
+		if(!pUnixFrameImpl->m_bDoZoomUpdate && (pUnixFrameImpl->m_iZoomUpdateID == 0))
+		{
+			pUnixFrameImpl->m_iZoomUpdateID = g_idle_add(static_cast<GSourceFunc>(do_ZoomUpdate), static_cast<gpointer>(pUnixFrameImpl));
+		}
 	}
 	return 1;
 }
@@ -627,6 +641,32 @@ gint XAP_UnixFrameImpl::_fe::configure_event(GtkWidget* w, GdkEventConfigure *e)
 gint XAP_UnixFrameImpl::_fe::motion_notify_event(GtkWidget* w, GdkEventMotion* e)
 {
 	XAP_UnixFrameImpl * pUnixFrameImpl = static_cast<XAP_UnixFrameImpl *>(g_object_get_data(G_OBJECT(w), "user_data"));
+	if(e->type == GDK_MOTION_NOTIFY)
+	{
+		//
+		// swallow queued drag events and just get the last one.
+		//
+		GdkEvent  * eNext = gdk_event_peek();
+		if(eNext && eNext->type == GDK_MOTION_NOTIFY)
+		{
+			g_object_unref(G_OBJECT(e));
+			e = reinterpret_cast<GdkEventMotion *>(eNext);
+			while(eNext && eNext->type == GDK_MOTION_NOTIFY)
+			{
+				xxx_UT_DEBUGMSG(("Swallowing drag event \n"));
+				gdk_event_free(eNext);
+				eNext = gdk_event_get();
+				gdk_event_free(reinterpret_cast<GdkEvent *>(e));
+				e = reinterpret_cast<GdkEventMotion *>(eNext);
+				eNext = gdk_event_peek();
+			}
+			if(eNext != NULL)
+			{
+				gdk_event_free(eNext);
+			}
+		}
+	}
+
 	XAP_Frame* pFrame = pUnixFrameImpl->getFrame();
 	pUnixFrameImpl->setTimeOfLastEvent(e->time);
 	AV_View * pView = pFrame->getCurrentView();
@@ -765,6 +805,10 @@ gint XAP_UnixFrameImpl::_fe::expose(GtkWidget * w, GdkEventExpose* pExposeEvent)
 {
 	XAP_UnixFrameImpl * pUnixFrameImpl = static_cast<XAP_UnixFrameImpl *>(g_object_get_data(G_OBJECT(w), "user_data"));
 	FV_View * pView = static_cast<FV_View *>(pUnixFrameImpl->getFrame()->getCurrentView());
+	if((pUnixFrameImpl->m_bDoZoomUpdate) || (pUnixFrameImpl->m_iZoomUpdateID != 0))
+	{
+		return TRUE;
+	}
 	if(pView)
 	{
 		GR_Graphics * pGr = pView->getGraphics ();
@@ -772,9 +816,9 @@ gint XAP_UnixFrameImpl::_fe::expose(GtkWidget * w, GdkEventExpose* pExposeEvent)
 		xxx_UT_DEBUGMSG(("Expose area: x %d y %d width %d  height %d \n",pExposeEvent->area.x,pExposeEvent->area.y,pExposeEvent->area.width,pExposeEvent->area.height));
 		rClip.left = pGr->tlu(pExposeEvent->area.x);
 		rClip.top = pGr->tlu(pExposeEvent->area.y);
-		rClip.width = pGr->tlu(pExposeEvent->area.width);
-		rClip.height = pGr->tlu(pExposeEvent->area.height);
-
+		rClip.width = pGr->tlu(pExposeEvent->area.width)+1;
+		rClip.height = pGr->tlu(pExposeEvent->area.height)+1;
+		pGr->setExposePending(false);
 		pView->draw(&rClip);
 	}
 	return FALSE;
@@ -867,7 +911,7 @@ void XAP_UnixFrameImpl::_initialize()
 {
     	// get a handle to our keyboard binding mechanism
  	// and to our mouse binding mechanism.
- 	EV_EditEventMapper * pEEM = getFrame()->getEditEventMapper();
+ 	EV_EditEventMapper * pEEM = XAP_App::getApp()->getEditEventMapper();
  	UT_ASSERT(pEEM);
 
 	m_pKeyboard = new ev_UnixKeyboard(pEEM);
@@ -1003,13 +1047,12 @@ void XAP_UnixFrameImpl::_setCursor(GR_Graphics::Cursor c)
 
 UT_sint32 XAP_UnixFrameImpl::_setInputMode(const char * szName)
 {
-	XAP_Frame*	pFrame = getFrame();
-	UT_sint32 result = pFrame->XAP_Frame::setInputMode(szName);
+	UT_sint32 result = XAP_App::getApp()->setInputMode(szName);
 	if (result == 1)
 	{
 		// if it actually changed we need to update keyboard and mouse
 
-		EV_EditEventMapper * pEEM = pFrame->getEditEventMapper();
+		EV_EditEventMapper * pEEM = XAP_App::getApp()->getEditEventMapper();
 		UT_ASSERT(pEEM);
 
 		m_pKeyboard->setEditEventMap(pEEM);
@@ -1226,8 +1269,8 @@ gint XAP_UnixFrameImpl::_imRetrieveSurrounding_cb (GtkIMContext *context, gpoint
 
 	PT_DocPosition begin_p, end_p, here;
 
-	begin_p = pView->mapDocPos (FV_DOCPOS_BOB);
-	end_p = pView->mapDocPos (FV_DOCPOS_EOB);
+	begin_p = pView->mapDocPosSimple (FV_DOCPOS_BOB);
+	end_p = pView->mapDocPosSimple (FV_DOCPOS_EOB);
 	here = pView->getInsPoint ();
 
 	UT_UCSChar * text = pView->getTextBetweenPos (begin_p, end_p);
@@ -1296,8 +1339,8 @@ void XAP_UnixFrameImpl::_setGeometry ()
 	// (ignore app_x, app_y & app_f since the WM will set them for us just fine)
 	  
 	// This is now done with --geometry parsing.
-	if (app_w == 0) app_w = 760;
-	if (app_h == 0) app_h = 520;
+	if (app_w == 0 || app_w > USHRT_MAX) app_w = 760;
+        if (app_h == 0 || app_h > USHRT_MAX) app_h = 520;
 
 	UT_DEBUGMSG(("xap_UnixFrameImpl: app-width=%lu, app-height=%lu\n",
 				 static_cast<unsigned long>(app_w),static_cast<unsigned long>(app_h)));
@@ -1350,13 +1393,16 @@ void XAP_UnixFrameImpl::_setGeometry ()
 			user_h = static_cast<guint>(app_h);
 		}
 
+	if (user_w > USHRT_MAX) user_w = app_w;
+        if (user_h > USHRT_MAX) user_h = app_h;
+
 	GdkGeometry geom;
 	geom.min_width   = 100;
 	geom.min_height  = 100;
 	if(getFrame()->getFrameMode() == XAP_NormalFrame)
 	{
 		gtk_window_set_geometry_hints (GTK_WINDOW(m_wTopLevelWindow), m_wTopLevelWindow, &geom,
-									static_cast<GdkWindowHints>(GDK_HINT_MIN_SIZE));
+									   static_cast<GdkWindowHints>(GDK_HINT_MIN_SIZE));
 
 		gtk_window_set_default_size (GTK_WINDOW(m_wTopLevelWindow), user_w, user_h);
 	}
@@ -1419,7 +1465,7 @@ void XAP_UnixFrameImpl::_rebuildToolbar(UT_uint32 ibar)
 	pToolbar = _newToolbar(m_pUnixApp, pFrame, szTBName,
 			       static_cast<const char *>(m_szToolbarLabelSetName));
 	static_cast<EV_UnixToolbar *>(pToolbar)->rebuildToolbar(oldpos);
-	m_vecToolbars.setNthItem(ibar, static_cast<void *>(pToolbar), NULL);
+	m_vecToolbars.setNthItem(ibar, pToolbar, NULL);
 	// Refill the framedata pointers
 
 	pFrame->refillToolbarsInFrameData();
@@ -1436,7 +1482,8 @@ bool XAP_UnixFrameImpl::_close()
 bool XAP_UnixFrameImpl::_raise()
 {
 	UT_ASSERT(m_wTopLevelWindow);
-	gdk_window_raise(m_wTopLevelWindow->window);
+
+	gtk_window_present(GTK_WINDOW (m_wTopLevelWindow));
 
 	return true;
 }
@@ -1458,18 +1505,11 @@ bool XAP_UnixFrameImpl::_updateTitle()
 		return false;
 	}
 
-	char buf[256];
-	buf[0] = 0;
-
-	const char * szAppName = m_pUnixApp->getApplicationTitleForTitleBar();
-
-	int len = 256 - strlen(szAppName) - 4;
-
-	const char * szTitle = getFrame()->getTitle(len);
-
-	sprintf(buf, "%s - %s", szTitle, szAppName);
 	if(getFrame()->getFrameMode() == XAP_NormalFrame)
-		gtk_window_set_title(GTK_WINDOW(m_wTopLevelWindow), buf);
+	{
+		const char * szTitle = getFrame()->getTitle(MAX_TITLE_LENGTH);
+		gtk_window_set_title(GTK_WINDOW(m_wTopLevelWindow), szTitle);
+	}
 	return true;
 }
 
@@ -1568,152 +1608,13 @@ EV_Menu* XAP_UnixFrameImpl::_getMainMenu()
 	return m_pUnixMenu;
 }
 
-UT_String XAP_UnixFrameImpl::_localizeHelpUrl (bool bLocal, const char * pathBefore, 
-											   const char * pathAfter)
-{
-#if 0 //def HAVE_GNOME
-	UT_String path (pathAfter);
-	path += ".html";
-	return path;
-#else
-	return XAP_FrameImpl::_localizeHelpUrl (bLocal, pathBefore, pathAfter);
-#endif
-}
-
-bool XAP_UnixFrameImpl::_openHelpURL(const char * szURL)
-{
-#if 0 //def HAVE_GNOME
-	GError * err = NULL;	
-
-	UT_DEBUGMSG(("DOM: Help URL: %s\n", szURL));
-
-	gnome_help_display (szURL, NULL, &err);
-	if (err != NULL) {
-		UT_DEBUGMSG(("DOM: help error: %s\n", err->message));
-		g_error_free (err);
-	}
-	return false;
-#else
-	return _openURL (szURL);
-#endif
-}
-
-bool XAP_UnixFrameImpl::_openURL(const char * szURL)
-{
-#ifdef HAVE_GNOME
-	gnome_url_show(szURL, NULL);
-	return false;
-#else
-	static char *fmtstring = NULL;
-  	char *execstring = NULL;
-
-	if(fmtstring)			// lookup browser result when we have
-	  {				// already calculated it once before
-		if(strstr(fmtstring, "netscape"))
-			execstring = g_strdup_printf(fmtstring, szURL, szURL);
-		else
-		  execstring = g_strdup_printf(fmtstring, szURL);
-
-		system(execstring);
-		g_free (execstring);
-		return false;					// why do we return false?
-	}
-
-	// TODO: we should move this into a preferences item,
-	// TODO: but every other platform has a default browser
-	// TODO: or text/html handler in a global registry
-
-	// ORDER:
-	// Use value of environment variable BROWSER, if valid; otherwise:
-	// 1) konqueror
-	// 2) mozilla
-	// 3) netscape
-	// 4) kdehelp
-	// 5) lynx in an xterm
-  	char *env_browser = getenv ("BROWSER");
-  	if (env_browser)
-  	{
-		if(progExists(env_browser))
-		{
-			if (strstr (env_browser, "netscape"))
-			{
-				fmtstring = g_strdup_printf("%s -remote openURL\\('%%s'\\) || %s '%%s' &", env_browser, env_browser);
-				if (fmtstring) execstring = g_strdup_printf(fmtstring, szURL, szURL);
-			}
-			else
-			{
-				fmtstring = g_strdup_printf("%s '%%s' &", env_browser);
-				if (fmtstring) execstring = g_strdup_printf(fmtstring, szURL);
-			}
-		}
-  	}
-	if (fmtstring == 0)
-	{
-		if(progExists("galeon"))
-		{
-		  	fmtstring = "galeon '%s' &";
-			execstring = g_strdup_printf(fmtstring, szURL);
-		}
-		// Anyone know how to find out where it might be, regardless?
-		else if(progExists("mozilla"))
-		{
-		        fmtstring = "mozilla '%s' &";
-			execstring = g_strdup_printf(fmtstring, szURL);
-		}
-		// Anyone know how to find out where it might be, regardless?
-		else if(progExists("phoenix"))
-		{
-		        fmtstring = "phoenix '%s' &";
-			execstring = g_strdup_printf(fmtstring, szURL);
-		}
-		else if(progExists("netscape"))
-		{
-			// Try to connect to a running Netscape, if not, start new one
-			fmtstring = "netscape -remote openURL\\('%s'\\) || netscape '%s' &";
-			execstring = g_strdup_printf(fmtstring, szURL, szURL);
-		}
-		else if(progExists("konqueror"))
-		{
-			fmtstring = "konqueror '%s' &";
-			execstring = g_strdup_printf(fmtstring, szURL);
-		}
-		else if(progExists("khelpcenter"))
-		{
-			fmtstring = "khelpcenter '%s' &";
-			execstring = g_strdup_printf(fmtstring, szURL);
-		}
-		else if(progExists("gnome-help-browser"))
-		{
-			fmtstring = "gnome-help-browser '%s' &";
-			execstring = g_strdup_printf(fmtstring, szURL);
-		}
-		else if(progExists("lynx"))
-		{
-			fmtstring = "xterm -e lynx '%s' &";
-			execstring = g_strdup_printf(fmtstring, szURL);
-		}
-		else if(progExists("w3m"))
-		{
-			fmtstring = "xterm -e w3m '%s' &";
-	 		execstring = g_strdup_printf(fmtstring, szURL);
-	 	}
-	}
-	if (execstring)
-	{
-		system (execstring);
-		g_free (execstring);
-	}
-
-	return false;
-#endif
-}
 
 void XAP_UnixFrameImpl::_setFullScreen(bool changeToFullScreen)
 {
-	wmspec_change_layer(changeToFullScreen, GTK_WIDGET(m_wTopLevelWindow)->window);
-	wmspec_change_state(changeToFullScreen, GTK_WIDGET(m_wTopLevelWindow)->window,
-			    gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN", TRUE),
-			    GDK_NONE);
+	if (changeToFullScreen)
+		gtk_window_fullscreen (GTK_WINDOW(m_wTopLevelWindow));
+	else
+		gtk_window_unfullscreen (GTK_WINDOW(m_wTopLevelWindow));
 }
 
 EV_Toolbar * XAP_UnixFrameImpl::_newToolbar(XAP_App *pApp, XAP_Frame *pFrame,
