@@ -52,7 +52,6 @@ XAP_Win32Dialog_Zoom::XAP_Win32Dialog_Zoom(XAP_DialogFactory * pDlgFactory,
 	: XAP_Dialog_Zoom(pDlgFactory,id)
 {
 	m_pPreviewWidget = NULL;
-	m_bEditPctChanged = false;
 	m_bEditPctEnabled = false;
 }
 
@@ -65,6 +64,8 @@ XAP_Win32Dialog_Zoom::~XAP_Win32Dialog_Zoom(void)
 
 void XAP_Win32Dialog_Zoom::runModal(XAP_Frame * pFrame)
 {
+	m_pFrame = pFrame;
+
 	/*
 	  This dialog is non-persistent.
 	  
@@ -85,6 +86,7 @@ void XAP_Win32Dialog_Zoom::runModal(XAP_Frame * pFrame)
 	  - Save the value in the Percent spin button box to "m_zoomPercent".
 
 	  On "Cancel" the dialog should:
+	  FIXME: remove the Cancel button from HEAD, since we have Quick Zoom now
 
 	  - Just quit, the data items will be ignored by the caller.
 
@@ -127,8 +129,7 @@ BOOL XAP_Win32Dialog_Zoom::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam
 	localizeControlText(XAP_RID_DIALOG_ZOOM_RADIO_WHOLE,	XAP_STRING_ID_DLG_Zoom_WholePage);
 	localizeControlText(XAP_RID_DIALOG_ZOOM_RADIO_PCT,		XAP_STRING_ID_DLG_Zoom_Percent);
 	localizeControlText(XAP_RID_DIALOG_ZOOM_TEXT_PREVIEW,	XAP_STRING_ID_DLG_Zoom_PreviewFrame);
-	localizeControlText(XAP_RID_DIALOG_ZOOM_BTN_OK,			XAP_STRING_ID_DLG_OK);
-	localizeControlText(XAP_RID_DIALOG_ZOOM_BTN_CANCEL,		XAP_STRING_ID_DLG_Cancel);
+	localizeControlText(XAP_RID_DIALOG_ZOOM_BTN_CLOSE,		XAP_STRING_ID_DLG_Close);
 
 	// set initial state
 	checkButton(XAP_RID_DIALOG_ZOOM_RADIO_200 + m_zoomType);
@@ -175,30 +176,35 @@ BOOL XAP_Win32Dialog_Zoom::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 	switch (wId)
 	{
 	case XAP_RID_DIALOG_ZOOM_RADIO_200:
-		_updatePreviewZoomPercent(200);
 		m_zoomType = XAP_Frame::z_200;
+		m_pFrame->setZoomType(m_zoomType);
+		_updatePreviewZoomPercent(200);
 		return 1;
 
 	case XAP_RID_DIALOG_ZOOM_RADIO_100:
-		_updatePreviewZoomPercent(100);
 		m_zoomType = XAP_Frame::z_100;
+		m_pFrame->setZoomType(m_zoomType);
+		_updatePreviewZoomPercent(100);
 		return 1;
 
 	case XAP_RID_DIALOG_ZOOM_RADIO_75:
-		_updatePreviewZoomPercent(75);
 		m_zoomType = XAP_Frame::z_75;
+		m_pFrame->setZoomType(m_zoomType);
+		_updatePreviewZoomPercent(75);
 		return 1;
 
 	case XAP_RID_DIALOG_ZOOM_RADIO_WIDTH:
-		newValue = getApp()->findValidFrame()->getCurrentView()->calculateZoomPercentForPageWidth();
-		_updatePreviewZoomPercent(newValue);
+		newValue = m_pFrame->getCurrentView()->calculateZoomPercentForPageWidth();
 		m_zoomType = XAP_Frame::z_PAGEWIDTH;
+		m_pFrame->setZoomType(m_zoomType);
+		_updatePreviewZoomPercent(newValue);
 		return 1;
 
 	case XAP_RID_DIALOG_ZOOM_RADIO_WHOLE:
-		newValue = getApp()->findValidFrame()->getCurrentView()->calculateZoomPercentForWholePage();
-		_updatePreviewZoomPercent(newValue);
+		newValue = m_pFrame->getCurrentView()->calculateZoomPercentForWholePage();
 		m_zoomType = XAP_Frame::z_WHOLEPAGE;
+		m_pFrame->setZoomType(m_zoomType);
+		_updatePreviewZoomPercent(newValue);
 		return 1;
 
 	case XAP_RID_DIALOG_ZOOM_RADIO_PCT:
@@ -206,43 +212,24 @@ BOOL XAP_Win32Dialog_Zoom::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		{
 			if (_getValueFromEditPct(&newValue))
 			{
-				_updatePreviewZoomPercent(newValue);
 				m_zoomType = XAP_Frame::z_PERCENT;
+				m_pFrame->setZoomType(m_zoomType);
 				m_zoomPercent = newValue;
+				_updatePreviewZoomPercent(newValue);
 			}
 		}
 		return 1;
 
 	case XAP_RID_DIALOG_ZOOM_EDIT_PCT:
-		switch (wNotifyCode)
+		if (_getValueFromEditPct(&newValue))
 		{
-		case EN_CHANGE:
-			m_bEditPctChanged = true;
-			return 1;
-
-		case EN_KILLFOCUS:
-			if (m_bEditPctChanged)
-			{
-				if (_getValueFromEditPct(&newValue))
-				{
-					_updatePreviewZoomPercent(newValue);
-					m_zoomType = XAP_Frame::z_PERCENT;
-					m_zoomPercent = newValue;
-				}
-			}
-			m_bEditPctChanged = false;
-			return 1;
-			
-		default:
-			return 1;
+			m_zoomPercent = newValue;
+			_updatePreviewZoomPercent(newValue);
 		}
-		
-	case IDCANCEL:						// also XAP_RID_DIALOG_ZOOM_BTN_CANCEL
-		m_answer = a_CANCEL;
-		EndDialog(hWnd,0);
 		return 1;
-
-	case IDOK:							// also XAP_RID_DIALOG_ZOOM_BTN_OK
+			
+	case IDCANCEL:						// Zoom is instant-apply, so treat cancel as OK
+	case IDOK:							// also XAP_RID_DIALOG_ZOOM_BTN_CLOSE
 		m_answer = a_OK; 
 		EndDialog(hWnd,0);
 		return 1;
