@@ -22,7 +22,6 @@
 #include "xap_Win32FrameImpl.h"
 #include "commctrl.h"
 #include <limits.h>					/* for INT_MAX */
-#include <winreg.h>
 
 #include "zmouse.h"
 #ifdef __MINGW32__
@@ -37,6 +36,9 @@
 #include "ie_impGraphic.h"
 #include "fv_View.h"
 #include "ap_Win32App.h"
+#include "fg_Graphic.h"
+#include "gr_Win32Graphics.h"
+
 
 #ifdef _MSC_VER
 #pragma warning(disable: 4355)	// 'this' used in base member initializer list
@@ -64,8 +66,6 @@
 
 WHICHPROC s_oldRedBar; 
 
-extern "C" const char * wvLIDToLangConverter(unsigned short langID);
-
 // Where the heck is this function????
 // TODO Fix the following header file. It seems to be incomplete
 // TODO #include <ap_EditMethods.h>
@@ -77,7 +77,7 @@ extern XAP_Dialog_MessageBox::tAnswer s_CouldNotLoadFileMessage(XAP_Frame * pFra
 */
 LRESULT CALLBACK s_rebarWndProc( HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
 {
-	switch (uMessage)	
+	switch (uMessage)
 	{		
 		case WM_DRAWITEM:
 		{
@@ -131,6 +131,11 @@ XAP_Win32FrameImpl::~XAP_Win32FrameImpl(void)
 	
 	DELETEP(m_pWin32Menu);
 	DELETEP(m_pWin32Popup);
+
+	// have to reset the window long, so our message functions do not
+	// try to derefernce it
+	SetWindowLong(m_hwndFrame, GWL_USERDATA,0);
+
 }
 
 
@@ -157,7 +162,7 @@ bool XAP_Win32FrameImpl::_updateTitle(void)
 	sTmp += " - ";
 	sTmp += XAP_App::getApp()->getApplicationTitleForTitleBar();
 	
-	SetWindowText(m_hwndFrame, (AP_Win32App::s_fromUTF8ToAnsi(sTmp.c_str())).c_str());
+	SetWindowText(m_hwndFrame, (AP_Win32App::s_fromUTF8ToWinLocale(sTmp.c_str())).c_str());
 
 	return true;
 }
@@ -169,7 +174,7 @@ void XAP_Win32FrameImpl::_initialize(void)
 	// get a handle to our keyboard binding mechanism
 	// and to our mouse binding mechanism.
 	
-	EV_EditEventMapper * pEEM = getFrame()->getEditEventMapper();
+	EV_EditEventMapper * pEEM = XAP_App::getApp()->getEditEventMapper();
 	UT_return_if_fail(pEEM);
 
 	m_pKeyboard = new ev_Win32Keyboard(pEEM);
@@ -235,7 +240,7 @@ void XAP_Win32FrameImpl::_createTopLevelWindow(void)
 	// synthesize a menu from the info in our
 	// base class and install it into the window.
 	m_pWin32Menu = new EV_Win32MenuBar(pWin32App,
-							getFrame()->getEditEventMapper(),
+							 XAP_App::getApp()->getEditEventMapper(),
 							m_szMenuLayoutName,
 							m_szMenuLabelSetName);
 	UT_ASSERT(m_pWin32Menu);
@@ -318,7 +323,6 @@ bool XAP_Win32FrameImpl::_close(void)
 	// last window closed is the one that the window state is stored from.
 	WINDOWPLACEMENT wndPlacement;
 	wndPlacement.length = sizeof(WINDOWPLACEMENT); // must do
-	
 	if (GetWindowPlacement(m_hwndFrame, &wndPlacement))
 	{
 		UT_uint32 nFlags = PREF_FLAG_GEOMETRY_POS | PREF_FLAG_GEOMETRY_SIZE;
@@ -379,7 +383,9 @@ EV_Toolbar * XAP_Win32FrameImpl::_newToolbar(XAP_App *app, XAP_Frame *frame, con
 	// for now, position each one manually
 	// TODO: put 'em all in a rebar instead
 	HWND hwndBar = result->getWindow();
-	
+	if (!hwndBar) 
+		return result;
+
 	RECT rcClient;
 	GetClientRect(hwndBar, &rcClient);
 	const UT_uint32 iHeight = rcClient.bottom - rcClient.top;
@@ -475,154 +481,6 @@ void XAP_Win32FrameImpl::_setFullScreen(bool isFullScreen)
 }
 
 
-bool XAP_Win32FrameImpl::_openURL(const char * szURL)
-{
-	// NOTE: could get finer control over browser window via DDE 
-	// NOTE: may need to fallback to WinExec for old NSCP versions
-
-	UT_String sURL = szURL;
-
-	// If this is a file:// URL, strip off file:// and make it backslashed
-	if (sURL.substr(0, 7) == "file://")
-	{
-		sURL = sURL.substr(7, sURL.size() - 7);
-
-		// Strip off the leading backslash
-		if (sURL.substr(0, 1) == "\\")
-			sURL = sURL.substr(2, sURL.size() - 2);
-
-		// View as WebPage likes to throw in an extra /\ just for fun, strip it off
-		if (sURL.substr(0, 2) == "/\\")
-			sURL = sURL.substr(2, sURL.size() - 2);
-
-		// Convert all forwardslashes to backslashes
-		for (unsigned int i=0; i<sURL.length();i++)	
-			if (sURL[i]=='/')	sURL[i]='\\';
-
-		// Convert from longpath to 8.3 shortpath, in case of spaces in the path
-		char* longpath = NULL;
-		char* shortpath = NULL;
-		longpath = new char[MAX_PATH];
-		shortpath = new char[MAX_PATH];
-		strcpy(longpath, sURL.c_str());
-		GetShortPathName(longpath, shortpath, MAX_PATH);
-		sURL = shortpath;
-		DELETEP(longpath);
-		DELETEP(shortpath);
-	}
-
-	// Query the registry for the default browser so we can directly invoke it
-	UT_String sBrowser;
-	HKEY hKey;
-	unsigned long lType;
-	DWORD dwSize;
-	unsigned char* szValue = NULL;
-
-	if (RegOpenKeyEx(HKEY_CLASSES_ROOT, "http\\shell\\open\\command", 0, KEY_READ, &hKey) == ERROR_SUCCESS)
-	{
-		if(RegQueryValueEx(hKey, NULL, NULL, &lType, NULL, &dwSize) == ERROR_SUCCESS)
-		{
-			szValue = new unsigned char[dwSize + 1];
-			RegQueryValueEx(hKey, NULL, NULL, &lType, szValue, &dwSize);
-			sBrowser = (char*) szValue;
-			DELETEP(szValue);
-		}
-		RegCloseKey(hKey);
-	}
-
-	/* Now that we have sBrowser from the registry, we need to parse it out.
-	 * If the first character is a double-quote, everything up to and including
-	 * the next double-quote is the sBrowser command. Everything after the
-	 * double-quote is appended to the parameters.
-	 * If the first character is NOT a double-quote, we assume
-	 * everything up to the first whitespace is the command and anything after
-	 * is appended to the parameters.
-	 */
-
-	int iDelimiter;
-	if (sBrowser.substr(0, 1) == "\"")
-		iDelimiter = UT_String_findCh(sBrowser.substr(1, sBrowser.length()-1), '"')+2;
-	else
-		iDelimiter = UT_String_findCh(sBrowser.substr(0, sBrowser.length()), ' ');
-
-	// Store params into a separate UT_String before we butcher sBrowser
-	UT_String sParams = sBrowser.substr(iDelimiter+1, sBrowser.length()-iDelimiter+1);
-	// Cut params off of sBrowser so all we're left with is the broweser path & executable
-	sBrowser = sBrowser.substr(0, iDelimiter);
-
-	// Check for a %1 passed in from the registry.  If we find it,
-	// substitute our URL for %1.  Otherwise, just append sURL to params.
-	char *pdest = strstr(sParams.c_str(), "%1");
-	if (pdest != NULL)
-	{
-		int i = pdest - sParams.c_str() + 1;
-		sParams = sParams.substr(0, i-1) + sURL + sParams.substr(i+1, sParams.length()-i+1);
-	}
-	else
-	{
-		sParams = sParams + " " + sURL;
-	}
-
-	// Win95 doesn't like the Browser command to be quoted, so strip em off.
-	if (sBrowser.substr(0, 1) == "\"")
-		sBrowser = sBrowser.substr(1, sBrowser.length() - 1);
-	if (sBrowser.substr(sBrowser.length()-1, 1) == "\"")
-		sBrowser = sBrowser.substr(0, sBrowser.length() - 1);
-
-	int res = (int) ShellExecute(m_hwndFrame /*(HWND) top level window */, "open", sBrowser.c_str(), sParams.c_str(), NULL, SW_SHOW);
-
-	// TODO: localized error messages
-	// added more specific error messages as documented in http://msdn.microsoft.com/library/default.asp?url=/library/en-us/debug/base/system_error_codes.asp
-
-	if (res <= 32)	// show error message if failed to launch browser to display URL
-	{
-		UT_String errMsg;
-		switch (res)
-		{
-			case ERROR_FILE_NOT_FOUND:
-				{
-					errMsg = "Error ("; 
-					errMsg += UT_String_sprintf("%d", res);
-					errMsg += ") displaying URL: The system cannot find the file specified.\n";
-					errMsg += " [ ";  errMsg += sURL;  errMsg += " ] ";
-					MessageBox(m_hwndFrame, errMsg.c_str(), "Error displaying URL", MB_OK|MB_ICONEXCLAMATION);
-				}
-				break;
-
-			case ERROR_PATH_NOT_FOUND:
-				{
-					errMsg = "Error ("; 
-					errMsg += UT_String_sprintf("%d", res);
-					errMsg += ") displaying URL: The system cannot find the path specified.\n";
-					errMsg += " [ ";  errMsg += sURL;  errMsg += " ] ";
-					MessageBox(m_hwndFrame, errMsg.c_str(), "Error displaying URL", MB_OK|MB_ICONEXCLAMATION);
-				}
-				break;
-
-			case SE_ERR_ACCESSDENIED:
-				{
-					errMsg = "Error ("; 
-					errMsg += UT_String_sprintf("%d", res);
-					errMsg += ") displaying URL: Access is denied.\n";
-					errMsg += " [ ";  errMsg += sURL;  errMsg += " ] ";
-					MessageBox(m_hwndFrame, errMsg.c_str(), "Error displaying URL", MB_OK|MB_ICONEXCLAMATION);
-				}
-				break;
-
-			default:
-				{
-					errMsg = "Error ("; 
-					errMsg += UT_String_sprintf("%d", res);
-					errMsg += ") displaying URL: \n";
-					errMsg += " [ ";  errMsg += sURL;  errMsg += " ] ";
-					MessageBox(m_hwndFrame, errMsg.c_str(), "Error displaying URL", MB_OK|MB_ICONEXCLAMATION);
-				}
-				break;
-		} /* switch (res) */
-	} /* if (res <= 32) */
-
-	return (res>32);
-}
 
 
 void XAP_Win32FrameImpl::_nullUpdate (void) const
@@ -639,7 +497,16 @@ void XAP_Win32FrameImpl::_nullUpdate (void) const
 
 void XAP_Win32FrameImpl::_setCursor(GR_Graphics::Cursor cursor)
 {
-	// TODO: currently does nothing
+	FV_View* pView = (FV_View *) getFrame()->getCurrentView();
+
+	// this is legitimate, when we first load, the frame has no view
+	if(!pView)
+		return;
+	
+	GR_Win32Graphics * pG = (GR_Win32Graphics*)pView->getGraphics();
+	UT_return_if_fail( pG );
+
+	pG->handleSetCursorMessage();
 }
 
 UT_RGBColor XAP_Win32FrameImpl::getColorSelBackground () const
@@ -715,9 +582,30 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 	switch (iMsg)
 	{
 	
-	
-	case WM_EXITMENULOOP:
 	case WM_SETFOCUS:
+		{
+			// on set focus we want to make sure that the kbd layout
+			// is the same as it was before we lost focus. If the user
+			// was in some other application and changed the layout in
+			// there to something else and now returns to AW then we
+			// should not be in that different layout (this should not
+			// happen as the layouts are supposed to be thread
+			// specific, but I noted that sometimes even thought we
+			// have the correct layout, the icon on the task bar is
+			// out of sync with our layout; this should fix that).
+			XAP_Win32App *pWin32App = static_cast<XAP_Win32App *>(XAP_App::getApp());
+			UT_ASSERT( pWin32App );
+
+			if(pWin32App)
+			{
+				ActivateKeyboardLayout(pWin32App->getHKL(),0);
+			}
+			
+			
+			// fall through ...
+		}
+			
+	case WM_EXITMENULOOP:
 		if (pView)
 		{
 			pView->focusChange(AV_FOCUS_HERE);
@@ -986,47 +874,20 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 
 	case WM_INPUTLANGCHANGE:
 	{
-		UT_DEBUGMSG(("Frame received input language change\n"));
+		UT_DEBUGMSG(("XAP_Win32FrameImpl::_FrameWndProc: received input language change\n"));
 
 		// This will remap the static tables used by all frames.
 		// (see the comment in ev_Win32Keyboard.cpp.)
 		ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(fimpl->m_pKeyboard);
 		pWin32Keyboard->remapKeyboard((HKL)lParam);
 
-		// we also want to automatically change the language at
-		// the insertion point, unless the user preferences tell us
-		// otherwise
+		// set the language
+		XAP_Win32App *pWin32App = static_cast<XAP_Win32App *>(XAP_App::getApp());
+		UT_ASSERT( pWin32App );
 
-	    bool bChangeLang = true;
-		XAP_App::getApp()->getPrefsValueBool(XAP_PREF_KEY_ChangeLanguageWithKeyboard, &bChangeLang);
+		if(pWin32App)
+			pWin32App->setKbdLanguage((HKL)lParam);
 
-		if(bChangeLang)
-		{
-			WORD langID = LANGIDFROMLCID(LOWORD((HKL)lParam));
-			const char * pszLang = wvLIDToLangConverter((unsigned short)langID);
-			UT_DEBUGMSG(("Keyboard Language: %s\n",pszLang));
-			
-			// now invoke the appropriet formatting method ...
-			EV_EditMethodCallData CallData(pszLang,strlen(pszLang));
-
-			XAP_App * pApp = f->getApp();
-			UT_ASSERT(pApp);
-
-			const EV_EditMethodContainer * pEMC = pApp->getEditMethodContainer();
-			UT_ASSERT(pEMC);
-
-			if(pEMC)
-			{
-				EV_EditMethod * pEM = pEMC->findEditMethodByName("language");
-				UT_ASSERT(pEM);						// make sure it's bound to something
-			
-				if (pEM)
-				{
-					pEM->Fn(pView,&CallData);
-				}
-			}
-		}
-		
 		// Do not propagate this message.
 		
 		return 1; //DefWindowProc(hwnd, iMsg, wParam, lParam);
@@ -1081,7 +942,7 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 					IEGraphicFileType iegft = IEGFT_Unknown;					
 					IE_ImpGraphic *pIEG;
 					FG_Graphic* pFG;
-					UT_Error errorCode;
+					UT_Error errorCode = UT_ERROR;
 
 					/*
 						The user may be dropping any kind of file
@@ -1093,26 +954,16 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 					if(errorCode == UT_OK)
 					{						
 						errorCode = pIEG->importGraphic(szFileName, &pFG);
-						if(errorCode != UT_OK || !pFG)
-						{
-							s_CouldNotLoadFileMessage(f, szFileName, errorCode);							
-							DELETEP(pIEG);
-							return 0;
-						}
 
-						DELETEP(pIEG);					
-
-						errorCode = pView->cmdInsertGraphic(pFG, szFileName);
-						if (errorCode != UT_OK)
+						DELETEP(pIEG);
+						if(errorCode == UT_OK && pFG)
 						{
-							s_CouldNotLoadFileMessage(f, szFileName, errorCode);							
-							DELETEP(pFG);
-							return 0;
+						  errorCode = pView->cmdInsertGraphic(pFG);
+						  DELETEP(pFG);
 						}
-						
-						DELETEP(pFG);
 					  }
-					else
+
+					if (errorCode != UT_OK)
 					{	
 						// Check if the current document is empty.
 						if (f->isDirty() || f->getFilename() ||
@@ -1121,8 +972,9 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 							pNewFrame = pApp->newFrame();
 							if (pNewFrame == NULL)
 							{
-								f->setStatusMessage("Could not open another window");
-								return 0;
+							  errorCode = UT_ERROR;
+							  f->setStatusMessage("Could not open another window");
+							  return 0;
 							}
 						}
 						else
@@ -1130,18 +982,22 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 							pNewFrame = f;
 						}
 
-						UT_Error error = pNewFrame->loadDocument(szFileName, IEFT_Unknown);
-						if (error != UT_OK)
+						errorCode = pNewFrame->loadDocument(szFileName, IEFT_Unknown);
+						if (errorCode != UT_OK)
 						{
 							if (f != pNewFrame)
 								pNewFrame->close();
-							s_CouldNotLoadFileMessage(f, szFileName, error);
 						}
 						else
 						{
 							pNewFrame->show();
 						}
 					  }
+
+					if (errorCode != UT_OK)
+					  {
+					    s_CouldNotLoadFileMessage(f, szFileName, errorCode);
+					  }
 				}
 				else
 				{
@@ -1158,4 +1014,3 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 
 	return DefWindowProc(hwnd, iMsg, wParam, lParam);
 }
-
