@@ -1,5 +1,6 @@
 /* AbiSource Application Framework
- * Copyright (C) 1998,1999 AbiSource, Inc.
+ * Copyright (C) 1998, 1999 AbiSource, Inc.
+ * Copyright (C) 2004 Hubert Figuiere
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,13 +25,18 @@
 
 #include "ut_types.h"
 #include "ut_assert.h"
+#include "ut_path.h"
 #include "ut_string.h"
 #include "ut_debugmsg.h"
+#include "ut_Language.h"
+#include "ut_uuid.h"
 #include "ev_EditMethod.h"
 #include "ev_Menu_Actions.h"
 #include "ev_Toolbar_Actions.h"
 #include "xap_App.h"
+#include "xap_AppImpl.h"
 #include "xap_Args.h"
+#include "xap_InputModes.h"
 #include "gr_Image.h"
 #include "xap_Frame.h"
 #include "xap_EditMethods.h"
@@ -42,10 +48,12 @@
 #include "xap_EncodingManager.h"
 #include "xap_Menu_Layouts.h"
 #include "xap_Toolbar_Layouts.h"
+#include "xav_View.h"
 #include "ut_rand.h"
-#include "ut_contextGlyph.h"
 #include "ut_map.h"
 #include "gr_CharWidthsCache.h"
+#include "gr_ContextGlyph.h"
+#include "xav_Listener.h"
 
 UT_Map * abi_ut_map_instance = 0;
 
@@ -73,11 +81,22 @@ XAP_App::XAP_App(XAP_Args * pArgs, const char * szAppName)
 	  m_bAreCustomized(true),
 	  m_bDebugBool(false),
 	  m_bBonoboRunning(false),
-	  m_bEnableSmoothScrolling(true)
+	  m_bEnableSmoothScrolling(true),
+	  m_pKbdLang(NULL), // must not be deleted by destructor !!!
+ 	  m_pUUIDGenerator(NULL),
+	  m_pGraphicsFactory(NULL),
+	  m_iDefaultGraphicsId(0),
+	  m_pInputModes(NULL),
+	  m_pImpl(NULL)
 {
 #ifdef DEBUG
 	_fundamentalAsserts(); // see the comments in the function itself
 #endif
+
+	m_pGraphicsFactory = new GR_GraphicsFactory;
+	UT_ASSERT( m_pGraphicsFactory );
+	
+	m_pImpl = XAP_AppImpl::static_constructor();
 	
 	UT_ASSERT(szAppName && *szAppName);
 	m_pApp = this;
@@ -88,6 +107,7 @@ XAP_App::XAP_App(XAP_Args * pArgs, const char * szAppName)
 	m_pMenuFactory = new XAP_Menu_Factory(this);
 	m_pToolbarFactory = new XAP_Toolbar_Factory(this);
 	clearIdTable();
+	m_vecPluginListeners.clear(); // Just to be safe
 
 	/* hack to force the linker to link in UT_Map functions
 	 */
@@ -123,12 +143,16 @@ XAP_App::~XAP_App()
 	// Delete the instance of the Encoding Manager.
 	XAP_EncodingManager::get_instance()->Delete_instance();
 
-	// This is to delete static data allocated by UT_contextGlyph; it
+	// This is to delete static data allocated by Gr_ContextGlyph; it
 	// is strictly speaking not necessary -- this is really to shut up
 	// the debugger complaining about memory leaks
-	UT_contextGlyph cg(true);
-	cg.static_destructor();
+	GR_ContextGlyph::static_destructor();
 	GR_CharWidthsCache::destroyCharWidthsCache();
+
+	DELETEP(m_pUUIDGenerator);
+	DELETEP(m_pGraphicsFactory);
+	DELETEP(m_pInputModes);
+	DELETEP(m_pImpl);
 }
 
 const char* XAP_App::getBuildId ()
@@ -182,11 +206,15 @@ EV_Toolbar_ActionSet *XAP_App::getToolbarActionSet()
 	return m_pToolbarActionSet;
 }
 
-bool XAP_App::initialize()
+bool XAP_App::initialize(const char * szKeyBindingsKey, const char * szKeyBindingsDefaultValue)
 {
 	// create application-wide resources that
 	// are shared by everything.
 
+	// init keyboard language (cannot be in the constructor as it
+	// requires the platform code already initialised
+	setKbdLanguage(_getKbdLanguage());
+	
 #if 0 
 	m_pEMC = AP_GetEditMethods();
 	UT_ASSERT(m_pEMC);
@@ -245,9 +273,163 @@ bool XAP_App::initialize()
 	UT_uint32 t = static_cast<UT_uint32>(time(NULL));
 	UT_srandom(t);
 
+	// Input mode initilization, taken out of the XAP_Frame
+	const char * szBindings = NULL;
+	EV_EditBindingMap * pBindingMap = NULL;
+
+	if ((getPrefsValue(szKeyBindingsKey,
+				 static_cast<const XML_Char**>(&szBindings))) && 
+	    (szBindings) && (*szBindings))
+		pBindingMap = m_pApp->getBindingMap(szBindings);
+	if (!pBindingMap)
+		pBindingMap = m_pApp->getBindingMap(szKeyBindingsDefaultValue);
+	UT_ASSERT(pBindingMap);
+
+	if (!m_pInputModes)
+	{
+		m_pInputModes = new XAP_InputModes();
+		UT_ASSERT(m_pInputModes);
+	}
+	bool bResult;
+	bResult = m_pInputModes->createInputMode(szBindings,pBindingMap);
+	UT_ASSERT(bResult);
+	bool bResult2;
+	bResult2 = m_pInputModes->setCurrentMap(szBindings);
+	UT_ASSERT(bResult2);
+
+	// check if the prefs are set to use specific graphics class
+	const char * pszGraphics = NULL;
+	if(getPrefsValue(XAP_PREF_KEY_DefaultGraphics, &pszGraphics))
+	{
+		UT_uint32 iID = 0;
+
+		// please leave this in hex format (the constants are defined in gr_Graphics.h as hex)
+		sscanf(pszGraphics,"%x", &iID);
+		if(iID != 0)
+		{
+			UT_DEBUGMSG(("Graphics %d requested as default\n", iID));
+
+			// first of all, check that it is registered
+			GR_GraphicsFactory * pGF = getGraphicsFactory();
+			UT_return_val_if_fail(pGF,false);
+
+			if(pGF->isRegistered(iID))
+			{
+				pGF->registerAsDefault(iID, true);
+				pGF->registerAsDefault(iID, false);
+			}
+			else
+			{
+				UT_DEBUGMSG(("Graphics not loaded\n"));
+			}
+		}
+	}
+	
 	return true;
 }
 
+/*!
+ * Plugins can register themselves here if they want to receive
+ * notifications of document changes.
+ */
+bool XAP_App::addListener(AV_Listener * pListener, 
+							 AV_ListenerId * pListenerId)
+{
+	UT_uint32 kLimit = m_vecPluginListeners.getItemCount();
+	UT_uint32 k;
+
+	// see if we can recycle a cell in the vector.
+	UT_DEBUGMSG(("Asked to register pListener %x \n",pListener));
+	
+	for (k=0; k<kLimit; k++)
+		if (m_vecPluginListeners.getNthItem(k) == 0)
+		{
+			static_cast<void>(m_vecPluginListeners.setNthItem(k,pListener,NULL));
+			goto ClaimThisK;
+		}
+
+	// otherwise, extend the vector for it.
+	
+	if (m_vecPluginListeners.addItem(pListener,&k) != 0)
+	{
+		UT_DEBUGMSG(("Failed! id %d \n",k));
+		return false;				// could not add item to vector
+	}
+
+  ClaimThisK:
+
+	// give our vector index back to the caller as a "Listener Id".
+	
+	*pListenerId = k;
+	return true;
+}
+
+
+/*!
+ * Plugins must remove themselves if they've registered.
+ */
+bool XAP_App::removeListener(AV_ListenerId listenerId)
+{
+	if (listenerId == (AV_ListenerId) -1)
+		return false;
+		
+	return (m_vecPluginListeners.setNthItem(listenerId,NULL,NULL) == 0);
+}
+
+/*!
+ * Send notifications to all the registered plugins
+ */
+bool XAP_App::notifyListeners(AV_View * pView, const AV_ChangeMask hint)
+{
+	/*
+		App-specific logic calls this virtual method when relevant portions of 
+		the view state *may* have changed.  (That's why it's called a hint.)
+
+		This base class implementation doesn't do any filtering of those 
+		hints, it just broadcasts those hints to all listeners.  
+
+		Subclasses are encouraged to improve the quality of those hints by 
+		filtering out mask bits which haven't *actually* changed since the 
+		last notification.  To do so, they would 
+
+			- copy the hint (it's passed as a const),
+			- clear any irrelevant bits, and 
+			- call this implementation on the way out
+
+		Good hinting logic is app-specific, and non-trivial to tune, but it's 
+		worth doing, because it helps minimizes flicker for things like 
+		toolbar button state.  
+	*/
+
+	// make sure there's something left
+
+	if (hint == AV_CHG_NONE)
+	{
+		return false;
+	}
+	
+	// notify listeners of a change.
+		
+	AV_ListenerId lid;
+	AV_ListenerId lidCount = m_vecPluginListeners.getItemCount();
+
+	// for each listener in our vector, we send a notification.
+	// we step over null listners (for listeners which have been
+	// removed (views that went away)).
+
+	for (lid=0; lid<lidCount; lid++)
+	{
+		AV_Listener * pListener = static_cast<AV_Listener *>(m_vecPluginListeners.getNthItem(lid));
+		if(pListener)
+		{
+				pListener->notify(pView,hint);
+		}
+	}
+
+	return true;
+}
+
+
 void XAP_App::resetToolbarsToDefault(void)
 {
 	//
@@ -255,18 +437,18 @@ void XAP_App::resetToolbarsToDefault(void)
 	//
 	m_pToolbarFactory->resetAllToolbarsToDefault();
 	UT_uint32 count = m_vecFrames.getItemCount();
-	UT_Vector vClones;
+	UT_GenericVector<XAP_Frame*> vClones;
 	UT_uint32 i = 0;
 	for(i=0; i< count; i++)
 	{
-		XAP_Frame * pFrame = static_cast<XAP_Frame *>(m_vecFrames.getNthItem(i));
+		XAP_Frame * pFrame = m_vecFrames.getNthItem(i);
 		if(pFrame->getViewNumber() > 0)
 		{
 			getClones(&vClones,pFrame);
 			UT_uint32 j=0;
 			for(j=0; j < vClones.getItemCount(); j++)
 			{
-				XAP_Frame * f = static_cast<XAP_Frame *>(vClones.getNthItem(j));
+				XAP_Frame * f = vClones.getNthItem(j);
 				f->rebuildAllToolbars();
 			}
 		}
@@ -351,37 +533,36 @@ bool XAP_App::rememberFrame(XAP_Frame * pFrame, XAP_Frame * pCloneOf)
 	if (pCloneOf)
 	{
 		// locate vector of this frame's clones
-		void * pEntry = const_cast<void *>(m_hashClones.pick(pCloneOf->getViewKey()));
-		UT_Vector * pvClones = NULL;
+		UT_GenericVector<XAP_Frame*> * pEntry = m_hashClones.pick(pCloneOf->getViewKey());
+		UT_GenericVector<XAP_Frame*> * pvClones = NULL;
 
 		if (pEntry)
 		{
 			// hash table entry already exists
-			pvClones = static_cast<UT_Vector *>(pEntry);
+			pvClones = pEntry;
 
 			if (!pvClones)
 			{
 				// nothing there, so create a new one
-				pvClones = new UT_Vector();
+				pvClones = new UT_GenericVector<XAP_Frame*>();
 				UT_ASSERT(pvClones);
 
 				pvClones->addItem(pCloneOf);
 
 				// reuse this slot
-				m_hashClones.set(pCloneOf->getViewKey(), 
-						 static_cast<void *>(pvClones));
+				m_hashClones.set(pCloneOf->getViewKey(), pvClones);
 			}
 		}
 		else
 		{
 			// create a new one
-			pvClones = new UT_Vector();
+			pvClones = new UT_GenericVector<XAP_Frame*>();
 			UT_ASSERT(pvClones);
 
 			pvClones->addItem(pCloneOf);
 
 			// add it to the hash table
-			m_hashClones.insert(pCloneOf->getViewKey(), static_cast<void *>(pvClones));
+			m_hashClones.insert(pCloneOf->getViewKey(), pvClones);
 		}
 
 		pvClones->addItem(pFrame);
@@ -389,7 +570,7 @@ bool XAP_App::rememberFrame(XAP_Frame * pFrame, XAP_Frame * pCloneOf)
 		// notify all clones of their new view numbers
 		for (UT_uint32 j=0; j<pvClones->getItemCount(); j++)
 		{
-			XAP_Frame * f = static_cast<XAP_Frame *>(pvClones->getNthItem(j));
+			XAP_Frame * f = pvClones->getNthItem(j);
 			UT_ASSERT(f);
 
 			f->setViewNumber(j+1);
@@ -400,6 +581,7 @@ bool XAP_App::rememberFrame(XAP_Frame * pFrame, XAP_Frame * pCloneOf)
 	}
 	
 	// TODO do something here...
+	notifyFrameCountChange();
 	return true;
 }
 
@@ -418,12 +600,12 @@ bool XAP_App::forgetFrame(XAP_Frame * pFrame)
 	if (pFrame->getViewNumber() > 0)
 	{
 		// locate vector of this frame's clones
-		void * pEntry = const_cast<void *>(m_hashClones.pick(pFrame->getViewKey()));
+		UT_GenericVector<XAP_Frame*>* pEntry = m_hashClones.pick(pFrame->getViewKey());
 		UT_ASSERT(pEntry);
 
 		if (pEntry)
 		{
-			UT_Vector * pvClones = static_cast<UT_Vector *>(pEntry);
+			UT_GenericVector<XAP_Frame*> * pvClones = pEntry;
 			UT_ASSERT(pvClones);
 
 			// remove this frame from the vector
@@ -443,7 +625,7 @@ bool XAP_App::forgetFrame(XAP_Frame * pFrame)
 			if (count == 1)
 			{
 				// remaining clone is now a singleton
-				f = static_cast<XAP_Frame *>(pvClones->getNthItem(count-1));
+				f = pvClones->getNthItem(count-1);
 				UT_ASSERT(f);
 
 				f->setViewNumber(0);
@@ -476,6 +658,7 @@ bool XAP_App::forgetFrame(XAP_Frame * pFrame)
 	if (ndx >= 0)
 	{
 		m_vecFrames.deleteNthItem(ndx);
+		notifyFrameCountChange();
 	}
 
 	notifyModelessDlgsCloseFrame(pFrame);
@@ -494,7 +677,7 @@ bool XAP_App::forgetClones(XAP_Frame * pFrame)
 		return forgetFrame(pFrame);
 	}
 
-	UT_Vector vClones;
+	UT_GenericVector<XAP_Frame*> vClones;
 	getClones(&vClones, pFrame);
 	
 	for (UT_uint32 i = 0; i < vClones.getItemCount(); i++)
@@ -506,17 +689,15 @@ bool XAP_App::forgetClones(XAP_Frame * pFrame)
 	return true;
 }
 
-bool XAP_App::getClones(UT_Vector *pvClonesCopy, XAP_Frame * pFrame)
+bool XAP_App::getClones(UT_GenericVector<XAP_Frame*> *pvClonesCopy, XAP_Frame * pFrame)
 {
 	UT_ASSERT(pvClonesCopy);
 	UT_ASSERT(pFrame);
 	UT_ASSERT(pFrame->getViewNumber() > 0);
 
 	// locate vector of this frame's clones
-	void * pEntry = const_cast<void *>(m_hashClones.pick(pFrame->getViewKey()));
-	UT_ASSERT(pEntry);
 
-	UT_Vector * pvClones = static_cast<UT_Vector *>(pEntry);
+	UT_GenericVector<XAP_Frame*> * pvClones = m_hashClones.pick(pFrame->getViewKey());
 	UT_ASSERT(pvClones);
 
 	return pvClonesCopy->copy(pvClones);
@@ -528,12 +709,12 @@ bool XAP_App::updateClones(XAP_Frame * pFrame)
 	UT_ASSERT(pFrame->getViewNumber() > 0);
 
 	// locate vector of this frame's clones
-	void * pEntry = const_cast<void *>(m_hashClones.pick(pFrame->getViewKey()));
+	UT_GenericVector<XAP_Frame*>* pEntry = m_hashClones.pick(pFrame->getViewKey());
 	UT_ASSERT(pEntry);
 
 	if (pEntry)
 	{
-		UT_Vector * pvClones = static_cast<UT_Vector *>(pEntry);
+		UT_GenericVector<XAP_Frame*>* pvClones = pEntry;
 		UT_ASSERT(pvClones);
 
 		UT_uint32 count = pvClones->getItemCount();
@@ -542,7 +723,7 @@ bool XAP_App::updateClones(XAP_Frame * pFrame)
 
 		for (UT_uint32 j=0; j<count; j++)
 		{
-			f = static_cast<XAP_Frame *>(pvClones->getNthItem(j));
+			f = pvClones->getNthItem(j);
 			UT_ASSERT(f);
 
 			f->updateTitle();
@@ -552,6 +733,11 @@ bool XAP_App::updateClones(XAP_Frame * pFrame)
 	return true;
 }
 
+void XAP_App::notifyFrameCountChange() // default is empty method
+{
+	UT_DEBUGMSG(("XAP_App::notifyFrameCountChange(): count=%lu\n", static_cast<unsigned long>(getFrameCount())));
+}
+
 UT_uint32 XAP_App::getFrameCount() const
 {
 	return m_vecFrames.getItemCount();
@@ -563,7 +749,7 @@ XAP_Frame * XAP_App::getFrame(UT_uint32 ndx) const
 	
 	if (ndx < m_vecFrames.getItemCount())
 	{
-		pFrame = static_cast<XAP_Frame *>(m_vecFrames.getNthItem(ndx));
+		pFrame = m_vecFrames.getNthItem(ndx);
 		UT_ASSERT(pFrame);
 	}
 	return pFrame;
@@ -597,7 +783,7 @@ UT_sint32 XAP_App::findFrame(const char * szFilename)
 void XAP_App::_setAbiSuiteLibDir(const char * sz)
 {
 	FREEP(m_szAbiSuiteLibDir);
-	UT_cloneString((char *&)m_szAbiSuiteLibDir,sz);
+	m_szAbiSuiteLibDir = UT_strdup(sz);		// don't use UT_cloneString as "" is a valid libdir value
 }
 
 const char * XAP_App::getAbiSuiteLibDir() const
@@ -605,6 +791,72 @@ const char * XAP_App::getAbiSuiteLibDir() const
 	return m_szAbiSuiteLibDir;
 }
 
+bool XAP_App::findAbiSuiteLibFile(UT_String & path, const char * filename, const char * subdir)
+{
+	if (!filename) return false;
+
+#if defined(WIN32)
+	const char * sep = "\\";
+#else
+	const char * sep = "/";
+#endif
+	bool bFound = false;
+
+	const char * dir = 0;
+	if (dir = getUserPrivateDirectory())
+	{
+		path = dir;
+		if (subdir)
+		{
+			path += sep;
+			path += subdir;
+		}
+		path += sep;
+		path += filename;
+		bFound = UT_isRegularFile (path.c_str ());
+	}
+	if (!bFound && (dir = getAbiSuiteLibDir()))
+	{
+		path = dir;
+		if (subdir)
+		{
+			path += sep;
+			path += subdir;
+		}
+		path += sep;
+		path += filename;
+		bFound = UT_isRegularFile (path.c_str ());
+	}
+	return bFound;
+}
+
+bool XAP_App::findAbiSuiteAppFile(UT_String & path, const char * filename, const char * subdir) 
+{
+	if (!filename) return false;
+
+#if defined(WIN32)
+	const char * sep = "\\";
+#else
+	const char * sep = "/";
+#endif
+	bool bFound = false;
+
+	const char * dir = 0;
+	if (dir = getAbiSuiteAppDir())
+	{
+		path = dir;
+		if (subdir)
+		{
+			path += sep;
+			path += subdir;
+		}
+		path += sep;
+		path += filename;
+		bFound = UT_isRegularFile (path.c_str ());
+	}
+	return bFound;
+}
+
 bool XAP_App::addWordToDict(const UT_UCSChar * pWord, UT_uint32 len)
 {
 	if (!m_pDict)
@@ -624,7 +876,7 @@ bool XAP_App::isWordInDict(const UT_UCSChar * pWord, UT_uint32 len) const
 /*!
  * Look up the custom dictionary for suggested words
  */
-void XAP_App::suggestWord(UT_Vector * pVecSuggestions, const UT_UCSChar * pWord, UT_uint32 lenWord)
+void XAP_App::suggestWord(UT_GenericVector<UT_UCSChar*> * pVecSuggestions, const UT_UCSChar * pWord, UT_uint32 lenWord)
 {
 	if(m_pDict)
 	{
@@ -858,6 +1110,208 @@ void XAP_App::parseAndSetGeometry(const char *string)
 	}
 } 
 
+/*!
+    translate given language tag into static UT_LangRecord stored in
+    UT_Language class and set m_pKbdLang to it; do addtional
+    processing to ensure the change propagates into the status bar
+*/
+void XAP_App::setKbdLanguage(const char * pszLang)
+{
+	if(!pszLang)
+	{
+		m_pKbdLang = NULL;
+	}
+	else
+	{
+		UT_Language Lang;
+		m_pKbdLang = Lang.getLangRecordFromCode(pszLang);
+
+		// ensure that the change is shown in our status bar
+	    bool bChangeLang = false;
+		getPrefsValueBool(XAP_PREF_KEY_ChangeLanguageWithKeyboard, &bChangeLang);
+
+		if(bChangeLang && m_pKbdLang)
+		{
+			UT_return_if_fail(m_pKbdLang->m_szLangCode);
+			
+			// invoke appropriate formatting method if it exists
+			const EV_EditMethodContainer * pEMC = getEditMethodContainer();
+
+			if(pEMC)
+			{
+				EV_EditMethod * pEM = pEMC->findEditMethodByName("language");
+			
+				if (pEM)
+				{
+					XAP_Frame * pFrame = getLastFocussedFrame();
+					
+					if(pFrame)
+					{
+						AV_View * pView = pFrame->getCurrentView();
+
+						if(pView)
+						{
+							EV_EditMethodCallData CallData(m_pKbdLang->m_szLangCode,
+														   strlen(m_pKbdLang->m_szLangCode));
+
+							pEM->Fn(pView,&CallData);
+						}
+					}
+				}
+			}
+		}
+	}
+}
+
+void XAP_App::enumerateFrames(UT_Vector & v)
+{
+	for(UT_uint32 i = 0; i < getFrameCount(); ++i)
+	{
+		XAP_Frame * pF = getFrame(i);
+		if(pF)
+		{
+			if (v.findItem((void*)pF) < 0)
+			{
+				v.addItem((void*)pF);
+			}
+		}
+	}
+}
+
+/*!
+    Enumerates currently open document associated with the
+    application, excluding document pointed to by pExclude
+
+    \param v: UT_Vector into which to store the document pointers
+    
+    \para pExclude: pointer to a document to exclude from enumeration,
+                    can be NULL (e.g., if this function is called from
+                    inside a document, it might be desirable to
+                    exclude that document)
+*/
+void XAP_App::enumerateDocuments(UT_Vector & v, const AD_Document * pExclude)
+{
+	UT_sint32 iIndx;
+
+	for(UT_uint32 i = 0; i < getFrameCount(); ++i)
+	{
+		XAP_Frame * pF = getFrame(i);
+
+		if(pF)
+		{
+			AD_Document * pD = pF->getCurrentDoc();
+
+			if(pD && pD != pExclude)
+			{
+				iIndx = v.findItem((void*)pD);
+
+				if(iIndx < 0)
+				{
+					v.addItem((void*)pD);
+				}
+			}
+		}
+	}
+}
+
+EV_EditEventMapper * XAP_App::getEditEventMapper(void) const
+{
+	UT_ASSERT(m_pInputModes);
+	return m_pInputModes->getCurrentMap();
+}
+
+UT_sint32 XAP_App::setInputMode(const char * szName)
+{
+	UT_uint32 i;
+	
+	UT_ASSERT(m_pInputModes);
+	const char * szCurrentName = m_pInputModes->getCurrentMapName();
+	if (UT_stricmp(szName,szCurrentName) == 0)
+		return -1;					// already set, no change required
+
+	EV_EditEventMapper * p = m_pInputModes->getMapByName(szName);
+	if (!p)
+	{
+		// map not previously loaded -- we need to install it first
+
+		EV_EditBindingMap * pBindingMap = m_pApp->getBindingMap(szName);
+		UT_ASSERT(pBindingMap);
+		bool bResult;
+		bResult = m_pInputModes->createInputMode(szName,pBindingMap);
+		UT_ASSERT(bResult);
+	}
+	
+	// note: derrived classes will need to update keyboard
+	// note: and mouse after we return.
+
+	UT_DEBUGMSG(("Setting InputMode to [%s] for the current window.\n",szName));
+	
+	bool bStatus = m_pInputModes->setCurrentMap(szName);
+	
+	// notify all the frames about the INPUTMODE change
+	for (i = 0; i < getFrameCount(); i++) {
+		getFrame(i)->getCurrentView()->notifyListeners(AV_CHG_INPUTMODE);
+	}
+	
+	return (bStatus);
+}
+
+const char * XAP_App::getInputMode(void) const
+{
+	return m_pInputModes->getCurrentMapName();
+}
+
+/*!
+    This is the primary function for allocating graphics classes. It
+    determines if screen or priter graphics is required and then
+    allocates appropriate graphics respecting current setting.
+*/
+GR_Graphics * XAP_App::newGraphics(GR_AllocInfo &param) const
+{
+	UT_return_val_if_fail(m_pGraphicsFactory, NULL);
+
+	if(param.isPrinterGraphics())
+	{
+		return m_pGraphicsFactory->newGraphics(GRID_DEFAULT_PRINT, param);
+	}
+	else
+	{
+		return m_pGraphicsFactory->newGraphics(GRID_DEFAULT, param);
+	}
+}
+
+/*!
+    Use only if you require specific graphics, otherwise use newGraphics(GR_AllocInfo*)
+*/
+GR_Graphics * XAP_App::newGraphics(UT_uint32 iClassId, GR_AllocInfo &param) const
+{
+	UT_return_val_if_fail(m_pGraphicsFactory, NULL);
+
+	return m_pGraphicsFactory->newGraphics(iClassId, param);
+}
+
+void XAP_App::setDefaultGraphicsId(UT_uint32 i)
+{
+	if(i == GRID_UNKNOWN)
+		return;
+	
+	m_iDefaultGraphicsId = i;
+
+	if(i < GRID_LAST_BUILT_IN && i > GRID_LAST_DEFAULT)
+	{
+		// change the preference settings
+		UT_return_if_fail(m_prefs)
+			
+		XAP_PrefsScheme *pPrefsScheme = m_prefs->getCurrentScheme();
+		UT_return_if_fail(pPrefsScheme);
+
+		UT_String s;
+		UT_String_sprintf(s, "%d", i);
+		
+		pPrefsScheme->setValue(XAP_PREF_KEY_DefaultGraphics, s.c_str());
+	}
+}
+
 #ifdef DEBUG
 /*!
     This function contains some of the most fundamental assumptions
