@@ -55,6 +55,11 @@
 #include "xap_Toolbar_Layouts.h"
 #include "ut_sleep.h"
 
+#ifdef _MSC_VER
+// MSVC++ warns about using 'this' in initializer list.
+#pragma warning(disable: 4355)
+#endif
+
 XAP_Frame::XAP_Frame(XAP_FrameImpl *pFrameImpl, XAP_App * pApp)
 	: m_pApp(pApp),
 	  m_pDoc(0),
@@ -68,11 +73,11 @@ XAP_Frame::XAP_Frame(XAP_FrameImpl *pFrameImpl, XAP_App * pApp)
 	  m_lidScrollbarViewListener(static_cast<AV_ListenerId>(-1)),
 	  m_zoomType(z_PAGEWIDTH),
 	  m_pData(0),
-	  m_pInputModes(0),
 	  m_iIdAutoSaveTimer(0),
 	  m_iAutoSavePeriod(0),
 	  m_stAutoSaveExt(),
 	  m_bBackupRunning(false),
+	  m_bBackupInProgress(false),
 	  m_isrcId(0),
 	  m_isrcTBNr(0),
 	  m_idestId(0),
@@ -105,7 +110,6 @@ XAP_Frame::XAP_Frame(XAP_Frame * f)
 	m_lidScrollbarViewListener(static_cast<AV_ListenerId>(-1)),
 	m_zoomType(f->m_zoomType),
 	m_pData(0),
-	m_pInputModes(0),
 	m_iIdAutoSaveTimer(0),
 	m_bBackupRunning(false),
 	m_isrcId(0),
@@ -127,20 +131,20 @@ XAP_Frame::XAP_Frame(XAP_Frame * f)
 
 XAP_Frame::~XAP_Frame(void)
 {
-  // if we're auto-saving files and now we're exiting normally
-  // delete/unlink the file
-  bool autosave = true;
-  getApp()->getPrefsValueBool(XAP_PREF_KEY_AutoSaveFile, &autosave);
-  if (autosave)
-    {
-      UT_String backupName = makeBackupName ();
-      UT_DEBUGMSG(("DOM: removing backup file %s\n", backupName.c_str()));
-      UT_unlink ( backupName.c_str() );
-    }
+	/* if we're auto-saving files and now we're exiting normally
+	 * delete/unlink the file
+	 */
+	// bool autosave = true;
+	// getApp()->getPrefsValueBool(XAP_PREF_KEY_AutoSaveFile, &autosave);
+	if (/* autosave && */ m_stAutoSaveNamePrevious.size())
+	{
+		UT_DEBUGMSG(("DOM: removing backup file %s\n", m_stAutoSaveNamePrevious.c_str()));
+		UT_unlink ( m_stAutoSaveNamePrevious.c_str() );
+	}
 
 	// only delete the things that we created...
-    // I do not like this; we should be deleting all our members,
-    // since they are no-one else's bussines (Tomas, Jan 30, 2003)
+	// I do not like this; we should be deleting all our members,
+	// since they are no-one else's bussines (Tomas, Jan 30, 2003)
 
 	if (m_pView)
 		m_pView->removeListener(m_lid);
@@ -154,7 +158,6 @@ XAP_Frame::~XAP_Frame(void)
 	UNREFP(m_pDoc);
 
 	DELETEP(m_pScrollObj);
-	DELETEP(m_pInputModes);
 
 	DELETEP(m_pScrollbarViewListener);
 
@@ -163,13 +166,13 @@ XAP_Frame::~XAP_Frame(void)
 		UT_Timer *timer = UT_Timer::findTimer(m_iIdAutoSaveTimer);
 		if (timer != 0)
 		{
-			UT_DEBUGMSG(("Stopping timer [%d]\n", m_iIdAutoSaveTimer));
+			UT_DEBUGMSG(("Stopping Autosave timer [%d]\n", m_iIdAutoSaveTimer));
 			timer->stop();
 			DELETEP(timer);
 		}
 		else
 		{
-			UT_DEBUGMSG(("Timer [%d] not found\n", m_iIdAutoSaveTimer));
+			UT_DEBUGMSG(("Autosave Timer [%d] not found\n", m_iIdAutoSaveTimer));
 		}
 	}
 }
@@ -193,34 +196,7 @@ bool XAP_Frame::initialize(const char * szKeyBindingsKey, const char * szKeyBind
 {
 	XAP_App * pApp = getApp();
 
-	//////////////////////////////////////////////////////////////////
-	// choose which set of key- and mouse-bindings to load
-	// create a EventMapper state-machine to process our events
-	//////////////////////////////////////////////////////////////////
 
-	const char * szBindings = NULL;
-	EV_EditBindingMap * pBindingMap = NULL;
-
-	if ((pApp->getPrefsValue(szKeyBindingsKey,
-				 static_cast<const XML_Char**>(&szBindings))) && 
-	    (szBindings) && (*szBindings))
-		pBindingMap = m_pApp->getBindingMap(szBindings);
-	if (!pBindingMap)
-		pBindingMap = m_pApp->getBindingMap(szKeyBindingsDefaultValue);
-	UT_ASSERT(pBindingMap);
-
-	if (!m_pInputModes)
-	{
-		m_pInputModes = new XAP_InputModes();
-		UT_ASSERT(m_pInputModes);
-	}
-	bool bResult;
-	bResult = m_pInputModes->createInputMode(szBindings,pBindingMap);
-	UT_ASSERT(bResult);
-	bool bResult2;
-	bResult2 = m_pInputModes->setCurrentMap(szBindings);
-	UT_ASSERT(bResult2);
-	
 	//////////////////////////////////////////////////////////////////
 	// select which menu bar we should use
 	//////////////////////////////////////////////////////////////////
@@ -312,7 +288,8 @@ bool XAP_Frame::initialize(const char * szKeyBindingsKey, const char * szKeyBind
 
 	if (autosave)
 		_createAutoSaveTimer();
-	
+	setAutoSaveFile(autosave);
+
 	//////////////////////////////////////////////////////////////////
 	// select the default zoom settings
 	//////////////////////////////////////////////////////////////////
@@ -401,7 +378,7 @@ bool XAP_Frame::initialize(const char * szKeyBindingsKey, const char * szKeyBind
 extern "C" {
 static void autoSaveCallback(UT_Worker *wkr)
 {
-	UT_DEBUGMSG(("Autosaving doc...\n"));
+	xxx_UT_DEBUGMSG(("Autosaving doc...\n"));
 	XAP_Frame *me = static_cast<XAP_Frame *> (wkr->getInstanceData());
 	AD_Document * pDoc = me->getCurrentDoc();
 	if(pDoc && pDoc->isPieceTableChanging())
@@ -413,9 +390,9 @@ static void autoSaveCallback(UT_Worker *wkr)
 		UT_Error error = me->backup();
 
 		if (!error)
-			UT_DEBUGMSG(("Document Auto saved\n"));
+			xxx_UT_DEBUGMSG(("Document Auto saved\n"));
 		else
-			UT_DEBUGMSG(("Error [%d] saving document.\n", error));
+			xxx_UT_DEBUGMSG(("Error [%d] saving document.\n", error));
 	}
 	else
 	{
@@ -464,7 +441,8 @@ void /* static*/ XAP_FrameImpl::viewAutoUpdater(UT_Worker *wkr)
 	XAP_FrameImpl *pFrameImpl = static_cast<XAP_FrameImpl *> (wkr->getInstanceData());
 	XAP_App *pApp = pFrameImpl->m_pFrame->getApp(); // WL_REFACTOR: may be redundant
 	const XAP_StringSet * pSS = pFrameImpl->m_pFrame->getApp()->getStringSet();
-	UT_String msg = pSS->getValue(XAP_STRING_ID_MSG_BuildingDoc, pApp->getDefaultEncoding());
+	UT_String msg;
+	pSS->getValue(XAP_STRING_ID_MSG_BuildingDoc, pApp->getDefaultEncoding(),msg);
 	pFrameImpl->_setCursor(GR_Graphics::GR_CURSOR_WAIT);
 	AV_View * pView = pFrameImpl->m_pFrame->getCurrentView();
 	UT_DEBUGMSG(("SEVIOR: frame view updater \n"));
@@ -527,46 +505,6 @@ UT_RGBColor XAP_Frame::getColorSelForeground () const
   return m_pFrameImpl->getColorSelForeground ();
 }
 
-EV_EditEventMapper * XAP_Frame::getEditEventMapper(void) const
-{
-	UT_ASSERT(m_pInputModes);
-	return m_pInputModes->getCurrentMap();
-}
-
-UT_sint32 XAP_Frame::setInputMode(const char * szName)
-{
-	UT_ASSERT(m_pInputModes);
-	const char * szCurrentName = m_pInputModes->getCurrentMapName();
-	if (UT_stricmp(szName,szCurrentName) == 0)
-		return -1;					// already set, no change required
-
-	EV_EditEventMapper * p = m_pInputModes->getMapByName(szName);
-	if (!p)
-	{
-		// map not previously loaded -- we need to install it first
-
-		EV_EditBindingMap * pBindingMap = m_pApp->getBindingMap(szName);
-		UT_ASSERT(pBindingMap);
-		bool bResult;
-		bResult = m_pInputModes->createInputMode(szName,pBindingMap);
-		UT_ASSERT(bResult);
-	}
-	
-	// note: derrived classes will need to update keyboard
-	// note: and mouse after we return.
-
-	UT_DEBUGMSG(("Setting InputMode to [%s] for the current window.\n",szName));
-	
-	bool bStatus = m_pInputModes->setCurrentMap(szName);
-	getCurrentView()->notifyListeners(AV_CHG_INPUTMODE);
-
-	return (bStatus);
-}
-
-const char * XAP_Frame::getInputMode(void) const
-{
-	return m_pInputModes->getCurrentMapName();
-}
 
 XAP_App * XAP_Frame::getApp(void) const
 {
@@ -692,7 +630,7 @@ EV_Toolbar *  XAP_Frame::getToolbar(UT_uint32 ibar)
 {
 	if(ibar >= m_pFrameImpl->m_vecToolbars.getItemCount())
 		return NULL;
-	return static_cast<EV_Toolbar *>(m_pFrameImpl->m_vecToolbars.getNthItem(ibar));
+	return m_pFrameImpl->m_vecToolbars.getNthItem(ibar);
 }
 
 bool XAP_Frame::repopulateCombos(void)
@@ -751,12 +689,15 @@ UT_sint32 XAP_Frame::findToolbarNr(EV_Toolbar * pTB)
 
 void XAP_Frame::setAutoSaveFile(bool b)
 {
+	m_bBackupRunning = b;
 	if (b && !m_iIdAutoSaveTimer)
 	{
 		UT_Timer *timer = UT_Timer::static_constructor(autoSaveCallback, this);
 		UT_ASSERT(m_iAutoSavePeriod != 0);
 		timer->set(m_iAutoSavePeriod * 60000);
 		m_iIdAutoSaveTimer = timer->getIdentifier();
+		timer->start();
+		return;
 	}
 
 	if (!b && m_iIdAutoSaveTimer)
@@ -769,6 +710,12 @@ void XAP_Frame::setAutoSaveFile(bool b)
 		if (timer)
 			timer->stop();
 	}
+	if(b)
+	{
+		UT_Timer *timer = UT_Timer::findTimer(m_iIdAutoSaveTimer);
+		timer->start();
+		return;
+	}
 }
 
 void XAP_Frame::setAutoSaveFilePeriod(int min)
@@ -803,13 +750,15 @@ XAP_Dialog_MessageBox * XAP_Frame::createMessageBox(XAP_String_Id id,
 	if (id > 0) {
 		char * szNewMessage = static_cast<char *>(malloc(sizeof(char) * 256));
 		const XAP_StringSet * pSS = getApp()->getStringSet();
+		UT_String s;
+		pSS->getValue(id, m_pApp->getDefaultEncoding(), s);
 		
 		va_list args;		
 		va_start(args, default_answer);		
-		vsprintf(szNewMessage, static_cast<const char*>(pSS->getValue(id, m_pApp->getDefaultEncoding()).c_str()), args);
+		vsprintf(szNewMessage, static_cast<const char*>(s.c_str()), args);
 		va_end(args);
 
-		pDialog->setMessage(szNewMessage);
+		pDialog->setMessage("%s", szNewMessage);
 		
 		// XAP_MessageBox makes a copy of the message, so free it
 		FREEP(szNewMessage);
@@ -861,11 +810,12 @@ UT_String XAP_Frame::makeBackupName(const char* szExt)
   UT_String ext(szExt ? szExt : m_stAutoSaveExt.c_str());
   UT_String oldName(m_pDoc->getFilename() ? m_pDoc->getFilename() : "");
   UT_String backupName;
-  
+  UT_DEBUGMSG(("In make Backup name. Old Name is  %s \n",oldName.c_str()));
   if (oldName.empty())
     {
       const XAP_StringSet * pSS = m_pApp->getStringSet();
-	  const UT_String sTmp = pSS->getValue(XAP_STRING_ID_UntitledDocument, m_pApp->getDefaultEncoding());
+	  UT_String sTmp;
+	  pSS->getValue(XAP_STRING_ID_UntitledDocument, m_pApp->getDefaultEncoding(), sTmp);
 	  UT_String_sprintf(oldName, sTmp.c_str(), m_iUntitled);
 
       UT_DEBUGMSG(("Untitled.  We will give it the name [%s]\n", oldName.c_str()));
@@ -885,9 +835,9 @@ UT_String XAP_Frame::makeBackupName(const char* szExt)
  * If the extension is empty, then it save the document with
  * the default extension (as defined in the preferences dialog box)
  */
-UT_Error XAP_Frame::backup(const char* szExt)
+UT_Error XAP_Frame::backup(const char* szExt, UT_sint32 iEFT)
 {
-	if (m_bBackupRunning)
+	if (m_bBackupInProgress)
 		return UT_OK;
 
 	if (!m_pDoc)
@@ -896,17 +846,66 @@ UT_Error XAP_Frame::backup(const char* szExt)
 		return UT_OK;
 	}
 
-	m_bBackupRunning = true;
+	m_bBackupInProgress = true;
 
 	UT_String backupName = makeBackupName ( szExt );
 
-	UT_Error error = m_pDoc->saveAs(backupName.c_str(), m_pDoc->getLastSavedAsType(), false);
+	if (m_stAutoSaveNamePrevious.size() && (backupName != m_stAutoSaveNamePrevious))
+	{
+		/* If the user does a Save-As to rename the file then the auto-save name also changes, so
+		 * need to remove the old backup file...
+		 */
+		UT_unlink(m_stAutoSaveNamePrevious.c_str());
+	}
+	m_stAutoSaveNamePrevious = backupName;
+	
+	UT_Error error;
+//
+// Don't put this auto-save in the most recent list.
+//
+	getApp()->getPrefs()->setIgnoreNextRecent();
+
+	if(iEFT < 0)
+	{
+		error = m_pDoc->saveAs(backupName.c_str(), m_pDoc->getLastSavedAsType(), false);
+	}
+	else
+	{
+		error = m_pDoc->saveAs(backupName.c_str(), iEFT, false);
+	}
+
 	UT_DEBUGMSG(("File %s saved.\n", backupName.c_str()));
 
-	m_bBackupRunning = false;
+	m_bBackupInProgress = false;
 	return error;
 }
 
+void XAP_Frame::quickZoom(void)
+{
+	if( !m_pView ) return;
+	UT_uint32 newZoom = 100;
+	switch( getZoomType() )
+	{
+	case z_PAGEWIDTH:
+		newZoom = m_pView->calculateZoomPercentForPageWidth();
+		if      (newZoom < XAP_DLG_ZOOM_MINIMUM_ZOOM) newZoom = XAP_DLG_ZOOM_MINIMUM_ZOOM;
+		else if (newZoom > XAP_DLG_ZOOM_MAXIMUM_ZOOM) newZoom = XAP_DLG_ZOOM_MAXIMUM_ZOOM;
+		XAP_Frame::setZoomPercentage( newZoom );
+		quickZoom( newZoom );
+		break;
+	case z_WHOLEPAGE:
+		newZoom = m_pView->calculateZoomPercentForWholePage() ;
+		if      (newZoom < XAP_DLG_ZOOM_MINIMUM_ZOOM) newZoom = XAP_DLG_ZOOM_MINIMUM_ZOOM;
+		else if (newZoom > XAP_DLG_ZOOM_MAXIMUM_ZOOM) newZoom = XAP_DLG_ZOOM_MAXIMUM_ZOOM;
+		XAP_Frame::setZoomPercentage( newZoom );
+		quickZoom( newZoom );
+		break;
+	default:
+		m_pView->updateScreen(false);
+       ;
+   }
+}
+
 void XAP_Frame::updateZoom(void)
 {
 	if( !m_pView ) return;
@@ -918,14 +917,16 @@ void XAP_Frame::updateZoom(void)
 		if      (newZoom < XAP_DLG_ZOOM_MINIMUM_ZOOM) newZoom = XAP_DLG_ZOOM_MINIMUM_ZOOM;
 		else if (newZoom > XAP_DLG_ZOOM_MAXIMUM_ZOOM) newZoom = XAP_DLG_ZOOM_MAXIMUM_ZOOM;
 		XAP_Frame::setZoomPercentage( newZoom );
-		setZoomPercentage( newZoom );
+		quickZoom( newZoom );
+//		setZoomPercentage( newZoom );
 		break;
 	case z_WHOLEPAGE:
 		newZoom = m_pView->calculateZoomPercentForWholePage() ;
 		if      (newZoom < XAP_DLG_ZOOM_MINIMUM_ZOOM) newZoom = XAP_DLG_ZOOM_MINIMUM_ZOOM;
 		else if (newZoom > XAP_DLG_ZOOM_MAXIMUM_ZOOM) newZoom = XAP_DLG_ZOOM_MAXIMUM_ZOOM;
 		XAP_Frame::setZoomPercentage( newZoom );
-		setZoomPercentage( newZoom );
+		quickZoom( newZoom );
+//		setZoomPercentage( newZoom );
 		break;
 	default:
        ;
@@ -1069,80 +1070,3 @@ void XAP_Frame::setFrameMode(XAP_FrameMode iFrameMode)
 	m_pFrameImpl->m_iFrameMode = iFrameMode;
 }
 
-//////////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////////
-
-XAP_InputModes::XAP_InputModes(void)
-{
-	m_indexCurrentEventMap = 0;
-}
-
-XAP_InputModes::~XAP_InputModes(void)
-{
-	UT_ASSERT(m_vecEventMaps.getItemCount() == m_vecNames.getItemCount());
-
-	UT_VECTOR_PURGEALL(EV_EditEventMapper *, m_vecEventMaps);
-	UT_VECTOR_FREEALL(char *, m_vecNames);
-}
-
-bool XAP_InputModes::createInputMode(const char * szName,
-										EV_EditBindingMap * pBindingMap)
-{
-	UT_ASSERT(szName && *szName);
-	UT_ASSERT(pBindingMap);
-	
-	char * szDup = NULL;
-	EV_EditEventMapper * pEEM = NULL;
-
-	UT_cloneString(szDup,szName);
-	UT_ASSERT(szDup);
-	
-	pEEM = new EV_EditEventMapper(pBindingMap);
-	UT_ASSERT(pEEM);
-
-	bool b1;
-	b1 = (m_vecEventMaps.addItem(pEEM) == 0);
-	bool b2;
-	b2 = (m_vecNames.addItem(szDup) == 0);
-    UT_ASSERT(b1 && b2);
-
-	return true;
-}
-
-bool XAP_InputModes::setCurrentMap(const char * szName)
-{
-	UT_uint32 kLimit = m_vecNames.getItemCount();
-	UT_uint32 k;
-
-	for (k=0; k<kLimit; k++)
-		if (UT_stricmp(szName,reinterpret_cast<const char *>(m_vecNames.getNthItem(k))) == 0)
-		{
-			m_indexCurrentEventMap = k;
-			return true;
-		}
-
-	return false;
-}
-
-EV_EditEventMapper * XAP_InputModes::getCurrentMap(void) const
-{
-	return static_cast<EV_EditEventMapper *>(m_vecEventMaps.getNthItem(m_indexCurrentEventMap));
-}
-
-const char * XAP_InputModes::getCurrentMapName(void) const
-{
-	return reinterpret_cast<const char *>(m_vecNames.getNthItem(m_indexCurrentEventMap));
-}
-
-EV_EditEventMapper * XAP_InputModes::getMapByName(const char * szName) const
-{
-	UT_uint32 kLimit = m_vecNames.getItemCount();
-	UT_uint32 k;
-
-	for (k=0; k<kLimit; k++)
-		if (UT_stricmp(szName,reinterpret_cast<const char *>(m_vecNames.getNthItem(k))) == 0)
-			return static_cast<EV_EditEventMapper *>(m_vecEventMaps.getNthItem(k));
-
-	return NULL;
-}
-
