@@ -111,6 +111,7 @@ public:
 	virtual void	      updateScreen(bool bDirtyRunsOnly=true) = 0;
 	virtual GR_Graphics * getGraphics(void) const = 0;
     virtual void          updateLayout(void) = 0;
+	virtual void          rebuildLayout(void) = 0;
 	virtual void	cmdScroll(AV_ScrollCmd cmd, UT_uint32 iPos = 0) = 0;
 	void			addScrollListener(AV_ScrollObj*);
 	void			removeScrollListener(AV_ScrollObj*);
@@ -154,6 +155,7 @@ public:
 	virtual UT_uint32	  getPoint(void) const =0;
 	virtual void setCursorWait(void) = 0;
 	virtual void clearCursorWait(void) = 0;
+
 protected:
 	XAP_App *			m_pApp;
 	void*				m_pParentData;
@@ -164,8 +166,8 @@ protected:
 	UT_uint32                       m_iTick; // Count changes
 	bool				m_bInsertMode;
 
-	UT_Vector			m_scrollListeners;
-	UT_Vector			m_vecListeners;
+	UT_GenericVector<AV_ScrollObj*>	m_scrollListeners;
+	UT_GenericVector<AV_Listener*>	m_vecListeners;
 
 private:
 	AV_View(const AV_View&);	// no impl.
