@@ -26,7 +26,8 @@
 #include "fp_Run.h"
 #include "fp_TextRun.h"
 #include "fp_Column.h"
-
+#include "fb_Alignment.h"
+#include "fp_Page.h"
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
 #include "ut_string.h"
@@ -57,7 +58,9 @@ fb_LineBreaker::~fb_LineBreaker(void)
   All trailing spaces should remain on the end of the line.
 */
 UT_sint32
-fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock, fp_Line * pLineToStartAt)
+fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock, 
+							   fp_Line * pLineToStartAt,
+							   fp_Page * pPage)
 {
 	// FIXME: This should:
 	//	 o Check the Runs for state signalling if they changed since
@@ -70,9 +73,14 @@ fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock, fp_Line * pLineToStartAt)
 	fp_Line* pLine = static_cast<fp_Line *>(pBlock->getFirstContainer());
 	UT_ASSERT(pLine);
 	xxx_UT_DEBUGMSG(("In LineBreaker max line width is %d pLineToStart %x \n",m_iMaxLineWidth,pLineToStartAt));
-	while(pLine)
+
+	// if the block is justified, the alignment has already been reset
+	// prior to entering this function -- do not do it again
+	bool bJustified  = (pBlock->getAlignment() && pBlock->getAlignment()->getType() == FB_ALIGNMENT_JUSTIFY);
+	
+	while(!bJustified && pLine)
 	{
-		pLine->resetJustification();
+		pLine->resetJustification(true); // permanent reset
 		pLine = static_cast<fp_Line *>(pLine->getNext());
 	}
 	pLine = static_cast<fp_Line *>(pBlock->getFirstContainer());
@@ -80,6 +88,7 @@ fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock, fp_Line * pLineToStartAt)
 	if(pLineToStartAt)
 	{
 		pLine = pLineToStartAt;
+		pLine->resetJustification(true); // permanent reset
 	}
 
 	while (pLine)
@@ -142,7 +151,7 @@ fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock, fp_Line * pLineToStartAt)
 						while(pRun && pRun != pOffendingRun)
 						{
 							m_iWorkingLineWidth -= pRun->getWidth();
-							pRun = pRun->getPrev();
+							pRun = pRun->getPrevRun();
 						}
 						// This run needs splitting.
 						xxx_UT_DEBUGMSG(("Break at 1 Trailing Space %d Wording width %d \n",iTrailingSpace,m_iWorkingLineWidth));
@@ -152,7 +161,7 @@ fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock, fp_Line * pLineToStartAt)
 					}
 					else
 					{
-						m_iWorkingLineWidth -= iTrailingSpace;
+						//m_iWorkingLineWidth -= iTrailingSpace;
 						if(pCurrentRun)
 						{
 							xxx_UT_DEBUGMSG(("Break at 2 Trailing Space %d \n",iTrailingSpace));
@@ -215,7 +224,7 @@ fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock, fp_Line * pLineToStartAt)
 						{
 							// if there are some runs between this tab ran and the last run that the line knows
 							// belongs to it; we have to add these as well
-							fp_Run * pRun = pOriginalLastOnLine->getNext();
+							fp_Run * pRun = pOriginalLastOnLine->getNextRun();
 							while(pRun)
 							{
 								fp_Line * pL = pRun->getLine();
@@ -227,7 +236,7 @@ fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock, fp_Line * pLineToStartAt)
 								if(pRun == pCurrentRun)
 									break;
 
-								pRun = pRun->getNext();
+								pRun = pRun->getNextRun();
 							}
 #if DEBUG
 							pLine->assertLineListIntegrity();
@@ -246,6 +255,7 @@ fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock, fp_Line * pLineToStartAt)
 					break;
 				}
 				case FPRUN_FMTMARK:
+				case FPRUN_DUMMY:
 					break;
 
 				case FPRUN_FIELD:
@@ -268,7 +278,7 @@ fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock, fp_Line * pLineToStartAt)
 				} // switch
 
 				pPreviousRun = pCurrentRun;
-				pCurrentRun = pCurrentRun->getNext();
+				pCurrentRun = pCurrentRun->getNextRun();
 
 				iIndx++;
 
@@ -285,7 +295,7 @@ fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock, fp_Line * pLineToStartAt)
 			  sure that no other runs are on this line.
 			*/
 
-			_breakTheLineAtLastRunToKeep(pLine, pBlock);
+			_breakTheLineAtLastRunToKeep(pLine, pBlock,pPage);
 
 			/*
 			  Now we know all the runs which belong on this line.
@@ -350,7 +360,7 @@ UT_sint32 fb_LineBreaker::_moveBackToFirstNonBlankData(fp_Run *pCurrentRun, fp_R
 			break;
 		}
 
-		pCurrentRun = pCurrentRun->getPrev();
+		pCurrentRun = pCurrentRun->getPrevRun();
 	}
 	while(pCurrentRun);
 
@@ -408,7 +418,7 @@ bool fb_LineBreaker::_splitAtOrBeforeThisRun(fp_Run *pCurrentRun, UT_sint32 iTra
 		fp_Run* pRunLookingBackwards = pCurrentRun;
 		while (pRunLookingBackwards != m_pFirstRunToKeep)
 		{
-			pRunLookingBackwards = pRunLookingBackwards->getPrev();
+			pRunLookingBackwards = pRunLookingBackwards->getPrevRun();
 
 			if ( !pRunLookingBackwards )
 			  {
@@ -481,7 +491,7 @@ bool fb_LineBreaker::_splitAtOrBeforeThisRun(fp_Run *pCurrentRun, UT_sint32 iTra
 				/*
 				  Force a break right before the offending run.
 				*/
-				m_pLastRunToKeep = pOffendingRun->getPrev();
+				m_pLastRunToKeep = pOffendingRun->getPrevRun();
 
 				bFoundBreakAfter = true;
 			}
@@ -499,7 +509,7 @@ bool fb_LineBreaker::_splitAtOrBeforeThisRun(fp_Run *pCurrentRun, UT_sint32 iTra
 		UT_ASSERT(!bFoundBreakAfter);
 
 		_splitRunAt(pRunToSplit, splitInfo);
-
+		m_pLastRunToKeep = pRunToSplit;
 	}
 
 
@@ -520,7 +530,7 @@ bool fb_LineBreaker::_splitAtNextNonBlank(fp_Run *pCurrentRun)
 	{
 		// cannot split run so split before.
 		xxx_UT_DEBUGMSG(("Cannot split at next non-blank \n"));
-		m_pLastRunToKeep = pCurrentRun->getPrev();
+		m_pLastRunToKeep = pCurrentRun->getPrevRun();
 	}
 
 	return true;
@@ -532,11 +542,11 @@ void fb_LineBreaker::_splitRunAt(fp_Run *pCurrentRun, fp_RunSplitInfo &splitInfo
 	fp_TextRun *pRunToSplit = static_cast<fp_TextRun*>(pCurrentRun);
 
 	pRunToSplit->split(splitInfo.iOffset + 1);	// TODO err check this
-	UT_ASSERT(pRunToSplit->getNext());
-	UT_ASSERT(pRunToSplit->getNext()->getType() == FPRUN_TEXT);
+	UT_ASSERT(pRunToSplit->getNextRun());
+	UT_ASSERT(pRunToSplit->getNextRun()->getType() == FPRUN_TEXT);
 
 	fp_TextRun *pOtherHalfOfSplitRun;
-	pOtherHalfOfSplitRun = static_cast<fp_TextRun*>(pRunToSplit->getNext());
+	pOtherHalfOfSplitRun = static_cast<fp_TextRun*>(pRunToSplit->getNextRun());
 
 // todo decide if we need to call recalcWidth() now on the 2 pieces.
 //							pRunToSplit->recalcWidth();
@@ -549,7 +559,7 @@ void fb_LineBreaker::_splitRunAt(fp_Run *pCurrentRun, fp_RunSplitInfo &splitInfo
 }
 
 void fb_LineBreaker::_breakTheLineAtLastRunToKeep(fp_Line *pLine,
-												  fl_BlockLayout *pBlock)
+												  fl_BlockLayout *pBlock,fp_Page * pPage)
 {
 
 	/*
@@ -579,7 +589,7 @@ void fb_LineBreaker::_breakTheLineAtLastRunToKeep(fp_Line *pLine,
 		else
 		{
 			pPrevRun = pCurrentRun;
-			pCurrentRun = pCurrentRun->getNext();
+			pCurrentRun = pCurrentRun->getNextRun();
 		}
 	}
 
@@ -593,7 +603,19 @@ void fb_LineBreaker::_breakTheLineAtLastRunToKeep(fp_Line *pLine,
 		pNextLine = static_cast<fp_Line *>(pLine->getNext());
 		if (!pNextLine)
 		{
-			fp_Line* pNewLine  = static_cast<fp_Line *>(pBlock->getNewContainer());
+			fp_Line* pNewLine = NULL;
+			if(pPage == NULL)
+			{
+				pNewLine  = static_cast<fp_Line *>(pBlock->getNewContainer());
+			}
+			else
+			{
+				UT_sint32 iX = pLine->getX();
+				iX += pLine->getMaxWidth();
+				pLine->recalcHeight(m_pLastRunToKeep);
+				UT_sint32 iHeight = pLine->getHeight() + pLine->getMarginAfter();
+				pNewLine = pBlock->getNextWrappedLine(iX,iHeight,pPage);
+			}
 			UT_ASSERT(pNewLine);	// TODO check for outofmem
 			pNextLine = pNewLine;
 			xxx_UT_DEBUGMSG(("!!!! Generated a new Line \n"));
@@ -615,7 +637,14 @@ void fb_LineBreaker::_breakTheLineAtLastRunToKeep(fp_Line *pLine,
 		{
 			UT_ASSERT(pRunToBump->getLine() == pLine);
 			xxx_UT_DEBUGMSG(("RunToBump %x Type %d Offset %d Length %d \n",pRunToBump,pRunToBump->getType(),pRunToBump->getBlockOffset(),pRunToBump->getLength()));
-			pLine->removeRun(pRunToBump);
+			if(!pLine->removeRun(pRunToBump))
+			{
+//
+// More repair code I think...
+// run is not on the Line! It's totally lost...
+//
+				pRunToBump->setLine(NULL);
+			}
 			
 			UT_ASSERT(pLine->getLastRun()->getType() != FPRUN_ENDOFPARAGRAPH);
 //
@@ -628,7 +657,7 @@ void fb_LineBreaker::_breakTheLineAtLastRunToKeep(fp_Line *pLine,
 			}
 			pNextLine->insertRun(pRunToBump);
 
-			pRunToBump = pRunToBump->getPrev();
+			pRunToBump = pRunToBump->getPrevRun();
 			xxx_UT_DEBUGMSG(("Next runToBump %x \n",pRunToBump));
 		}
 	}
