@@ -31,6 +31,7 @@
 #include "fl_Layout.h"
 #include "fl_DocLayout.h"
 #include "fl_BlockLayout.h"
+#include "fl_TOCLayout.h"
 #include "fl_TableLayout.h"
 #include "fp_TableContainer.h"
 #include "fp_Page.h"
@@ -45,6 +46,10 @@
 #include "ut_debugmsg.h"
 #include "ut_assert.h"
 #include "ut_units.h"
+#include "fl_FrameLayout.h"
+#include "fp_FrameContainer.h"
+#include "fl_AutoNum.h"
+
 
 fl_ContainerLayout::fl_ContainerLayout(fl_ContainerLayout* pMyLayout, PL_StruxDocHandle sdh, PT_AttrPropIndex indexAP, PTStruxType iStrux, fl_ContainerType iType)
 	: fl_Layout(iStrux, sdh),
@@ -56,8 +61,10 @@ fl_ContainerLayout::fl_ContainerLayout(fl_ContainerLayout* pMyLayout, PL_StruxDo
 	  m_pLastL(NULL),
 	  m_pFirstContainer(NULL),
 	  m_pLastContainer(NULL),
-	  m_eHidden(FP_VISIBLE)
+	  m_eHidden(FP_VISIBLE),
+	  m_iFoldedLevel(0)
 {
+//	UT_ASSERT(pMyLayout != NULL);
 	setAttrPropIndex(indexAP);
 	if(pMyLayout)
 	{
@@ -78,20 +85,262 @@ fl_ContainerLayout::~fl_ContainerLayout()
 #endif
 }
 
+void fl_ContainerLayout::lookupProperties(void)
+{
+	// first of all, call getAP() which will set default visibility
+	// for us (either visible, or hidden revision)
+	const PP_AttrProp* pAP = NULL;
+	FPVisibility eVisibility = getAP(pAP);
+	UT_return_if_fail(pAP);
+
+	setVisibility(eVisibility);
+	
+	//  Find the folded Level of the strux
+	lookupFoldedLevel();
+	if((isHidden() == FP_VISIBLE) && (getFoldedLevel() > 0) && (getLevelInList() > getFoldedLevel()) )
+	{
+		xxx_UT_DEBUGMSG(("Table set to hidden folded \n"));
+		setVisibility(FP_HIDDEN_FOLDED);
+	}
+	// evaluate "display" property
+	// display property
+	const char* pszDisplay = NULL;
+	pAP->getProperty("display", (const XML_Char *&)pszDisplay);
+	if(isHidden() == FP_VISIBLE && pszDisplay && !UT_strcmp(pszDisplay, "none"))
+	{
+		setVisibility(FP_HIDDEN_TEXT);
+	}
+
+	// other common properties should come here
+
+	// this should only implement class-specific properties ...
+	_lookupProperties(pAP);
+}
+
+
+/*!
+    retrieves AP associated with this layout, corretly processing any
+    revision information;
+
+    /return return value indicates whether the layout is hidden due to
+    current revision settings or not
+*/
+FPVisibility fl_ContainerLayout::getAP(const PP_AttrProp *& pAP)const
+{
+	FL_DocLayout* pDL =	getDocLayout();
+	UT_return_val_if_fail(pDL,FP_VISIBLE);
+
+	FV_View* pView = pDL->getView();
+	UT_return_val_if_fail(pView,FP_VISIBLE);
+
+	UT_uint32 iId  = pView->getRevisionLevel();
+	bool bShow     = pView->isShowRevisions();
+	bool bHiddenRevision = false;
+
+	getAttrProp(&pAP,NULL,bShow,iId,bHiddenRevision);
+
+	if(bHiddenRevision)
+	{
+		return FP_HIDDEN_REVISION;
+	}
+	else
+	{
+		return FP_VISIBLE;
+	}
+}
+
+void fl_ContainerLayout::getSpanAP(UT_uint32 blockPos, bool bLeft, const PP_AttrProp * &pSpanAP) const
+{
+	//first we need to ascertain if this revision is visible
+	FL_DocLayout* pDL =	getDocLayout();
+	UT_return_if_fail(pDL);
+
+	FV_View* pView = pDL->getView();
+	UT_return_if_fail(pView);
+
+	UT_uint32 iId  = pView->getRevisionLevel();
+	bool bShow     = pView->isShowRevisions();
+	bool bHiddenRevision = false;
+
+	getSpanAttrProp(blockPos, bLeft, &pSpanAP,NULL,bShow,iId,bHiddenRevision);
+}
+
+
+const char * fl_ContainerLayout::getContainerString(void)
+{
+	switch(getContainerType())
+	{
+	case FL_CONTAINER_BLOCK:
+		return "FL_CONTAINER_BLOCK";
+	case  FL_CONTAINER_DOCSECTION:
+		return "FL_CONTAINER_DOCSECTION";
+	case FL_CONTAINER_HDRFTR:
+		return "FL_CONTAINER_HDRFTR";
+	case FL_CONTAINER_SHADOW:
+		return "FL_CONTAINER_SHADOW";
+	case FL_CONTAINER_FOOTNOTE:
+		return "FL_CONTAINER_FOOTNOTE";
+	case FL_CONTAINER_ENDNOTE:
+		return "FL_CONTAINER_ENDNOTE";
+	case FL_CONTAINER_MARGINNOTE:
+		return "FL_CONTAINER_MARGINNOTE";
+	case FL_CONTAINER_TABLE:
+		return "FL_CONTAINER_TABLE";
+	case FL_CONTAINER_CELL:
+		return "FL_CONTAINER_CELL";
+	case FL_CONTAINER_FRAME:
+		return "FL_CONTAINER_FRAME";
+	case FL_CONTAINER_TOC:
+		return "FL_CONTAINER_TOC";
+	default:
+		return "NOT_IMPLEMENTED";
+	}
+	return "NOT IMPLEMENTED";
+}
+
+
+
 /*!
  * Return the value of the attribute keyed by pszName
  */
 const char*	fl_ContainerLayout::getAttribute(const char * pszName) const
 {
 	const PP_AttrProp * pAP = NULL;
-	getAttrProp(&pAP);
-
+	getAP(pAP);
+	UT_return_val_if_fail(pAP, NULL);
+	
 	const XML_Char* pszAtt = NULL;
 	pAP->getAttribute(static_cast<const XML_Char*>(pszName), pszAtt);
 
 	return pszAtt;
 }
 
+/*!
+ * Return the nested List level of this structure.
+ */
+UT_sint32 fl_ContainerLayout::getLevelInList(void)
+{
+      fl_BlockLayout * pBList = NULL;
+      if(getContainerType() == FL_CONTAINER_BLOCK)
+      {
+	   pBList = static_cast<fl_BlockLayout * >(this);
+      }
+      else
+      {
+	   pBList = getPrevBlockInDocument();
+      }
+      UT_sint32 iLevel = 0;
+      bool bLoop = true;
+      while(pBList && bLoop)
+      {
+	  while(pBList && !pBList->isListItem())
+	  {
+	       pBList = pBList->getPrevBlockInDocument();
+	  }
+	  if(pBList == NULL)
+	  {
+	       bLoop = false;
+	       break;
+	  }
+	  const PP_AttrProp * pAP = NULL;
+	  pBList->getAP(pAP);
+	  const XML_Char * szLid=NULL;
+	  UT_uint32 id=0;
+
+	  if (!pAP || !pAP->getAttribute(PT_LISTID_ATTRIBUTE_NAME, szLid))
+	       szLid = NULL;
+	  if (szLid)
+	  {
+	       id = atoi(szLid);
+		
+	  }
+	  else
+	  {
+		id = 0;
+	  }
+	  if(id == 0)
+	  {
+	        bLoop = false;
+	        break;
+	  }
+	  PD_Document * pDoc = getDocLayout()->getDocument();
+	  fl_AutoNum * pAuto = pDoc->getListByID( id);
+	  if(pAuto->getLastItem() == pBList->getStruxDocHandle())
+	  {
+	        if(pAuto->getLastItem() == getStruxDocHandle())
+		{
+		     iLevel = pAuto->getLevel();
+		     bLoop = false;
+		     break;
+		}
+	        iLevel = pAuto->getLevel() -1;
+		if(iLevel < 0)
+		{
+		      iLevel = 0;
+		}
+	  }
+	  else
+	  {
+	        if(pBList == this)
+	        { 
+		      iLevel = pAuto->getLevel();
+		}
+		else
+		{
+		      iLevel = pAuto->getLevel() + 1;
+		}
+	  }
+	  bLoop = false;
+	  break;
+      }
+      return iLevel;
+}
+/*!
+ * This method returns the folded level of the text.
+ */
+UT_sint32 fl_ContainerLayout::getFoldedLevel(void)
+{
+	return m_iFoldedLevel;
+}
+
+
+/*!
+ * This method returns the ID of the list that is folded.
+ */
+UT_uint32 fl_ContainerLayout::getFoldedID(void)
+{
+	return m_iFoldedID;
+}
+
+/*!
+ * This Method looks up the folded level of the strux.
+ */
+void fl_ContainerLayout::lookupFoldedLevel(void)
+{
+ 	const PP_AttrProp* pSectionAP = NULL;
+
+	getAP(pSectionAP);
+
+	const XML_Char *pszTEXTFOLDED = NULL;
+	if(!pSectionAP || !pSectionAP->getProperty("text-folded",pszTEXTFOLDED))
+	{
+		m_iFoldedLevel = 0;
+	}
+	else
+	{
+		m_iFoldedLevel = atoi(pszTEXTFOLDED);
+	}
+	xxx_UT_DEBUGMSG(("FOlded Level is %d \n",m_iFoldedLevel));
+    pszTEXTFOLDED = NULL;
+	if(!pSectionAP || !pSectionAP->getProperty("text-folded-id",pszTEXTFOLDED))
+	{
+		m_iFoldedID = 0;
+	}
+	else
+	{
+		m_iFoldedID = atoi(pszTEXTFOLDED);
+	}
+}
 
 /*!
  * This method appends all the text in the current layout to the supplied 
@@ -179,6 +428,26 @@ fl_ContainerLayout * fl_ContainerLayout::myContainingLayout(void) const
 	return m_pMyLayout;
 }
 
+/*!
+ * If this container is contained by a HdrFtrSectionLayout return it. 
+ * Otherwise return
+ * NULL.
+ */
+fl_HdrFtrSectionLayout * fl_ContainerLayout::getHdrFtrLayout(void)
+{
+	fl_ContainerLayout * pCL = this;
+	while(pCL && (pCL->getContainerType() != FL_CONTAINER_HDRFTR) && 
+		  (pCL->getContainerType() != FL_CONTAINER_DOCSECTION))
+	{
+		pCL = pCL->myContainingLayout();
+	}
+	if(pCL && (pCL->getContainerType() ==  FL_CONTAINER_HDRFTR))
+	{
+		return static_cast<fl_HdrFtrSectionLayout *>(pCL);
+	}
+	return NULL;
+}
+
 FL_DocLayout* fl_ContainerLayout::getDocLayout(void) const
 {
 	const fl_ContainerLayout * pMyContainer = static_cast<const fl_ContainerLayout *>(this);
@@ -191,6 +460,7 @@ FL_DocLayout* fl_ContainerLayout::getDocLayout(void) const
 
 void fl_ContainerLayout::setContainingLayout(fl_ContainerLayout * pL)
 {
+//	UT_ASSERT(pL != NULL); // useful for debugging
 	m_pMyLayout = pL;
 }
 
@@ -248,7 +518,10 @@ fl_BlockLayout* fl_ContainerLayout::getNextBlockInDocument(void) const
 				pOld = myContainingLayout();
 			}
 			depth++;
-			pNext = pOld->getNext();
+			if(pOld != NULL) // HdrFtr's have myContainingLayout == NULL
+			{
+				pNext = pOld->getNext();
+			}
 			if(pPrevOld == pOld)
 			{
 				pOld = NULL;
@@ -270,10 +543,29 @@ fl_BlockLayout* fl_ContainerLayout::getNextBlockInDocument(void) const
 		{
 			pNext = pNext->getFirstLayout();
 		}
+		else if(pNext->getContainerType() == FL_CONTAINER_FRAME)
+		{
+			if(pNext->getFirstLayout() == NULL)
+			{
+			     pNext = pNext->getNext();
+			}
+			else
+			{
+			     pNext = pNext->getFirstLayout();
+			}
+		}
 		else if(pNext->getContainerType() == FL_CONTAINER_CELL)
 		{
 			pNext = pNext->getFirstLayout();
 		}
+		else if(pNext->getContainerType() == FL_CONTAINER_TOC)
+		{
+			pNext = pNext->getNext();
+			if(pNext == NULL)
+			{
+				goto next_is_null;
+			}
+		}
 		else if(pNext->getContainerType() == FL_CONTAINER_FOOTNOTE)
 		{
 			pNext = pNext->getNext();
@@ -322,8 +614,11 @@ fl_BlockLayout* fl_ContainerLayout::getPrevBlockInDocument(void) const
 				pOld = myContainingLayout();
 			}
 			depth++;
-			pPrev = pOld->getPrev();
-			if(pPrevOld == pOld)
+			if(pOld != NULL) // HdrFtr's can have NULL myContainingLayout's
+			{
+				pPrev = pOld->getPrev();
+			}
+			if(pPrevOld == pOld )
 			{
 				pOld = NULL;
 			}
@@ -340,6 +635,17 @@ fl_BlockLayout* fl_ContainerLayout::getPrevBlockInDocument(void) const
 		{
 			pPrev = pPrev->getLastLayout();
 		}
+		else if(pPrev->getContainerType() == FL_CONTAINER_FRAME)
+		{
+			if(pPrev->getLastLayout() == NULL)
+			{
+			     pPrev = pPrev->getPrev();
+			}
+			else
+			{
+			     pPrev = pPrev->getLastLayout();
+			}
+		}
 		else if(pPrev->getContainerType() == FL_CONTAINER_TABLE)
 		{
 			pPrev = pPrev->getLastLayout();
@@ -352,6 +658,10 @@ fl_BlockLayout* fl_ContainerLayout::getPrevBlockInDocument(void) const
 		{
 			pPrev = pPrev->getLastLayout();
 		}
+		else if(pPrev->getContainerType() == FL_CONTAINER_TOC)
+		{
+			pPrev = pPrev->getLastLayout();
+		}
 		else if(pPrev->getContainerType() == FL_CONTAINER_ENDNOTE)
 		{
 			pPrev = pPrev->getLastLayout();
@@ -363,7 +673,10 @@ fl_BlockLayout* fl_ContainerLayout::getPrevBlockInDocument(void) const
 		}
 		if(pPrev == NULL)
 		{
-			pPrev = pOld->myContainingLayout()->getPrev();
+			if(pOld && pOld->myContainingLayout())
+			{
+				pPrev = pOld->myContainingLayout()->getPrev();
+			}
 		}
 	}
 	return NULL;
@@ -413,32 +726,18 @@ fl_ContainerLayout * fl_ContainerLayout::insert(PL_StruxDocHandle sdh, fl_Contai
 	switch (iType)
 	{
 	case FL_CONTAINER_BLOCK:
+		// we have a problem here -- the block needs to be in the list before the consturction is completed
 		if(getContainerType() ==  FL_CONTAINER_HDRFTR)
 		{
-			pL = static_cast<fl_ContainerLayout *>(new fl_BlockLayout(sdh, static_cast<fl_BlockLayout *>(pPrev), static_cast<fl_SectionLayout *>(this), indexAP,true));
-			if (pPrev)
-				pPrev->_insertIntoList(pL);
-			else
-			{ 
-				pL->setNext(getFirstLayout()); 
-				if (getFirstLayout()) getFirstLayout()->setPrev(pL); 
-			}
+			pL = static_cast<fl_ContainerLayout *>(new fl_BlockLayout(sdh, pPrev, static_cast<fl_SectionLayout *>(this), indexAP,true));
 		}
 		else if ((pPrev!= NULL) && (pPrev->getContainerType() == FL_CONTAINER_TABLE))
 		{
-			pL = static_cast<fl_ContainerLayout *>(new fl_BlockLayout(sdh, static_cast<fl_BlockLayout *>(pPrev), static_cast<fl_SectionLayout *>(pPrev->myContainingLayout()), indexAP));
-			pPrev->_insertIntoList(pL);
+			pL = static_cast<fl_ContainerLayout *>(new fl_BlockLayout(sdh,pPrev, static_cast<fl_SectionLayout *>(pPrev->myContainingLayout()), indexAP));
 		}
 		else
 		{
 			pL = static_cast<fl_ContainerLayout *>(new fl_BlockLayout(sdh, static_cast<fl_BlockLayout *>(pPrev), static_cast<fl_SectionLayout *>(this), indexAP));
-			if (pPrev)
-				pPrev->_insertIntoList(pL);
-			else
-			{ 
-				pL->setNext(getFirstLayout()); 
-				if (getFirstLayout()) getFirstLayout()->setPrev(pL); 
-			}
 		}
 		break;
 	case FL_CONTAINER_TABLE:
@@ -452,12 +751,48 @@ fl_ContainerLayout * fl_ContainerLayout::insert(PL_StruxDocHandle sdh, fl_Contai
 			fp_TableContainer * pTab = static_cast<fp_TableContainer *>(static_cast<fl_TableLayout *>(pL)->getLastContainer());
 			static_cast<fl_TableLayout *>(pL)->insertTableContainer(static_cast<fp_TableContainer *>(pTab));
 		}
+		if(getContainerType() == FL_CONTAINER_CELL)
+		{
+			fl_CellLayout * pCell = static_cast<fl_CellLayout *>(this);
+			pCell->incNumNestedTables();
+			fl_TableLayout * pTab = static_cast<fl_TableLayout *>(pCell->myContainingLayout());
+			pTab->incNumNestedTables();
+		}
 		break;
 	case FL_CONTAINER_CELL:
 		pL = static_cast<fl_ContainerLayout *>(new fl_CellLayout(getDocLayout(),sdh, indexAP, this));
 		if (pPrev)
+		{
 			pPrev->_insertIntoList(pL);
+		}
+		else
+		{
+			_insertFirst(pL);
+		}
 		break;
+	case FL_CONTAINER_FRAME:
+	{
+		fl_DocSectionLayout * pDSL = getDocSectionLayout();
+		pL = static_cast<fl_ContainerLayout *>(new fl_FrameLayout(getDocLayout(), 
+					  pDSL, 
+					  sdh, indexAP, this));
+		if (pPrev)
+		{
+			while(pPrev && pPrev->getContainerType() != FL_CONTAINER_BLOCK)
+			{
+				pPrev = pPrev->getPrev();
+			}
+//
+// Add the frame to the list in te previous block.
+//
+			if(pPrev)
+			{
+				pPrev->_insertIntoList(pL);
+				pPrev->addFrame(static_cast<fl_FrameLayout *>(pL));
+			}
+		}
+		break;
+	}
 	case FL_CONTAINER_FOOTNOTE:
 	{
 		fl_DocSectionLayout * pDSL = getDocSectionLayout();
@@ -468,6 +803,17 @@ fl_ContainerLayout * fl_ContainerLayout::insert(PL_StruxDocHandle sdh, fl_Contai
 			pPrev->_insertIntoList(pL);
 		break;
 	}
+	case FL_CONTAINER_TOC:
+	{
+		fl_DocSectionLayout * pDSL = getDocSectionLayout();
+		pL = static_cast<fl_ContainerLayout *>(new fl_TOCLayout(getDocLayout(), 
+					  pDSL, 
+					  sdh, indexAP, this));
+		if (pPrev)
+			pPrev->_insertIntoList(pL);
+		static_cast<fl_TOCLayout *>(pL)->getNewContainer(NULL);
+		break;
+	}
 	case FL_CONTAINER_ENDNOTE:
 	{
 		fl_DocSectionLayout * pDSL = getDocSectionLayout();
@@ -525,6 +871,27 @@ void fl_ContainerLayout::_insertIntoList(fl_ContainerLayout * pL)
 		pNext->setPrev(pL);
 }
 
+
+/*! 
+   Inserts pL into the containment hierarchy at First Location;
+*/
+void fl_ContainerLayout::_insertFirst(fl_ContainerLayout * pL)
+{
+	if(m_pFirstL == NULL)
+	{
+		m_pFirstL = pL;
+		pL->setPrev(NULL);
+		pL->setNext(NULL);
+		m_pLastL = pL;
+		return;
+	}
+	fl_ContainerLayout * pOldFirst = m_pFirstL;
+	m_pFirstL = pL;
+	pL->setNext(pOldFirst);
+	pL->setPrev(NULL);
+	pOldFirst->setPrev(pL);
+}
+
 /*!
  * Remove a containerLayout class from the linked list held here.
  */
@@ -588,11 +955,17 @@ fp_Container* fl_ContainerLayout::getLastContainer() const
 
 void fl_ContainerLayout::setFirstContainer(fp_Container * pCon)
 {
+	if(pCon && getContainerType() == FL_CONTAINER_BLOCK)
+	{
+		UT_ASSERT(pCon->getContainerType() == FP_CONTAINER_LINE);
+	}
+	xxx_UT_DEBUGMSG(("Set FirstContainer of %x to %x \n",this,pCon));
 	m_pFirstContainer = pCon;
 }
 
 void fl_ContainerLayout::setLastContainer(fp_Container * pCon)
 {
+	xxx_UT_DEBUGMSG(("Set LastContainer of %x to %x \n",this,pCon));
 	m_pLastContainer = pCon;
 }
 
@@ -620,7 +993,7 @@ fp_Run * fl_ContainerLayout::getFirstRun(void) const
 UT_uint32 fl_ContainerLayout::getPosition(bool bActualBlockPos) const
 {
 	const fl_ContainerLayout * pL = this;
-    if(!bActualBlockPos)
+    if(!bActualBlockPos && (getContainerType() != FL_CONTAINER_TOC))
 	{
 		while(pL->getContainerType() != FL_CONTAINER_BLOCK && pL->getFirstLayout())
 		{
@@ -633,7 +1006,7 @@ UT_uint32 fl_ContainerLayout::getPosition(bool bActualBlockPos) const
 		}
 		return 0;
 	}
-	PT_DocPosition pos = getDocLayout()->getDocument()->getStruxPosition(m_sdh);
+	PT_DocPosition pos = getDocLayout()->getDocument()->getStruxPosition(getStruxDocHandle());
 	return pos;
 }
 
@@ -688,8 +1061,8 @@ bool fl_ContainerLayout::isOnScreen() const
 	if(bHidden)
 		return false;
 	
-	UT_Vector vRect;
-	UT_Vector vPages;
+	UT_GenericVector<UT_Rect*> vRect;
+	UT_GenericVector<fp_Page*> vPages;
 
 	pView->getVisibleDocumentPagesAndRectangles(vRect, vPages);
 
@@ -714,12 +1087,12 @@ bool fl_ContainerLayout::isOnScreen() const
 		{
 			for(UT_uint32 i = 0; i < iCount; i++)
 			{
-				fp_Page * pPage = static_cast<fp_Page*>(vPages.getNthItem(i));
+				fp_Page * pPage = vPages.getNthItem(i);
 
 				if(pPage == pMyPage)
 				{
 					UT_Rect r;
-					UT_Rect *pR = static_cast<UT_Rect*>(vRect.getNthItem(i));
+					UT_Rect *pR = vRect.getNthItem(i);
 
 					if(!pC->getPageRelativeOffsets(r))
 						break;
@@ -741,3 +1114,58 @@ bool fl_ContainerLayout::isOnScreen() const
 	return bRet;
 }
 
+// Frames stuff
+
+void fl_ContainerLayout::addFrame(fl_FrameLayout * pFrame)
+{
+	UT_DEBUGMSG(("Adding frame %x to list in container %x \n",pFrame,this));
+	UT_sint32 i = m_vecFrames.findItem(pFrame);
+	if(i>= 0)
+	{
+		UT_DEBUGMSG(("Adding already existing frame \n"));
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		return;
+	}
+	m_vecFrames.addItem(pFrame);
+}
+
+UT_sint32 fl_ContainerLayout::getNumFrames(void) const
+{
+	return static_cast<UT_sint32>(m_vecFrames.getItemCount());
+}
+
+fl_FrameLayout * fl_ContainerLayout::getNthFrameLayout(UT_sint32 i) const
+{
+	if(i> getNumFrames())
+	{
+		return NULL;
+	}
+	return m_vecFrames.getNthItem(i);
+}
+
+
+fp_FrameContainer * fl_ContainerLayout::getNthFrameContainer(UT_sint32 i) const
+{
+	if(i> getNumFrames())
+	{
+		return NULL;
+	}
+	fl_FrameLayout * pFrame= m_vecFrames.getNthItem(i);
+	fp_FrameContainer * pFC = static_cast<fp_FrameContainer *>(pFrame->getFirstContainer());
+	return pFC;
+}
+
+void fl_ContainerLayout:: removeFrame(fl_FrameLayout * pFrame)
+{
+	UT_DEBUGMSG(("Remove Frame %x from this container %x \n",pFrame,this));
+	UT_sint32 i = m_vecFrames.findItem(pFrame);
+	if(i >= 0)
+	{
+		m_vecFrames.deleteNthItem(i);
+	}
+	else
+	{
+		UT_DEBUGMSG((" Requested Frame not found \n"));
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	}
+}
