@@ -36,7 +36,8 @@ typedef enum _fl_ContainerType
 	FL_CONTAINER_MARGINNOTE,
 	FL_CONTAINER_TABLE,
 	FL_CONTAINER_CELL,
-	FL_CONTAINER_FRAME
+	FL_CONTAINER_FRAME,
+	FL_CONTAINER_TOC
 } fl_ContainerType;
 
 // this enum is used here and by the run classes
@@ -45,7 +46,7 @@ typedef enum {FP_VISIBLE = 0,
 			  FP_HIDDEN_TEXT,
 			  FP_HIDDEN_REVISION,
 			  FP_HIDDEN_REVISION_AND_TEXT,
-			  FP_COLLAPSED
+			  FP_HIDDEN_FOLDED
 } FPVisibility;
 
 
@@ -55,6 +56,8 @@ typedef enum {FP_VISIBLE = 0,
 #include "fl_Layout.h"
 #include "ut_debugmsg.h"
 #include "ut_misc.h" // for UT_RGBColor
+#include "gr_Image.h"
+#include "gr_Graphics.h"
 
 class FL_DocLayout;
 class PD_Document;
@@ -67,16 +70,19 @@ class fp_Container;
 class fp_Run;
 class fl_DocSectionLayout;
 class UT_GrowBuf;
+class fl_FrameLayout;
+class fp_FrameContainer;
 class fl_BlockLayout;
 
 class ABI_EXPORT fl_ContainerLayout : public fl_Layout
 {
+	friend class fl_BlockLayout;
 public:
 	fl_ContainerLayout(fl_ContainerLayout* pLayout, PL_StruxDocHandle sdh, PT_AttrPropIndex ap, PTStruxType iStrux,fl_ContainerType iType);
 	virtual ~fl_ContainerLayout();
 
 	fl_ContainerType    getContainerType(void) const { return m_iConType; }
-
+	const char *        getContainerString(void);
 	const char *                getAttribute(const char * pKey) const;
 	virtual fp_Container*		getFirstContainer() const;
 	virtual fp_Container*		getLastContainer() const;
@@ -87,6 +93,7 @@ public:
 
 	virtual fp_Container*		getNewContainer(fp_Container * pFirstContainer = NULL) = 0;
 	virtual FL_DocLayout *      getDocLayout(void) const;
+	UT_sint32           getLevelInList(void);
 	virtual void		format(void) = 0;
 	virtual void        appendTextToBuf(UT_GrowBuf & buf);
 	virtual void		updateLayout(void) = 0;
@@ -98,12 +105,12 @@ public:
 	virtual bool        isCollapsed(void) const = 0;
 	virtual bool        needsReformat(void) const = 0;
 	virtual bool        needsRedraw(void) const = 0;
-	virtual void        updateBackgroundColor() = 0;
 	virtual fl_DocSectionLayout * getDocSectionLayout(void);
 	virtual fl_SectionLayout * getSectionLayout() const = 0;
 	virtual fl_HdrFtrSectionLayout * getHdrFtrSectionLayout(void) const;
 	void				setContainingLayout(fl_ContainerLayout*);
 	fl_ContainerLayout * myContainingLayout(void) const;
+	fl_HdrFtrSectionLayout *   getHdrFtrLayout(void);
 	void				setPrev(fl_ContainerLayout*);
 	void				setNext(fl_ContainerLayout*);
 	fl_ContainerLayout * getPrev(void) const;
@@ -123,18 +130,40 @@ public:
 	void                    setVisibility(FPVisibility eVis) {m_eHidden = eVis;}
 	bool                    isOnScreen() const;
 
- 	fl_BlockLayout*         getNextBlockInDocument(void) const;
- 	fl_BlockLayout*         getPrevBlockInDocument(void) const;
+// Frames stuff	
+	UT_sint32               getNumFrames(void) const;
+	fl_FrameLayout *        getNthFrameLayout(UT_sint32 i) const;
+	fp_FrameContainer *     getNthFrameContainer(UT_sint32 i) const;
+	void                    addFrame(fl_FrameLayout * pFrame);
+	void                    removeFrame(fl_FrameLayout * pFrame);
 	
+// For Folded Text
+
+	UT_sint32               getFoldedLevel(void);
+	UT_uint32               getFoldedID(void);
+    void                    lookupFoldedLevel(void);
+
+	void                    lookupProperties(void);
+	
+	fl_BlockLayout*         getNextBlockInDocument(void) const;
+	fl_BlockLayout*         getPrevBlockInDocument(void) const;
+
+	FPVisibility            getAP(const PP_AttrProp *& pAP)const;
+	void                    getSpanAP(UT_uint32 blockPos, bool bLeft, const PP_AttrProp * &pSpanAP) const;
 
 #ifdef FMT_TEST
 	virtual void		__dump(FILE * fp) const;
 #endif
 protected:
-private:
-	virtual bool            _canContainPoint() const {return true;}
+	// need this to be protected, so fl_BlockLayout can call it
 	void	                _insertIntoList(fl_ContainerLayout * pL);
 
+private:
+	virtual void            _lookupProperties(const PP_AttrProp* pAP) = 0;
+	
+	virtual bool            _canContainPoint() const {return true;}
+	void	                _insertFirst(fl_ContainerLayout * pL);
+
 	fl_ContainerType	        m_iConType;
 	fl_ContainerLayout*		    m_pMyLayout;
 
@@ -146,6 +175,9 @@ private:
 	fp_Container *              m_pFirstContainer;
 	fp_Container *              m_pLastContainer;
 	FPVisibility                m_eHidden;
+	UT_GenericVector<fl_FrameLayout *> m_vecFrames;
+    UT_sint32                   m_iFoldedLevel;
+    UT_uint32                   m_iFoldedID;
 };
 
 #endif /* CONTAINERLAYOUT_H */
