@@ -36,6 +36,7 @@ class fl_EndnoteSectionLayout;
 class fl_HdrFtrSectionLayout;
 class fl_DocSectionLayout;
 class fl_SectionLayout;
+class fp_TOCContainer;
 class fl_HdrFtrShadow;
 class fp_Page;
 class PP_AttrProp;
@@ -51,7 +52,9 @@ public:
 	virtual ~fp_VerticalContainer();
 
 	virtual void		setWidth(UT_sint32);
+	void		        _setWidth(UT_sint32 iWidth) {m_iWidth = iWidth;}
 	virtual void		setHeight(UT_sint32);
+	void		        _setHeight(UT_sint32 iHeight) {m_iHeight = iHeight;}
 	virtual void		setMaxHeight(UT_sint32);
 	virtual void		setX(UT_sint32, bool bDontClearIfNeeded=false);
 	virtual void		setY(UT_sint32);
@@ -74,6 +77,7 @@ public:
 	void        _setX( UT_sint32 iX) { m_iX = iX;}
 
 	virtual UT_sint32	getY(void) const;
+	UT_sint32	        getY(GR_Graphics * pG) const;
 
 	void        _setY( UT_sint32 iY) { m_iY = iY;}
 	/*!
@@ -105,23 +109,27 @@ public:
 	bool				insertContainerAfter(fp_Container* pNewContainer, fp_Container*	pAfterContainer);
 	bool				insertContainer(fp_Container*);
 	bool				addContainer(fp_Container*);
-	void				removeContainer(fp_Container*);
+	void				removeContainer(fp_Container* pContainer, bool bClear = false);
 
 	virtual UT_uint32 	distanceFromPoint(UT_sint32 x, UT_sint32 y);
 
 	virtual void		mapXYToPosition(UT_sint32 xPos,
 										UT_sint32 yPos,
 										PT_DocPosition& pos,
-										bool& bBOL, bool& bEOL);
+										bool& bBOL, bool& bEOL, bool &isTOC);
 
 	void		 		getOffsets(fp_ContainerObject* pContainer,
 								   UT_sint32& xoff,
 								   UT_sint32& yoff);
 	fp_TableContainer * getCorrectBrokenTable(fp_Container * pLine);
+	fp_TOCContainer *   getCorrectBrokenTOC(fp_Container * pLine);
 	void		 		getScreenOffsets(fp_ContainerObject* pContainer,
 										 UT_sint32& xoff,
 										 UT_sint32& yoff);
 
+    virtual UT_Rect *       getScreenRect();
+    virtual void            markDirtyOverlappingRuns(UT_Rect & recScreen);
+
 	virtual void		draw(dg_DrawArgs*);
 	virtual void		draw(GR_Graphics*) {}
 
@@ -144,13 +152,19 @@ public:
 	bool                validate(void);
 	FV_View*			getView(void) const
 		{ return getPage()->getDocLayout()->getView(); }
-
+	UT_sint32           getNumWrapped(void) const
+		{ return m_vecWrappedLines.getItemCount();}
+	void                addWrappedLine(fp_Line * pLine)
+		{ m_vecWrappedLines.addItem(pLine);}
+	void                clearWrappedLines(void)
+		{ m_vecWrappedLines.clear();}
+	fp_Line *           getNthWrappedLine(UT_sint32 i)
+		{ return m_vecWrappedLines.getNthItem(i);}
 protected:
     void                _setMaxContainerHeight(UT_sint32 iContainerHeight);
 	UT_sint32           _getMaxContainerHeight(void) const;
 
 	virtual void			_drawBoundaries(dg_DrawArgs* pDA);
-
 private:
 
 	/*!
@@ -186,6 +200,7 @@ private:
 	 */
 	bool					m_bIntentionallyEmpty;
 	UT_sint32               m_imaxContainerHeight;
+    UT_GenericVector<fp_Line *> m_vecWrappedLines;
 };
 
 class ABI_EXPORT fp_Column : public fp_VerticalContainer
@@ -205,7 +220,7 @@ public:
 	  \return Page
 	*/
 
-	void				setPage(fp_Page* pPage) {m_pPage = pPage ;}
+	void				setPage(fp_Page* pPage);
 	virtual fp_Page*	getPage(void) const
 		{ return m_pPage; }
 	/*!
@@ -216,7 +231,7 @@ public:
  	
 	void				layout(void);
 
-
+	void                collapseEndnotes(void);
 #ifdef FMT_TEST
 	void				__dump(FILE * fp) const;
 #endif
@@ -246,9 +261,10 @@ public:
  	virtual void		draw(dg_DrawArgs*);
  	virtual void		draw(GR_Graphics*) {};
   	virtual void		layout(void);
+	void                layout(bool bForce);
  	virtual void		clearScreen(void);
 	void                clearHdrFtrBoundaries(void);
-	void				setPage(fp_Page* pPage) {m_pPage = pPage ;}
+	void				setPage(fp_Page* pPage);
 	virtual inline fp_Page*		getPage(void) const
 		{ return m_pPage; }
 protected:
