@@ -20,111 +20,72 @@
 #ifndef FV_VIEW_H
 #define FV_VIEW_H
 
-#include "ut_misc.h"
+#include "xap_Features.h"
+
+//#include "ut_misc.h"
 #include "ut_types.h"
+
 #include "xav_View.h"
+
 #include "pt_Types.h"
-#include "gr_DrawArgs.h"
-#include "gr_Graphics.h"
-#include "ev_EditBits.h"
 #include "ie_types.h"
-#include "xap_Prefs.h"
-#include "ap_Dialog_Goto.h"
-#include "fl_AutoLists.h"
-#include "fl_SectionLayout.h"
-#include "pp_Revision.h"
-#include "ap_TopRuler.h"
-#include "ap_LeftRuler.h"
+#include "ap_types.h"
+#include "fp_types.h"
+
+#include "ev_EditBits.h"
+
+
+// have to include these as they are instantiated in the FV_View
+// class definition
+#include "fv_FrameEdit.h"
+#include "fv_VisualDragText.h"
+#include "fv_Selection.h"
+
 #define AUTO_SCROLL_MSECS	100
 
 class FL_DocLayout;
+class FV_Caret_Listener;
+
+class fl_DocSectionLayout;
+class fl_HdrFtrSectionLayout;
 class fl_DocListener;
 class fl_BlockLayout;
 class fl_PartOfBlock;
+class fl_AutoNum;
+class fl_EndnoteLayout;
+
+class fp_PageSize;
 class fp_Page;
 class fp_Run;
 class fp_HyperlinkRun;
+class fp_CellContainer;
+
 class FG_Graphic;
+
 class PD_Document;
+class PP_AttrProp;
+class PP_RevisionAttr;
+
 class GR_Graphics;
+struct dg_DrawArgs;
+
 class UT_Worker;
 class UT_Timer;
+class UT_UTF8String;
+
 class AP_TopRulerInfo;
 class AP_LeftRulerInfo;
-class XAP_App;
-class XAP_Prefs;
-class UT_StringPtrMap;
-class PP_AttrProp;
-class fl_AutoNum;
-class fp_PageSize;
 class AP_TopRuler;
 class AP_LeftRuler;
+
+class AP_Dialog_SplitCells;
+
+class XAP_App;
+class XAP_Prefs;
+
 class SpellChecker;
-class FV_Caret_Listener;
 
-typedef enum _FVDocPos
-{
-	FV_DOCPOS_BOB, FV_DOCPOS_EOB,	// block
-	FV_DOCPOS_BOD, FV_DOCPOS_EOD,	// document
-	FV_DOCPOS_BOP, FV_DOCPOS_EOP,	// page
-	FV_DOCPOS_BOL, FV_DOCPOS_EOL,	// line
-	FV_DOCPOS_BOS, FV_DOCPOS_EOS,	// sentence
-	FV_DOCPOS_BOW, FV_DOCPOS_EOW_MOVE, FV_DOCPOS_EOW_SELECT // word
-} FV_DocPos;
-
-typedef enum _ToggleCase
-{
-  CASE_SENTENCE,
-  CASE_LOWER,
-  CASE_UPPER,
-  CASE_TITLE,
-  CASE_TOGGLE,
-  CASE_FIRST_CAPITAL,
-  CASE_ROTATE
-} ToggleCase;
-
-typedef enum _FormatTable
-{
-	FORMAT_TABLE_SELECTION,
-	FORMAT_TABLE_ROW,
-	FORMAT_TABLE_COLUMN,
-	FORMAT_TABLE_TABLE
-} FormatTable;
-
-typedef enum
-{
-	BreakSectionContinuous,
-	BreakSectionNextPage,
-	BreakSectionEvenPage,
-	BreakSectionOddPage
-} BreakSectionType;
-
-typedef enum
-{
-  VIEW_PRINT,
-  VIEW_NORMAL,
-  VIEW_WEB,
-  VIEW_PREVIEW
-} ViewMode;
-
-typedef enum
-{
-  PREVIEW_NONE,
-  PREVIEW_ZOOMED,
-  PREVIEW_ADJUSTED_PAGE,
-  PREVIEW_CLIPPED,
-  PREVIEW_ZOOMED_SCROLL,
-  PREVIEW_ADJUSTED_PAGE_SCROLL,
-  PREVIEW_CLIPPED_SCROLL
-} PreViewMode;
-
-typedef enum _AP_JumpTarget
-{
-	AP_JUMPTARGET_PAGE, 			// beginning of page
-	AP_JUMPTARGET_LINE,
-	AP_JUMPTARGET_BOOKMARK,
-	AP_JUMPTARGET_PICTURE // TODO
-} AP_JumpTarget;
+class CellLine;
 
 struct fv_ChangeState
 {
@@ -165,6 +126,13 @@ private:
 	XML_Char **       m_pszProps;
 };
 
+enum FV_BIDI_Order
+{
+	FV_Order_Visual = 0,
+	FV_Order_Logical_LTR = UT_BIDI_LTR,
+	FV_Order_Logical_RTL = UT_BIDI_RTL
+};
+
 								
 class ABI_EXPORT FV_View : public AV_View
 {
@@ -174,14 +142,22 @@ class ABI_EXPORT FV_View : public AV_View
 	friend class fl_Squiggles;
 	friend class fl_DocSectionLayout;
 	friend class GR_Caret;
+	friend class FV_FrameEdit;
+	friend class FV_VisualDragText;
+	friend class FV_Selection;
+	friend class CellLine;
 
 public:
 	FV_View(XAP_App*, void*, FL_DocLayout*);
 	virtual ~FV_View();
 
 	virtual inline GR_Graphics*    getGraphics(void) const { return m_pG; }
-	virtual inline UT_uint32	  getPoint(void) const { return m_iInsPoint; }
-	inline UT_uint32		getSelectionAnchor(void) const { return m_bSelection? m_iSelectionAnchor : m_iInsPoint; }
+	void  setGraphics(GR_Graphics *pG);
+	void  replaceGraphics(GR_Graphics *pG);
+	
+	virtual inline PT_DocPosition   getPoint(void) const { return m_iInsPoint; }
+	PT_DocPosition	getSelectionAnchor(void) const;
+	UT_uint32       getSelectionLength(void) const;
 
 	virtual void focusChange(AV_Focus focus);
 
@@ -190,9 +166,12 @@ public:
 	virtual void	cmdScroll(AV_ScrollCmd cmd, UT_uint32 iPos = 0);
 
 	virtual void	cmdHyperlinkJump(UT_sint32 xPos, UT_sint32 yPos);
+	void	        cmdHyperlinkJump(PT_DocPosition pos);
 
 	virtual void	draw(const UT_Rect* pRect=static_cast<UT_Rect*>(NULL));
 
+	const PP_AttrProp * getAttrPropForPoint();
+
 	virtual bool	notifyListeners(const AV_ChangeMask hint);
 
 	virtual bool	canDo(bool bUndo) const;
@@ -207,11 +186,13 @@ public:
 	UT_Error		cmdInsertBookmark(const char* szName);
 	UT_Error		cmdDeleteBookmark(const char* szName);
 	UT_Error		cmdInsertHyperlink(const char* szName);
+	fp_Run *        getHyperLinkRun(PT_DocPosition pos);
 	UT_Error		cmdDeleteHyperlink();
+	UT_Error		cmdInsertTOC(void);
 	UT_Error		cmdHyperlinkStatusBar(UT_sint32 xPos, UT_sint32 yPos);
 
-	UT_Error		cmdInsertGraphic(FG_Graphic*, const char*);
-
+	UT_Error		cmdInsertGraphic(FG_Graphic*);
+	UT_Error        cmdInsertGraphicAtStrux(FG_Graphic* pFG, PT_DocPosition iPos, PTStruxType iStruxType);
 	virtual void	toggleCase(ToggleCase c);
 	virtual void	setPaperColor(const XML_Char * clr);
 
@@ -221,6 +202,10 @@ public:
 	virtual void	cmdPaste(bool bHonorFormatting = true);
 	virtual void	cmdPasteSelectionAt(UT_sint32 xPos, UT_sint32 yPos);
 
+	void            pasteFromLocalTo(PT_DocPosition pos);
+	void            _pasteFromLocalTo(PT_DocPosition pos);
+	void            copyToLocal(PT_DocPosition pos1, PT_DocPosition pos2);
+
 	virtual void	getTopRulerInfo(AP_TopRulerInfo * pInfo);
 	virtual void	getTopRulerInfo(PT_DocPosition pos, AP_TopRulerInfo * pInfo);
 	virtual void	getLeftRulerInfo(AP_LeftRulerInfo * pInfo);
@@ -231,12 +216,15 @@ public:
 	virtual EV_EditMouseContext getMouseContext(UT_sint32 xPos, UT_sint32 yPos);
 	virtual EV_EditMouseContext getInsertionPointContext(UT_sint32 * pxPos, UT_sint32 * pyPos);
 	virtual void        updateLayout(void);
+	virtual void        rebuildLayout(void);
 	virtual bool		isSelectionEmpty(void) const;
 	virtual void		cmdUnselectSelection(void);
 	void				getDocumentRangeOfCurrentSelection(PD_DocumentRange * pdr);
 	PT_DocPosition		mapDocPos( FV_DocPos dp );
+	PT_DocPosition		mapDocPosSimple( FV_DocPos dp );
 	PT_DocPosition saveSelectedImage (const char * toFile );
 	PT_DocPosition saveSelectedImage (const UT_ByteBuf ** outByteBuf);
+	PT_DocPosition getSelectedImage(const char **dataId);
 
 	void            getTextInCurrentBlock(UT_GrowBuf & buf);
 	void            getTextInCurrentSection(UT_GrowBuf & buf);
@@ -262,25 +250,28 @@ public:
 	bool	getSectionFormat(const XML_Char *** properties);
 
 	bool	setBlockIndents(bool doLists, double indentChange, double page_size);
+	bool    setCollapsedRange(PT_DocPosition posLow,PT_DocPosition posHigh, const XML_Char * properties[]);
 	bool	setBlockFormat(const XML_Char * properties[]);
 	bool	getBlockFormat(const XML_Char *** properties,bool bExpandStyles=true);
+	bool    removeStruxAttrProps(PT_DocPosition ipos1, PT_DocPosition ipos2, PTStruxType iStrux,const XML_Char * attributes[] ,const XML_Char * properties[]);
+	bool    isImageAtStrux(PT_DocPosition ipos1, PTStruxType iStrux);
 
 	bool	processPageNumber(HdrFtrType hfType, const XML_Char ** atts);
 
 	bool	isTextMisspelled()const ;
 	bool	isTabListBehindPoint(void);
 	bool	isTabListAheadPoint(void);
-	void	processSelectedBlocks(List_Type listType);
-	void	getBlocksInSelection( UT_Vector * vBlock);
+	void	processSelectedBlocks(FL_ListType listType);
+	void	getBlocksInSelection(UT_GenericVector<fl_BlockLayout*> * vBlock);
 	UT_sint32 getNumColumnsInSelection(void);
 	UT_sint32 getNumRowsInSelection(void);
-	void	getAllBlocksInList( UT_Vector * vBlock);
+	void	getAllBlocksInList(UT_GenericVector<fl_BlockLayout *> * vBlock);
 	bool	isPointBeforeListLabel(void);
 	bool	isCurrentListBlockEmpty(void);
 	bool	cmdStartList(const XML_Char * style);
 	bool	cmdStopList(void);
 	void	changeListStyle(fl_AutoNum* pAuto,
-							List_Type lType,
+							FL_ListType lType,
 							UT_uint32 startv,
 							const XML_Char* pszDelim,
 							const XML_Char* pszDecimal,
@@ -291,7 +282,18 @@ public:
 	void	setDontChangeInsPoint(void);
 	void	allowChangeInsPoint(void);
 
+	bool    getAttributes(const PP_AttrProp ** ppSpanAP, const PP_AttrProp ** ppBlockAP = NULL, PT_DocPosition posStart = 0);
+
+	/* Experimental, for the moment; use with caution. - fjf, 24th Oct. '04
+	 */
+	// - begin
+	bool    getAllAttrProp(const PP_AttrProp *& pSpanAP, const PP_AttrProp *& pBlockAP, const PP_AttrProp *& pSectionAP, const PP_AttrProp *& pDocAP) const;
+	bool	queryCharFormat(const XML_Char * szProperty, UT_UTF8String & szValue, bool & bExplicitlyDefined, bool & bMixedSelection) const;
+	bool	queryCharFormat(const XML_Char * szProperty, UT_UTF8String & szValue, bool & bExplicitlyDefined, PT_DocPosition position) const;
+	// - end
+
 	bool	setCharFormat(const XML_Char * properties[], const XML_Char * attribs[] = NULL);
+	bool	resetCharFormat(bool bAll);
 	bool	getCharFormat(const XML_Char *** properties,bool bExpandStyles=true);
 	bool	getCharFormat(const XML_Char *** properties,bool bExpandStyles, PT_DocPosition posStart);
 	fl_BlockLayout * getBlockFromSDH(PL_StruxDocHandle sdh);
@@ -305,6 +307,8 @@ public:
 
 	bool	getEditableBounds(bool bEnd, PT_DocPosition & docPos, bool bOverride=false)const;
 
+	bool    isParaBreakNeededAtPos(PT_DocPosition pos);
+	bool    insertParaBreakIfNeededAtPos(PT_DocPosition pos);
 	void	insertParagraphBreak(void);
 	void	insertParagraphBreaknoListUpdate(void);
 	void	insertSectionBreak( BreakSectionType type);
@@ -314,12 +318,16 @@ public:
 	// ----------------------
 	bool			isLeftMargin(UT_sint32 xPos, UT_sint32 yPos);
 	void			cmdSelect(UT_sint32 xPos, UT_sint32 yPos, FV_DocPos dpBeg, FV_DocPos dpEnd);
-	void					cmdSelect(PT_DocPosition dpBeg, PT_DocPosition dpEnd);
+	void			cmdSelectTOC(UT_sint32 xPos, UT_sint32 yPos);
+	bool            isTOCSelected(void);
+	bool            setTOCProps(PT_DocPosition pos, const char * szProps);
+
+	void			cmdSelect(PT_DocPosition dpBeg, PT_DocPosition dpEnd);
 	void			cmdCharMotion(bool bForward, UT_uint32 count);
 	bool			cmdCharInsert(const UT_UCSChar * text, UT_uint32 count, bool bForce = false);
 	void			cmdCharDelete(bool bForward, UT_uint32 count);
 	void			delTo(FV_DocPos dp);
-	UT_UCSChar *	getSelectionText(void);
+	void            getSelectionText(UT_UCS4Char *& text);
 
 	UT_UCSChar *	getTextBetweenPos(PT_DocPosition pos1, PT_DocPosition pos2);
 	inline PT_DocPosition  getInsPoint () const { return m_iInsPoint; }
@@ -343,18 +351,49 @@ public:
 
 	void endDragSelection(UT_sint32 xPos, UT_sint32 yPos);
 
-	PT_DocPosition  getDocPositionFromXY(UT_sint32 xpos, UT_sint32 ypos);
+	PT_DocPosition  getDocPositionFromXY(UT_sint32 xpos, UT_sint32 ypos, bool bNotFrames = false);
 	PT_DocPosition  getDocPositionFromLastXY(void);
 
 	fl_BlockLayout* getBlockAtPosition(PT_DocPosition pos) const {return _findBlockAtPosition(pos);};
 	virtual void	updateScreen(bool bDirtyRunsOnly=true);
+	bool            isInDocSection(PT_DocPosition pos = 0);
 
+//---------
+//Visual Drag stuff
+//
+	void            cutVisualText(UT_sint32 x, UT_sint32 y);
+	void            copyVisualText(UT_sint32 x, UT_sint32 y);
+	void            dragVisualText(UT_sint32 x, UT_sint32 y);
+	void            pasteVisualText(UT_sint32 x, UT_sint32 y);
+	void            btn0VisualDrag(UT_sint32 x, UT_sint32 y);
+// -------
+// Frame stuff
+//
+	FV_FrameEdit *  getFrameEdit(void);
+	void            btn0Frame(UT_sint32 x, UT_sint32 y);
+	void            btn1Frame(UT_sint32 x, UT_sint32 y);
+	void            dragFrame(UT_sint32 x, UT_sint32 y);
+	void            releaseFrame(UT_sint32 x, UT_sint32 y);
+	bool            isInFrame(PT_DocPosition pos);
+	void            deleteFrame(void);
+	void            cutFrame(void);
+	void            selectFrame(void);
+	fl_FrameLayout * getFrameLayout(PT_DocPosition pos);
+	fl_FrameLayout * getFrameLayout(void);
+	void            setFrameFormat(const XML_Char ** props);
+	void            setFrameFormat(const XML_Char ** props,FG_Graphic * pFG, UT_String & dataID);
+	void            convertInLineToPositioned(PT_DocPosition pos, 
+											const XML_Char ** attribs);
 
 // ----------------------
 
 	bool			isPosSelected(PT_DocPosition pos) const;
 	bool			isXYSelected(UT_sint32 xPos, UT_sint32 yPos) const;
-
+	FV_SelectionMode getSelectionMode(void) const;
+	FV_SelectionMode getPrevSelectionMode(void) const;
+	PD_DocumentRange * getNthSelection(UT_sint32 i);
+	UT_sint32          getNumSelections(void) const;
+	void            setSelectionMode(FV_SelectionMode selMode);
 // ----------------------
 // Stuff for spellcheck context menu
 //
@@ -365,6 +404,7 @@ public:
 // ----------------------
 // Stuff for edittable Headers/Footers
 //
+	bool                isInHdrFtr(PT_DocPosition pos);
 	void				setHdrFtrEdit(fl_HdrFtrShadow * pShadow);
 	void				clearHdrFtrEdit(void);
 	bool				isHdrFtrEdit(void);
@@ -446,10 +486,12 @@ public:
 	bool			findReplaceReverse(bool& bDoneEntireDocument);
 
 	bool			_findReplaceReverse(UT_uint32* pPrefix,
-								 bool& bDoneEntireDocument);
+										bool& bDoneEntireDocument,
+										bool bNoUpdate);
 
 	bool			_findReplace(UT_uint32* pPrefix,
-								bool& bDoneEntireDocument);
+								 bool& bDoneEntireDocument,
+								 bool bNoUpdate);
 
 
 	bool			findReplace(bool& bDoneEntireDocument);
@@ -497,17 +539,37 @@ public:
 	/* Revision related functions */
 	void                toggleMarkRevisions();
 	void                cmdAcceptRejectRevision(bool bReject, UT_sint32 x, UT_sint32 y);
-	bool                isMarkRevisions();
-	void                cmdSetRevisionLevel(UT_uint32 i){m_iViewRevision = i;}
-	UT_uint32           getRevisionLevel()const{return m_iViewRevision;}
+	// NB: 'mark revisions' state is document-wide
+	bool                isMarkRevisions() const;
+	// NB: 'show revisions' state is view-specific
+	bool                isShowRevisions() const {return m_bShowRevisions;}
+	void                toggleShowRevisions();
+	void                setShowRevisions(bool bShow);
+	
+	void                cmdSetRevisionLevel(UT_uint32 i);
+	UT_uint32           getRevisionLevel()const;
+	void                setRevisionLevel(UT_uint32 i);
 
+	bool                cmdFindRevision(bool bNext, UT_sint32 xPos, UT_sint32 yPos);
+	bool                doesSelectionContainRevision() const;
+
+	void                updateRevisionMode();
+  protected:
+	void                _fixInsertionPointAfterRevision();
+	bool                _makePointLegal(void);
+  public:
+	
 	/* Table related functions */
 	bool                isPointLegal(PT_DocPosition pos);
 	bool                isPointLegal(void);
 	bool				isInTable();
+	fl_TableLayout *    getTableAtPos(PT_DocPosition);
 	bool				isInTable(PT_DocPosition pos);
+	bool				isInTableForSure(PT_DocPosition pos);
 	bool                cmdAutoSizeCols(void);
+	bool                cmdTextToTable(bool bIgnoreSpaces);
 	bool                cmdAutoSizeRows(void);
+	bool                cmdAdvanceNextPrevCell(bool bGoNext);
 	fp_CellContainer *  getCellAtPos(PT_DocPosition pos);
 	PT_DocPosition      findCellPosAt(PT_DocPosition posTable, UT_sint32 row, UT_sint32 col);
 	bool                _deleteCellAt(PT_DocPosition posTable,UT_sint32 row, UT_sint32 col);
@@ -520,17 +582,21 @@ public:
 	bool                cmdDeleteTable(PT_DocPosition pos);
 	bool                cmdInsertRow(PT_DocPosition posTable, bool bBfore);
 	bool                cmdInsertCol(PT_DocPosition posTable, bool bBefore);
+	bool                cmdSplitCells(AP_CellSplitType iSplitType);
+	bool                cmdSelectColumn(PT_DocPosition posOfColumn);
+	bool                cmdAutoFitTable(void);
 	bool                cmdMergeCells(PT_DocPosition posSource, PT_DocPosition posDestination);
 	bool                _MergeCells( PT_DocPosition posDestination,PT_DocPosition posSource, bool bBefore);
 	bool                getCellParams(PT_DocPosition posCol, UT_sint32 *iLeft, 
 									  UT_sint32 *iRight,UT_sint32 *iTop, UT_sint32 *iBot);
 	bool				getCellLineStyle(PT_DocPosition posCell, UT_sint32 * pLeft, UT_sint32 * pRight,
 										 UT_sint32 * pTop, UT_sint32 * pBot);
-	bool				setCellFormat(const XML_Char * properties[], FormatTable applyTo);
-	bool				getCellBGColor(XML_Char * &color);
+	bool				setCellFormat(const XML_Char * properties[], FormatTable applyTo, FG_Graphic * pFG, UT_String & sDataID);
+	bool				getCellProperty(XML_Char * szPropName, XML_Char * &szPropValue);
 	bool	            setTableFormat(const XML_Char * properties[]);
 	bool	            setTableFormat(PT_DocPosition pos,const XML_Char * properties[]);
-	
+	bool                getCellFormat(PT_DocPosition pos, UT_String & sCellProps);
+
 	UT_Error            cmdInsertTable(UT_sint32 numRows, UT_sint32 numCols, 
 									   const XML_Char * pPropsArray[]);
 	bool                _changeCellTo(PT_DocPosition posTable,UT_sint32 rowOld, 
@@ -563,7 +629,8 @@ public:
 	UT_RGBColor			getColorHdrFtr(void) const { return m_colorHdrFtr; }
 	UT_RGBColor			getColorColumnLine(void) const { return m_colorColumnLine; }
 
-	void                getVisibleDocumentPagesAndRectangles(UT_Vector &vRect, UT_Vector &vPages) const;
+	void                getVisibleDocumentPagesAndRectangles(UT_GenericVector<UT_Rect*> &vRect, 
+															 UT_GenericVector<fp_Page*> &vPages) const;
 
 	//
 	// image selection && resizing && dragging functions
@@ -574,6 +641,10 @@ public:
 	GR_Graphics::Cursor getImageSelCursor();
 	void				setCurImageSel(UT_Rect r);
 	UT_Rect				getCurImageSel();
+#if XAP_DONTUSE_XOR
+	void				setCurImageSelCache(GR_Image* cache);
+	GR_Image* 			getCurImageSelCache();
+#endif
 	bool				isOverImageResizeBox(GR_Graphics::Cursor &cur, UT_uint32 xPos, UT_uint32 yPos);
 	void				startImageResizing(UT_sint32 xPos, UT_sint32 yPos);
 	void				stopImageResizing();
@@ -584,6 +655,8 @@ public:
 	void				drawDraggedImage(UT_sint32 xPos, UT_sint32 yPos);
 	void				stopImageDrag(UT_sint32 xPos, UT_sint32 yPos);
 
+	bool                isImageSelected(void);
+
 //
 // Table resizing
 //
@@ -601,6 +674,10 @@ public:
 		                { return m_pLeftRuler;}
 	
 
+	const XML_Char **   getViewPersistentProps();
+	FV_BIDI_Order	    getBidiOrder()const {return m_eBidiOrder;}
+	void                setBidiOrder(FV_BIDI_Order o) {m_eBidiOrder = o;}
+
 protected:
 	void				_saveAndNotifyPieceTableChange(void);
 	void				_restorePieceTableState(void);
@@ -610,7 +687,7 @@ protected:
 
 	void				_drawBetweenPositions(PT_DocPosition left, PT_DocPosition right);
 	bool				_clearBetweenPositions(PT_DocPosition left, PT_DocPosition right, bool bFullLineHeightRect);
-	bool                _drawOrClearBetweenPositions(PT_DocPosition iPos1, PT_DocPosition iPos2, bool bClear, bool bFullLineHeight);
+	bool                		_drawOrClearBetweenPositions(PT_DocPosition iPos1, PT_DocPosition iPos2, bool bClear, bool bFullLineHeight);
 	bool				_ensureInsertionPointOnScreen();
 	void				_moveInsPtNextPrevPage(bool bNext);
 	void				_moveInsPtNextPrevScreen(bool bNext);
@@ -640,13 +717,15 @@ protected:
 									  UT_sint32 yPos,
 									  UT_sint32& xClick,
 									  UT_sint32& yClick) const;
-
+	bool                _insertField(const char* szName, 
+									 const XML_Char ** extra_attrs = NULL, 
+									 const XML_Char ** extra_props = NULL);
 	void				_moveToSelectionEnd(bool bForward);
 	void				_eraseSelection(void);
 	void				_clearSelection(void);
 	void				_resetSelection(void);
 	void				_setSelectionAnchor(void);
-	void				_deleteSelection(PP_AttrProp *p_AttrProp_Before = NULL);
+	void				_deleteSelection(PP_AttrProp *p_AttrProp_Before = NULL, bool bNoUpdate = false);
 	bool				_insertFormatPair(const XML_Char * szName, const XML_Char * properties[]);
 	void				_updateInsertionPoint();
 	void				_fixInsertionPointCoords();
@@ -663,7 +742,7 @@ protected:
 	// localize handling of insertion point logic
 	void				_setPoint(PT_DocPosition pt, bool bEOL = false);
 	UT_uint32			_getDataCount(UT_uint32 pt1, UT_uint32 pt2);
-	bool				_charMotion(bool bForward,UT_uint32 countChars);
+	bool				_charMotion(bool bForward,UT_uint32 countChars, bool bSkipCannotContainPoint = true);
 	void				_doPaste(bool bUseClipboard, bool bHonorFormatting = true);
 	void				_clearIfAtFmtMark(PT_DocPosition dpos);
 
@@ -677,12 +756,10 @@ protected:
 	UT_Error			_deleteHyperlink(PT_DocPosition &i, bool bSignal);
 	fp_HyperlinkRun *   _getHyperlinkInRange(PT_DocPosition &posStart,
 											 PT_DocPosition &posEnd);
+	bool			    _charInsert(const UT_UCSChar * text, UT_uint32 count, bool bForce = false);
 
-	void                _acceptRejectRevision(bool bReject,
-											  PT_DocPosition iStart,
-											  PT_DocPosition iEnd,
-											  const PP_RevisionAttr * pRevAttr);
-
+	void                _adjustDeletePosition(UT_uint32 &iDocPos, UT_uint32 &iCount);
+	
 private:
 	PT_DocPosition		m_iInsPoint;
 	UT_sint32			m_xPoint;
@@ -704,11 +781,6 @@ private:
 	GR_Graphics*		m_pG;
 	void*				m_pParentData;
 
-	PT_DocPosition		m_iSelectionAnchor;
-	PT_DocPosition		m_iSelectionLeftAnchor;
-	PT_DocPosition		m_iSelectionRightAnchor;
-	bool				m_bSelection;
-
 	// autoscroll stuff
 	UT_Timer *			m_pAutoScrollTimer;
 	UT_sint32			m_xLastMouse;
@@ -767,13 +839,15 @@ private:
 
 	// properties for image selection
 	UT_Rect				m_selImageRect;
-	UT_uint32			m_iImageSelBoxSize;
+	UT_uint32			m_iImageSelBoxSize;  // in device units!
 	GR_Graphics::Cursor	m_imageSelCursor;
 	UT_sint32			m_ixResizeOrigin;
 	UT_sint32			m_iyResizeOrigin;
 	bool				m_bIsResizingImage;
 	UT_Rect				m_curImageSel;
-
+#if XAP_DONTUSE_XOR
+	GR_Image*			m_curImageSelCache;
+#endif
 	// properties for image dragging
 	bool				m_bIsDraggingImage;
 	fp_Run *			m_pDraggedImageRun;
@@ -807,6 +881,17 @@ private:
 	fv_PropCache        m_CharProps;
 	fv_PropCache        m_BlockProps;
 	fv_PropCache        m_SecProps;
+	AV_ListenerId       m_CaretListID;
+	FV_FrameEdit        m_FrameEdit;
+	FV_VisualDragText   m_VisualDragText;
+	FV_Selection        m_Selection;
+	bool                m_bShowRevisions;
+
+	FV_BIDI_Order       m_eBidiOrder;
+	UT_uint32           m_iFreePass;
+	bool                m_bDontNotifyListeners;
+	UT_ByteBuf *        m_pLocalBuf;
+	UT_sint32           m_iGrabCell;
 };
 
 #endif /* FV_VIEW_H */
