@@ -39,6 +39,13 @@
   TODO do we want this list of last-resort default settings to be here?
   It seems out of place... --EWS
 */
+/*
+	Response: I agree that it seems out of place.  It's inconsistent with
+				the per-class definition and organization of the properties.
+				IE, the pd_Document use of dom-dir.  We don't even have a
+				PP_LEVEL_DOC represented here.		-MG
+*/
+
 /*
 	We need to be able to change the BiDi relevant dafault properties at runtime
 	in response to the user changing the default direction in preferences.
@@ -68,10 +75,18 @@ static PP_Property _props[] =
 {
 	{ "background-color",      "transparent",     false, NULL, PP_LEVEL_SECT},
 	{ "bgcolor",               "transparent",     true,  NULL, PP_LEVEL_CHAR},
+	{ "bot-attach",            "",               false,  NULL, PP_LEVEL_TABLE},
 	{ "bot-color",             "000000",          false, NULL, PP_LEVEL_TABLE},
 	{ "bot-style",             "solid",           false, NULL, PP_LEVEL_TABLE},
 	{ "bot-thickness",         "1px",             false, NULL, PP_LEVEL_TABLE},
 
+	{ "bounding-space",        "0.05in",          false, NULL, PP_LEVEL_FRAME},
+
+	{ "cell-margin-bottom",   "0.002in",         false,  NULL, PP_LEVEL_TABLE},
+	{ "cell-margin-left",     "0.002in",         false,  NULL, PP_LEVEL_TABLE},
+	{ "cell-margin-right",    "0.002in",         false,  NULL, PP_LEVEL_TABLE},
+	{ "cell-margin-top",      "0.002in",         false,  NULL, PP_LEVEL_TABLE},
+
 	{ "color",                 "000000",          true,  NULL, PP_LEVEL_CHAR},
 	{ "column-gap",	           "0.25in",          false, NULL, PP_LEVEL_SECT},
 	{ "column-line",           "off",	          false, NULL, PP_LEVEL_SECT},
@@ -96,6 +111,17 @@ static PP_Property _props[] =
 	{ "footer-last",           "",                false, NULL, PP_LEVEL_SECT},
 	{ "format",                "%*%d.",           true,  NULL, PP_LEVEL_BLOCK},
 
+	{"frame-col-xpos",         "0.0in",           false, NULL, PP_LEVEL_FRAME},
+	{"frame-col-ypos",         "0.0in",           false, NULL, PP_LEVEL_FRAME},
+	{"frame-height",           "0.0in",           false, NULL, PP_LEVEL_FRAME},
+	{"frame-page-xpos",        "0.0in",           false, NULL, PP_LEVEL_FRAME},
+	{"frame-page-ypos",        "0.0in",           false, NULL, PP_LEVEL_FRAME},
+	{"frame-position-to",      "block-above-text",false, NULL, PP_LEVEL_FRAME},
+	{"frame-type",             "textbox",         false, NULL, PP_LEVEL_FRAME},
+	{"frame-width",            "0.0in",           false, NULL, PP_LEVEL_FRAME},
+	{"frame-xpos",             "0.0in",           false, NULL, PP_LEVEL_FRAME},
+	{"frame-ypos",             "0.0in",           false, NULL, PP_LEVEL_FRAME},
+
 	{ "header",                "",                false, NULL, PP_LEVEL_SECT},
 	{ "header-even",           "",                false, NULL, PP_LEVEL_SECT},
 	{ "header-first",          "",                false, NULL, PP_LEVEL_SECT},
@@ -107,6 +133,7 @@ static PP_Property _props[] =
 	{ "keep-with-next",        "no",              false, NULL, PP_LEVEL_BLOCK},
 
 	{ "lang",                  "en-US",           true,  NULL, PP_LEVEL_CHAR},
+	{ "left-attach",           "",               false,  NULL, PP_LEVEL_TABLE},
 	{ "left-color",            "000000",          false, NULL, PP_LEVEL_TABLE},
 	{ "left-style",            "solid",           false, NULL, PP_LEVEL_TABLE},
 	{ "left-thickness",        "1px",             false, NULL, PP_LEVEL_TABLE},
@@ -130,10 +157,13 @@ static PP_Property _props[] =
 	{ "page-margin-right",     "1in",             false, NULL, PP_LEVEL_SECT},
 	{ "page-margin-top",       "1in",             false, NULL, PP_LEVEL_SECT},
 
+	{ "right-attach",         "",               false,  NULL, PP_LEVEL_TABLE},
 	{ "right-color",           "000000",          false, NULL, PP_LEVEL_TABLE},
 	{ "right-style",           "solid",           false, NULL, PP_LEVEL_TABLE},
 	{ "right-thickness",       "1px",             false, NULL, PP_LEVEL_TABLE},
 
+	{ "section-footnote-line-thickness","0.005in",false, NULL, PP_LEVEL_SECT},
+	{ "section-footnote-yoff", "0.01in",          false, NULL, PP_LEVEL_SECT},
 	{ "section-max-column-height", "0in",         false, NULL, PP_LEVEL_SECT},
 	{ "section-restart",       "",                false, NULL, PP_LEVEL_SECT},
 	{ "section-restart-value", "",                false, NULL, PP_LEVEL_SECT},
@@ -141,28 +171,82 @@ static PP_Property _props[] =
 	{ "start-value",           "1",               true,  NULL, PP_LEVEL_BLOCK},
 
 	{ "table-border",          "0.1in",           false, NULL, PP_LEVEL_TABLE},
-	{ "table-col-spacing",     "0.05in",          false, NULL, PP_LEVEL_TABLE},
+	{ "table-col-spacing",     "0.03in",          false, NULL, PP_LEVEL_TABLE},
 	{ "table-column-leftpos",  "0.0in",           false, NULL, PP_LEVEL_TABLE},
 	{ "table-column-props",    "",                false, NULL, PP_LEVEL_TABLE},
-	{ "table-line-thickness",  "1",               false, NULL, PP_LEVEL_TABLE},
+	{ "table-line-thickness",  "0.8pt",               false, NULL, PP_LEVEL_TABLE},
 	{ "table-line-type",       "1",               false, NULL, PP_LEVEL_TABLE},
-	{ "table-margin-bottom",   "0.1in",           false, NULL, PP_LEVEL_TABLE},
- 	{ "table-margin-left",     "0.1in",           false, NULL, PP_LEVEL_TABLE},
-	{ "table-margin-right",    "0.1in",           false, NULL, PP_LEVEL_TABLE},
-	{ "table-margin-top",      "0.1in",           false, NULL, PP_LEVEL_TABLE},
+	{ "table-margin-bottom",   "0.01in",           false, NULL, PP_LEVEL_TABLE},
+ 	{ "table-margin-left",     "0.005in",           false, NULL, PP_LEVEL_TABLE},
+	{ "table-margin-right",    "0.005in",           false, NULL, PP_LEVEL_TABLE},
+	{ "table-margin-top",      "0.01in",           false, NULL, PP_LEVEL_TABLE},
 	{ "table-row-props",       "",                false, NULL, PP_LEVEL_TABLE},
-	{ "table-row-spacing",     "0.1in",           false, NULL, PP_LEVEL_TABLE},
+	{ "table-row-spacing",     "0.01in",           false, NULL, PP_LEVEL_TABLE},
 	{ "tabstops",              "",                false, NULL, PP_LEVEL_BLOCK},
 	{ "text-align",            text_align,	      true,  NULL, PP_LEVEL_BLOCK},
 	{ "text-decoration",       "none",            true,  NULL, PP_LEVEL_CHAR},
+	{ "text-folded",           "0",               false, NULL, PP_LEVEL_BLOCK},
+	{ "text-folded-id",        "0",               false, NULL, PP_LEVEL_BLOCK},
 	{ "text-indent",           "0in",             false, NULL, PP_LEVEL_BLOCK},
 	{ "text-position",         "normal",          true,  NULL, PP_LEVEL_CHAR},
+	{ "toc-dest-style1",      "Contents 1"   ,   false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-dest-style2",      "Contents 2",      false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-dest-style3",      "Contents 3",      false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-dest-style4",      "Contents 4",      false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-has-heading",       "1",               false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-has-label1",       "1",                false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-has-label2",       "1",                false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-has-label3",       "1",                false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-has-label4",       "1",                false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-heading",       "Table of Contents",   false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-heading-style",  "Contents Header",    false, NULL, PP_LEVEL_BLOCK},
+    { "toc-id",                "0",               false, NULL, PP_LEVEL_SECT},
+	{ "toc-indent1",           "0.5in",           false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-indent2",           "0.5in",           false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-indent3",           "0.5in",           false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-indent4",           "0.5in",           false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-after1",       "",               false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-after2",       "",               false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-after3",       "",               false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-after4",       "",               false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-before1",       "",              false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-before2",       "",              false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-before3",       "",              false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-before4",       "",              false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-inherits1",       "1",           false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-inherits2",       "1",           false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-inherits3",       "1",           false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-inherits4",       "1",           false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-start1",       "1",              false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-start2",       "1",              false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-start3",       "1",              false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-start4",       "1",              false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-type1",       "numeric",         false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-type2",       "numeric",         false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-type3",       "numeric",         false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-label-type4",       "numeric",         false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-page-type1",        "numeric",         false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-page-type2",        "numeric",         false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-page-type3",        "numeric",         false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-page-type4",        "numeric",         false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-range-bookmark",    "",                false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-source-style1",     "Heading 1",       false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-source-style2",     "Heading 2",       false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-source-style3",     "Heading 3",       false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-source-style4",     "Heading 4",       false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-tab-leader1",       "dot",             false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-tab-leader2",       "dot",             false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-tab-leader3",       "dot",             false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-tab-leader4",       "dot",             false, NULL, PP_LEVEL_BLOCK},
+
+	{ "top-attach",             "",               false, NULL, PP_LEVEL_TABLE},
 	{ "top-color",             "000000",          false, NULL, PP_LEVEL_TABLE},
 	{ "top-style",             "solid",           false, NULL, PP_LEVEL_TABLE},
 	{ "top-thickness",         "1px",             false, NULL, PP_LEVEL_TABLE},
 
 	{ "widows",                "2",               false, NULL, PP_LEVEL_BLOCK},
-	{ "width",                 "0in",             false, NULL, PP_LEVEL_CHAR} 
+	{ "width",                 "0in",             false, NULL, PP_LEVEL_CHAR}, 
+	{ "wrap-mode",             "above-text",      false, NULL, PP_LEVEL_FRAME} 
 };
 
 #ifdef __MRC__
@@ -198,15 +282,15 @@ void PP_resetInitialBiDiValues(const XML_Char * pszValue)
 
 	for (i=0; i<count; i++)
 	{
-		if (/*(0 == UT_stricmp(_props[i].m_pszName, "dir"))
-		  ||*/(0 == UT_stricmp(_props[i].m_pszName, "dom-dir"))
-		  /*||(0 == UT_stricmp(_props[i].m_pszName, "column-order"))*/)
+		if (/*(0 == strcmp(_props[i].m_pszName, "dir"))
+		  ||*/(0 == strcmp(_props[i].m_pszName, "dom-dir"))
+		  /*||(0 == strcmp(_props[i].m_pszName, "column-order"))*/)
 		  //this last one is not necessary since dom-dir and column-order
 		  //share the same physical string
 		{
 			UT_XML_strncpy(_props[i].m_pszInitial, 3,pszValue);
 		}
-		else if ((0 == UT_stricmp(_props[i].m_pszName, "text-align")))
+		else if ((0 == strcmp(_props[i].m_pszName, "text-align")))
 		{
 			UT_DEBUGMSG(("reseting text-align (%s)\n", pszValue));
 			if(pszValue[0] == (XML_Char)'r')
@@ -238,7 +322,7 @@ static PD_Style * _getStyle(const PP_AttrProp * pAttrProp, PD_Document * pDoc)
 //
 	if (pAttrProp->getAttribute(PT_NAME_ATTRIBUTE_NAME, szValue))
 	{
-		UT_ASSERT(szValue && szValue[0]);
+		UT_return_val_if_fail (szValue && szValue[0], NULL);
 		if (pDoc)
 			pDoc->getStyle(reinterpret_cast<const char*>(szValue), &pStyle);
 
@@ -246,7 +330,7 @@ static PD_Style * _getStyle(const PP_AttrProp * pAttrProp, PD_Document * pDoc)
 	}
     else if(pAttrProp->getAttribute(PT_STYLE_ATTRIBUTE_NAME, szValue))
 	{
-		UT_ASSERT(szValue && szValue[0]);
+		UT_return_val_if_fail (szValue && szValue[0], NULL);
 		if (pDoc)
 			pDoc->getStyle(reinterpret_cast<const char*>(szValue), &pStyle);
 
@@ -323,130 +407,151 @@ const XML_Char * PP_evalProperty (const XML_Char *  pszName,
 	// TODO: ?? make lookup more efficient by tagging each property with scope (block, char, section)
 
 	if (pSpanAttrProp)
-		{
-			szValue = s_evalProperty (pProp, pSpanAttrProp, pDoc, bExpandStyles);
+	{
+		szValue = s_evalProperty (pProp, pSpanAttrProp, pDoc, bExpandStyles);
 
-			if (szValue)
-				if (UT_strcmp (szValue, "inherit") == 0)
+		if (szValue)
+			if (UT_strcmp (szValue, "inherit") == 0)
+			{
+				szValue = NULL;
+				bInherit = true;
+			}
+		if ((szValue == NULL) && (bInherit || pProp->canInherit ()))
+		{
+			bInherit = false;
+
+			if (pBlockAttrProp)
+			{
+				szValue = s_evalProperty (pProp, pBlockAttrProp, pDoc, bExpandStyles);
+
+				if (szValue)
+					if (UT_strcmp (szValue, "inherit") == 0)
 					{
 						szValue = NULL;
 						bInherit = true;
 					}
-			if ((szValue == NULL) && (bInherit || pProp->canInherit ()))
-				{
-					bInherit = false;
-
-					if (pBlockAttrProp)
-						{
-							szValue = s_evalProperty (pProp, pBlockAttrProp, pDoc, bExpandStyles);
-
-							if (szValue)
-								if (UT_strcmp (szValue, "inherit") == 0)
-									{
-										szValue = NULL;
-										bInherit = true;
-									}
-							if ((szValue == NULL) && (bInherit || pProp->canInherit ()))
-								{
-									bInherit = false;
-
-									if (pSectionAttrProp)
-										{
-											szValue = s_evalProperty (pProp, pSectionAttrProp, pDoc, bExpandStyles);
-
-											if (szValue)
-												if (UT_strcmp (szValue, "inherit") == 0)
-													{
-														szValue = NULL;
-														bInherit = true;
-													}
-											if ((szValue == NULL) && (bInherit || pProp->canInherit ()))
-												{
-													const PP_AttrProp * pDocAP = pDoc->getAttrProp ();
-													if (pDocAP)
-														pDocAP->getProperty (pszName, szValue);
-												}
-										}
-								}
-						}
-				}
-		}
-	else if (pBlockAttrProp)
-		{
-			szValue = s_evalProperty (pProp, pBlockAttrProp, pDoc, bExpandStyles);
-
-			if (szValue)
-				if (UT_strcmp (szValue, "inherit") == 0)
-					{
-						szValue = NULL;
-						bInherit = true;
-					}
-			if ((szValue == NULL) && (bInherit || pProp->canInherit ()))
+				if ((szValue == NULL) && (bInherit || pProp->canInherit ()))
 				{
 					bInherit = false;
 
 					if (pSectionAttrProp)
+					{
+						szValue = s_evalProperty (pProp, pSectionAttrProp, pDoc, bExpandStyles);
+
+						if (szValue)
+							if (UT_strcmp (szValue, "inherit") == 0)
+							{
+								szValue = NULL;
+								bInherit = true;
+							}
+						if ((szValue == NULL) && (bInherit || pProp->canInherit ()))
 						{
-							szValue = s_evalProperty (pProp, pSectionAttrProp, pDoc, bExpandStyles);
-
-							if (szValue)
-								if (UT_strcmp (szValue, "inherit") == 0)
-									{
-										szValue = NULL;
-										bInherit = true;
-									}
-							if ((szValue == NULL) && (bInherit || pProp->canInherit ()))
-								{
-									const PP_AttrProp * pDocAP = pDoc->getAttrProp ();
-									if (pDocAP)
-										pDocAP->getProperty (pszName, szValue);
-								}
+							const PP_AttrProp * pDocAP = pDoc->getAttrProp ();
+							if (pDocAP)
+								pDocAP->getProperty (pszName, szValue);
 						}
+					}
 				}
+			}
 		}
-	else if (pSectionAttrProp)
-		{
-			szValue = s_evalProperty (pProp, pSectionAttrProp, pDoc, bExpandStyles);
+	}
+	else if (pBlockAttrProp)
+	{
+		szValue = s_evalProperty (pProp, pBlockAttrProp, pDoc, bExpandStyles);
 
-			if (szValue)
-				if (UT_strcmp (szValue, "inherit") == 0)
+		if (szValue)
+			if (UT_strcmp (szValue, "inherit") == 0)
+			{
+				szValue = NULL;
+				bInherit = true;
+			}
+		if ((szValue == NULL) && (bInherit || pProp->canInherit ()))
+		{
+			bInherit = false;
+
+			if (pSectionAttrProp)
+			{
+				szValue = s_evalProperty (pProp, pSectionAttrProp, pDoc, bExpandStyles);
+
+				if (szValue)
+					if (UT_strcmp (szValue, "inherit") == 0)
 					{
 						szValue = NULL;
 						bInherit = true;
 					}
-			if ((szValue == NULL) && (bInherit || pProp->canInherit ()))
+				if ((szValue == NULL) && (bInherit || pProp->canInherit ()))
 				{
 					const PP_AttrProp * pDocAP = pDoc->getAttrProp ();
 					if (pDocAP)
 						pDocAP->getProperty (pszName, szValue);
 				}
+			}
 		}
-	else
+	}
+	else if (pSectionAttrProp)
+	{
+		szValue = s_evalProperty (pProp, pSectionAttrProp, pDoc, bExpandStyles);
+
+		if (szValue)
+			if (UT_strcmp (szValue, "inherit") == 0)
+			{
+				szValue = NULL;
+				bInherit = true;
+			}
+		if ((szValue == NULL) && (bInherit || pProp->canInherit ()))
 		{
 			const PP_AttrProp * pDocAP = pDoc->getAttrProp ();
 			if (pDocAP)
 				pDocAP->getProperty (pszName, szValue);
 		}
+	}
+	else
+	{
+		const PP_AttrProp * pDocAP = pDoc->getAttrProp ();
+		if (pDocAP)
+		{
+			pDocAP->getProperty (pszName, szValue);
+
+			// dom-dir requires special treatment at document level
+			if(szValue && UT_strcmp(pszName, "dom-dir") == 0)
+			{
+				if(   UT_strcmp(szValue, "logical-ltr") == 0
+				   || UT_strcmp(szValue, "logical-rtl") == 0)
+					szValue += 8;
+			}
+		}
+	}
 	if (szValue)
 		if (UT_strcmp (szValue, "inherit") == 0) // shouldn't happen, but doesn't hurt to check
 			szValue = NULL;
 
 	if (szValue == NULL)
 		if (bExpandStyles)
+		{
+			PD_Style * pStyle = 0;
+
+			if (pDoc->getStyle ("Normal", &pStyle))
 			{
-				PD_Style * pStyle = 0;
+				/* next to last resort -- check for this property in the Normal style
+				 */
+				pStyle->getProperty (pszName, szValue);
 
-				if (pDoc->getStyle ("Normal", &pStyle))
-					{
-						/* next to last resort -- check for this property in the Normal style
-						 */
-						pStyle->getProperty (pszName, szValue);
-
-						if (szValue)
-							if (UT_strcmp (szValue, "inherit") == 0)
-								szValue = NULL;
-					}
+				if (szValue)
+					if (UT_strcmp (szValue, "inherit") == 0)
+						szValue = NULL;
 			}
+		}
+
+	if(szValue == NULL && pDoc && (bInherit || pProp->canInherit ()))
+	{
+		// see if the doc has a value for this prop
+		const PP_AttrProp *  pAP = pDoc->getAttrProp();
+		if(pAP)
+		{
+			pAP->getProperty(pszName, szValue);
+		}
+	}
+	
 	if (szValue == NULL)
 		szValue = pProp->getInitial (); // which may itself be NULL, but that is a bad thing - FIXME!!
 
@@ -617,7 +722,7 @@ PP_PropertyType *PP_PropertyType::createPropertyType(tProperty_type Type, const
 		break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
