@@ -19,7 +19,11 @@
 
 #include "pp_Revision.h"
 #include "pp_AttrProp.h"
+#include "pd_Style.h"
+#include "pd_Document.h"
 #include "ut_debugmsg.h"
+#include "ut_misc.h"
+
 //#include <limits.h>
 
 PP_Revision::PP_Revision(UT_uint32 Id, PP_RevisionType eType, const XML_Char * props, const XML_Char * attrs):
@@ -28,28 +32,33 @@ PP_Revision::PP_Revision(UT_uint32 Id, PP_RevisionType eType, const XML_Char * p
 	if(!props && !attrs)
 		return;
 
+	const char * empty = "";
+	
 	if(props)
 	{
 		char * pProps = UT_strdup(props);
-
-		UT_ASSERT(pProps);
-		if(!pProps)
-		{
-			UT_DEBUGMSG(("PP_Revision::PP_Revision: out of memory\n"));
-			return;
-		}
+		UT_return_if_fail (pProps);
 
 		char * p = strtok(pProps, ":");
 
 		while(p)
 		{
 			char * n = p;
-			p = strtok(NULL, ";");
-			UT_ASSERT(p && n);
 
-			if(p && n)
+			// skip over spaces ...
+			while(n && *n == ' ')
+				++n;
+			
+			p = strtok(NULL, ";");
+
+			// if we have no p, that means the property is being removed ...
+			const char * v = p ? p : empty;
+			if(! UT_strcmp(v, "-/-"))
+				v = empty;
+		
+			if(n)
 			{
-				setProperty(n,p);
+				setProperty(n,v);
 				p = strtok(NULL,":");
 			}
 			else
@@ -69,7 +78,7 @@ PP_Revision::PP_Revision(UT_uint32 Id, PP_RevisionType eType, const XML_Char * p
 	{
 		char * pAttrs = UT_strdup(attrs);
 
-		UT_ASSERT(pAttrs);
+		UT_ASSERT_HARMLESS(pAttrs);
 		if(!pAttrs)
 		{
 			UT_DEBUGMSG(("PP_Revision::PP_Revision: out of memory\n"));
@@ -82,11 +91,14 @@ PP_Revision::PP_Revision(UT_uint32 Id, PP_RevisionType eType, const XML_Char * p
 		{
 			char * n = p;
 			p = strtok(NULL, ";");
-			UT_ASSERT(p && n);
 
-			if(p && n)
+			const char * v = p ? p : empty;
+			if(! UT_strcmp(v, "-/-"))
+				v = empty;
+			
+			if(n)
 			{
-				setAttribute(n,p);
+				setAttribute(n,v);
 				p = strtok(NULL,":");
 			}
 			else
@@ -120,6 +132,53 @@ PP_Revision::PP_Revision(UT_uint32 Id, PP_RevisionType eType, const XML_Char **
 	}
 }
 
+/*!
+    Sets attributes taking care of any nested revision attribute (which needs to be parsed
+    and combined with the current AP set.
+*/
+bool PP_Revision::setAttributes(const XML_Char ** attributes)
+{
+	if(!PP_AttrProp::setAttributes(attributes))
+		return false;
+	
+	return _handleNestedRevAttr();
+}
+
+
+
+bool PP_Revision::_handleNestedRevAttr()
+{
+	const XML_Char * pNestedRev = NULL;
+	getAttribute("revision", pNestedRev);
+	
+	if(pNestedRev)
+	{
+		PP_RevisionAttr NestedAttr(pNestedRev);
+
+		// now remove "revision"
+		setAttribute("revision", NULL);
+		prune();
+
+		// overlay the attrs and props from the revision attribute
+		for(UT_uint32 i = 0; i < NestedAttr.getRevisionsCount(); ++i)
+		{
+			const PP_Revision * pRev = NestedAttr.getNthRevision(i);
+			UT_return_val_if_fail( pRev, false );
+					
+			// ignore inserts and deletes
+			if(pRev->getType() == PP_REVISION_ADDITION || pRev->getType() == PP_REVISION_DELETION)
+				continue;
+
+			setProperties(pRev->getProperties());
+			setAttributes(pRev->getAttributes());
+		}
+
+		prune();
+	}
+
+	return true;
+}
+
 
 /*! converts the internal vector of properties into XML string */
 const XML_Char * PP_Revision::getPropsString()
@@ -150,7 +209,14 @@ void PP_Revision::_refreshString()
 
 	for(i = 0; i < iCount; i++)
 	{
-		getNthProperty(i,n,v);
+		if(!getNthProperty(i,n,v))
+		{
+			// UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+			continue;
+		}
+		
+		if(!v || !*v) v = "-/-";
+		
 		m_sXMLProps += n;
 		m_sXMLProps += ":";
 		m_sXMLProps += v;
@@ -161,7 +227,14 @@ void PP_Revision::_refreshString()
 	iCount = getAttributeCount();
 	for(i = 0; i < iCount; i++)
 	{
-		getNthAttribute(i,n,v);
+		if(!getNthAttribute(i,n,v))
+		{
+			// UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+			continue;
+		}
+		
+		if(!v || !*v) v = "-/-";
+
 		m_sXMLAttrs += n;
 		m_sXMLAttrs += ":";
 		m_sXMLAttrs += v;
@@ -232,6 +305,15 @@ PP_RevisionAttr::PP_RevisionAttr(const XML_Char * r):
 	_init(r);
 }
 
+/*! create class instance from a single revision data */
+PP_RevisionAttr::PP_RevisionAttr(UT_uint32 iId, PP_RevisionType eType,
+								 const XML_Char ** pAttrs, const XML_Char ** pProps)
+{
+	PP_Revision * pRevision = new PP_Revision((UT_uint32)iId, eType, pProps, pAttrs);
+	m_vRev.addItem((void*)pRevision);
+}
+
+
 PP_RevisionAttr::~PP_RevisionAttr()
 {
 	_clear();
@@ -313,14 +395,14 @@ void PP_RevisionAttr::_init(const XML_Char *r)
 		cl_brace = strchr(t, '}');
 		op_brace = strchr(t, '{');
 
-		if(!cl_brace && !op_brace)
+		if(!cl_brace || !op_brace)
 		{
 			// no props
 			if(eType == PP_REVISION_FMT_CHANGE)
 			{
 				// malformed token, move onto the next one
 				UT_DEBUGMSG(("PP_RevisionAttr::_init: invalid ! token [%s]\n",t));
-				continue;
+				goto skip_this_token;
 			}
 			pProps = NULL;
 			pAttrs = NULL;
@@ -333,7 +415,7 @@ void PP_RevisionAttr::_init(const XML_Char *r)
 			{
 				// malformed token, move onto the next one
 				UT_DEBUGMSG(("PP_RevisionAttr::_init: invalid - token [%s]\n",t));
-				continue;
+				goto skip_this_token;
 			}
 
 			// insert null as needed to be able to parse the id and props
@@ -366,10 +448,13 @@ void PP_RevisionAttr::_init(const XML_Char *r)
 		// now we can retrieve the id
 		iId = atol(t);
 
-		PP_Revision * pRevision = new PP_Revision((UT_uint32)iId, eType, pProps, pAttrs);
-
-		m_vRev.addItem((void*)pRevision);
+		{
+			PP_Revision * pRevision = new PP_Revision((UT_uint32)iId, eType, pProps, pAttrs);
 
+			m_vRev.addItem((void*)pRevision);
+		}
+		
+	skip_this_token:
 		if(next_s < end_s)
 			t = strtok(next_s,",");
 		else
@@ -382,6 +467,118 @@ void PP_RevisionAttr::_init(const XML_Char *r)
 	m_pLastRevision = NULL;
 }
 
+/*!
+    changes the type of revision with id iId to eType; if revision
+    with that id is not present, returns false
+ */
+bool PP_RevisionAttr::changeRevisionType(UT_uint32 iId, PP_RevisionType eType)
+{
+	for(UT_uint32 i = 0; i < m_vRev.getItemCount(); i++)
+	{
+		PP_Revision * r = (PP_Revision *)m_vRev.getNthItem(i);
+
+		if(iId == r->getId())
+		{
+			r->setType(eType);
+			m_bDirty = true;
+			return true;
+		}
+	}
+
+	return false;
+}
+
+bool PP_RevisionAttr::changeRevisionId(UT_uint32 iOldId, UT_uint32 iNewId)
+{
+	UT_return_val_if_fail(iNewId >= iOldId, false);
+	
+	for(UT_uint32 i = 0; i < m_vRev.getItemCount(); i++)
+	{
+		PP_Revision * r = (PP_Revision *)m_vRev.getNthItem(i);
+
+		if(iOldId == r->getId())
+		{
+			r->setId(iNewId);
+			m_bDirty = true;
+			return true;
+		}
+	}
+
+	return false;
+}
+
+/*!
+    this function removes any revisions that no-longer contribute to the cumulative effect
+    it is used in full-history mode when transfering attrs and props from the revision attribute
+    into the main attrs and props
+*/
+void PP_RevisionAttr::pruneForCumulativeResult(PD_Document * pDoc)
+{
+	// first we are looking for any deletions which cancel anything below them
+	bool bDelete = false;
+	UT_sint32 i;
+	if(m_vRev.getItemCount() == 0)
+	{
+		return;
+	}
+	
+	for(i = m_vRev.getItemCount()-1; i >=0; --i)
+	{
+		PP_Revision * r = (PP_Revision *)m_vRev.getNthItem(i);
+		
+		if(r->getType() == PP_REVISION_DELETION)
+			bDelete = true;
+
+		if(bDelete)
+		{
+			delete r;
+			m_vRev.deleteNthItem(i);
+		}
+	}
+
+	// now we merge props and attrs in what is left
+	if(m_vRev.getItemCount() == 0)
+	{
+		return;
+	}
+
+	PP_Revision * r0 = (PP_Revision *)m_vRev.getNthItem(0);
+	UT_return_if_fail(r0);
+	
+	for(i = 1; i < (UT_sint32)m_vRev.getItemCount(); ++i)
+	{
+		PP_Revision * r = (PP_Revision *)m_vRev.getNthItem(i);
+		UT_return_if_fail( r );
+		
+		r0->setProperties(r->getProperties());
+		r0->setAttributes(r->getAttributes());
+
+		delete r;
+		m_vRev.deleteNthItem(i);
+		--i;
+	}
+
+    // explode the style if present
+	if(pDoc)
+		r0->explodeStyle(pDoc);
+
+#if 0
+	// I do not think we should do this -- the emptiness indicates that the props and
+	// attributes should be removed from the format; if we remove them, we irreversibly
+	// lose this information
+	
+	// get rid of any empty props and attrs
+	r0->prune();
+#endif
+	
+	// finally, remove the revision attribute if present
+	const XML_Char * v;
+	if(r0->getAttribute("revision", v))
+		r0->setAttribute("revision", NULL);
+
+	UT_ASSERT_HARMLESS( m_vRev.getItemCount() == 1 );
+}
+
 
 /*! return highest revision associated with this revision attribute
     that has ID at most equal to id; the returned PP_Revision can be
@@ -389,17 +586,25 @@ void PP_RevisionAttr::_init(const XML_Char *r)
     in revision with the original id
 
     \param UT_uint32 id : the id of this revision
+    \param PP_Revision ** ppR: location where to store pointer to one
+                               of the special revisions in case return
+                               value is NULL
+                              
     \return : pointer to PP_Revision, or NULL if the revision
     attribute should be ignored for the present level
 */
 
 // these are special instances of PP_Revision that are used to in the
 // following function to handle special cases
-static const PP_Revision s_hidden(0, PP_REVISION_DELETION, (XML_Char*)0, (XML_Char*)0);
-static const PP_Revision s_visible(0, PP_REVISION_ADDITION, (XML_Char*)0, (XML_Char*)0);
+static const PP_Revision s_del(0, PP_REVISION_DELETION, (XML_Char*)0, (XML_Char*)0);
+static const PP_Revision s_add(0, PP_REVISION_ADDITION, (XML_Char*)0, (XML_Char*)0);
 
-const PP_Revision *  PP_RevisionAttr::getGreatestLesserOrEqualRevision(UT_uint32 id)
+const PP_Revision *  PP_RevisionAttr::getGreatestLesserOrEqualRevision(UT_uint32 id,
+																	   const PP_Revision ** ppR)
 {
+	if(ppR)
+		*ppR = NULL;
+	
 	if(id == 0)
 		return getLastRevision();
 
@@ -433,33 +638,61 @@ const PP_Revision *  PP_RevisionAttr::getGreatestLesserOrEqualRevision(UT_uint32
 
 	// now that we have the biggest revision with ID lesser or equal
 	// id, we have to deal with the special case when this is NULL
-	// i.e., this fragment only figures in revisions > id the problem
+	// i.e., this fragment only figures in revisions > id; the problem
 	// with NULL is that it is visible if the smallest revision ID is
-	// negative, and hidden in the opposite case -- if this is to be
-	// visible we return the special static variables s_visible and
-	// s_hidden
+	// negative, and hidden in the opposite case -- we use the special
+	// static variables s_del and s_add to indicate what should
+	// be done
 
-	if(r == NULL)
+	if(r == NULL && ppR)
 	{
 		if(!m)
 		{
-			//UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			//UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 			// this happens when there was no revision attribute
 			return NULL;
 		}
 
 		if(m->getType() == PP_REVISION_DELETION)
-			return &s_visible;
+			*ppR = &s_del;
 		else if((m->getType() == PP_REVISION_ADDITION)
 				||(m->getType() == PP_REVISION_ADDITION_AND_FMT))
-			return &s_hidden;
+			*ppR = &s_add;
 		else // the initial revision was fmt change, so ignore it
-			return NULL;
+			*ppR = NULL;
 	}
-	else
-		return r;
+
+	return r;
 }
 
+const PP_Revision * PP_RevisionAttr::getLowestGreaterOrEqualRevision(UT_uint32 id)
+{
+	if(id == 0)
+		return NULL;
+
+	const PP_Revision *r = NULL; // this will be the revision we are looking for
+	UT_uint32 r_id = PD_MAX_REVISION;
+
+	for(UT_uint32 i = 0; i < m_vRev.getItemCount(); i++)
+	{
+		const PP_Revision * t = (const PP_Revision *) m_vRev.getNthItem(i);
+		UT_uint32 t_id = t->getId();
+
+		// the special case speedup - if we hit our id, then we can return immediately
+		if(t_id == id)
+			return t;
+
+		if((t_id > id) && (t_id < r_id))
+		{
+			r = t;
+			r_id = t_id;
+		}
+	}
+
+	return r;
+}
+
+
 /*! finds the highest revision number in this attribute
  */
 const PP_Revision * PP_RevisionAttr::getLastRevision()
@@ -484,50 +717,103 @@ const PP_Revision * PP_RevisionAttr::getLastRevision()
 		}
 	}
 
+	// UT_ASSERT_HARMLESS( m_pLastRevision );
+	// it is legal for this to be NULL -- it happens when the revision was pruned for
+	// cumulative effect and the last revision was a deletion.
 	return m_pLastRevision;
 }
 
+/*!
+   find revision with id == iId; if revision is not found minId
+   contains the smallest id in this set greater than iId; if return value is and minId
+   is PD_MAX_REVISION then there are revisions preset
+*/
+const PP_Revision * PP_RevisionAttr::getRevisionWithId(UT_uint32 iId, UT_uint32 &minId)
+{
+	minId = PD_MAX_REVISION;
+
+	for(UT_uint32 i = 0; i < m_vRev.getItemCount(); i++)
+	{
+		const PP_Revision * t = (const PP_Revision *)m_vRev.getNthItem(i);
+		UT_uint32 t_id = t->getId();
+
+		if(t_id == iId)
+		{
+			return t;
+		}
+
+		if(minId > t_id && t_id > iId)
+			minId = t_id;
+	}
+
+	return NULL;
+}
+
 
 /*! given revision level id, this function returns true if given
     segment of text is to be visible, false if it is to be hidden
 */
 bool PP_RevisionAttr::isVisible(UT_uint32 id)
 {
-	PP_RevisionType eType = getGreatestLesserOrEqualRevision(id)->getType();
+	if(id == 0)
+	{
+		// id 0 means show all revisions
+		return true;
+	}
 
-	return ((eType == PP_REVISION_ADDITION) || (eType == PP_REVISION_ADDITION_AND_FMT));
+	const PP_Revision * pSpecial;
+	const PP_Revision * pR = getGreatestLesserOrEqualRevision(id, &pSpecial);
+
+	if(pR)
+	{
+		// found compliant revision ...
+		return true;
+	}
+	
+
+	if(pSpecial)
+	{
+		// pSpecial is of the same type as the revision with the
+		// lowest id
+		PP_RevisionType eType = pSpecial->getType();
+
+		// deletions and fmt changes can be ignored; insertions need
+		// to be hidden
+		return ((eType != PP_REVISION_ADDITION) && (eType == PP_REVISION_ADDITION_AND_FMT));
+	}
+
+	// the revision with the lowest id is a change of format, this
+	// text has to remain visible
+	return true;
 }
 
-/*! returns true if the text should be visible according to the last
-    revision
-*/
-
-bool PP_RevisionAttr::isVisible()
-{
-	PP_RevisionType eType = getLastRevision()->getType();
-
-	return ((eType == PP_REVISION_ADDITION) || (eType == PP_REVISION_ADDITION_AND_FMT));
-}
 
 /*! adds id to the revision vector handling the special cases where id
     is already present in this attribute.
 */
-void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType, const XML_Char ** pAttrs, const XML_Char ** pProps)
+void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType,
+								  const XML_Char ** pAttrs, const XML_Char ** pProps)
 {
-	for(UT_uint32 i = 0; i < m_vRev.getItemCount(); i++)
+	UT_uint32 i;
+
+	for(i = 0; i < m_vRev.getItemCount(); i++)
 	{
 		PP_Revision * r = (PP_Revision*) m_vRev.getNthItem(i);
 		UT_uint32 r_id = r->getId();
 		PP_RevisionType r_type = r->getType();
 
-		if(iId == r_id && eType != r_type)
+		if(iId != r_id)
+			continue;
+		
+		if(eType != r_type)
 		{
 			// we are trying to add a revision id already in the vector
 			// but of a different -- this is legal, i.e., the
 			// editor just changed his mind
 			// we need to make distinction between different cases
 
-			if((eType == PP_REVISION_DELETION) && (r_type == PP_REVISION_ADDITION))
+			if((eType == PP_REVISION_DELETION) && (   r_type == PP_REVISION_ADDITION
+												   || r_type == PP_REVISION_ADDITION_AND_FMT))
 			{
 				// the editor originally inserted a new segment of
 				// text but now wants it out; we cannot just remove
@@ -584,6 +870,7 @@ void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType, const XM
 				m_vRev.deleteNthItem(i);
 
 				const PP_Revision * pRevision = new PP_Revision(iId, eType, pProps, pAttrs);
+				
 				m_vRev.addItem((void*)pRevision);
 			}
 			else if((eType == PP_REVISION_FMT_CHANGE) && (r_type == PP_REVISION_ADDITION))
@@ -593,10 +880,12 @@ void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType, const XM
 				// so we will keep the old revision record, but need
 				// to merge any existing props in the revision with
 				// the new ones
-
-				// I will implement this here, but this case should
-				// really be handled higher up and the property
-				// changes should be applied directly to the fragments props
+				r->setProperties(pProps);
+				r->setAttributes(pAttrs);
+			}
+			else if((eType == PP_REVISION_FMT_CHANGE) && (r_type == PP_REVISION_ADDITION_AND_FMT))
+			{
+				// addition with a fmt change, just add our changes to it
 				r->setProperties(pProps);
 				r->setAttributes(pAttrs);
 			}
@@ -605,17 +894,25 @@ void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType, const XM
 			m_pLastRevision = NULL;
 			return;
 		}
-
-		if(iId == r_id && eType == r_type)
+		else //(eType == r_type)
 		{
-			// we are trying to add an id already in the vector
-			// do nothing
+			// we are trying to add a type already in the vector; this is legal but makes sense only
+			// if both are fmt changes
+			if(!((eType == PP_REVISION_FMT_CHANGE) && (r_type == PP_REVISION_FMT_CHANGE)))
+				return;
+			
+			r->setProperties(pProps);
+			r->setAttributes(pAttrs);
+			
+			m_bDirty = true;
+			m_pLastRevision = NULL;
 			return;
 		}
 	}
 
 	// if we got here then the item is not in our vector so add it
 	const PP_Revision * pRevision = new PP_Revision(iId, eType, pProps, pAttrs);
+	
 	m_vRev.addItem((void*)pRevision);
 	m_bDirty = true;
 	m_pLastRevision = NULL;
@@ -718,12 +1015,19 @@ void PP_RevisionAttr::_refreshString()
 		sprintf(buf,"%d",r->getId()* ((r_type == PP_REVISION_DELETION)?-1:1));
 		m_sXMLstring += buf;
 
-		if((r_type == PP_REVISION_FMT_CHANGE)||(r_type == PP_REVISION_ADDITION_AND_FMT))
+		if(r_type != PP_REVISION_DELETION)
 		{
-			m_sXMLstring += "{";
-			m_sXMLstring += r->getPropsString();
-			m_sXMLstring += "}";
-
+			// if we have no props but have attribs, we have to issue empty braces so as not to
+			// confuse attribs with props
+			if(r->hasProperties() || r->hasAttributes())
+				m_sXMLstring += "{";
+			
+			if(r->hasProperties())
+				m_sXMLstring += r->getPropsString();
+			
+			if(r->hasProperties() || r->hasAttributes())
+				m_sXMLstring += "}";
+			
 			if(r->hasAttributes())
 			{
 				m_sXMLstring += "{";
@@ -765,7 +1069,7 @@ bool PP_RevisionAttr::isFragmentSuperfluous() const
 	// and the fragment belongs only to a single revision level
 	if(m_iSuperfluous != 0 && m_vRev.getItemCount() == 1)
 	{
-		UT_ASSERT(((PP_Revision *)m_vRev.getNthItem(0))->getId() == m_iSuperfluous);
+		UT_return_val_if_fail (((PP_Revision *)m_vRev.getNthItem(0))->getId() == m_iSuperfluous,false);
 		return true;
 	}
 	else
@@ -795,8 +1099,13 @@ bool PP_RevisionAttr::operator == (const PP_RevisionAttr &op2) const
 */
 bool PP_RevisionAttr::hasProperty(UT_uint32 iId, const XML_Char * pName, const XML_Char * &pValue)
 {
-	const PP_Revision * r = getGreatestLesserOrEqualRevision(iId);
-	return r->getProperty(pName, pValue);
+	const PP_Revision * s;
+	const PP_Revision * r = getGreatestLesserOrEqualRevision(iId, &s);
+
+	if(r)
+		return r->getProperty(pName, pValue);
+
+	return false;
 }
 
 /*! returns true if after the last revision this fragment carries revised
@@ -813,7 +1122,15 @@ bool PP_RevisionAttr::hasProperty(const XML_Char * pName, const XML_Char * &pVal
  */
 PP_RevisionType PP_RevisionAttr::getType(UT_uint32 iId)
 {
-	const PP_Revision * r = getGreatestLesserOrEqualRevision(iId);
+	const PP_Revision * s;
+	const PP_Revision * r = getGreatestLesserOrEqualRevision(iId,&s);
+
+	if(!r)
+	{
+		// HACK need to return something
+		return PP_REVISION_FMT_CHANGE;
+	}
+	
 	return r->getType();
 }
 
