@@ -19,7 +19,7 @@
  * 02111-1307, USA.
  */
 
-
+#include <ctype.h>
 #include "ut_types.h"
 #include "ut_misc.h"
 #include "ut_string.h"
@@ -68,20 +68,235 @@ bool pt_PieceTable::appendStrux(PTStruxType pts, const XML_Char ** attributes, p
 	return true;
 }
 
+pf_Frag * pt_PieceTable::_findLastStruxOfType(pf_Frag * pfStart, PTStruxType pst, bool bSkipEmbededSections)
+{
+	UT_return_val_if_fail( pfStart, NULL );
+
+	pf_Frag * pf = pfStart;
+	
+	while(pf)
+	{
+		if(pf->getType() == pf_Frag::PFT_Strux)
+		{
+			pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux*>(pf);
+
+			if(pfs->getStruxType() == pst)
+				break;
+
+			if(bSkipEmbededSections)
+			{
+				if(pfs->getStruxType() == PTX_EndTOC)
+				{
+					while(pf)
+					{
+						if(pf->getType() == pf_Frag::PFT_Strux)
+						{
+							pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux*>(pf);
+
+							if(pfs->getStruxType() == PTX_SectionTOC)
+								break;
+						}
+
+						pf = pf->getPrev();
+					}
+				}
+				if(pfs->getStruxType() == PTX_EndFrame)
+				{
+					while(pf)
+					{
+						if(pf->getType() == pf_Frag::PFT_Strux)
+						{
+							pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux*>(pf);
+
+							if(pfs->getStruxType() == PTX_SectionFrame)
+								break;
+						}
+
+						pf = pf->getPrev();
+					}
+				}
+
+				if(pfs->getStruxType() == PTX_EndEndnote)
+				{
+					while(pf)
+					{
+						if(pf->getType() == pf_Frag::PFT_Strux)
+						{
+							pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux*>(pf);
+
+							if(pfs->getStruxType() == PTX_SectionEndnote)
+								break;
+						}
+
+						pf = pf->getPrev();
+					}
+				}
+				if(pfs->getStruxType() == PTX_EndFootnote)
+				{
+					while(pf)
+					{
+						if(pf->getType() == pf_Frag::PFT_Strux)
+						{
+							pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux*>(pf);
+
+							if(pfs->getStruxType() == PTX_SectionFootnote)
+								break;
+						}
+
+						pf = pf->getPrev();
+					}
+				}
+				if(pfs->getStruxType() == PTX_EndMarginnote)
+				{
+					while(pf)
+					{
+						if(pf->getType() == pf_Frag::PFT_Strux)
+						{
+							pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux*>(pf);
+
+							if(pfs->getStruxType() == PTX_SectionMarginnote)
+								break;
+						}
+
+						pf = pf->getPrev();
+					}
+				}
+				if(pfs->getStruxType() == PTX_EndCell)
+				{
+					while(pf)
+					{
+						if(pf->getType() == pf_Frag::PFT_Strux)
+						{
+							pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux*>(pf);
+
+							if(pfs->getStruxType() == PTX_SectionCell)
+								break;
+						}
+
+						pf = pf->getPrev();
+					}
+				}
+				if(pfs->getStruxType() == PTX_EndTable)
+				{
+					while(pf)
+					{
+						if(pf->getType() == pf_Frag::PFT_Strux)
+						{
+							pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux*>(pf);
+
+							if(pfs->getStruxType() == PTX_SectionTable)
+								break;
+						}
+
+						pf = pf->getPrev();
+					}
+				}
+			
+			}
+		}
+		if(pf)
+			pf = pf->getPrev();
+	}
+
+	return pf;
+}
+
+
+/*!
+    Changes formating of the last strux of type pts
+    bSkipEmbededSections indicates whether when an end of an embeded section is
+    encountered, the entire section is to be skipped over, for example if the end of the
+    document looks like
+
+    <p><footnote><p></p></footnote>
+
+    when searching for <p> if bSkipEmbededSections == true the paragraph before <footnote>
+    will be modified
+*/
+bool pt_PieceTable::appendLastStruxFmt(PTStruxType pst, const XML_Char ** attributes, const XML_Char ** props,
+									   bool bSkipEmbededSections)
+{
+	// can only be used while loading the document
+	UT_return_val_if_fail (m_pts==PTS_Loading,false);
+
+	// Only a strux can be appended to an empty document
+	UT_return_val_if_fail (NULL != m_fragments.getFirst(), false);
+	if (!m_fragments.getFirst())
+		return false;
+
+	pf_Frag * pf = m_fragments.getLast();
+
+	UT_return_val_if_fail ( pf, false );
+
+	pf = _findLastStruxOfType(pf, pst, bSkipEmbededSections);
+	
+	UT_return_val_if_fail( pf, false );
+	
+	PT_AttrPropIndex currentAP = pf->getIndexAP();
+
+	const PP_AttrProp * pOldAP;
+    if(!getAttrProp(currentAP,&pOldAP))
+		return false;
+
+	PP_AttrProp * pNewAP = pOldAP->cloneWithReplacements(attributes,props,false);
+	pNewAP->markReadOnly();
+
+	PT_AttrPropIndex indexAP;
+	if (!m_varset.addIfUniqueAP(pNewAP,&indexAP))
+		return false;
+
+	pf->setIndexAP(indexAP);
+
+	return true;
+}
+
+/*!
+    As above, but props represented by a single XML string
+    
+*/
+bool pt_PieceTable::appendLastStruxFmt(PTStruxType pst, const XML_Char ** attributes, const XML_Char * props,
+									   bool bSkipEmbededSections)
+{
+	if(props && *props)
+	{
+		// we parse the xml props string into separate field by simply duplicating it and then
+		// replacing ; and : with '0';
+	
+		// foolproofing
+		if(*props == ';')
+			props++;
+		
+		char * pProps = UT_strdup(props);
+
+		const XML_Char ** pPropsArray = UT_splitPropsToArray(pProps);
+		UT_return_val_if_fail( pPropsArray, false );
+		
+		bool bRet = appendLastStruxFmt(pst, attributes, pPropsArray, bSkipEmbededSections);
+
+		delete [] pPropsArray;
+		FREEP(pProps);
+
+		return bRet;
+	}
+	else
+	{
+		const XML_Char ** pPropsArray = NULL;
+		return appendLastStruxFmt(pst, attributes, pPropsArray, bSkipEmbededSections);
+	}
+}
+
 /*! changes formatting of a strux while loading document */
 bool pt_PieceTable::appendStruxFmt(pf_Frag_Strux * pfs, const XML_Char ** attributes)
 {
 	// can only be used while loading the document
-	UT_ASSERT(m_pts==PTS_Loading);
+	UT_return_val_if_fail (m_pts==PTS_Loading,false);
 
 	// Only a strux can be appended to an empty document
-	UT_ASSERT(NULL != m_fragments.getFirst());
+	UT_return_val_if_fail (NULL != m_fragments.getFirst(), false);
 	if (!m_fragments.getFirst())
 		return false;
 
-	UT_ASSERT( pfs );
-	if(!pfs)
-		return false;
+	UT_return_val_if_fail ( pfs, false );
 
 	PT_AttrPropIndex currentAP = pfs->getIndexAP();
 
@@ -104,12 +319,10 @@ bool pt_PieceTable::appendStruxFmt(pf_Frag_Strux * pfs, const XML_Char ** attrib
 bool pt_PieceTable::appendFmt(const XML_Char ** attributes)
 {
 	// can only be used while loading the document
-	UT_ASSERT(m_pts==PTS_Loading);
+	UT_return_val_if_fail (m_pts==PTS_Loading, false);
 
 	// Only a strux can be appended to an empty document
-	UT_ASSERT(NULL != m_fragments.getFirst());
-	if (!m_fragments.getFirst())
-		return false;
+	UT_return_val_if_fail (NULL != m_fragments.getFirst(), false);
 
 	// create a new Attribute/Property structure in the table
 	// and set the current index to it.  the next span of text
@@ -125,15 +338,13 @@ bool pt_PieceTable::appendFmt(const XML_Char ** attributes)
 	return true;
 }
 
-bool pt_PieceTable::appendFmt(const UT_Vector * pVecAttributes)
+bool pt_PieceTable::appendFmt(const UT_GenericVector<XML_Char*> * pVecAttributes)
 {
 	// can only be used while loading the document
-	UT_ASSERT(m_pts==PTS_Loading);
+	UT_return_val_if_fail (m_pts==PTS_Loading, false);
 
 	// Only a strux can be appended to an empty document
-	UT_ASSERT(NULL != m_fragments.getFirst());
-	if (!m_fragments.getFirst())
-		return false;
+	UT_return_val_if_fail (NULL != m_fragments.getFirst(),false);
 
 	if (!m_varset.storeAP(pVecAttributes,&loading.m_indexCurrentInlineAP))
 		return false;
@@ -144,12 +355,10 @@ bool pt_PieceTable::appendFmt(const UT_Vector * pVecAttributes)
 bool pt_PieceTable::appendSpan(const UT_UCSChar * pbuf, UT_uint32 length)
 {
 	// can only be used while loading the document
-	UT_ASSERT(m_pts==PTS_Loading);
+	UT_return_val_if_fail (m_pts==PTS_Loading, false);
 
 	// Only a strux can be appended to an empty document
-	UT_ASSERT(NULL != m_fragments.getFirst());
-	if (!m_fragments.getFirst())
-		return false;
+	UT_return_val_if_fail (NULL != m_fragments.getFirst(),false);
 
 	// append the text data to the end of the buffer.
 
@@ -230,12 +439,10 @@ bool pt_PieceTable::insertStruxBeforeFrag(pf_Frag * pF, PTStruxType pts,
 bool pt_PieceTable::insertSpanBeforeFrag(pf_Frag * pf, const UT_UCSChar * p, UT_uint32 length)
 {
 	// can only be used while loading the document
-	UT_ASSERT(m_pts==PTS_Loading);
+	UT_return_val_if_fail (m_pts==PTS_Loading, false);
 
 	// Only a strux can be appended to an empty document
-	UT_ASSERT(NULL != m_fragments.getFirst());
-	if (!m_fragments.getFirst())
-		return false;
+	UT_return_val_if_fail (NULL != m_fragments.getFirst(),false);
 
 	// cannot insert before first fragment (i.e., span cannot start a document)
 	UT_return_val_if_fail(pf && pf->getPrev() && pf != m_fragments.getFirst(), false);
@@ -317,11 +524,25 @@ bool pt_PieceTable::insertFmtMarkBeforeFrag(pf_Frag * pF)
 	return true;
 }
 
+
+bool pt_PieceTable::insertFmtMarkBeforeFrag(pf_Frag * pF, const XML_Char ** attributes)
+{
+	// cannot insert before first fragment
+	UT_return_val_if_fail(pF && pF->getPrev() && pF != m_fragments.getFirst(), false);
+
+	pf_Frag_FmtMark * pff = NULL;
+	if (!_makeFmtMark(pff,attributes) || !pff)
+		return false;
+
+	m_fragments.insertFragBefore(pF, pff);
+	return true;
+}
+
 bool pt_PieceTable::_makeStrux(PTStruxType pts, const XML_Char ** attributes, pf_Frag_Strux * &pfs)
 {
 	// create a new structure fragment at the current end of the document.
 	// this function can only be called while loading the document.
-	UT_ASSERT(m_pts==PTS_Loading);
+	UT_return_val_if_fail (m_pts==PTS_Loading, false);
 
 	// first, store the attributes and properties and get an index to them.
 
@@ -362,12 +583,10 @@ bool pt_PieceTable::_makeObject(PTObjectType pto, const XML_Char ** attributes,
 {
 	// create a new object fragment at the current end of the document.
 	// this function can only be called while loading the document.
-	UT_ASSERT(m_pts==PTS_Loading);
+	UT_return_val_if_fail (m_pts==PTS_Loading, false);
 
 	// Only a strux can be appended to an empty document
-	UT_ASSERT(NULL != m_fragments.getFirst());
-	if (!m_fragments.getFirst())
-		return false;
+	UT_return_val_if_fail (NULL != m_fragments.getFirst(), false);
 
 	// first, store the attributes and properties and get an index to them.
 
@@ -384,12 +603,10 @@ bool pt_PieceTable::_makeObject(PTObjectType pto, const XML_Char ** attributes,
 bool pt_PieceTable::_makeFmtMark(pf_Frag_FmtMark * &pff)
 {
 	// this function can only be called while loading the document.
-	UT_ASSERT(m_pts==PTS_Loading);
+	UT_return_val_if_fail (m_pts==PTS_Loading,false);
 
 	// Only a strux can be appended to an empty document
-	UT_ASSERT(NULL != m_fragments.getFirst());
-	if (!m_fragments.getFirst())
-		return false;
+	UT_return_val_if_fail (NULL != m_fragments.getFirst(),false);
 
 	pff = new pf_Frag_FmtMark(this,loading.m_indexCurrentInlineAP);
 	if (!pff)
@@ -397,3 +614,26 @@ bool pt_PieceTable::_makeFmtMark(pf_Frag_FmtMark * &pff)
 
 	return true;
 }
+
+
+bool pt_PieceTable::_makeFmtMark(pf_Frag_FmtMark * &pff, const XML_Char ** attributes)
+{
+	// this function can only be called while loading the document.
+	UT_return_val_if_fail (m_pts==PTS_Loading,false);
+
+	// Only a strux can be appended to an empty document
+	UT_return_val_if_fail (NULL != m_fragments.getFirst(), false);
+	if(attributes == NULL)
+		{
+			return _makeFmtMark(pff);
+		}
+	PT_AttrPropIndex indexAP;
+	if (!m_varset.storeAP(attributes,&indexAP))
+		return false;
+
+	pff = new pf_Frag_FmtMark(this,indexAP);
+	if (!pff)
+		return false;
+
+	return true;
+}
