@@ -99,19 +99,20 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 		// Tomas, Oct 28, 2003
 
 		bool bRet = false;
-		for(PT_DocPosition curPos = dpos1; curPos < dpos2; )
+		while(dpos1 < dpos2)
 		{
 			// first retrive the starting and ending fragments
 			pf_Frag * pf1, * pf2;
 			PT_BlockOffset Offset1, Offset2;
+			bool bTableStrux = false;
+			bool bHasEndStrux = false;
+			UT_sint32 iTableDepth = 0;
 
-			if(!getFragsFromPositions(curPos,dpos2, &pf1, &Offset1, &pf2, &Offset2))
+			if(!getFragsFromPositions(dpos1,dpos2, &pf1, &Offset1, &pf2, &Offset2))
 				return bRet;
 			else
 				bRet = true;
 
-			curPos += pf1->getLength();
-							
 			// get attributes for this fragement
 			const PP_AttrProp * pAP;
 			pf_Frag::PFType eType = pf1->getType();
@@ -135,22 +136,39 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 					case PTX_Block:
 						iLen = pf_FRAG_STRUX_BLOCK_LENGTH;
 						break;
-
-					case PTX_Section:
-					case PTX_SectionHdrFtr:
-					case PTX_SectionEndnote:
+						
 					case PTX_SectionTable:
+						iTableDepth = 1;
+						// fall through
 					case PTX_SectionCell:
-					case PTX_SectionFootnote:
+						bTableStrux = true;
+						bHasEndStrux = true;
+						iLen = pf_FRAG_STRUX_SECTION_LENGTH;
+						break;
+						
 					case PTX_EndCell:
 					case PTX_EndTable:
+						bTableStrux = true;
+						iLen = pf_FRAG_STRUX_SECTION_LENGTH;
+						break;
+						
+					case PTX_SectionEndnote:
+					case PTX_SectionFootnote:
+					case PTX_SectionFrame:
+					case PTX_SectionTOC:
+						bHasEndStrux = true;
+						// fall through ...
+					case PTX_Section:
+					case PTX_SectionHdrFtr:
 				    case PTX_EndFootnote:
 				    case PTX_EndEndnote:
+				    case PTX_EndFrame:
+					case PTX_EndTOC:
 						iLen = pf_FRAG_STRUX_SECTION_LENGTH;
 						break;
 
 					default:
-						UT_ASSERT(UT_NOT_IMPLEMENTED);
+						UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED);
 						iLen = 1;
 						break;
 				}
@@ -161,12 +179,30 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 				if(!getAttrProp(static_cast<pf_Frag_Object*>(pf1)->getIndexAP(),&pAP))
 					return false;
 			}
+			else if(eType == pf_Frag:: PFT_FmtMark)
+			{
+				iLen = 0;
+				if(!getAttrProp(static_cast<pf_Frag_Object*>(pf1)->getIndexAP(),&pAP))
+					return false;
+				// something that does not carry AP
+			}
+			else if (eType == pf_Frag:: PFT_EndOfDoc)
+			{
+				break;
+			}
 			else
 			{
-				// something that does not carry AP
-				continue;
+				UT_ASSERT(0); // Dunno what this could be
+				break;
 			}
 
+			if(bTableStrux && !bDeleteTableStruxes)
+			{
+				// skip over this frag
+				dpos1 += iLen;
+				continue;
+			}
+			
 			if(!pAP->getAttribute(name, pRevision))
 				pRevision = NULL;
 
@@ -175,14 +211,15 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 			// now we need to see if revision with this id is already
 			// present, and if it is, whether it might not be addition
 			UT_uint32 iId = m_pDocument->getRevisionId();
-			const PP_Revision * pRev = Revisions.getGreatestLesserOrEqualRevision(iId);
+			const PP_Revision * pS;
+			const PP_Revision * pRev = Revisions.getGreatestLesserOrEqualRevision(iId, &pS);
 
 			PT_DocPosition dposEnd = UT_MIN(dpos2,dpos1 + pf1->getLength());
 
 			if(pRev && iId == pRev->getId())
 			{
 				// OK, we already have a revision with this id here,
-				// which means that the editor made a change earlier
+				// which means that the user made a change earlier
 				// (insertion or format change) but now wants this deleted
 				//
 				// so if the previous revision is an addition, we just
@@ -190,13 +227,80 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 				if(   (pRev->getType() == PP_REVISION_ADDITION)
 				   || (pRev->getType() == PP_REVISION_ADDITION_AND_FMT ))
 				{
+					// if this fragment is one of the struxes that is paired with an end-strux, we
+					// need to delete both struxes and everything in between
+					if(bHasEndStrux)
+					{
+						PT_DocPosition posEnd = dposEnd;
+						for(pf_Frag * pf = pf1->getNext(); pf != NULL; pf = pf->getNext())
+						{
+							posEnd += pf->getLength();
+							
+							if(pf_Frag::PFT_Strux != pf->getType())
+								continue;
+
+							pf_Frag_Strux * pfs = (pf_Frag_Strux*) pf;
+							PTStruxType eStrux2Type = pfs->getStruxType();
+
+							if(eStrux2Type == PTX_SectionTable)
+								iTableDepth++;
+							else if (eStrux2Type == PTX_EndTable)
+								iTableDepth--;
+								
+							
+							switch(eStruxType )
+							{
+								case PTX_SectionEndnote:
+									if(eStrux2Type != PTX_EndEndnote)
+										continue;
+									break;
+								case PTX_SectionTable:
+									if(iTableDepth > 0 || eStrux2Type != PTX_EndTable)
+										continue;
+									break;
+								case PTX_SectionCell:
+									if(iTableDepth > 0 || eStrux2Type != PTX_EndCell)
+										continue;
+									break;
+								case PTX_SectionFootnote:
+									if(eStrux2Type != PTX_EndFootnote)
+										continue;
+									break;
+								case PTX_SectionFrame:
+									if(eStrux2Type != PTX_EndFrame)
+										continue;
+									break;
+								case PTX_SectionTOC:
+									if(eStrux2Type != PTX_EndTOC)
+										continue;
+									break;
+							    default:
+									break;
+							}
+
+							// if we got this far, we found what we are looking for and we have the
+							// correct end position
+							break;
+						}
+						
+						dposEnd = posEnd;
+					}
+					
 					if(!_realDeleteSpan(dpos1, dposEnd, p_AttrProp_Before,bDeleteTableStruxes,
 										bDontGlob))
 						return false;
 
-					iRealDeleteCount += dposEnd - dpos1;
+					UT_uint32 iDeleteThisStep = dposEnd - dpos1;
+					
+					iRealDeleteCount += iDeleteThisStep;
 
-					dpos1 = dposEnd;
+					// because we removed stuff, the position dpos1 remains the same and dpos2 needs
+					// to be adjusted
+					if(dpos2 > iDeleteThisStep)
+						dpos2 -= iDeleteThisStep;
+					else
+						dpos2 = 0;
+					
 					continue;
 				}
 			}
@@ -209,21 +313,22 @@ bool pt_PieceTable::deleteSpan(PT_DocPosition dpos1,
 
 			switch (eType)
 			{
+				case pf_Frag::PFT_Object:
 				case pf_Frag::PFT_Text:
 					if(! _realChangeSpanFmt(PTC_AddFmt, dpos1, dposEnd, ppRevAttrib,NULL))
 						return false;
 					break;
 
 				case pf_Frag::PFT_Strux:
-					if(! _realChangeStruxFmt(PTC_AddFmt, dpos1 + iLen, dpos1 + 2*iLen, ppRevAttrib,NULL,eStruxType))
+					// _realChangeStruxFmt() changes the strux
+					// *containing* the given position, hence we pass
+					// it the position immediately after the strux; we
+					// only want the one strux changed, so we pass
+					// identical position in both parameters
+					if(! _realChangeStruxFmt(PTC_AddFmt, dpos1 + iLen, dpos1 + iLen /*2*iLen*/, ppRevAttrib,NULL,eStruxType))
 						return false;
 					break;
-#if 0
-				case pf_Frag::PFT_Object:
-					if(! _realChangeStruxFmt(PTC_AddFmt, dpos1, dposEnd, ppRevAttrib,NULL))
-						return false;
-					break;
-#endif
+
 				default:;
 			}
 
@@ -246,7 +351,7 @@ bool pt_PieceTable::_deleteSpan(pf_Frag_Text * pft, UT_uint32 fragOffset,
 	// perform simple delete of a span of text.
 	// we assume that it is completely contained within this fragment.
 
-	UT_ASSERT(fragOffset+length <= pft->getLength());
+	UT_return_val_if_fail (fragOffset+length <= pft->getLength(),false);
 
 	SETP(ppfEnd, pft);
 	SETP(pfragOffsetEnd, fragOffset);
@@ -294,7 +399,7 @@ bool pt_PieceTable::_deleteSpan(pf_Frag_Text * pft, UT_uint32 fragOffset,
 	UT_uint32 lenTail = pft->getLength() - startTail;
 	PT_BufIndex biTail = m_varset.getBufIndex(pft->getBufIndex(),startTail);
 	pf_Frag_Text * pftTail = new pf_Frag_Text(this,biTail,lenTail,pft->getIndexAP(),pft->getField());
-	UT_ASSERT(pftTail);
+	UT_return_val_if_fail (pftTail, false);
 	pft->changeLength(fragOffset);
 	m_fragments.insertFrag(pft,pftTail);
 
@@ -313,7 +418,7 @@ bool pt_PieceTable::_deleteSpanWithNotify(PT_DocPosition dpos,
 {
 	// create a change record for this change and put it in the history.
 
-	UT_ASSERT(pfs);
+	UT_return_val_if_fail (pfs, false);
 
 	if (length == 0)					// TODO decide if this is an error.
 	{
@@ -334,7 +439,7 @@ bool pt_PieceTable::_deleteSpanWithNotify(PT_DocPosition dpos,
 								   dpos, pft->getIndexAP(),
 								   m_varset.getBufIndex(pft->getBufIndex(),fragOffset),
 								   length,blockOffset,pft->getField());
-	UT_ASSERT(pcr);
+	UT_return_val_if_fail (pcr, false);
 
 	bool bResult = _deleteSpan(pft,fragOffset,pft->getBufIndex(),length,ppfEnd,pfragOffsetEnd);
 
@@ -359,7 +464,7 @@ bool pt_PieceTable::_canCoalesceDeleteSpan(PX_ChangeRecord_Span * pcrSpan) const
 {
 	// see if this record can be coalesced with the most recent undo record.
 
-	UT_ASSERT(pcrSpan->getType() == PX_ChangeRecord::PXT_DeleteSpan);
+	UT_return_val_if_fail (pcrSpan->getType() == PX_ChangeRecord::PXT_DeleteSpan, false);
 
 	PX_ChangeRecord * pcrUndo;
 	if (!m_history.getUndo(&pcrUndo))
@@ -415,7 +520,7 @@ bool pt_PieceTable::_isSimpleDeleteSpan(PT_DocPosition dpos1,
 	PT_BlockOffset fragOffset_End;
 
 	bool bFound = getFragsFromPositions(dpos1,dpos2,&pf_First,&fragOffset_First,&pf_End,&fragOffset_End);
-	UT_ASSERT(bFound);
+	UT_return_val_if_fail (bFound, false);
 
 	if ((fragOffset_End==0) && pf_End->getPrev() && (pf_End->getPrev()->getType() == pf_Frag::PFT_Text))
 	{
@@ -430,6 +535,9 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 										 PT_DocPosition & dpos2,
 										 UT_Stack * pstDelayStruxDelete) const
 {
+	if(m_bDoNotTweakPosition)
+		return true;
+	
 	//  Our job is to adjust the end positions of the delete
 	//  operating to delete those structural object that the
 	//  user will expect to have deleted, even if the dpositions
@@ -442,27 +550,27 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 	PT_BlockOffset fragOffset_End;
 
 	bool bFound = getFragsFromPositions(dpos1,dpos2,&pf_First,&fragOffset_First,&pf_End,&fragOffset_End);
-	UT_ASSERT(bFound);
+	UT_return_val_if_fail (bFound, false);
 
 	pf_Frag_Strux * pfsContainer = NULL;
 	bool bFoundStrux = _getStruxFromPosition(dpos1,&pfsContainer);
-	UT_ASSERT(bFoundStrux);
+	UT_return_val_if_fail (bFoundStrux,false);
 
     _tweakFieldSpan(dpos1,dpos2);
 
 	switch (pfsContainer->getStruxType())
 	{
 	default:
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 		return false;
 
 	case PTX_Section:
 #if 0
 		// if the previous container is a section, then pf_First
 		// must be the first block in the section.
-		UT_ASSERT((pf_First->getPrev() == pfsContainer));
-		UT_ASSERT((pf_First->getType() == pf_Frag::PFT_Strux));
-		UT_ASSERT(((static_cast<pf_Frag_Strux *>(pf_First))->getStruxType() == PTX_Block));
+		UT_return_val_if_fail ((pf_First->getPrev() == pfsContainer),false);
+		UT_return_val_if_fail ((pf_First->getType() == pf_Frag::PFT_Strux),false);
+		UT_return_val_if_fail (((static_cast<pf_Frag_Strux *>(pf_First))->getStruxType() == PTX_Block),false);
 		// since, we cannot delete the first block in a section, we
 		// secretly translate this into a request to delete the section;
 		// the block we have will then be slurped into the previous
@@ -473,9 +581,9 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 	case PTX_SectionHdrFtr:
 		// if the previous container is a Header/Footersection, then pf_First
 		// must be the first block in the section.
-		UT_ASSERT((pf_First->getPrev() == pfsContainer));
-		UT_ASSERT((pf_First->getType() == pf_Frag::PFT_Strux));
-		UT_ASSERT(((static_cast<pf_Frag_Strux *>(pf_First))->getStruxType() == PTX_Block));
+		UT_return_val_if_fail ((pf_First->getPrev() == pfsContainer),false);
+		UT_return_val_if_fail ((pf_First->getType() == pf_Frag::PFT_Strux),false);
+		UT_return_val_if_fail (((static_cast<pf_Frag_Strux *>(pf_First))->getStruxType() == PTX_Block),false);
 		// since, we cannot delete the first block in a section, we
 		// secretly translate this into a request to delete the section;
 		// the block we have will then be slurped into the previous
@@ -485,8 +593,12 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 
 	case PTX_SectionTable:
 	case PTX_SectionCell:
+	case PTX_SectionFrame:
 	case PTX_EndTable:
 	case PTX_EndCell:
+	case PTX_EndFrame:
+	case PTX_SectionTOC:
+	case PTX_EndTOC:
 //
 // We've set things up so that deleting table struxes is done very deliberately.//  Don't mess with the end points here
 //
@@ -527,7 +639,7 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 			break;
 
 		case PTX_Section:
-			UT_ASSERT(fragOffset_First == 0);
+			UT_return_val_if_fail (fragOffset_First == 0,false);
 			if (dpos2 == dpos1 + pf_First->getLength())
 			{
 				//  If we are just deleting a section break, then
@@ -535,14 +647,14 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 				//  next section, combining the blocks before and
 				//  after the section break.
 				pf_Other = pf_First->getNext();
-				UT_ASSERT(pf_Other);
-				UT_ASSERT(pf_Other->getType() == pf_Frag::PFT_Strux);
-				UT_ASSERT(((static_cast<pf_Frag_Strux *>(pf_Other))->getStruxType() == PTX_Block));
+				UT_return_val_if_fail (pf_Other, false);
+				UT_return_val_if_fail (pf_Other->getType() == pf_Frag::PFT_Strux,false);
+				UT_return_val_if_fail (((static_cast<pf_Frag_Strux *>(pf_Other))->getStruxType() == PTX_Block),false);
 				dpos2 += pf_Other->getLength();
 				return true;
 			}
 		case PTX_SectionHdrFtr:
-			UT_ASSERT(fragOffset_First == 0);
+			UT_return_val_if_fail (fragOffset_First == 0, false);
 			if (dpos2 == dpos1 + pf_First->getLength())
 			{
 				//  If we are just deleting a section break, then
@@ -550,9 +662,9 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 				//  next section, combining the blocks before and
 				//  after the section break.
 				pf_Other = pf_First->getNext();
-				UT_ASSERT(pf_Other);
-				UT_ASSERT(pf_Other->getType() == pf_Frag::PFT_Strux);
-				UT_ASSERT(((static_cast<pf_Frag_Strux *>(pf_Other))->getStruxType() == PTX_Block));
+				UT_return_val_if_fail (pf_Other,false);
+				UT_return_val_if_fail (pf_Other->getType() == pf_Frag::PFT_Strux,false);
+				UT_return_val_if_fail (((static_cast<pf_Frag_Strux *>(pf_Other))->getStruxType() == PTX_Block),false);
 				dpos2 += pf_Other->getLength();
 				return true;
 			}
@@ -561,6 +673,14 @@ bool pt_PieceTable::_tweakDeleteSpanOnce(PT_DocPosition & dpos1,
 		}
 	}
 
+	if(pf_End->getType() ==  pf_Frag::PFT_Strux)
+	{
+	    if(static_cast<pf_Frag_Strux *>(pf_End)->getStruxType() == PTX_EndTOC)
+		{
+			dpos2++;
+		}
+	}
+
 	if (fragOffset_First == 0 && fragOffset_End == 0 && pf_First != pf_End)
 	{
 		pf_Frag * pf_Before = pf_First->getPrev();
@@ -625,6 +745,134 @@ bool pt_PieceTable::_tweakDeleteSpan(PT_DocPosition & dpos1,
 									 PT_DocPosition & dpos2,
 									 UT_Stack * pstDelayStruxDelete) const
 {
+	if(m_bDoNotTweakPosition)
+		return true;
+	
+	//
+// First we want to handle hyperlinks. If we delete all the text within
+// a hyperlink, then we should also delete the start and end point of the
+// hyperlink.
+//
+	pf_Frag * pf_First;
+	pf_Frag * pf_End;
+	PT_BlockOffset fragOffset_First;
+	PT_BlockOffset fragOffset_End;
+
+	bool bFound = getFragsFromPositions(dpos1,dpos2,&pf_First,&fragOffset_First,&pf_End,&fragOffset_End);
+	UT_return_val_if_fail (bFound,false);
+	while(pf_First && (pf_First->getLength() == 0))
+	{
+		pf_First = pf_First->getNext();
+	}
+	if(pf_First)
+	{
+		while(pf_End && (pf_End->getLength() == 0))
+		{
+			pf_End = pf_End->getPrev();
+		}
+		bool bDoit = false;
+		if(pf_End && ((pf_End->getPos() + pf_End->getLength() - pf_First->getPos())  == (dpos2 - dpos1 +1)))
+		{
+			if((pf_First->getType() == pf_Frag::PFT_Text) && (pf_First->getLength() == 2))
+			{
+				bDoit = false;
+			}
+			else if((pf_First->getType() == pf_Frag::PFT_Text) && (pf_End->getType() == pf_Frag::PFT_Text) && (pf_First != pf_End))
+			{
+				bDoit = false;
+			}
+			else
+			{
+				bDoit = true;
+			}
+		}
+		if(pf_End && ((pf_End->getPos() + pf_End->getLength() - pf_First->getPos())  == (dpos2 - dpos1)))
+		{
+			bDoit = true;
+		}
+		if(bDoit)
+		{
+//
+// OK these frags are entirely contained by dpos1  and dpos2
+// OK now look to see if there is a hyperlink just before and after these
+//
+			if(pf_End->getType() != pf_Frag::PFT_Object)
+			{
+				pf_End = pf_End->getNext();
+			}
+			while(pf_End && (pf_End->getLength() == 0))
+			{
+				pf_End = pf_End->getNext();
+			}
+			if(pf_First->getType() != pf_Frag::PFT_Object)
+			{
+				pf_First = pf_First->getPrev();
+			}
+			while(pf_First && (pf_First->getLength() == 0))
+			{
+				pf_First = pf_First->getPrev();
+			}
+			if(pf_First && (pf_First->getType() == pf_Frag::PFT_Object))
+			{
+				pf_Frag_Object *pFO = static_cast<pf_Frag_Object *>(pf_First);
+				bool bFoundBook = false;
+				bool bFoundHype = false;
+				if(pFO->getObjectType() == PTO_Bookmark)
+				{
+					bFoundBook = true;
+				}
+				if(pFO->getObjectType() == PTO_Hyperlink)
+				{
+					bFoundHype = true;
+				}
+				if(pf_End && (pf_End->getType() == pf_Frag::PFT_Object) && (pf_End != pf_First))
+				{
+					pFO = static_cast<pf_Frag_Object *>(pf_End);
+					if(pFO->getObjectType() == PTO_Bookmark && bFoundBook)
+					{
+//
+// Found a bookmark which will have all contents deleted so delete it too
+//
+						dpos1--;
+						dpos2++;
+					}
+					else if(pFO->getObjectType() == PTO_Hyperlink && bFoundHype)
+					{
+//
+// Found a Hyperlink which will have all contents deleted so delte it too
+//
+						dpos1--;
+						dpos2++;
+					}
+				}
+			}
+		}
+	}
+//
+// Can't handle a delete span start from an endTOC. sum1 has arranged corner
+// cases where this is possible. HAndle this corner case by starting at the 
+// next strux
+//
+	if(pf_First->getType() == pf_Frag::PFT_Strux)
+	{
+		pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *>(pf_First);
+		if(pfs->getStruxType() == PTX_EndTOC)
+		{
+			pf_Frag * pf = pf_First->getNext();
+			while(pf && pf->getLength() == 0)
+			{
+				pf = pf->getNext();
+			}
+			if(pf && pf->getType() ==  pf_Frag::PFT_Strux)
+			{
+				pfs = static_cast<pf_Frag_Strux *>(pf);
+				if(pfs->getStruxType() == PTX_Block)
+				{
+					dpos1++;
+				}
+			}
+		}
+	}
 	//  We want to keep tweaking the delete span until there is nothing
 	//  more to tweak.  We check to see if nothing has changed in the
 	//  last tweak, and if so, we are done.
@@ -652,7 +900,7 @@ bool pt_PieceTable::_deleteFormatting(PT_DocPosition dpos1,
 	PT_BlockOffset fragOffset_End;
 
 	bool bFound = getFragsFromPositions(dpos1,dpos2,&pf_First,&fragOffset_First,&pf_End,&fragOffset_End);
-	UT_ASSERT(bFound);
+	UT_return_val_if_fail (bFound,false);
 
 	// before we delete the content, we do a quick scan and delete
 	// any FmtMarks first -- this let's us avoid problems with
@@ -679,10 +927,10 @@ bool pt_PieceTable::_deleteFormatting(PT_DocPosition dpos1,
 				_getStruxFromFragSkip(pfsContainerTemp,&pfsContainerTemp);
 				xxx_UT_DEBUGMSG(("_deleteSpan 6: After skip  pfsContainer %x type %d \n",pfsContainerTemp,pfsContainerTemp->getStruxType()));
 			}
-			UT_ASSERT(bFoundStrux);
+			UT_return_val_if_fail (bFoundStrux,false);
 			bool bResult = _deleteFmtMarkWithNotify(dposTemp,static_cast<pf_Frag_FmtMark *>(pfTemp),
 										 pfsContainerTemp,&pfNewTemp,&fragOffsetNewTemp);
-			UT_ASSERT(bResult);
+			UT_return_val_if_fail (bResult,false);
 
 			// FmtMarks have length zero, so we don't need to update dposTemp.
 			pfTemp = pfNewTemp;
@@ -712,35 +960,46 @@ bool pt_PieceTable::_deleteFormatting(PT_DocPosition dpos1,
 }
 
 /*!
- * Returns true if pfs is not a strux connected with a table
+ * Returns true if pfs is not a strux connected with a table or frame
  */
 bool pt_PieceTable::_StruxIsNotTable(pf_Frag_Strux * pfs)
 {
 	PTStruxType its = pfs->getStruxType();
 	bool b = ((its != PTX_SectionTable) && (its != PTX_SectionCell)
-			  && (its != PTX_EndTable) && (its != PTX_EndCell));
+			  && (its != PTX_EndTable) && (its != PTX_EndCell)
+			  && (its != PTX_SectionFrame) && (its != PTX_EndFrame));
 	return b;
 }
 
-bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
-									   PT_DocPosition dpos2,
+/*
+    Because complex span can involve deletion of a bookmark the comrade of which is outside of the
+    deletion range, this function can change dpos1 and dpos2 to indicate which document positions
+    after the deletion correspond to the original values passed to the function -- the caller needs
+    to take this into account
+ */
+bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition & origPos1,
+									   PT_DocPosition & origPos2,
 									   UT_Stack * stDelayStruxDelete)
 {
 	pf_Frag * pfNewEnd;
 	UT_uint32 fragOffsetNewEnd;
 	bool bPrevWasCell = false;
 	bool bPrevWasEndTable = false;
+	bool bPrevWasFrame = false;
 	pf_Frag * pf_First;
 	pf_Frag * pf_End;
 	PT_BlockOffset fragOffset_First;
 	PT_BlockOffset fragOffset_End;
 
+	PT_DocPosition dpos1 = origPos1;
+	PT_DocPosition dpos2 = origPos2;
+
 	bool bFound = getFragsFromPositions(dpos1,dpos2,&pf_First,&fragOffset_First,&pf_End,&fragOffset_End);
-	UT_ASSERT(bFound);
+	UT_return_val_if_fail (bFound, false);
 	UT_DEBUGMSG(("deleteComplex span dpos1 %d dpos2 %d pf_First %x pf_First pos %d \n",dpos1,dpos2,pf_First,pf_First->getPos()));
 	pf_Frag_Strux * pfsContainer = NULL;
 	bool bFoundStrux = _getStruxFromPosition(dpos1,&pfsContainer);
-	UT_ASSERT(bFoundStrux);
+	UT_return_val_if_fail (bFoundStrux, false);
 	if(isEndFootnote(pfsContainer))
 	{
 		xxx_UT_DEBUGMSG(("_deleteSpan 3: orig pfsContainer %x type %d \n",pfsContainer,pfsContainer->getStruxType()));
@@ -754,6 +1013,10 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 	UT_uint32 length = dpos2 - dpos1;
 	UT_uint32 iTable = 0;
 	UT_sint32 iFootnoteCount = 0;
+	if(pfsContainer->getStruxType() == PTX_SectionFrame)
+	{
+		bPrevWasFrame = true;
+	}
 	if(pfsContainer->getStruxType() == PTX_SectionCell)
 	{
 		bPrevWasCell = true;
@@ -774,7 +1037,7 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 		{
 		case pf_Frag::PFT_EndOfDoc:
 		default:
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 			return false;
 
 		case pf_Frag::PFT_Strux:
@@ -791,6 +1054,18 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 //
 // Dont' delete this Block strux if the previous strux is a cell and this
 // is the start of the delete phase.
+//
+				pfNewEnd = pfs->getNext();
+				fragOffsetNewEnd = 0;
+				pfsContainer = pfs;
+				dpos1 = dpos1 +  lengthInFrag;
+				break;
+			}
+			if(bPrevWasFrame && (iLoopCount == 0))
+			{
+//
+// Dont' delete this Block strux if the previous strux is a cell and this
+// is the start of the delete phase.
 //
 				pfNewEnd = pfs->getNext();
 				fragOffsetNewEnd = 0;
@@ -800,12 +1075,13 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 			}
 			if(_StruxIsNotTable(pfs))
 			{
-				if((bPrevWasCell || bPrevWasFootnote || bPrevWasEndTable) 
+				if((bPrevWasCell || bPrevWasFootnote || bPrevWasEndTable|| bPrevWasFrame) 
 				   && pfs->getStruxType()== PTX_Block)
 				{
 					bPrevWasCell = false;
 					bPrevWasFootnote = false;
 					bPrevWasEndTable = false;
+					bPrevWasFrame = false;
 					pfNewEnd = pfs->getNext();
 					fragOffsetNewEnd = 0;
 					pfsContainer = pfs;
@@ -824,6 +1100,7 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 						bResult = _deleteStruxWithNotify(dpos1,pfs,
 												  &pfNewEnd,&fragOffsetNewEnd);
 						bPrevWasCell = false;
+						bPrevWasFrame = false;
 						bPrevWasEndTable = false;
 						break;
 					}
@@ -858,6 +1135,14 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 				{
 					bPrevWasCell = false;
 				}
+				if(pfs->getStruxType() == PTX_SectionFrame )
+				{
+					bPrevWasFrame = true;
+				}
+				else
+				{
+					bPrevWasFrame = false;
+				}
 				if(pfs->getStruxType() == PTX_EndTable)
 				{
 					bPrevWasEndTable = true;
@@ -892,31 +1177,33 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 // Deleting the table so don't delay deleting the following strux.
 //
 					bPrevWasEndTable = false;
-					xxx_UT_DEBUGMSG(("Doing Table delete immediately \n"));
-					iTable = 1;
+					UT_DEBUGMSG(("Doing Table delete immediately \n"));
+//					iTable = 1;
+					UT_sint32 myTable =1;
 //
 // First delete the EndTable Strux
 //
 					stDelayStruxDelete->pop(reinterpret_cast<void **>(&pfs));
+					if(m_fragments.areFragsDirty())
+					{
+						m_fragments.cleanFrags();
+					}
 					PT_DocPosition myPos = pfs->getPos();
 					_deleteFormatting(myPos - pfs->getLength(), myPos);
+					UT_DEBUGMSG(("DELeteing EndTable Strux, pos= %d \n",pfs->getPos()));
 					bResult = _deleteStruxWithNotify(myPos, pfs,
 													  &pfNewEnd,
 													  &fragOffsetNewEnd);
-					while(bResult && iTable > 0)
+					while(bResult && myTable > 0)
 					{
 						stDelayStruxDelete->pop(reinterpret_cast<void **>(&pfs));
-						if(m_fragments.areFragsDirty())
-						{
-							m_fragments.cleanFrags();
-						}
 						if(pfs->getStruxType() == PTX_SectionTable)
 						{
-							iTable--;
+							myTable--;
 						}
 						else if(pfs->getStruxType() == PTX_EndTable)
 						{
-							iTable++;
+							myTable++;
 						}
 						PT_DocPosition myPos = pfs->getPos();
 						_deleteFormatting(myPos - pfs->getLength(), myPos);
@@ -932,17 +1219,9 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 //
 					bFoundStrux = _getStruxFromPosition(dpos1,&pfsContainer);
 				}
+				UT_DEBUGMSG(("Finished doing table delete -1 \n"));
 				break;
 			}
-			else if(isEndFootnote(pfs))
-			{
-//
-// Attempting to delete an EndFootnote end strux without a matching begin.
-// terminate the loop now.
-//
-				return false;
-
-			}
 //
 // Look to see if we've reached the end of a footnote section.
 //
@@ -995,12 +1274,75 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 				}
 //
 // Now we have to update pfsContainer from dpos1
+//
+				bFoundStrux = _getStruxFromPosition(dpos1,&pfsContainer);
+				break;
+			}
+			else if(isEndFootnote(pfs))
+			{
+//
+// Attempting to delete an EndFootnote end strux without a matching begin.
+// terminate the loop now.
+//
+				return false;
+
+			}
+//
+// Look to see if we've reached the end of a Frame section.
+//
+			if (pfs->getStruxType() == PTX_EndFrame)
+			{
+//
+// First delete the EndFrame Strux
+//
+				UT_DEBUGMSG(("Doing Frame delete now \n"));
+				stDelayStruxDelete->pop(reinterpret_cast<void **>(&pfs));
+				if(m_fragments.areFragsDirty())
+				{
+					m_fragments.cleanFrags();
+				}
+				PT_DocPosition myPos = pfs->getPos();
+				_deleteFormatting(myPos - pfs->getLength(), myPos);
+				bool isFrame =  pfs->getStruxType() == PTX_SectionFrame;
+				bResult = _deleteStruxWithNotify(myPos, pfs,
+												  &pfNewEnd,
+												  &fragOffsetNewEnd);
+//
+// Each strux is one in length, we've added one while delaying the delete
+// so subtract it now.
+//
+				dpos1 -= 1;
+//
+// Now delete the Frame strux. Doing things in this order works for
+// the layout classes for both the delete (needs the endFrame strux 
+// deleted first) and for
+// undo where we want the Frame Strux inserted first.
+//
+				while(bResult && !isFrame)
+				{
+					stDelayStruxDelete->pop(reinterpret_cast<void **>(&pfs));
+					if(m_fragments.areFragsDirty())
+					{
+						m_fragments.cleanFrags();
+					}
+					PT_DocPosition myPos = pfs->getPos();
+					isFrame =  pfs->getStruxType() == PTX_SectionFrame;
+					_deleteFormatting(myPos - pfs->getLength(), myPos);
+					bResult = _deleteStruxWithNotify(myPos, pfs, &pff, &dpp);
+//
+// Each strux is one in length, we've added one while delaying the delete
+// so subtract it now.
+//
+					dpos1 -= 1;
+				}
+//
+// Now we have to update pfsContainer from dpos1
 //
 				bFoundStrux = _getStruxFromPosition(dpos1,&pfsContainer);
 				break;
 			}
 			
-			UT_ASSERT(bResult);
+			UT_return_val_if_fail (bResult,false);
 			// we do not update pfsContainer because we just deleted pfs.
 		}
 		break;
@@ -1014,7 +1356,7 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 			bool bResult = _deleteSpanWithNotify(dpos1,static_cast<pf_Frag_Text *>(pf_First),
 									  fragOffset_First,lengthThisStep,
 									  pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
-			UT_ASSERT(bResult);
+			UT_return_val_if_fail (bResult, false);
 		}
 		break;
 		// the bookmark and hyperlink objects require special treatment; since
@@ -1039,7 +1381,7 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 					pf_Frag_Strux * pfsContainer2 = NULL;
 
 					po_Bookmark * pB = pO->getBookmark();
-					UT_ASSERT(pB);
+					UT_return_val_if_fail (pB, false);
 					pf_Frag * pF;
 					if(pB->getBookmarkType() == po_Bookmark::POBOOKMARK_END)
 					{
@@ -1050,14 +1392,19 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 							{
 								pf_Frag_Object *pOb = static_cast<pf_Frag_Object*>(pF);
 								po_Bookmark * pB1 = pOb->getBookmark();
-								if(!strcmp(pB->getName(),pB1->getName()))
+								if(pB1 && !strcmp(pB->getName(),pB1->getName()))
 								{
-
 									m_pDocument->removeBookmark(pB1->getName());
 
 									posComrade = getFragPosition(pOb);
-									bFoundStrux2 = _getStruxFromPosition(posComrade,&pfsContainer2);
-									UT_ASSERT(bFoundStrux2);
+									
+									if(posComrade < origPos1)
+									{
+										origPos1--;
+									}
+									
+									bFoundStrux2 = _getStruxFromFragSkip(pOb,&pfsContainer2);
+									UT_return_val_if_fail (bFoundStrux2, false);
 
 									bResult2 =
 											_deleteObjectWithNotify(posComrade,pOb,0,1,
@@ -1093,8 +1440,8 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 									m_pDocument->removeBookmark(pB1->getName());
 
 									posComrade = getFragPosition(pOb);
-									bool bFoundStrux2 = _getStruxFromPosition(posComrade,&pfsContainer2);
-									UT_ASSERT(bFoundStrux2);
+									bool bFoundStrux2 = _getStruxFromFragSkip(pOb,&pfsContainer2);
+									UT_return_val_if_fail (bFoundStrux2, false);
 
 									bResult2 =
 											_deleteObjectWithNotify(posComrade,pOb,0,1,
@@ -1129,7 +1476,7 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 
 		    		const PP_AttrProp * pAP = NULL;
 				    pO->getPieceTable()->getAttrProp(pO->getIndexAP(),&pAP);
-				    UT_ASSERT(pAP);
+				    UT_return_val_if_fail (pAP, false);
 				    const XML_Char* pszHref = NULL;
 				    const XML_Char* pszHname  = NULL;
 		            UT_uint32 k = 0;
@@ -1156,8 +1503,8 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 								if(pOb->getObjectType() == PTO_Hyperlink)
 								{
 									posComrade = getFragPosition(pOb);
-									bFoundStrux2 = _getStruxFromPosition(posComrade,&pfsContainer2);
-									UT_ASSERT(bFoundStrux2);
+									bFoundStrux2 = _getStruxFromFragSkip(pOb,&pfsContainer2);
+									UT_return_val_if_fail (bFoundStrux2, false);
 
 									bResult2 =
 											_deleteObjectWithNotify(posComrade,pOb,0,1,
@@ -1196,8 +1543,8 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 								if(pOb->getObjectType() == PTO_Hyperlink)
 								{
 									posComrade = getFragPosition(pOb);
-									bFoundStrux2 = _getStruxFromPosition(posComrade,&pfsContainer2);
-									UT_ASSERT(bFoundStrux2);
+									bFoundStrux2 = _getStruxFromFragSkip(pOb,&pfsContainer2);
+									UT_return_val_if_fail (bFoundStrux2, false);
 					                // delete the original start marker
 									bResult
 										= _deleteObjectWithNotify(dpos1,pO,fragOffset_First,lengthThisStep,
@@ -1237,13 +1584,13 @@ bool pt_PieceTable::_deleteComplexSpan(PT_DocPosition dpos1,
 			}
 
 
-			UT_ASSERT(bResult);
+			UT_return_val_if_fail (bResult, false);
 		}
 		break;
 
 		case pf_Frag::PFT_FmtMark:
 			// we already took care of these...
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 			break;
 
 		}
@@ -1282,11 +1629,11 @@ bool pt_PieceTable::_deleteComplexSpan_norec(PT_DocPosition dpos1,
 	PT_BlockOffset fragOffset_End;
 
 	bool bFound = getFragsFromPositions(dpos1,dpos2,&pf_First,&fragOffset_First,&pf_End,&fragOffset_End);
-	UT_ASSERT(bFound);
+	UT_return_val_if_fail (bFound, false);
 
 	pf_Frag_Strux * pfsContainer = NULL;
 	bool bFoundStrux = _getStruxFromPosition(dpos1,&pfsContainer);
-	UT_ASSERT(bFoundStrux);
+	UT_return_val_if_fail (bFoundStrux, false);
 
 	// loop to delete the amount requested, one text fragment at a time.
 	// if we encounter any non-text fragments along the way, we delete
@@ -1302,7 +1649,7 @@ bool pt_PieceTable::_deleteComplexSpan_norec(PT_DocPosition dpos1,
 		{
 		case pf_Frag::PFT_EndOfDoc:
 		default:
-			UT_ASSERT(0);
+			UT_ASSERT_HARMLESS(0);
 			return false;
 
 		case pf_Frag::PFT_Strux:
@@ -1312,7 +1659,7 @@ bool pt_PieceTable::_deleteComplexSpan_norec(PT_DocPosition dpos1,
 			bool bResult = _deleteStruxWithNotify(dpos1,pfs,
 									   &pfNewEnd,&fragOffsetNewEnd,
 									   false);
-			UT_ASSERT(bResult);
+			UT_return_val_if_fail (bResult, false);
 			// we do not update pfsContainer because we just deleted pfs.
 		}
 		break;
@@ -1324,22 +1671,23 @@ bool pt_PieceTable::_deleteComplexSpan_norec(PT_DocPosition dpos1,
 									  fragOffset_First,lengthThisStep,
 									  pfsContainer,&pfNewEnd,
 									  &fragOffsetNewEnd, false);
-			UT_ASSERT(bResult);
+			UT_return_val_if_fail (bResult, false);
 		}
 		break;
 
 		case pf_Frag::PFT_Object:
 		{
-			bool bResult = _deleteObject_norec(dpos1,static_cast<pf_Frag_Object *>(pf_First),
+			bool bResult = _deleteObjectWithNotify(dpos1,static_cast<pf_Frag_Object *>(pf_First),
 									fragOffset_First,lengthThisStep,
-									pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
-			UT_ASSERT(bResult);
+									pfsContainer,&pfNewEnd,&fragOffsetNewEnd,
+									false);
+			UT_return_val_if_fail (bResult, false);
 		}
 		break;
 
 		case pf_Frag::PFT_FmtMark:
 			// we already took care of these...
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 			break;
 
 		}
@@ -1375,8 +1723,8 @@ bool pt_PieceTable::_realDeleteSpan(PT_DocPosition dpos1,
 {
 	// remove (dpos2-dpos1) characters from the document at the given position.
 
-	UT_ASSERT(m_pts==PTS_Editing);
-	UT_ASSERT(dpos2 > dpos1);
+	UT_return_val_if_fail (m_pts==PTS_Editing, false);
+	UT_return_val_if_fail (dpos2 > dpos1, false);
 
 	bool bSuccess = true;
 	UT_Stack stDelayStruxDelete;
@@ -1404,10 +1752,13 @@ bool pt_PieceTable::_realDeleteSpan(PT_DocPosition dpos1,
 		{
 			const PP_AttrProp *p_AttrProp;
 			getAttrProp(static_cast<pf_Frag_Text *>(pf1)->getIndexAP(), &p_AttrProp);
-
+		  
 			AttrProp_Before = *p_AttrProp;
 			if(p_AttrProp_Before)
 				*p_AttrProp_Before = *p_AttrProp;
+
+			// we do not want to inherit revision attribute
+			AttrProp_Before.setAttribute("revision", "");
 		}
 	}
 
@@ -1516,7 +1867,8 @@ bool pt_PieceTable::_realDeleteSpan(PT_DocPosition dpos1,
 		// All text in paragraph is deleted so insert a text format.
 		// Except if we're realy don't want it. We know we dont if
 		// bDontGlob is true.
-		if(!bDontGlob)
+		pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *>(p_frag_before);
+		if(!bDontGlob && ((pfs->getStruxType() == PTX_Block) || (p_frag_before->getType() == pf_Frag::PFT_EndOfDoc) ))
 			_insertFmtMarkFragWithNotify(PTC_AddFmt, dpos1, &AttrProp_Before);
 
 	}
@@ -1536,13 +1888,13 @@ bool pt_PieceTable::deleteFieldFrag(pf_Frag * pf)
 {
 
 
-	UT_ASSERT(m_pts==PTS_Editing);
+	UT_return_val_if_fail (m_pts==PTS_Editing,false);
 
 	bool bSuccess = true;
 	UT_Stack stDelayStruxDelete;
 
 	PT_DocPosition dpos1 = getFragPosition(pf);
-	UT_ASSERT(dpos1);
+	UT_return_val_if_fail (dpos1,false);
 	PT_DocPosition dpos2 = dpos1 + pf->getLength();
 
 
@@ -1556,6 +1908,9 @@ bool pt_PieceTable::deleteFieldFrag(pf_Frag * pf)
 void pt_PieceTable::_tweakFieldSpan(PT_DocPosition & dpos1,
                                     PT_DocPosition & dpos2) const
 {
+	if(m_bDoNotTweakPosition)
+		return;
+	
 	//  Our job is to adjust the end positions of the delete
 	//  operating to delete those structural object that the
 	//  user will expect to have deleted, even if the dpositions
@@ -1568,11 +1923,11 @@ void pt_PieceTable::_tweakFieldSpan(PT_DocPosition & dpos1,
 	PT_BlockOffset fragOffset_End;
 
 	bool bFound = getFragsFromPositions(dpos1,dpos2,&pf_First,&fragOffset_First,&pf_End,&fragOffset_End);
-	UT_ASSERT(bFound);
+	UT_return_if_fail (bFound);
 
 	pf_Frag_Strux * pfsContainer = NULL;
 	bool bFoundStrux = _getStruxFromPosition(dpos1,&pfsContainer);
-	UT_ASSERT(bFoundStrux);
+	UT_return_if_fail (bFoundStrux);
 
     // if start in middle of field widen to include object
     if ((pf_First->getType() == pf_Frag::PFT_Text)&&
@@ -1585,14 +1940,14 @@ void pt_PieceTable::_tweakFieldSpan(PT_DocPosition & dpos1,
         while (pft->getPrev()->getType() == pf_Frag::PFT_Text)
         {
             pft2 = static_cast<pf_Frag_Text *>(pft->getPrev());
-            UT_ASSERT(pft->getField() == pft2->getField());
+            UT_ASSERT_HARMLESS(pft->getField() == pft2->getField());
             pft = pft2;
         }
-        UT_ASSERT(pft->getPrev()->getType() == pf_Frag::PFT_Object);
+        UT_return_if_fail (pft->getPrev()->getType() == pf_Frag::PFT_Object);
         pf_Frag_Object *pfo =
             static_cast<pf_Frag_Object *>(pft->getPrev());
-        UT_ASSERT(pfo->getObjectType()==PTO_Field);
-        UT_ASSERT(pfo->getField()==pft->getField());
+        UT_return_if_fail (pfo->getObjectType()==PTO_Field);
+        UT_return_if_fail (pfo->getField()==pft->getField());
         dpos1 = getFragPosition(pfo);
     }
     // if end in middle of field widen to include whole Frag_Text
@@ -1604,13 +1959,13 @@ void pt_PieceTable::_tweakFieldSpan(PT_DocPosition & dpos1,
 								->getObjectType()==PTO_Field))*/)
     {
         fd_Field * pField = pf_End->getField();
-        UT_ASSERT(pField);
+        UT_return_if_fail (pField);
         pf_Other = pf_End->getNext();
-        UT_ASSERT(pf_Other);
+        UT_return_if_fail (pf_Other);
         while (pf_Other->getField()==pField)
         {
             pf_Other = pf_Other->getNext();
-            UT_ASSERT(pf_Other);
+            UT_return_if_fail (pf_Other);
         }
         dpos2 = getFragPosition(pf_Other);
     }
