@@ -59,6 +59,14 @@ bool pt_PieceTable::_unlinkStrux(pf_Frag_Strux * pfs,
 	{
 		UT_DEBUGMSG(("_unlink Strux EndCell %x \n",pfs));
 	}
+	else if(pfs->getStruxType() == PTX_SectionFrame)
+	{
+		UT_DEBUGMSG(("_unlink Strux SectionFrame %x \n",pfs));
+	}
+	else if(pfs->getStruxType() == PTX_EndFrame)
+	{
+		UT_DEBUGMSG(("_unlink Strux EndFrame %x \n",pfs));
+	}
 	else if(pfs->getStruxType() == PTX_Block)
 	{
 		UT_DEBUGMSG(("_unlink Strux Block %x \n",pfs));
@@ -87,7 +95,15 @@ bool pt_PieceTable::_unlinkStrux(pf_Frag_Strux * pfs,
 	{
 		UT_DEBUGMSG(("_unlink Strux EndEndnote %x \n",pfs));
 	}
-	m_pDocument->miniDump((PL_StruxDocHandle) pfs, 2);
+	else if(pfs->getStruxType() == PTX_SectionTOC)
+	{
+		UT_DEBUGMSG(("_unlink Strux SectionTOC %x \n",pfs));
+	}
+	else if(pfs->getStruxType() == PTX_EndTOC)
+	{
+		UT_DEBUGMSG(("_unlink Strux EndTOC %x \n",pfs));
+	}
+//	m_pDocument->miniDump((PL_StruxDocHandle) pfs, 2);
 #endif
 	switch (pfs->getStruxType())
 	{
@@ -95,19 +111,23 @@ bool pt_PieceTable::_unlinkStrux(pf_Frag_Strux * pfs,
 	case PTX_SectionHdrFtr:
 	case PTX_SectionEndnote:
 	case PTX_SectionTable:
+	case PTX_SectionFrame:
 	case PTX_SectionCell:
 	case PTX_SectionFootnote:
+	case PTX_SectionTOC:
 	case PTX_EndCell:
 	case PTX_EndTable:
 	case PTX_EndFootnote:
 	case PTX_EndEndnote:
+	case PTX_EndFrame:
+	case PTX_EndTOC:
 		return _unlinkStrux_Section(pfs,ppfEnd,pfragOffsetEnd);
 
 	case PTX_Block:
 		return _unlinkStrux_Block(pfs,ppfEnd,pfragOffsetEnd);
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		return false;
 	}
 }
@@ -115,7 +135,7 @@ bool pt_PieceTable::_unlinkStrux(pf_Frag_Strux * pfs,
 bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 										  pf_Frag ** ppfEnd, UT_uint32 * pfragOffsetEnd)
 {
-	UT_ASSERT(pfs->getStruxType()==PTX_Block);
+	UT_return_val_if_fail (pfs->getStruxType()==PTX_Block,false);
 
 	// unlink this Block strux from the document.
 	// the caller is responsible for deleting pfs.
@@ -129,7 +149,7 @@ bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 
 	pf_Frag_Strux * pfsPrev = NULL;
 	_getStruxFromPosition(pfs->getPos(),&pfsPrev, true); // should that really skip footnotes?
-	UT_ASSERT(pfsPrev);			// we have a block that's not in a section ??
+	UT_return_val_if_fail (pfsPrev, false);			// we have a block that's not in a section ??
 	//
 	// Code to prevent a crash. But this should not happen and if it does not everything will
     // be deleted - Sevior.
@@ -151,7 +171,6 @@ bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
 		return true;
 
-	case PTX_SectionEndnote:
 	case PTX_Section:
 		// we are the first paragraph in this section.  if we have
 		// content, we cannot be deleted, since there is no one to
@@ -161,7 +180,7 @@ bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 		{
 			// TODO decide if this should assert or just fail...
 			UT_DEBUGMSG(("Cannot delete first paragraph with content.\n"));
-			UT_ASSERT(0);
+			UT_ASSERT_HARMLESS(0);
 			return false;
 		}
 
@@ -175,7 +194,7 @@ bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 		{
 			// TODO decide if this should assert or just fail...
 			UT_DEBUGMSG(("Cannot delete first paragraph with content.\n"));
-			UT_ASSERT(0);
+			UT_ASSERT_HARMLESS(0);
 			return false;
 		}
 
@@ -184,7 +203,8 @@ bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
 		return true;
 
-
+	case PTX_SectionFrame:
+    case PTX_EndFrame:
 	case PTX_SectionTable:
 	case PTX_SectionCell:
 	case PTX_EndCell:
@@ -198,7 +218,7 @@ bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 
 
 	default:
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 		return false;
 	}
 }
@@ -206,17 +226,21 @@ bool pt_PieceTable::_unlinkStrux_Block(pf_Frag_Strux * pfs,
 bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 											pf_Frag ** ppfEnd, UT_uint32 * pfragOffsetEnd)
 {
-	UT_ASSERT(pfs->getStruxType()==PTX_Section
+	UT_return_val_if_fail (pfs->getStruxType()==PTX_Section
 			  || pfs->getStruxType()==PTX_SectionHdrFtr
 			  || pfs->getStruxType()==PTX_SectionEndnote
 			  || pfs->getStruxType()==PTX_SectionTable
+			  || pfs->getStruxType()==PTX_SectionFrame
 			  || pfs->getStruxType()==PTX_SectionCell
 			  || pfs->getStruxType()==PTX_EndCell
 			  || pfs->getStruxType()==PTX_EndTable 
+			  || pfs->getStruxType()==PTX_EndFrame 
 			  || pfs->getStruxType()==PTX_SectionFootnote 
 			  || pfs->getStruxType()==PTX_EndFootnote 
 			  || pfs->getStruxType()==PTX_SectionEndnote 
-			  || pfs->getStruxType()==PTX_EndEndnote );
+			  || pfs->getStruxType()==PTX_EndEndnote
+			  || pfs->getStruxType()==PTX_SectionTOC 
+			  || pfs->getStruxType()==PTX_EndTOC, false );
 
 	// unlink this Section strux from the document.
 	// the caller is responsible for deleting pfs.
@@ -230,7 +254,7 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 
 	pf_Frag_Strux * pfsPrev = NULL;
 	pf_Frag * pf = pfs->getPrev();
-	while (pf && !pfsPrev || isFootnote(pf) || isEndFootnote(pf))
+	while (pf && (!pfsPrev || isFootnote(pf) || isEndFootnote(pf)))
 	{
 		if (pf->getType() == pf_Frag::PFT_Strux)
 			pfsPrev = static_cast<pf_Frag_Strux *> (pf);
@@ -242,7 +266,7 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 		// first section in the document cannot be deleted.
 		// TODO decide if this should assesrt or just file...
 		UT_DEBUGMSG(("Cannot delete first section in document.\n"));
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 		return false;
 	}
 
@@ -259,6 +283,18 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 	case PTX_SectionTable:
         //
         // deleting tables is a multi-step process that can't make assumptions
+        // on a single step
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+	case PTX_SectionFrame:
+        //
+        // deleting Frames is a multi-step process that can't make assumptions
+        // on a single step
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+	case PTX_EndFrame:
+        //
+        // deleting Frames is a multi-step process that can't make assumptions
         // on a single step
 		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
 		return true;
@@ -301,6 +337,23 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 		return true;
 
 
+	case PTX_SectionTOC:
+        //
+        // deleting TOC is a multi-step process that can't make 
+        // assumptions
+        // on a single step
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+
+	case PTX_EndTOC:
+        //
+        // deleting TOC is a multi-step process that can't make 
+        // assumptions
+        // on a single step
+		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
+		return true;
+
+
 	case PTX_SectionFootnote:
         //
         // deleting Footnotes is a multi-step process that can't make 
@@ -323,7 +376,7 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
 		// and the previous section.  this is not possible.
 		// TODO decide if this should assert or just fail...
 		UT_DEBUGMSG(("No blocks between sections ??\n"));
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 		return false;
 
 
@@ -337,11 +390,11 @@ bool pt_PieceTable::_unlinkStrux_Section(pf_Frag_Strux * pfs,
         // but no even pages exist yet.
 		UT_DEBUGMSG(("No blocks between sections ??\n"));
 //		_unlinkFrag(pfs,ppfEnd,pfragOffsetEnd);
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 		return false;
 
 	default:
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 		return false;
 	}
 }
@@ -355,7 +408,7 @@ bool pt_PieceTable::_deleteStruxWithNotify(PT_DocPosition dpos,
 	PX_ChangeRecord_Strux * pcrs
 		= new PX_ChangeRecord_Strux(PX_ChangeRecord::PXT_DeleteStrux,
 									dpos, pfs->getIndexAP(), pfs->getStruxType());
-	UT_ASSERT(pcrs);
+	UT_return_val_if_fail (pcrs, false);
 
 	if (!_unlinkStrux(pfs,ppfEnd,pfragOffsetEnd))
 		return false;
@@ -562,44 +615,45 @@ void pt_PieceTable::_deleteHdrFtrStruxWithNotify( pf_Frag_Strux * pfFragStruxHdr
 // Now find the end of the text in the header/footer
 //
 	bool foundEnd = false;
-	if(!bStop)
+	while(!foundEnd)
 	{
-		while(!foundEnd)
+		foundEnd = pfFrag == getFragments().getLast();
+		if(!foundEnd && pfFrag->getType() == pf_Frag::PFT_Strux)
 		{
-			foundEnd = pfFrag == getFragments().getLast();
-			if(!foundEnd && pfFrag->getType() == pf_Frag::PFT_Strux)
-			{
-				const pf_Frag_Strux * pfFragStrux = static_cast<const pf_Frag_Strux *>(pfFrag);
-				foundEnd = pfFragStrux->getStruxType() != PTX_Block;
-			}
-			if(!foundEnd)
-			{
-				pfFrag = pfFrag->getNext();
-			}
+			const pf_Frag_Strux * pfFragStrux = static_cast<const pf_Frag_Strux *>(pfFrag);
+			foundEnd = ((pfFragStrux->getStruxType() != PTX_Block) &&
+						(pfFragStrux->getStruxType() != PTX_SectionTable) &&
+						(pfFragStrux->getStruxType() != PTX_SectionCell) &&
+						(pfFragStrux->getStruxType() != PTX_EndTable) &&
+						(pfFragStrux->getStruxType() != PTX_EndCell));
 		}
-		PT_DocPosition TextEndPos = 0;
-		TextEndPos = getFragPosition(pfFrag);
-		if(pfFrag == getFragments().getLast())
+		if(!foundEnd)
 		{
-			TextEndPos = getFragPosition(pfFrag->getPrev()) + pfFrag->getPrev()->getLength();
+			pfFrag = pfFrag->getNext();
 		}
-		UT_DEBUGMSG(("SEVIOR: Deleting hdrftr Text End Pos = %d \n",TextEndPos));
+	}
+	PT_DocPosition TextEndPos = 0;
+	TextEndPos = getFragPosition(pfFrag);
+	if(pfFrag == getFragments().getLast())
+	{
+		TextEndPos = getFragPosition(pfFrag->getPrev()) + pfFrag->getPrev()->getLength();
+	}
+	UT_DEBUGMSG(("SEVIOR: Deleting hdrftr Text End Pos = %d \n",TextEndPos));
 //
 // OK delete the text
 //
-		if(TextEndPos > TextStartPos)
-		{
-			UT_uint32 iRealDeleteCount;
-			deleteSpan(TextStartPos,TextEndPos,NULL,iRealDeleteCount,true);
-			// TODO -- is this right with revisions ???
-		}
+	if(TextEndPos > TextStartPos)
+	{
+		UT_uint32 iRealDeleteCount;
+		deleteSpan(TextStartPos,TextEndPos,NULL,iRealDeleteCount,true);
+		// TODO -- is this right with revisions ???
 	}
 //
 // Now delete the struxes at the start.
 //
 //
 	UT_uint32 count = vecFragStrux.getItemCount();
-	UT_ASSERT(count > 1);
+	UT_return_if_fail (count > 1);
 	UT_uint32 i=0;
 	bool bres = false;
 //
@@ -607,6 +661,7 @@ void pt_PieceTable::_deleteHdrFtrStruxWithNotify( pf_Frag_Strux * pfFragStruxHdr
 // the HdrFtr to be properly recreated on undo (Since it needs blocks to be
 // present before it can be created.)
 //
+	m_fragments.cleanFrags();
 	bres = _deleteStruxWithNotify(pfFragStruxHdrFtr->getPos(),pfFragStruxHdrFtr,NULL,NULL);
 	m_fragments.cleanFrags();
 	for(i=1; i<count; i++)
@@ -615,16 +670,16 @@ void pt_PieceTable::_deleteHdrFtrStruxWithNotify( pf_Frag_Strux * pfFragStruxHdr
 		if(static_cast<pf_Frag *>(pfs) ==  getFragments().getLast())
 		{
 			UT_DEBUGMSG(("Delete Last Strux type %d \n",pfs->getStruxType()));
-			UT_ASSERT(0);
+			UT_ASSERT_HARMLESS(0);
 		}
 		UT_DEBUGMSG(("Delete Strux at %d strux type is %d \n",pfs->getPos(),pfs->getStruxType()));
 		if(pfs->getStruxType() != PTX_SectionHdrFtr)
 		{
 			bres = _deleteStruxWithNotify(pfs->getPos(),pfs,NULL,NULL);
 		}
-		UT_ASSERT(bres);
+		UT_return_if_fail (bres);
 	}
-	UT_ASSERT(bres);
+	UT_return_if_fail (bres);
 //	deleteSpan(HdrFtrPos,TextStartPos,NULL,true);
 }
 
