@@ -49,21 +49,28 @@ bool pt_PieceTable::insertSpan(PT_DocPosition dpos,
 							   UT_uint32 length, fd_Field * pField,
 							   bool bAddChangeRec)
 {
-	if(!_realInsertSpan(dpos, p, length, pField, bAddChangeRec))
-		return false;
-
 	if(bAddChangeRec && m_pDocument->isMarkRevisions())
 	{
 		PP_RevisionAttr Revisions(NULL);
-		Revisions.addRevision(m_pDocument->getRevisionId(),PP_REVISION_ADDITION,NULL,NULL);
+		const XML_Char ** ppRevAttrib = NULL;
+		const XML_Char ** ppRevProps  = NULL;
 
-		const XML_Char name[] = "revision";
-		const XML_Char * ppRevAttrib[3];
-		ppRevAttrib[0] = name;
-		ppRevAttrib[1] = Revisions.getXMLstring();
-		ppRevAttrib[2] = NULL;
+		pf_Frag * pf = NULL;
+		PT_BlockOffset fragOffset = 0;
+		bool bFound = getFragFromPosition(dpos,&pf,&fragOffset);
+		UT_return_val_if_fail( bFound, false );
 
-		return _realChangeSpanFmt(PTC_AddFmt, dpos, dpos + length, ppRevAttrib,NULL);
+		if(pf->getType() == pf_Frag::PFT_EndOfDoc)
+			pf = pf->getPrev();
+
+		UT_return_val_if_fail( pf, false );
+		
+		PT_AttrPropIndex indexAP = pf->getIndexAP();
+
+		_translateRevisionAttribute(Revisions, indexAP, PP_REVISION_ADDITION, ppRevAttrib, ppRevProps, 0, 0);
+		
+		//return _realChangeSpanFmt(PTC_AddFmt, dpos, dpos + length, ppRevAttrib, ppRevProps);
+		return _realInsertSpan(dpos, p, length, ppRevAttrib, ppRevProps, pField, bAddChangeRec);
 	}
 	else if(bAddChangeRec)
 	{
@@ -94,21 +101,24 @@ bool pt_PieceTable::insertSpan(PT_DocPosition dpos,
 			{
 				// if we have no revision attribute, then everything
 				// is OK
-				return true;
+				return _realInsertSpan(dpos, p, length, NULL, NULL, pField, bAddChangeRec);
 			}
 
-			if(!_realChangeSpanFmt(PTC_RemoveFmt, dpos, dpos+length, ppRevAttrib,NULL))
-				return false;
+			//if(!_realChangeSpanFmt(PTC_RemoveFmt, dpos, dpos+length, ppRevAttrib,NULL))
+			//	return false;
+			return _realInsertSpan(dpos, p, length, ppRevAttrib, NULL, pField, bAddChangeRec);
 		}
 		else
 		{
 			// no AP, this is probably OK
 			UT_DEBUGMSG(("pt_PieceTable::insertSpan: no AP\n"));
-			return true;
+			return _realInsertSpan(dpos, p, length, NULL, NULL, pField, bAddChangeRec);
 		}
 	}
-	
-	return true;
+	else
+	{
+		return _realInsertSpan(dpos, p, length, NULL, NULL, pField, bAddChangeRec);
+	}
 }
 
 
@@ -127,7 +137,7 @@ bool pt_PieceTable::_insertSpan(pf_Frag * pf,
 	switch (pf->getType())
 	{
 	default:
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 		return false;
 
 	case pf_Frag::PFT_EndOfDoc:
@@ -161,7 +171,7 @@ bool pt_PieceTable::_insertSpan(pf_Frag * pf,
 		// we need to replace the FmtMark with a Text frag with
 		// the same API.  This needs to be handled at the higher
 		// level (so the glob markers can be set).
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		return false;
 	}
 
@@ -291,7 +301,7 @@ bool pt_PieceTable::_insertSpan(pf_Frag * pf,
 	// if the change is in the middle of the fragment, we construct
 	// a second new text fragment for the portion after the insert.
 
-	UT_ASSERT(pft);
+	UT_return_val_if_fail (pft,false);
 
 	UT_uint32 lenTail = pft->getLength() - fragOffset;
 	PT_BufIndex biTail = m_varset.getBufIndex(pft->getBufIndex(),fragOffset);
@@ -337,25 +347,28 @@ bool pt_PieceTable::_lastUndoIsThisFmtMark(PT_DocPosition dpos)
 		}
 	}
 
-	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 	return false;
 }
 
 bool pt_PieceTable::_realInsertSpan(PT_DocPosition dpos,
 									const UT_UCSChar * p,
-									UT_uint32 length, fd_Field * pField,
+									UT_uint32 length,
+									const XML_Char ** attributes,
+									const XML_Char ** properties,
+									fd_Field * pField,
 									bool bAddChangeRec)
 {
 	// insert character data into the document at the given position.
 
-	UT_ASSERT(m_pts==PTS_Editing);
+	UT_return_val_if_fail (m_pts==PTS_Editing, false);
 
 	// get the fragment at the given document position.
 
 	pf_Frag * pf = NULL;
 	PT_BlockOffset fragOffset = 0;
 	bool bFound = getFragFromPosition(dpos,&pf,&fragOffset);
-	UT_ASSERT(bFound);
+	UT_return_val_if_fail (bFound,false);
 
 
 	// append the text data to the end of the current buffer.
@@ -367,12 +380,16 @@ bool pt_PieceTable::_realInsertSpan(PT_DocPosition dpos,
 	bool bSuccess = false;
 	pf_Frag_Strux * pfs = NULL;
 	bool bFoundStrux = _getStruxFromFrag(pf,&pfs);
-	UT_ASSERT(bFoundStrux);
+	UT_return_val_if_fail (bFoundStrux,false);
 	if(isEndFootnote((pf_Frag *)pfs))
 	{
 		bFoundStrux = _getStruxFromFragSkip((pf_Frag *) pfs,&pfs);
 	}
-	UT_ASSERT(pfs);
+	UT_return_val_if_fail (pfs,false);
+	if(pfs->getStruxType() == PTX_EndFrame)
+	{
+		bFoundStrux = _getStruxFromFragSkip((pf_Frag *) pfs,&pfs);
+	}
 	// we just did a getFragFromPosition() which gives us the
 	// the thing *starting* at that position.  if we have a
 	// fragment boundary at that position, it's sort of arbitrary
@@ -450,15 +467,15 @@ bool pt_PieceTable::_realInsertSpan(PT_DocPosition dpos,
 			// but with a few shortcuts.
 
 			bFound = getFragFromPosition(dpos,&pf,&fragOffset);
-			UT_ASSERT(bFound);
+			UT_return_val_if_fail (bFound, false);
 
 			bFoundStrux = _getStruxFromFrag(pf,&pfs);
-			UT_ASSERT(bFoundStrux);
+			UT_return_val_if_fail (bFoundStrux,false);
 			if(isEndFootnote((pf_Frag *)pfs))
 			{
 				bFoundStrux = _getStruxFromFragSkip((pf_Frag *)pfs,&pfs);
 			}
-			UT_ASSERT(bFoundStrux);
+			UT_return_val_if_fail (bFoundStrux, false);
 			xxx_UT_DEBUGMSG(("Got FragStrux at Pos %d \n",pfs->getPos()));
 
 			// with the FmtMark now gone, we make a minor adjustment so that we
@@ -498,7 +515,6 @@ bool pt_PieceTable::_realInsertSpan(PT_DocPosition dpos,
 
 		indexAP = _chooseIndexAP(pf,fragOffset);
 	}
-
 	PT_BlockOffset blockOffset = _computeBlockOffset(pfs,pf) + fragOffset;
 	PX_ChangeRecord_Span * pcr = NULL;
 
@@ -528,6 +544,18 @@ bool pt_PieceTable::_realInsertSpan(PT_DocPosition dpos,
 			return false;
 	}
 
+	if(attributes || properties)
+	{
+		// we need to add the attrs and props passed to us ...
+		PT_AttrPropIndex indexNewAP;
+		bool bMerged;
+		bMerged = m_varset.mergeAP(PTC_AddFmt,indexAP,attributes,properties,&indexNewAP,getDocument());
+		UT_ASSERT_HARMLESS( bMerged );
+
+		if(bMerged)
+			indexAP = indexNewAP;
+	}
+	
 	if (!_insertSpan(pf,bi,fragOffset,length,indexAP,pField))
 		goto Finish;
 
@@ -539,7 +567,7 @@ bool pt_PieceTable::_realInsertSpan(PT_DocPosition dpos,
 	pcr = new PX_ChangeRecord_Span(PX_ChangeRecord::PXT_InsertSpan,
 								   dpos,indexAP,bi,length,
                                    blockOffset, pField);
-	UT_ASSERT(pcr);
+	UT_return_val_if_fail (pcr, false);
 
 	if (!bAddChangeRec || _canCoalesceInsertSpan(pcr))
 	{
@@ -568,7 +596,7 @@ bool pt_PieceTable::_canCoalesceInsertSpan(PX_ChangeRecord_Span * pcrSpan) const
 {
 	// see if this record can be coalesced with the most recent undo record.
 
-	UT_ASSERT(pcrSpan->getType() == PX_ChangeRecord::PXT_InsertSpan);
+	UT_return_val_if_fail (pcrSpan->getType() == PX_ChangeRecord::PXT_InsertSpan, false);
 
 	PX_ChangeRecord * pcrUndo;
 	if (!m_history.getUndo(&pcrUndo))
@@ -677,7 +705,7 @@ PT_AttrPropIndex pt_PieceTable::_chooseIndexAP(pf_Frag * pf, PT_BlockOffset frag
 				return 0;
 
 			default:
-				UT_ASSERT(0);
+				UT_ASSERT_HARMLESS(0);
 				return 0;
 			}
 		}
@@ -691,7 +719,7 @@ PT_AttrPropIndex pt_PieceTable::_chooseIndexAP(pf_Frag * pf, PT_BlockOffset frag
 		}
 
 	default:
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 		return 0;
 	}
 }
