@@ -17,8 +17,7 @@
  * 02111-1307, USA.
  */
 
-#include <locale.h>
-
+#include "ut_locale.h"
 #include "ut_types.h"
 #include "ut_misc.h"
 #include "ut_string.h"
@@ -72,6 +71,12 @@ ST_LOCALISED_STYLES stLocalised[] =
 	{"Heading 1",           	XAP_STRING_ID_STYLE_HEADING1},		
 	{"Heading 2",           	XAP_STRING_ID_STYLE_HEADING2},		
 	{"Heading 3",           	XAP_STRING_ID_STYLE_HEADING3},		
+	{"Heading 4",           	XAP_STRING_ID_STYLE_HEADING4},		
+	{"Contents Header",         XAP_STRING_ID_STYLE_TOCHEADING},		
+	{"Contents 1",              XAP_STRING_ID_STYLE_TOCHEADING1},		
+	{"Contents 2",              XAP_STRING_ID_STYLE_TOCHEADING2},		
+	{"Contents 3",              XAP_STRING_ID_STYLE_TOCHEADING3},		
+	{"Contents 4",              XAP_STRING_ID_STYLE_TOCHEADING4},		
 	{"Normal",         	        XAP_STRING_ID_STYLE_NORMAL},		              
 	{"Block Text",              XAP_STRING_ID_STYLE_BLOCKTEXT},	
 	{"Lower Case List",         XAP_STRING_ID_STYLE_LOWERCASELIST},
@@ -105,28 +110,23 @@ ST_LOCALISED_STYLES stLocalised[] =
 /*
 	Gets a style name and returns its localised name
 */
-const char* pt_PieceTable::s_getLocalisedStyleName(const char *szStyle)
+void pt_PieceTable::s_getLocalisedStyleName(const char *szStyle, UT_UTF8String &utf8)
 {		
   static XAP_App * pApp = XAP_App::getApp();
 
 	const XAP_StringSet * pSS = pApp->getStringSet();
-	const char*	pRslt = szStyle;
-	int n;	
+	utf8 = szStyle;
+	int n;		
 		
 	for (n=0; stLocalised[n].pStyle; n++)
 	{
 		if (strcmp(szStyle, stLocalised[n].pStyle)==0)
 		{
-			XML_Char * szName = (XML_Char *) pSS->getValue(stLocalised[n].nID);
-
-			if (szName)
-				pRslt = szName;
-			
+			pSS->getValueUTF8(stLocalised[n].nID, utf8);			
 			break;
 		}
 	}		
 	
-	return pRslt;
 }
 
 bool pt_PieceTable::_loadBuiltinStyles(void)
@@ -135,7 +135,8 @@ bool pt_PieceTable::_loadBuiltinStyles(void)
 		!!! if adding or removing properties to the list_fmt, you have to make also changes to
 		pt_VarSet.cpp mergeAP()
 	*/
-	char* old_locale = setlocale (LC_NUMERIC, "C");
+	UT_LocaleTransactor t(LC_NUMERIC, "C");
+
 	char* list_fmt = " list-style:%s; start-value:%s; margin-left:%fin; text-indent:-%fin; "
 		"field-color:%s;list-delim:%s; field-font:%s; list-decimal:%s";
 	UT_String list_fmt_tmp;
@@ -169,6 +170,7 @@ bool pt_PieceTable::_loadBuiltinStyles(void)
 	_s("Heading 2",	"P", "Normal", "Normal", stTmp.c_str());
 	UT_String_sprintf(stTmp, szFmt, pszFamily, 12);
 	_s("Heading 3",	"P", "Normal", "Normal", stTmp.c_str());
+	_s("Heading 4",	"P", "Normal", "Normal", stTmp.c_str());
 	_s("Plain Text","P", "Normal", "Current Settings", "font-family:Courier New");
 	_s("Block Text","P", "Normal", "Current Settings", "margin-left:1in; margin-right:1in; margin-bottom:6pt");
 
@@ -225,10 +227,30 @@ bool pt_PieceTable::_loadBuiltinStyles(void)
 					  "field-color:transparent; list-delim:%%L.; field-font:%s; "
 					  "list-decimal:", pszFamily);
 
+
     _s("Numbered Heading 1","P","Heading 1","Normal", stTmp.c_str());
     _s("Numbered Heading 2","P","Heading 2","Normal", stTmp.c_str());
     _s("Numbered Heading 3","P","Heading 3","Normal", stTmp.c_str());
 
+	// pszFamily is the nearest font to Arial found in the system
+
+	UT_String_sprintf(stTmp, list_fmt, "Numbered List", "1",LIST_DEFAULT_INDENT, LIST_DEFAULT_INDENT_LABEL, "transparent", "%L.", "NULL", ".");
+
+    _s("Contents 1","P","Normal","Normal", stTmp.c_str());
+
+	UT_String_sprintf(stTmp, list_fmt, "Numbered List", "1",2*LIST_DEFAULT_INDENT, LIST_DEFAULT_INDENT_LABEL, "transparent", "%L.", "NULL", ".");
+    _s("Contents 2","P","Normal","Normal", stTmp.c_str());
+	UT_String_sprintf(stTmp, list_fmt, "Numbered List", "1",3*LIST_DEFAULT_INDENT, LIST_DEFAULT_INDENT_LABEL, "transparent", "%L.", "NULL", ".");
+    _s("Contents 3","P","Normal","Normal", stTmp.c_str());
+	UT_String_sprintf(stTmp, list_fmt, "Numbered List", "1",4*LIST_DEFAULT_INDENT, LIST_DEFAULT_INDENT_LABEL, "transparent", "%L.", "NULL", ".");
+    _s("Contents 4","P","Normal","Normal", stTmp.c_str());
+
+	
+	szFmt = "font-family:%s; font-size:%dpt; font-weight:bold; margin-top:12pt; margin-bottom:6pt; text-align:center; keep-with-next:1";
+	UT_String_sprintf(stTmp, szFmt, pszFamily, 16);
+    _s("Contents Header","P","Normal","Normal", stTmp.c_str());
+
+
 	szFmt = "tabstops:1.1in/L0; list-style:Numbered List; "
 		"start-value:1; margin-left:0.0in; text-indent:0.0in; "
 		"field-color:transparent; list-delim:%s %%L.; "
@@ -246,18 +268,16 @@ bool pt_PieceTable::_loadBuiltinStyles(void)
 	_s("Footnote Reference","C", "None", "Current Settings", "text-position:superscript; font-size:10pt");
 	_s("Footnote Text","C", "Normal", "Current Settings", "text-position:normal; font-size:10pt");
 
-	setlocale (LC_NUMERIC, old_locale);
 	return true;
 
 Failed:
-	setlocale (LC_NUMERIC, old_locale);
 	return false;
 }
 
 bool pt_PieceTable::_createBuiltinStyle(const char * szName, const XML_Char ** attributes)
 {
 	// this function can only be called before loading the document.
-	UT_ASSERT(m_pts==PTS_Create);
+	UT_return_val_if_fail (m_pts==PTS_Create, false);
 
 	PT_AttrPropIndex indexAP;
 	if (!m_varset.storeAP(attributes,&indexAP))
@@ -270,7 +290,7 @@ bool pt_PieceTable::_createBuiltinStyle(const char * szName, const XML_Char ** a
 
 	pStyle = new PD_BuiltinStyle(this, indexAP, szName);
 	if (pStyle)
-		m_hashStyles.insert(szName, (void*) pStyle);
+		m_hashStyles.insert(szName, pStyle);
 
 	return true;
 }
@@ -289,19 +309,19 @@ bool pt_PieceTable::appendStyle(const XML_Char ** attributes)
 
 	// verify unique name
 
-	UT_ASSERT(sizeof(char) == sizeof(XML_Char));
+	UT_ASSERT_HARMLESS(sizeof(char) == sizeof(XML_Char));
 	const char * szName = UT_getAttribute(PT_NAME_ATTRIBUTE_NAME, attributes);
 	if (!szName || !*szName)
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		return true;		// silently ignore unnamed styles
 	}
-	xxx_UT_DEBUGMSG(("SEVIOR: Appending style %s \n",szName));
+	UT_DEBUGMSG(("STYLE: Appending style %s \n",szName));
 	PD_Style * pStyle = NULL;
 	if (getStyle(szName,&pStyle) == true)
 	{
 		// duplicate name
-		UT_ASSERT(pStyle);
+		UT_return_val_if_fail (pStyle, false);
 		if (pStyle->isUserDefined())
 		{
 			// already loaded, ignore redefinition
@@ -316,31 +336,23 @@ bool pt_PieceTable::appendStyle(const XML_Char ** attributes)
 	{
 		// this is a new name
 		pStyle = new PD_Style(this, indexAP, szName);
-		xxx_UT_DEBUGMSG(("SEVIOR: Creating new style %s \n",szName));
+		UT_DEBUGMSG(("STYLE: Creating new style %s saving in hash \n",szName));
+
 //
-//TODO: Learn how to use Dom's AbiObject instead of this hack.
+// TODO: Learn how to use Dom's AbiObject instead of this hack.
 //
+
 		if (pStyle)
-			m_hashStyles.insert(szName,(void *)pStyle);
-//
-// Diagonostic on Append...
-//
-		const PD_Style * pdStyle = NULL;
-		const char * psdName =NULL;
-		UT_uint32 i = 0;
-		for(i=0; i<getStyleCount(); i++)
-		{
-			enumStyles(i,&psdName,&pdStyle);
-			xxx_UT_DEBUGMSG(("SEVIOR: Found %d style name %s \n",i,psdName));
-		}
+			m_hashStyles.insert(szName,pStyle);
+
 		return true;
 	}
 }
 
 bool pt_PieceTable::removeStyle (const XML_Char * szName)
 {
-	UT_ASSERT(szName);
-	UT_ASSERT(sizeof(char) == sizeof(XML_Char));
+	UT_return_val_if_fail (szName, false);
+	UT_ASSERT_HARMLESS(sizeof(char) == sizeof(XML_Char));
 
 	UT_DEBUGMSG(("DOM: remove the style, maybe recode the hash-table\n"));
 
@@ -364,13 +376,10 @@ bool pt_PieceTable::getStyle(const char * szName, PD_Style ** ppStyle) const
 {
 	//UT_ASSERT(szName && *szName);
 
-	const void * pHashEntry = m_hashStyles.pick (szName);
-	if (!pHashEntry)
+	PD_Style * pStyle = m_hashStyles.pick (szName);
+	if (!pStyle)
 		return false;
 
-	PD_Style * pStyle = (PD_Style *) pHashEntry;
-	//UT_ASSERT(pStyle);
-
 	if (ppStyle)
 	{
 		*ppStyle = pStyle;
@@ -412,11 +421,11 @@ bool pt_PieceTable::enumStyles(UT_uint32 k,
 	if (k >= kLimit)
 		return false;
 
-	UT_Vector * vStyle = m_hashStyles.enumerate() ;
-	vStyle->qsort(compareStyleNames);
+	UT_GenericVector<PD_Style*> * vStyle = m_hashStyles.enumerate() ;
+	//vStyle->qsort(compareStyleNames);
 
-	PD_Style * pStyle = (PD_Style *) vStyle->getNthItem(k);
-	UT_ASSERT(pStyle);
+	PD_Style * pStyle = vStyle->getNthItem(k);
+	UT_return_val_if_fail (pStyle,false);
 
 	if (ppStyle)
 	{
@@ -427,7 +436,7 @@ bool pt_PieceTable::enumStyles(UT_uint32 k,
 	{
 	  *pszName = pStyle->getName();
 	}
-	UT_ASSERT(*pszName);
+	UT_ASSERT_HARMLESS(*pszName);
 
 	delete vStyle;
 
