@@ -32,6 +32,8 @@
 #include "px_ChangeHistory.h"
 #include "pf_Fragments.h"
 #include "pt_VarSet.h"
+#include "pp_Revision.h"
+
 class pf_Frag_Object;
 class pf_Frag_FmtMark;
 class pf_Frag_Text;
@@ -70,25 +72,29 @@ public:
 	UT_uint32                               undoCount(bool bUndo) const;
 	bool					undoCmd(void);
 	bool					redoCmd(void);
+
+	void                    clearUndo() {m_history.clearHistory();}
 	
-	static const char* 		s_getLocalisedStyleName(const char *szStyle);
+	static void		s_getLocalisedStyleName(const char *szStyle, UT_UTF8String &utf8);
 
 protected:
 	bool					_realInsertObject(PT_DocPosition dpos,
-										 PTObjectType pto,
-										 const XML_Char ** attributes,
-										 const XML_Char ** properties);
+											  PTObjectType pto,
+											  const XML_Char ** attributes,
+											  const XML_Char ** properties);
 
 	bool					_realInsertObject(PT_DocPosition dpos,
-										 PTObjectType pto,
-										 const XML_Char ** attributes,
-										 const XML_Char ** properties, pf_Frag_Object ** ppfo );
+											  PTObjectType pto,
+											  const XML_Char ** attributes,
+											  const XML_Char ** properties, pf_Frag_Object ** ppfo );
 
 	bool					_realInsertSpan(PT_DocPosition dpos,
-									   const UT_UCSChar * p,
-									   UT_uint32 length,
-									   fd_Field * pField = NULL,
-									   bool bAddChangeRec = true);
+											const UT_UCSChar * p,
+											UT_uint32 length,
+											const XML_Char ** attributes,
+											const XML_Char ** properties,
+											fd_Field * pField = NULL,
+											bool bAddChangeRec = true);
 
 	bool					_realDeleteSpan(PT_DocPosition dpos1,
 											PT_DocPosition dpos2,
@@ -196,13 +202,28 @@ public:
 										   PT_DocPosition dpos2,
 										   const XML_Char ** attributes,
 										   const XML_Char ** properties,
-										   PTStruxType pts);
+										   PTStruxType pts=PTX_StruxDummy);
+
+
+	bool					changeStruxFmtNoUndo(PTChangeFmt ptc,
+										   pf_Frag_Strux * pfs,
+										   const XML_Char ** attributes,
+										   const XML_Char ** properties);
+
+	bool                    changeStruxFormatNoUpdate(PTChangeFmt ptc, pf_Frag_Strux * pfs,const XML_Char ** attributes);
 
 	bool                    changeStruxForLists(PL_StruxDocHandle sdh,
 												const char * pszParentID);
     bool                    changeSectionAttsNoUpdate(pf_Frag_Strux * pfStrux, const char * attr, const char * attvalue);
 	bool                    deleteStruxNoUpdate(PL_StruxDocHandle sdh);
 	bool                    insertStruxNoUpdateBefore(PL_StruxDocHandle sdh, PTStruxType pts,const XML_Char ** attributes );
+	bool                    changeLastStruxFmtNoUndo(PT_DocPosition dpos, PTStruxType pts,
+													 const XML_Char ** attrs, const XML_Char ** props,
+													 bool bSkipEmbededSections);
+	
+	bool                    changeLastStruxFmtNoUndo(PT_DocPosition dpos, PTStruxType pts,
+													 const XML_Char ** attrs, const XML_Char * props,
+													 bool bSkipEmbededSections);
 
 
 	/**********************	END OF REVISIONS AWARE METHODS ******/
@@ -218,8 +239,13 @@ public:
 	PD_Document *			getDocument(void);
 	bool					appendStrux(PTStruxType pts, const XML_Char ** attributes, pf_Frag_Strux ** ppfs_ret = 0);
 	bool					appendStruxFmt(pf_Frag_Strux * pfs, const XML_Char ** attributes);
+	bool                    appendLastStruxFmt(PTStruxType pts, const XML_Char ** attrs, const XML_Char ** props,
+											   bool bSkipEmbededSections);
+	bool                    appendLastStruxFmt(PTStruxType pts, const XML_Char ** attrs, const XML_Char * props,
+											   bool bSkipEmbededSections);
+	
 	bool					appendFmt(const XML_Char ** attributes);
-	bool					appendFmt(const UT_Vector * pVecAttributes);
+	bool					appendFmt(const UT_GenericVector<XML_Char*> * pVecAttributes);
 	bool					appendSpan(const UT_UCSChar * p, UT_uint32 length);
 	bool					appendObject(PTObjectType pto, const XML_Char ** attributes);
 	bool					appendFmtMark(void);
@@ -231,6 +257,7 @@ public:
 	bool					insertObjectBeforeFrag(pf_Frag * pF, PTObjectType pto,
 												   const XML_Char ** attributes);
 	bool					insertFmtMarkBeforeFrag(pf_Frag * pF);
+	bool					insertFmtMarkBeforeFrag(pf_Frag * pF, const XML_Char ** attributes);
 	
 	bool					removeStyle(const XML_Char * name);
 	size_t					getStyleCount(void);
@@ -256,8 +283,6 @@ public:
 			return m_varset.getPointer(bi);
 		}
 
-	bool					getSpanPtr(PL_StruxDocHandle sdh, UT_uint32 offset,
-									   const UT_UCSChar ** ppSpan, UT_uint32 * pLength) const;
 	bool					getBlockBuf(PL_StruxDocHandle sdh, UT_GrowBuf * pgb) const;
 
 	bool					getBounds(bool bEnd, PT_DocPosition & docPos) const;
@@ -290,6 +315,7 @@ public:
 	bool					enumStyles(UT_uint32 k,
 									   const char ** pszName, const PD_Style ** ppStyle) const;
 
+	const UT_GenericStringMap<PD_Style *> & getAllStyles()const {return m_hashStyles;}
 	bool                    isEndFootnote(pf_Frag * pf) const;
 	bool                    isFootnote(pf_Frag * pf) const;
 
@@ -298,6 +324,12 @@ public:
     pt_VarSet &             getVarSet(void) {return m_varset;};
     pf_Fragments &          getFragments(void) {return m_fragments;};
 
+	bool                    purgeFmtMarks();
+	bool                    isDoingTheDo(void) const
+	{	return m_bDoingTheDo;}
+
+	void                    setDoNotTweakPosition(bool b) {m_bDoNotTweakPosition = b;}
+
 #ifdef PT_TEST
 	UT_TestStatus			__test_VerifyCoalescedFrags(FILE * fp) const;
 	void					__dump(FILE * fp) const;
@@ -307,6 +339,10 @@ public:
 
 protected:
 
+	pf_Frag *               _findLastStruxOfType(pf_Frag * pfStart, PTStruxType pst, bool bSkipEmbeded);
+	pf_Frag *               _findPrevHyperlink(pf_Frag * pfStart);
+	pf_Frag *               _findNextHyperlink(pf_Frag * pfStart);
+
 	bool					_tellAndMaybeAddListener(PL_Listener * pListener,
 													 PL_ListenerId listenerId,
 													 bool bAdd);
@@ -381,8 +417,8 @@ protected:
 	bool					_deleteFormatting(PT_DocPosition dpos1,
 											  PT_DocPosition dpos2);
 
-	bool					_deleteComplexSpan(PT_DocPosition dpos1,
-											   PT_DocPosition dpos2,
+	bool					_deleteComplexSpan(PT_DocPosition & dpos1,
+											   PT_DocPosition & dpos2,
 											   UT_Stack *stDelayStruxDelete);
 
 
@@ -396,7 +432,8 @@ protected:
 													pf_Frag_Object * pfo, UT_uint32 fragOffset,
 													UT_uint32 length,
 													pf_Frag_Strux * pfs,
-													pf_Frag ** ppfEnd, UT_uint32 * pfragOffsetEnd);
+													pf_Frag ** ppfEnd, UT_uint32 * pfragOffsetEnd,
+                                                    bool bAddChangeRec = true);
 
 
 
@@ -423,6 +460,11 @@ protected:
 													  pf_Frag_Strux * pfs,
 													  const XML_Char ** attributes,
 													  const XML_Char ** properties);
+	bool					_fmtChangeStruxWithNotify(PTChangeFmt ptc,
+													  pf_Frag_Strux * pfs,
+													  const XML_Char ** attributes,
+													  const XML_Char ** properties,
+													  bool bDoAll);
 
 	bool					_fmtChangeSpan(pf_Frag_Text * pft, UT_uint32 fragOffset, UT_uint32 length,
 										   PT_AttrPropIndex indexNewAP,
@@ -456,6 +498,7 @@ protected:
 	bool					_getStruxFromFrag(pf_Frag * pfStart, pf_Frag_Strux ** ppfs) const;
 	bool					_getStruxFromFragSkip(pf_Frag * pfStart, pf_Frag_Strux ** ppfs) const;
 
+	bool                    _getNextStruxAfterFragSkip(pf_Frag *pfStart, pf_Frag_Strux ** ppfs);
 
 	bool                    _getStruxFromPositionSkip(PT_DocPosition docPos,
 													  pf_Frag_Strux ** ppfs) const;
@@ -504,13 +547,22 @@ protected:
 	bool					_makeObject(PTObjectType pto, const XML_Char ** attributes,
 										pf_Frag_Object * &pfo);
 	bool					_makeFmtMark(pf_Frag_FmtMark * &pff);
+	bool                    _makeFmtMark(pf_Frag_FmtMark * &pff, const XML_Char ** attributes);
+
+	// implemented in pt_PT_InsertStrux.cpp
+	bool                    _translateRevisionAttribute(PP_RevisionAttr & Revisions, PT_AttrPropIndex indexAP,
+														PP_RevisionType eType,
+														const XML_Char ** & ppRevAttrib,
+														const XML_Char ** & ppRevProps,
+														const XML_Char **   ppAttrib,
+														const XML_Char **   ppProps);
 	
 
 	PTState					m_pts;		/* are we loading or editing */
 	pt_VarSet				m_varset;
 	px_ChangeHistory		m_history;
 	pf_Fragments			m_fragments;
-	UT_StringPtrMap		    m_hashStyles;
+	UT_GenericStringMap<PD_Style *> m_hashStyles;
 
 	struct {
 		PT_AttrPropIndex	m_indexCurrentInlineAP;
@@ -518,7 +570,9 @@ protected:
 
 	PD_Document *			m_pDocument; /* back pointer to our document */
 
-	UT_uint32 m_atomicGlobCount;
+	UT_uint32               m_atomicGlobCount;
+	bool                    m_bDoingTheDo;
+	bool                    m_bDoNotTweakPosition;
 };
 
 #endif /* PT_PIECETABLE_H */
