@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
  * Copyright (C) 2003 Hubert Figuiere
@@ -37,6 +39,13 @@
 
 /*****************************************************************/
 
+enum {
+	GUI_LIST_NONE = 0,
+	GUI_LIST_BULLETED = 1,
+	GUI_LIST_NUMBERED = 2
+};
+
+
 static AP_CocoaDialog_Lists * Current_Dialog;
 
 
@@ -63,27 +72,10 @@ AP_CocoaDialog_Lists::~AP_CocoaDialog_Lists(void)
 		DELETEP (m_pPreviewWidget);
 }
 
-#if 0
-static gboolean s_preview_exposed(GtkWidget * widget, gpointer /* data */, AP_CocoaDialog_Lists * me)
-{
-	UT_ASSERT(widget && me);
-	me->previewExposed();
-	return FALSE;
-}
-
-
-static gboolean s_window_exposed(GtkWidget * widget, gpointer /* data */, AP_CocoaDialog_Lists * me)
-{
-	UT_ASSERT(widget && me);
-	me->previewExposed();
-	return FALSE;
-}
-#endif
-
 
 void AP_CocoaDialog_Lists::runModal( XAP_Frame * pFrame)
 {
-	List_Type  savedListType;
+	FL_ListType  savedListType;
 	setModal();
 	NSWindow* window;
 	m_dlg = [[AP_CocoaDialog_ListsController alloc] initFromNib];
@@ -104,7 +96,9 @@ void AP_CocoaDialog_Lists::runModal( XAP_Frame * pFrame)
 
 
 	NSView* preview = [m_dlg preview];
-	m_pPreviewWidget = new GR_CocoaGraphics(preview, m_pApp);
+	//m_pPreviewWidget = new GR_CocoaGraphics(preview, m_pApp);
+	GR_CocoaAllocInfo ai(preview, m_pApp);
+	m_pPreviewWidget = (GR_CocoaGraphics*)XAP_App::getApp()->newGraphics(ai);
 
 	// let the widget materialize
 
@@ -130,7 +124,6 @@ void AP_CocoaDialog_Lists::runModal( XAP_Frame * pFrame)
 
 void AP_CocoaDialog_Lists::runModeless (XAP_Frame * pFrame)
 {
-	NSWindow* window;
 	m_dlg = [[AP_CocoaDialog_ListsController alloc] initFromNib];
 	[m_dlg setXAPOwner:this];
 	
@@ -138,18 +131,21 @@ void AP_CocoaDialog_Lists::runModeless (XAP_Frame * pFrame)
 	// Save dialog the ID number and pointer to the widget
 	UT_sint32 sid = (UT_sint32) getDialogId ();
 	m_pApp->rememberModelessId(sid, (XAP_Dialog_Modeless *) m_pDialog);
-	window = [m_dlg window];
+
+	NSPanel * window = (NSPanel *) [m_dlg window];
 
 	// Populate the dialog
 	updateDialog();
 	m_bDontUpdate = false;
 
 	// Now Display the dialog
+	[window setBecomesKeyOnlyIfNeeded:YES];
 	[window orderFront:m_dlg];
 
 	// make a new Cocoa GC
 	NSView* preview = [m_dlg preview];
-	m_pPreviewWidget = new GR_CocoaGraphics(preview, m_pApp);
+	GR_CocoaAllocInfo ai(preview, m_pApp);
+	m_pPreviewWidget = static_cast<GR_CocoaGraphics*>(XAP_App::getApp()->newGraphics(ai));
 
 	// let the widget materialize
 
@@ -194,6 +190,16 @@ void AP_CocoaDialog_Lists::autoupdateLists(UT_Worker * pWorker)
 	}
 }
 
+void AP_CocoaDialog_Lists::setFoldLevelInGUI(void)
+{
+	UT_ASSERT(0);
+}
+
+bool AP_CocoaDialog_Lists::isPageLists(void)
+{
+	UT_ASSERT(0);
+	return true;
+}
 
 void AP_CocoaDialog_Lists::previewExposed(void)
 {
@@ -220,9 +226,13 @@ void AP_CocoaDialog_Lists::destroy(void)
 		setAnswer(AP_Dialog_Lists::a_CLOSE);
 
 		modeless_cleanup();
-		[[m_dlg window] performClose:m_dlg];
+
+		[m_dlg close];
+		[m_dlg release];
+		m_dlg = 0;
+
 		DELETEP(m_pAutoUpdateLists);
-		DELETEP (m_pPreviewWidget);
+		DELETEP(m_pPreviewWidget);
 	}
 }
 
@@ -244,25 +254,25 @@ void AP_CocoaDialog_Lists::notifyActiveFrame(XAP_Frame *pFrame)
 	previewExposed();
 }
 
-
-void  AP_CocoaDialog_Lists::styleChanged(int type)
+void  AP_CocoaDialog_Lists::typeChanged(int type)
 {
 	//
 	// code to change list list
 	//
 	[m_dlg setStyleMenu:type];
 	
-	if(type == 0)
-	{
+	switch (type) {
+	case GUI_LIST_NONE:
 		setNewListType(NOT_A_LIST);
-	}
-	else if(type == 1)
-	{
+		break;
+	case GUI_LIST_BULLETED:
 		setNewListType(BULLETED_LIST);
-	}
-	else if(type == 2)
-	{
+		break;
+	case GUI_LIST_NUMBERED:
 		setNewListType(NUMBERED_LIST);
+		break;
+	default:
+		UT_ASSERT_NOT_REACHED();
 	}
 //
 // This methods needs to be called from loadXPDataIntoLocal to set the correct
@@ -284,8 +294,9 @@ void  AP_CocoaDialog_Lists::styleChanged(int type)
  */
 void  AP_CocoaDialog_Lists::setListTypeFromWidget(void)
 {
-	NSMenuItem* item = [m_dlg selectedListType];
-	setNewListType((List_Type) [item tag]);
+	NSMenuItem* item = [m_dlg selectedListStyle];
+
+	setNewListType((FL_ListType)[item tag]);
 }
 
 /*!
@@ -306,17 +317,17 @@ void  AP_CocoaDialog_Lists::setXPFromLocal(void)
 // Now read the toggle button state and set the member variables from them
 //
 	switch ([m_dlg listAction]) {
-	case 0:
+	case GUI_LIST_NONE:
 		setbStartNewList(true);
 		setbApplyToCurrent(false);
 		setbResumeList(false);
 		break;
-	case 1:
+	case GUI_LIST_BULLETED:
 		setbStartNewList(false);
 		setbApplyToCurrent(true);
 		setbResumeList(false);
 		break;
-	case 2:
+	case GUI_LIST_NUMBERED:
 		setbStartNewList(false);
 		setbApplyToCurrent(false);
 		setbResumeList(true);
@@ -385,18 +396,27 @@ void AP_CocoaDialog_Lists::_fillFontMenu(NSPopUpButton* menu)
 	[menu removeAllItems];
 	AppendLocalizedMenuItem(menu, pSS, AP_STRING_ID_DLG_Lists_Current_Font, 0);
 
-	NSArray * list = [[NSFontManager sharedFontManager] availableFontFamilies];
+	NSArray * list = [[[NSFontManager sharedFontManager] availableFontFamilies] 
+	                      sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
 	nfonts = [list count];
 
 	for(i = 0; i < nfonts; i++)
 	{
+		[menu addItemWithTitle:[list objectAtIndex:i]];
+		[[menu lastItem] setTag:(i + 1)];
+#if 0
 		NSMenuItem* item = [[NSMenuItem alloc] initWithTitle:[list objectAtIndex:i]
 				action:nil keyEquivalent:@""];
 		[item setTag:i+1];
 		[[menu menu] addItem:item];
+#endif
 	}
 }
 
+const XML_Char* AP_CocoaDialog_Lists::_getDingbatsFontName() const
+{
+	return "Zapf Dingbats";
+}
 
 void AP_CocoaDialog_Lists::_fillNoneStyleMenu(NSMenu *listmenu)
 {
@@ -573,25 +593,26 @@ void AP_CocoaDialog_Lists::loadXPDataIntoLocal(void)
 
 	//
 	// Now set the list type and style
-	List_Type save = getNewListType();
+	FL_ListType save = getNewListType();
 	if(getNewListType() == NOT_A_LIST)
 	{
-		styleChanged(0);
+		typeChanged(GUI_LIST_NONE);
 		setNewListType(save);
+		[m_dlg->_typePopup selectItemAtIndex:GUI_LIST_NONE];
 		[m_dlg->_stylePopup selectItemAtIndex:[m_dlg->_stylePopup indexOfItemWithTag:(int)NOT_A_LIST]];
 	}
 	else if(getNewListType() >= BULLETED_LIST)
 	{
-		styleChanged(1);
+		typeChanged(GUI_LIST_BULLETED);
 		setNewListType(save);
-		[m_dlg->_typePopup selectItemAtIndex:1];
-		[m_dlg->_stylePopup selectItemAtIndex:[m_dlg->_stylePopup indexOfItemWithTag:(int)getNewListType() - BULLETED_LIST]];
+		[m_dlg->_typePopup selectItemAtIndex:GUI_LIST_BULLETED];
+		[m_dlg->_stylePopup selectItemAtIndex:[m_dlg->_stylePopup indexOfItemWithTag:(int)getNewListType()]];
 	}
 	else
 	{
-		styleChanged(2);
+		typeChanged(GUI_LIST_NUMBERED);
 	    setNewListType(save);
-		[m_dlg->_typePopup selectItemAtIndex:2];
+		[m_dlg->_typePopup selectItemAtIndex:GUI_LIST_NUMBERED];
 		[m_dlg->_stylePopup selectItemAtIndex:[m_dlg->_stylePopup indexOfItemWithTag:(int)getNewListType()]];
 	}
 
@@ -682,14 +703,14 @@ void AP_CocoaDialog_Lists::_gatherData(void)
 		LocalizeControl(_cancelBtn, pSS, XAP_STRING_ID_DLG_Cancel);
 	}
 	[_typePopup removeAllItems];
-	AppendLocalizedMenuItem(_typePopup, pSS, AP_STRING_ID_DLG_Lists_Type_none, 0);
-	AppendLocalizedMenuItem(_typePopup, pSS, AP_STRING_ID_DLG_Lists_Type_bullet, 1);
-	AppendLocalizedMenuItem(_typePopup, pSS, AP_STRING_ID_DLG_Lists_Type_numbered, 2);
-	[_typePopup selectItemAtIndex:0];
+	AppendLocalizedMenuItem(_typePopup, pSS, AP_STRING_ID_DLG_Lists_Type_none, GUI_LIST_NONE);
+	AppendLocalizedMenuItem(_typePopup, pSS, AP_STRING_ID_DLG_Lists_Type_bullet, GUI_LIST_BULLETED);
+	AppendLocalizedMenuItem(_typePopup, pSS, AP_STRING_ID_DLG_Lists_Type_numbered, GUI_LIST_NUMBERED);
+	[_typePopup selectItemAtIndex:GUI_LIST_NONE];
 	_xap->_fillNoneStyleMenu(m_listStyleNone_menu);
 	_xap->_fillNumberedStyleMenu(m_listStyleNumbered_menu);
 	_xap->_fillBulletedStyleMenu(m_listStyleBulleted_menu);
-	[_stylePopup setMenu:m_listStyleNumbered_menu];
+	[_stylePopup setMenu:m_listStyleNone_menu];
 	
 	LocalizeControl(_typeLabel, pSS, AP_STRING_ID_DLG_Lists_Type);
 	LocalizeControl(_styleLabel, pSS, AP_STRING_ID_DLG_Lists_Style);
@@ -727,14 +748,23 @@ void AP_CocoaDialog_Lists::_gatherData(void)
 - (void)setStyleMenu:(int)type
 {
 	switch(type) {
-	case 0:
-		[_stylePopup setMenu:m_listStyleNone_menu];
+	case GUI_LIST_NONE:
+		if ([_stylePopup menu] != m_listStyleNone_menu) {
+			[_stylePopup selectItemAtIndex:0];	// make sure first item is selected before changing.
+			[_stylePopup setMenu:m_listStyleNone_menu];
+		}
 		break;
-	case 1:
-		[_stylePopup setMenu:m_listStyleNumbered_menu];
+	case GUI_LIST_BULLETED:
+		if ([_stylePopup menu] != m_listStyleBulleted_menu) {
+			[_stylePopup selectItemAtIndex:0];	// make sure first item is selected before changing.
+			[_stylePopup setMenu:m_listStyleBulleted_menu];
+		}
 		break;
-	case 2:
-		[_stylePopup setMenu:m_listStyleBulleted_menu];
+	case GUI_LIST_NUMBERED:
+		if ([_stylePopup menu] != m_listStyleNumbered_menu) {
+			[_stylePopup selectItemAtIndex:0];	// make sure first item is selected before changing.
+			[_stylePopup setMenu:m_listStyleNumbered_menu];
+		}
 		break;
 	default:
 		UT_ASSERT_NOT_REACHED();
@@ -761,7 +791,7 @@ void AP_CocoaDialog_Lists::_gatherData(void)
 - (IBAction)applyAction:(id)sender
 {
 	_xap->applyClicked();
-	[NSApp stopModal];
+	// [NSApp stopModal];
 }
 
 - (IBAction)cancelAction:(id)sender
@@ -771,10 +801,14 @@ void AP_CocoaDialog_Lists::_gatherData(void)
 
 - (IBAction)labelAlignAction:(id)sender
 {
+	[_labelAlignData setFloatValue:[_labelAlignStepper floatValue]];
+	[self valueChangedAction:sender];
 }
 
 - (IBAction)labelAlignActionStepper:(id)sender
 {
+	[_labelAlignStepper setFloatValue:[_labelAlignData floatValue]];
+	[self valueChangedAction:sender];
 }
 
 - (IBAction)setDefaultAction:(id)sender
@@ -785,31 +819,17 @@ void AP_CocoaDialog_Lists::_gatherData(void)
 
 - (IBAction)startAtAction:(id)sender
 {
+	[_startAtStepper setFloatValue:[_startAtData floatValue]];
+	[self valueChangedAction:sender];
 }
 
 - (IBAction)startAtStepperAction:(id)sender
 {
+	[_startAtData setFloatValue:[_startAtStepper floatValue]];
+	[self valueChangedAction:sender];
 }
 
 - (IBAction)styleChangedAction:(id)sender
-{
-	int style = [[sender selectedItem] tag];
-	_xap->setDirty();
-	if (style) {
-		_xap->fillUncustomizedValues();
-	}
-	_xap->styleChanged(style);
-}
-
-- (IBAction)textAlignAction:(id)sender
-{
-}
-
-- (IBAction)textAlignActionStepper:(id)sender
-{
-}
-
-- (IBAction)typeChangedAction:(id)sender
 {
 	if(_xap->dontUpdate()) {
 		return;
@@ -821,6 +841,26 @@ void AP_CocoaDialog_Lists::_gatherData(void)
 	_xap->previewExposed();
 }
 
+- (IBAction)textAlignAction:(id)sender
+{
+	[_textAlignStepper setFloatValue:[_textAlignData floatValue]];
+	[self valueChangedAction:sender];
+}
+
+- (IBAction)textAlignActionStepper:(id)sender
+{
+	[_textAlignData setFloatValue:[_textAlignStepper floatValue]];
+	[self valueChangedAction:sender];
+}
+
+- (IBAction)typeChangedAction:(id)sender
+{
+	int style = [[sender selectedItem] tag];
+	[self setStyleMenu:style];
+	/* force the update.*/
+	[self styleChangedAction:_stylePopup];
+}
+
 - (IBAction)valueChangedAction:(id)sender
 {
 	if(_xap->dontUpdate()) {
