@@ -64,6 +64,7 @@
 
 
 #include "gr_Graphics.h"
+#include "gr_Painter.h"
 #include "gr_QNXGraphics.h"
 #include "gr_Image.h"
 #include "ut_bytebuf.h"
@@ -159,7 +160,7 @@ bool AP_QNXApp::initialize(void)
 	m_pToolbarActionSet = AP_CreateToolbarActionSet();
 	UT_ASSERT(m_pToolbarActionSet);
 
-	if (! XAP_QNXApp::initialize())
+	if (! AP_App::initialize())
 		return false;
 	
 	//////////////////////////////////////////////////////////////////
@@ -223,16 +224,18 @@ bool AP_QNXApp::initialize(void)
 	// CHECK THIS: the following was added by a Linux developer who can't test
 	// on QNX, or even compile, so someone with a QNX box (or fridge? ;-)) needs to check it
 	int i;
-	
+	UT_UTF8String s;
 	for (i = 0; fp_FieldTypes[i].m_Type != FPFIELDTYPE_END; i++)
 	{
-			(&fp_FieldTypes[i])->m_Desc = strdup(m_pStringSet->getValueUTF8(fp_FieldTypes[i].m_DescId).utf8_str());
+		m_pStringSet->getValueUTF8(fp_FieldTypes[i].m_DescId,s);
+		(&fp_FieldTypes[i])->m_Desc = strdup(s.utf8_str());
 	    UT_DEBUGMSG(("Setting field type desc for type %d, desc=%s\n", fp_FieldTypes[i].m_Type, fp_FieldTypes[i].m_Desc));
 	}
 
 	for (i = 0; fp_FieldFmts[i].m_Tag != NULL; i++)
 	{
-			(&fp_FieldFmts[i])->m_Desc = strdup(m_pStringSet->getValueUTF8(fp_FieldFmts[i].m_DescId).utf8_str());
+		m_pStringSet->getValueUTF8(fp_FieldFmts[i].m_DescId,s);
+		(&fp_FieldFmts[i])->m_Desc = strdup(s.utf8_str());
 	    UT_DEBUGMSG(("Setting field desc for field %s, desc=%s\n", fp_FieldFmts[i].m_Tag, fp_FieldFmts[i].m_Desc));
 	}
 
@@ -251,7 +254,13 @@ bool AP_QNXApp::initialize(void)
 
 	getMenuFactory()->buildMenuLabelSet(szMenuLabelSetName);
 
-	loadAllPlugins();
+	bool bLoadPlugins = true;
+	bool bFound = getPrefsValueBool(XAP_PREF_KEY_AutoLoadPlugins,&bLoadPlugins);
+	if(bLoadPlugins || !bFound)
+	{
+		loadAllPlugins();
+	}
+
 #ifdef ABI_OPT_PERL
     // hack to keep the perl bindings working on unix
     UT_ScriptLibrary& instance = UT_ScriptLibrary::instance(); 
@@ -422,7 +431,8 @@ static int s_hideSplash(PtWidget_t *w, void *data, PtCallbackInfo_t *info)
 static int s_drawingarea_expose(PtWidget_t *widget, PhTile_t *damage) {
 
 	if (pQNXGraphics && pSplashImage) {
-		pQNXGraphics->drawImage(pSplashImage, 0, 0);
+		GR_Painter painter(pQNXGraphics);
+		painter.drawImage(pSplashImage, 0, 0);
 
 		// on the first full paint of the image, start a 2 second timer
 		if (!firstExpose) {
@@ -492,7 +502,10 @@ static GR_Image * _showSplash(PtWidget_t *spwin, UT_uint32 delay)
 
 		// create image context
 		// TODO: find an XAP_App pointer for the following call:
-		pQNXGraphics = new GR_QNXGraphics(spwin, da, 0);
+		//pQNXGraphics = new GR_QNXGraphics(spwin, da, 0);
+		GR_QNXAllocInfo ai(spwin, da, 0);
+		pQNXGraphics = (GR_QNXGraphics*) XAP_App::getApp()->newGraphics(ai);
+		
 		pSplashImage = pQNXGraphics->createNewImage("splash", pBB, pQNXGraphics->tlu(iSplashWidth), pQNXGraphics->tlu(iSplashHeight));
 
 		PtRealizeWidget(spwin);
@@ -510,6 +523,7 @@ PtWidget_t	*gTimerWidget = NULL;
 int AP_QNXApp::main(const char * szAppName, int argc, const char ** argv)
 {
 	// This is a static function.
+	PtWidget_t *spwin;
 		   
 	UT_DEBUGMSG(("Build ID:\t%s\n", XAP_App::s_szBuild_ID));
 	UT_DEBUGMSG(("Version:\t%s\n", XAP_App::s_szBuild_Version));
@@ -522,10 +536,41 @@ int AP_QNXApp::main(const char * szAppName, int argc, const char ** argv)
 
 	XAP_Args XArgs = XAP_Args(argc,argv);
 
+	AP_QNXApp * pMyQNXApp = new AP_QNXApp(&XArgs, szAppName);
+	AP_Args Args = AP_Args(&XArgs,szAppName,pMyQNXApp);
+	Args.parsePoptOpts();
 
-	//TODO: Do a PtAppInit() here with the main window being the splash screen
-	PtWidget_t *spwin;
-	spwin = PtAppInit(NULL, NULL /* XArgs.m_argc */, NULL /* XArgs.m_argv */, 0, NULL);
+	// if the initialize fails, we don't have icons, fonts, etc.
+	if (!pMyQNXApp->initialize())
+	{
+		delete pMyQNXApp;
+		return -1;	// make this something standard?
+	}
+
+  // Setup signal handlers, primarily for segfault
+  // If we segfaulted before here, we *really* blew it
+
+  struct sigaction sa;
+
+  sa.sa_handler = signalWrapper;
+
+  sigfillset(&sa.sa_mask);  // We don't want to hear about other signals
+  sigdelset(&sa.sa_mask, SIGABRT); // But we will call abort(), so we can't ignore that
+
+  sa.sa_flags = SA_NODEFER | SA_RESETHAND; // Don't handle nested signals
+
+  sigaction(SIGSEGV, &sa, NULL);
+  sigaction(SIGBUS, &sa, NULL);
+  sigaction(SIGILL, &sa, NULL);
+  sigaction(SIGQUIT, &sa, NULL);
+  sigaction(SIGFPE, &sa, NULL);
+
+  if (!Args.doWindowlessArgs()) {
+		delete pMyQNXApp;
+		return 0;
+	}
+
+	spwin = PtAppInit(NULL, &XArgs.m_argc,(char **)XArgs.m_argv, 0, NULL);
 
 	/* Add all PhAB widgets we'll use */
 	ApAddClass("PtWindow",&PtWindow);
@@ -545,16 +590,8 @@ int AP_QNXApp::main(const char * szAppName, int argc, const char ** argv)
 	ApAddClass("PtDivider",&PtDivider);
 	ApAddClass("PtRaw",&PtRaw);
 
-	AP_QNXApp * pMyQNXApp = new AP_QNXApp(&XArgs, szAppName);
-	AP_Args Args = AP_Args(&XArgs,szAppName,pMyQNXApp);
-	Args.parsePoptOpts();
 
-	// if the initialize fails, we don't have icons, fonts, etc.
-	if (!pMyQNXApp->initialize())
-	{
-		delete pMyQNXApp;
-		return -1;	// make this something standard?
-	}
+
 
 	//This is used by all the timer classes, and should probably be in the XAP contructor
 	PtArg_t args[2];
@@ -571,40 +608,17 @@ int AP_QNXApp::main(const char * szAppName, int argc, const char ** argv)
  const XAP_Prefs * pPrefs = pMyQNXApp->getPrefs();
  UT_ASSERT(pPrefs);
  bool bSplashPref = true;
- if (pPrefs && 
-	pPrefs->getPrefsValueBool (AP_PREF_KEY_ShowSplash, &bSplashPref))
+ if (pPrefs && pPrefs->getPrefsValueBool (AP_PREF_KEY_ShowSplash, &bSplashPref))
   {
   	bShowSplash = bShowSplash && bSplashPref;
   }
 	if (bShowSplash) {
-		_showSplash(spwin, 2000);
+		_showSplash(spwin, 1500);
 	}
 	else {
 		PtDestroyWidget(spwin);
-
-  if (!Args.doWindowlessArgs())
-    return false;
-	
-    // Setup signal handlers, primarily for segfault
-    // If we segfaulted before here, we *really* blew it
-
-    struct sigaction sa;
-
-    sa.sa_handler = signalWrapper;
-
-    sigfillset(&sa.sa_mask);  // We don't want to hear about other signals
-    sigdelset(&sa.sa_mask, SIGABRT); // But we will call abort(), so we can't ignore that
-
-    sa.sa_flags = SA_NODEFER | SA_RESETHAND; // Don't handle nested signals
-
-    sigaction(SIGSEGV, &sa, NULL);
-    sigaction(SIGBUS, &sa, NULL);
-    sigaction(SIGILL, &sa, NULL);
-    sigaction(SIGQUIT, &sa, NULL);
-    sigaction(SIGFPE, &sa, NULL);
-    // TODO: handle SIGABRT
-
 	}
+
 	// this function takes care of all the command line args.
 	// if some args are botched, it returns false and we should
 	// continue out the door.
@@ -652,11 +666,12 @@ void AP_QNXApp::catchSignals(int sig_num)
 	UT_DEBUGMSG(("Oh no - we just segfaulted!\n"));
 
 	UT_uint32 i = 0;
+	IEFileType abiType = IE_Imp::fileTypeForSuffix("abw");
 	for(;i<m_vecFrames.getItemCount();i++)
 	{
 		AP_QNXFrame * curFrame = (AP_QNXFrame*) m_vecFrames[i];
 		UT_ASSERT(curFrame);
-		curFrame->backup();
+		curFrame->backup(".abw.saved", abiType);
 	}
 
 	fflush(stdout);
@@ -777,7 +792,7 @@ void AP_QNXApp::errorMsgBadFile(XAP_Frame * pFrame, const char * file,
 bool AP_QNXApp::doWindowlessArgs(const AP_Args *Args)
 {
 
-return false;
+return true;
 }
 
 XAP_Frame * AP_QNXApp::newFrame(AP_App * app)
