@@ -211,7 +211,10 @@ void AP_QNXDialog_Lists::runModeless(XAP_Frame * pFrame)
 		UT_ASSERT(m_wPreviewArea);
 
 		// make a new QNX GC
-		m_pPreviewWidget = new GR_QNXGraphics(m_mainWindow, m_wPreviewArea, pFrame->getApp());
+		//m_pPreviewWidget = new GR_QNXGraphics(m_mainWindow, m_wPreviewArea, pFrame->getApp());
+		GR_QNXAllocInfo ai(m_mainWindow, m_wPreviewArea, pFrame->getApp());
+		m_pPreviewWidget = (GR_QNXGraphics*) XAP_App::getApp()->newGraphics(ai);
+
 		unsigned short w, h;
 
 		// let the widget materialize
@@ -271,6 +274,18 @@ void AP_QNXDialog_Lists::previewExposed(void)
 }
 
 
+void AP_QNXDialog_Lists::setFoldLevelInGUI(void)
+{
+	UT_ASSERT(0);
+}
+
+bool AP_QNXDialog_Lists::isPageLists(void)
+{
+	UT_ASSERT(0);
+	return true;
+}
+
+
 /*
  THIS FUNCTION IS MIS-NAMED:  This is really the case of the
 _type_ changing, which results in a changing of the styles
@@ -316,9 +331,9 @@ void  AP_QNXDialog_Lists::setXPFromLocal(void)
 		UT_ASSERT(0);
 		setNewListType(NOT_A_LIST);
 	} else {
-		void *junk;
+		const void *junk;
 		junk = m_styleVector.getNthItem(*value - 1);
-		setNewListType((enum List_Type)((int)junk));
+		setNewListType((enum FL_ListType)((int)junk));
 	}
 
 	_gatherData();
@@ -397,38 +412,44 @@ PtWidget_t * AP_QNXDialog_Lists::_constructWindow (void)
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 
 	ConstructWindowName();
-	m_mainWindow = abiCreatePhabDialog("ap_QNXDialog_Lists",(char*)getWindowName()); 
+	m_mainWindow = abiCreatePhabDialog("ap_QNXDialog_Lists",pSS,XAP_STRING_ID_DLG_Cancel);
+
+	PtSetResource(m_mainWindow,Pt_ARG_WINDOW_TITLE,getWindowName(),NULL); 
 
 	SetupContextHelp(m_mainWindow,this);
 	PtAddHotkeyHandler(m_mainWindow,Pk_F1,0,Pt_HOTKEY_SYM,this,OpenHelp);
 	PtAddCallback(m_mainWindow, Pt_CB_WINDOW_CLOSING, s_deleteClicked, this);
 
-	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblType"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Type), 0);
+	localizeLabel(abiPhabLocateWidget(m_mainWindow,"lblType"), pSS, AP_STRING_ID_DLG_Lists_Type);
 	
 	listType = abiPhabLocateWidget(m_mainWindow,"comboType");
 	PtAddCallback(listType, Pt_CB_SELECTION, s_typeChanged, this);
 
 	const char *text;
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_none).utf8_str();
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_none,s);
+	text = s.utf8_str();
 	PtListAddItems(listType, &text, 1, 0);
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_bullet).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_bullet,s);
+	text = s.utf8_str();
 	PtListAddItems(listType, &text, 1, 0);
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_numbered).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_numbered,s);
+	text = s.utf8_str();
 	PtListAddItems(listType, &text, 1, 0);
 
-	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblStyle"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Style), 0);
+	localizeLabel(abiPhabLocateWidget(m_mainWindow,"lblStyle"), pSS, AP_STRING_ID_DLG_Lists_Style);
 
 	listStyle = abiPhabLocateWidget(m_mainWindow,"comboStyle"); 
 	PtAddCallback(listStyle, Pt_CB_SELECTION, s_styleChanged, this);
 
-	PtSetResource(abiPhabLocateWidget(m_mainWindow,"grpCustomLists"),Pt_ARG_TITLE,_(AP,DLG_Lists_Customize ),0);
+	localizeLabel(abiPhabLocateWidget(m_mainWindow,"grpCustomLists"),pSS,AP_STRING_ID_DLG_Lists_Customize );
 
-	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblFormat"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Format), 0);
-	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblFont"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Font), 0);
-	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblLevelDelimiter"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_DelimiterString), 0);
-	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblStartAt"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Start), 0);
-	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblTextAlign"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Align), 0);
-	PtSetResource(abiPhabLocateWidget(m_mainWindow,"lblLabelAlign"), Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Indent), 0);
+	localizeLabel(abiPhabLocateWidget(m_mainWindow,"lblFormat"), pSS, AP_STRING_ID_DLG_Lists_Format);
+	localizeLabel(abiPhabLocateWidget(m_mainWindow,"lblFont"), pSS, AP_STRING_ID_DLG_Lists_Font);
+	localizeLabel(abiPhabLocateWidget(m_mainWindow,"lblLevelDelimiter"), pSS, AP_STRING_ID_DLG_Lists_DelimiterString);
+	localizeLabel(abiPhabLocateWidget(m_mainWindow,"lblStartAt"), pSS, AP_STRING_ID_DLG_Lists_Start);
+	localizeLabel(abiPhabLocateWidget(m_mainWindow,"lblTextAlign"), pSS, AP_STRING_ID_DLG_Lists_Align);
+	localizeLabel(abiPhabLocateWidget(m_mainWindow,"lblLabelAlign"), pSS, AP_STRING_ID_DLG_Lists_Indent);
 
 	lblFormat = abiPhabLocateWidget(m_mainWindow,"textFormat"); 
 	PtAddCallback(lblFormat, Pt_CB_ACTIVATE, s_somethingChanged, this);
@@ -453,20 +474,20 @@ text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_numbered).utf8_str();
 	numIndentAlign =abiPhabLocateWidget(m_mainWindow,"NumericLabelAlign"); 
 	PtAddCallback(numIndentAlign, Pt_CB_NUMERIC_CHANGED, s_somethingChanged, this);
 
-	 PtSetResource(abiPhabLocateWidget(m_mainWindow,"grpPreview"),Pt_ARG_TITLE,_(AP,DLG_Lists_Preview),0);
+	 localizeLabel(abiPhabLocateWidget(m_mainWindow,"grpPreview"),pSS,AP_STRING_ID_DLG_Lists_Preview);
 
 	m_wPreviewArea = abiPhabLocateWidget(m_mainWindow,"rawPreview"); 
 	PtSetResource(m_wPreviewArea, Pt_ARG_POINTER, this, 0);
 	PtSetResource(m_wPreviewArea, Pt_ARG_RAW_DRAW_F, &s_preview_exposed, 1);
 
 	radnewlist = abiPhabLocateWidget(m_mainWindow,"toggleNew"); 
-	PtSetResource(radnewlist, Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Start_New), 0);
+	localizeLabel(radnewlist, pSS, AP_STRING_ID_DLG_Lists_Start_New);
 
 	radexisting = abiPhabLocateWidget(m_mainWindow,"toggleCurrent");
-	PtSetResource(radexisting, Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Apply_Current), 0);
+	localizeLabel(radexisting, pSS, AP_STRING_ID_DLG_Lists_Apply_Current);
 
 	radsublist = abiPhabLocateWidget(m_mainWindow,"togglePrevious"); 
-	PtSetResource(radsublist, Pt_ARG_TEXT_STRING, _(AP,DLG_Lists_Start_Sub), 0);
+	localizeLabel(radsublist, pSS, AP_STRING_ID_DLG_Lists_Start_Sub);
 
 	butOK = abiPhabLocateWidget(m_mainWindow,"btnApply"); 
 	PtSetResource(butOK, Pt_ARG_TEXT_STRING, pSS->getValue (XAP_STRING_ID_DLG_Apply), 0);
@@ -510,8 +531,9 @@ void AP_QNXDialog_Lists::_fillNoneStyleMenu( PtWidget_t *listmenu)
 
 	PtListDeleteAllItems(listmenu);
 	m_styleVector.clear();
-
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_none).utf8_str();
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Type_none,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)NOT_A_LIST);
 }
@@ -523,30 +545,37 @@ void AP_QNXDialog_Lists::_fillNumberedStyleMenu( PtWidget_t *listmenu)
 
 	PtListDeleteAllItems(listmenu);
 	m_styleVector.clear();
-
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Numbered_List).utf8_str();
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Numbered_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)NUMBERED_LIST);
-
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Lower_Case_List).utf8_str();
+	
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Lower_Case_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)LOWERCASE_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Upper_Case_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Upper_Case_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)UPPERCASE_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Lower_Roman_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Lower_Roman_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)LOWERROMAN_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Upper_Roman_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Upper_Roman_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)UPPERROMAN_LIST);
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Arabic_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Arabic_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)ARABICNUMBERED_LIST);
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Hebrew_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Hebrew_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)HEBREW_LIST);
 }
@@ -555,51 +584,63 @@ void AP_QNXDialog_Lists::_fillBulletedStyleMenu( PtWidget_t *listmenu)
 {
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 	const char *text;
-
+	UT_UTF8String s;
+	
 	PtListDeleteAllItems(listmenu);
 	m_styleVector.clear();
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Bullet_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Bullet_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)BULLETED_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Dashed_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Dashed_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)DASHED_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Square_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Square_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)SQUARE_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Triangle_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Triangle_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)TRIANGLE_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Diamond_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Diamond_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)DIAMOND_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Star_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Star_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)STAR_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Implies_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Implies_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)IMPLIES_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Tick_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Tick_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)TICK_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Box_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Box_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)BOX_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Hand_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Hand_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)HAND_LIST);
 
-text = pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Heart_List).utf8_str();
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Heart_List,s);
+	text = s.utf8_str();
 	PtListAddItems(listmenu, &text, 1, 0);
 	m_styleVector.addItem((void *)HEART_LIST);
 }
@@ -608,10 +649,13 @@ void AP_QNXDialog_Lists::_setRadioButtonLabels(void)
 {
 	//	char *tmp;
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
+	UT_UTF8String s;
 	PopulateDialogData();
 	// Button 0 is Start New List, button 2 is resume list
-PtSetResource(m_wStartNewList, Pt_ARG_TEXT_STRING, pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Start_New).utf8_str(), 0);
-PtSetResource(m_wStartSubList, Pt_ARG_TEXT_STRING, pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Resume).utf8_str(), 0);
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Start_New,s);
+	PtSetResource(m_wStartNewList, Pt_ARG_TEXT_STRING, s.utf8_str(), 0);
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Resume,s);
+	PtSetResource(m_wStartSubList, Pt_ARG_TEXT_STRING, s.utf8_str(), 0);
 }
 
 //
@@ -651,7 +695,7 @@ void AP_QNXDialog_Lists::_loadXPDataIntoLocal(void)
 
 	//
 	// Now set the list type and style
-	List_Type save = getNewListType();
+	FL_ListType save = getNewListType();
 	if(save == NOT_A_LIST) {
 		styleChanged(0);
 		setNewListType(save);
@@ -668,9 +712,9 @@ void AP_QNXDialog_Lists::_loadXPDataIntoLocal(void)
 
 	/* Determine which of the styles should be set */
 	for (i=0; i<m_styleVector.getItemCount(); i++) {
-		void *junk;
+		const void *junk;
 		junk = m_styleVector.getNthItem(i);
-		if ((enum List_Type)((int)junk) == getNewListType()) {
+		if ((enum FL_ListType)((int)junk) == getNewListType()) {
 			PtSetResource(m_wListStyle_menu, Pt_ARG_CBOX_SEL_ITEM, i+1, 0);
 			break;
 		}
