@@ -61,20 +61,23 @@
 #include "xap_Toolbar_ActionSet.h"
 #include "xap_Toolbar_Layouts.h"
 #include "xav_View.h"
-
+#include <gdk/gdk.h>
 #include "gr_Graphics.h"
 #include "gr_UnixGraphics.h"
 #include "gr_Image.h"
+#include "gr_UnixImage.h"
 #include "ut_bytebuf.h"
 #include "ut_png.h"
 #include "xap_UnixDialogHelper.h"
-
+#include "ut_iconv.h"
 #include "fv_View.h"
 #include "fp_Run.h"
 
 #include "ut_string_class.h"
 #include "xap_EncodingManager.h"
 
+#include "ie_types.h"
+
 #include "ie_impexp_Register.h"
 #include "xap_EditMethods.h"
 #include "ev_EditMethod.h"
@@ -90,6 +93,7 @@
 #include "ie_imp_RTF.h"
 #include "ie_imp_Text.h"
 #include "ie_imp_XHTML.h"
+#include "gr_DrawArgs.h"
 
 #include "xap_Prefs.h"
 #include "ap_Prefs_SchemeIds.h"
@@ -99,6 +103,11 @@
 #include "xap_UnixPSGraphics.h"
 #include "abiwidget.h"
 #include "ut_sleep.h"
+#include "gr_Painter.h"
+#include "ap_Preview_Abi.h"
+#include "xap_UnixDialogHelper.h"
+//#include <gtk/gtk.h>
+
 
 #ifdef GTK_WIN_POS_CENTER_ALWAYS
 #define WIN_POS GTK_WIN_POS_CENTER_ALWAYS
@@ -107,7 +116,6 @@
 #endif
 
 #include <popt.h>
-
 #include "ie_impGraphic.h"
 #include "ut_math.h"
 
@@ -117,10 +125,10 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <bonobo/bonobo-macros.h>
 #include <bonobo/bonobo-object.h>
-
+#include <libart_lgpl/art_affine.h>
 #include "xap_UnixGnomePrintGraphics.h"
 #include "ap_EditMethods.h"
-
+#include <libgnomeprint/gnome-print.h>
 static int mainBonobo(int argc, const char ** argv);
 #endif
 #ifdef LOGFILE
@@ -149,6 +157,7 @@ AP_UnixApp::AP_UnixApp(XAP_Args * pArgs, const char * szAppName)
 	  m_bSelectionInFlux(false),
 	  m_pViewSelection(0),
 	  m_cacheSelectionView(0),
+	  m_cacheDeferClear(0),
 	  m_pFrameSelection(0)
 {
 #ifndef HAVE_GNOME
@@ -238,7 +247,7 @@ bool AP_UnixApp::initialize(bool has_display)
 		// assume we will be using the builtin set (either as the main
 		// set or as the fallback set).
 	
-		AP_BuiltinStringSet * pBuiltinStringSet = new AP_BuiltinStringSet(this,static_cast<XML_Char*>(AP_PREF_DEFAULT_StringSet));
+		AP_BuiltinStringSet * pBuiltinStringSet = new AP_BuiltinStringSet(this,static_cast<const XML_Char*>(AP_PREF_DEFAULT_StringSet));
 		UT_ASSERT(pBuiltinStringSet);
 		m_pStringSet = pBuiltinStringSet;
 		// see if we should load an alternative set from the disk
@@ -299,7 +308,7 @@ bool AP_UnixApp::initialize(bool has_display)
     m_pToolbarActionSet = AP_CreateToolbarActionSet();
     UT_ASSERT(m_pToolbarActionSet);
     
-    if (! XAP_UNIXBASEAPP::initialize(has_display))
+    if (! AP_App::initialize())
 		return false;
 
 	//////////////////////////////////////////////////////////////////
@@ -590,6 +599,7 @@ void AP_UnixApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipb
     UT_uint32 iLen = 0;
 
     bool bFoundOne = false;
+	bool bSuccess = false;
     
     if ( bHonorFormatting )
       bFoundOne = m_pClipboard->getSupportedData(tFrom,reinterpret_cast<const void **>(&pData),&iLen,&szFormatFound);
@@ -605,20 +615,30 @@ void AP_UnixApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipb
     if (AP_UnixClipboard::isRichTextTag(szFormatFound))
     {
 		iLen = UT_MIN(iLen,strlen(reinterpret_cast<const char *>(pData)));
-		UT_DEBUGMSG(("PasteFromClipboard: pasting %d bytes in format [%s].\n",iLen,szFormatFound));
 
 		IE_Imp_RTF * pImpRTF = new IE_Imp_RTF(pDocRange->m_pDoc);
-		pImpRTF->pasteFromBuffer(pDocRange,pData,iLen);
+		bSuccess = pImpRTF->pasteFromBuffer(pDocRange,pData,iLen);
 		DELETEP(pImpRTF);
     }
 	else if (AP_UnixClipboard::isHTMLTag (szFormatFound))
 	{
-		iLen = UT_MIN(iLen,strlen(reinterpret_cast<const char *>(pData)));
-		UT_DEBUGMSG(("PasteFromClipboard: pasting %d bytes in format [%s].\n",iLen,szFormatFound));
-
-		IE_Imp_XHTML * pImpHTML = new IE_Imp_XHTML(pDocRange->m_pDoc);
-		pImpHTML->pasteFromBuffer(pDocRange,pData,iLen);
-		DELETEP(pImpHTML);
+		IE_Imp_Text_Sniffer SniffBuf;
+		const char * szRes = SniffBuf.recognizeContentsType(reinterpret_cast<const char *>(pData),iLen);
+		if(UT_strcmp(szRes,"none") != 0)
+		{
+			UT_uint32 iread,iwritten = 0;
+			const char * szutf8= static_cast<const char *>(UT_convert(reinterpret_cast<const char *>(pData),iLen,szRes,"UTF-8",&iread,&iwritten));
+			IE_Imp_XHTML * pImpHTML = new IE_Imp_XHTML(pDocRange->m_pDoc);
+			bSuccess = pImpHTML->pasteFromBuffer(pDocRange,reinterpret_cast<const unsigned char *>(szutf8),iwritten,"UTF-8");
+			free(const_cast<char *>(szutf8));
+			DELETEP(pImpHTML);
+		}
+		else
+		{
+			IE_Imp_XHTML * pImpHTML = new IE_Imp_XHTML(pDocRange->m_pDoc);
+			bSuccess = pImpHTML->pasteFromBuffer(pDocRange,reinterpret_cast<const unsigned char *>(pData),iLen);
+			DELETEP(pImpHTML);
+		}
 	}
     else if (AP_UnixClipboard::isImageTag(szFormatFound))
       {
@@ -627,8 +647,6 @@ void AP_UnixApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipb
 		  IEGraphicFileType iegft = IEGFT_Unknown;
 		  UT_Error error = UT_OK;
 		  
-		  XAP_Frame * pFrame = getLastFocussedFrame ();
-		  
 		  UT_ByteBuf * bytes = new UT_ByteBuf( iLen );
 		  
 		  bytes->append (pData, iLen);
@@ -639,7 +657,7 @@ void AP_UnixApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipb
 			  UT_DEBUGMSG(("DOM: could not construct importer (%d)\n", 
 						   error));
 			  DELETEP(bytes);
-			  return;
+			  goto retry_text;
 		  }
 		  
 		  error = pIEG->importGraphic(bytes, &pFG);
@@ -648,35 +666,40 @@ void AP_UnixApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipb
 			  UT_DEBUGMSG(("DOM: could not import graphic (%d)\n", error));
 			  DELETEP(bytes);
 			  DELETEP(pIEG);
-			  return;
+			  goto retry_text;
 		  }
 		  
 		  // at this point, 'bytes' is owned by pFG
-		  FV_View * pView = static_cast<FV_View*>(pFrame->getCurrentView());
-		  
-		  UT_String newName = UT_String_sprintf ( "paste_image_%d", UT_newNumber() ) ;
+		  FV_View * pView = static_cast<FV_View*>(getLastFocussedFrame ()->getCurrentView());
 		  
 		  DELETEP(pIEG);
 		  
-		  error = pView->cmdInsertGraphic(pFG, newName.c_str());
-		  if (error)
-		  {
-			  UT_DEBUGMSG(("DOM: could not insert graphic (%d)\n", error));
-			  DELETEP(pFG);
-			  return;
-		  }
-		  
+		  error = pView->cmdInsertGraphic(pFG);
 		  DELETEP(pFG);
+		  if (!error)
+			  bSuccess = true;
       }
     else // ( AP_UnixClipboard::isTextTag(szFormatFound) )
     {
 		iLen = UT_MIN(iLen,strlen(reinterpret_cast<const char *>(pData)));
-		UT_DEBUGMSG(("PasteFromClipboard: pasting %d bytes in format [%s].\n",iLen,szFormatFound));
 		
 		IE_Imp_Text * pImpText = new IE_Imp_Text(pDocRange->m_pDoc,"UTF-8");
-		pImpText->pasteFromBuffer(pDocRange,pData,iLen);
+		bSuccess = pImpText->pasteFromBuffer(pDocRange,pData,iLen);
 		DELETEP(pImpText);
     }
+
+ retry_text:
+
+	// we failed to paste *anything.* try plaintext as a last-ditch effort
+	if(!bSuccess && m_pClipboard->getTextData(tFrom,reinterpret_cast<const void **>(&pData),&iLen, &szFormatFound)) {
+		UT_DEBUGMSG(("DOM: pasting text as an absolute fallback (bug 7666)\n"));
+		
+		iLen = UT_MIN(iLen,strlen(reinterpret_cast<const char *>(pData)));
+		
+		IE_Imp_Text * pImpText = new IE_Imp_Text(pDocRange->m_pDoc,"UTF-8");
+		bSuccess = pImpText->pasteFromBuffer(pDocRange,pData,iLen);
+		DELETEP(pImpText);
+	}
 }
 
 bool AP_UnixApp::canPasteFromClipboard(void)
@@ -686,7 +709,7 @@ bool AP_UnixApp::canPasteFromClipboard(void)
 
 extern "C" {
 
-	// return > 0 for directory entries ending in ".so"
+	// return > 0 for directory entries ending in ".so" ".sl" and the like
 #if defined (__APPLE__) || defined (__FreeBSD__) || defined (__OpenBSD__) \
 	|| defined(_AIX) || defined(__sgi)
 	static int so_only (struct dirent *d)
@@ -702,8 +725,8 @@ extern "C" {
 			
 			if (len >= 3)
 			{
-				if(!strcmp(name+(len-3), ".so"))
-	    return 1;
+				if(!strcmp(name+(len-3), "."G_MODULE_SUFFIX))
+					return 1;
 			}
 		}
 		return 0;
@@ -734,7 +757,7 @@ void AP_UnixApp::loadAllPlugins ()
 
   // the user-local plugin directory
   pluginDir = getUserPrivateDirectory ();
-  pluginDir += "/AbiWord-2.0/plugins/";
+  pluginDir += "/AbiWord-2.2/plugins/";
   pluginList[1] = pluginDir;
 
   for(UT_uint32 i = 0; i < (sizeof(pluginList)/sizeof(pluginList[0])); i++)
@@ -759,7 +782,7 @@ void AP_UnixApp::loadAllPlugins ()
 				  free(namelist[n]);
 				  continue;
 			  }
-			  if(strcmp (namelist[n]->d_name+(len-3), ".so") != 0)
+			  if(strcmp (namelist[n]->d_name+(len-3), "."G_MODULE_SUFFIX) != 0)
 			  {
 				  UT_DEBUGMSG(("FJF: not really a plugin?\n"));
 				  free(namelist[n]);
@@ -1020,6 +1043,76 @@ bool AP_UnixApp::getCurrentSelection(const char** formatList,
     return true;
 }
 
+bool AP_UnixApp:: makePngPreview(const char * pszInFile, const char * pszPNGFile, UT_sint32 iWidth, UT_sint32 iHeight)
+{
+	//
+	// Create a private visual to draw with
+	//
+	GdkVisual * vis = gdk_visual_get_system (); // don't delete this!
+	//
+	// Create a private color map to draw with.
+	//
+	GdkColormap*  visColorMap = gdk_colormap_new(vis,true);
+	//
+	// Create a private set of attributes for our GdkWindow
+	// 
+	GdkWindowAttr attributes;
+	attributes.title = NULL;
+	attributes.event_mask = 0;
+	attributes.x = 0;
+	attributes.y = 0;
+	attributes.width = iWidth;
+	attributes.height = iHeight;
+	attributes.wclass = 	GDK_INPUT_OUTPUT ;
+	attributes.visual = vis;
+	attributes.colormap = visColorMap;
+	attributes.window_type = GDK_WINDOW_CHILD;
+	attributes.cursor = NULL;
+	attributes.wmclass_name = NULL;
+	attributes.wmclass_class = NULL;
+	attributes.override_redirect = true;
+
+	gint attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_COLORMAP | GDK_WA_VISUAL;
+	GdkWindow*  pWindow = gdk_window_new (NULL, &attributes,
+                                             attributes_mask);
+	GdkPixmap * pMap =  gdk_pixmap_new(pWindow,iWidth,iHeight,-1); 	
+
+	//GR_UnixGraphics * pG =  new GR_UnixGraphics(pMap, getFontManager(), this,true);
+	GR_UnixAllocInfo ai(pMap, getFontManager(), this,true);
+	GR_UnixGraphics * pG = (GR_UnixGraphics*) XAP_App::getApp()->newGraphics(ai);
+
+	pG->createCaret();
+	UT_Error error = UT_OK;
+	PD_Document * pNewDoc = new PD_Document(this);
+	error = pNewDoc->readFromFile(pszInFile,IEFT_Unknown, NULL);
+
+	if (error != UT_OK) 
+	{
+		return false;
+	}
+	AP_Preview_Abi * pPrevAbi = new AP_Preview_Abi(pG,iWidth,iHeight,NULL, PREVIEW_ZOOMED,pNewDoc);
+	dg_DrawArgs da;
+	memset(&da, 0, sizeof(da));
+	da.pG = pG;
+	pPrevAbi->getView()->draw(0, &da);
+	GR_Painter * pPaint = new GR_Painter(pG);
+	UT_Rect r;
+	r.left = 0;
+	r.top = 0;
+	r.width = pG->tlu(iWidth);
+	r.height = pG->tlu(iHeight);
+	GR_Image * pImage = pPaint->genImageFromRectangle(r);
+	DELETEP(pPaint);
+	static_cast<GR_UnixImage *>(pImage)->saveToPNG( pszPNGFile);
+	DELETEP(pImage);
+	DELETEP(pG);
+	delete pWindow;
+	delete pMap;
+	delete visColorMap;
+	DELETEP(pPrevAbi); // This deletes pNewDoc
+	return true;
+}
+
 /*****************************************************************/
 /*****************************************************************/
 
@@ -1041,7 +1134,7 @@ static gint s_hideSplash(gpointer /*data*/)
 {
     if (wSplash)
     {
-		gtk_timeout_remove(death_timeout_handler);
+		g_source_remove(death_timeout_handler);
 		gtk_widget_destroy(wSplash);
 		wSplash = NULL;
 		DELETEP(pUnixGraphics);
@@ -1071,14 +1164,15 @@ static gint s_drawingarea_expose(GtkWidget * /* widget */,
 {
     if (pUnixGraphics && pSplashImage)
     {
-		pUnixGraphics->drawImage(pSplashImage, 0, 0);
+		GR_Painter painter (pUnixGraphics);
+		painter.drawImage(pSplashImage, 0, 0);
 
 		// on the first full paint of the image, start a 2 second timer
 		if (!firstExpose)
 		{
 			firstExpose = true;
 			// kill the window after splashTimeoutValue ms
-			death_timeout_handler = gtk_timeout_add(splashTimeoutValue, s_hideSplash, NULL);
+			death_timeout_handler = g_timeout_add_full(0,splashTimeoutValue, s_hideSplash, NULL,NULL);
 		}
     }
 
@@ -1179,11 +1273,10 @@ GR_Image * AP_UnixApp::_showSplash(UT_uint32 delay)
 		gtk_widget_show(wSplash);
 		
 		// create image context
-#ifndef WITH_PANGO
-		pUnixGraphics = new GR_UnixGraphics(da->window, NULL, m_pApp);
-#else
-		pUnixGraphics = new GR_UnixGraphics(da->window, m_pApp);
-#endif
+		//pUnixGraphics = new GR_UnixGraphics(da->window, NULL, m_pApp);
+		GR_UnixAllocInfo ai(da->window, NULL, m_pApp);
+		pUnixGraphics = (GR_UnixGraphics*) XAP_App::getApp()->newGraphics(ai);
+		
 		pSplashImage = pUnixGraphics->createNewImage("splash", pBB, pUnixGraphics->tlu(iSplashWidth), 
 													 pUnixGraphics->tlu(iSplashHeight));
 
@@ -1238,7 +1331,7 @@ int AP_UnixApp::main(const char * szAppName, int argc, const char ** argv)
 #ifdef LOGFILE
 	fprintf(logfile,"About to start gnome_program_init \n");
 #endif
-	  GnomeProgram * program = gnome_program_init ("AbiWord", ABI_BUILD_VERSION, 
+	  GnomeProgram * program = gnome_program_init ("AbiWord-" PACKAGE_VERSION, ABI_BUILD_VERSION, 
 												   LIBGNOMEUI_MODULE, XArgs.m_argc, const_cast<char **>(XArgs.m_argv),
 												   GNOME_PARAM_APP_PREFIX, ABIWORD_PREFIX,
 												   GNOME_PARAM_APP_SYSCONFDIR, ABIWORD_SYSCONFDIR,
@@ -1312,8 +1405,10 @@ int AP_UnixApp::main(const char * szAppName, int argc, const char ** argv)
 	
     // Step 2: Handle all non-window args.
     
-    if (!Args.doWindowlessArgs())
-      return false;
+    if (!Args.doWindowlessArgs()) {
+		delete pMyUnixApp;
+		return 0;
+	}
 
 	if (have_display) {
 
@@ -1340,7 +1435,7 @@ int AP_UnixApp::main(const char * szAppName, int argc, const char ** argv)
 #endif
 			pMyUnixApp->shutdown();
 			delete pMyUnixApp;
-		  return rtn;
+			return rtn;
 		}
 #endif
 		
@@ -1370,7 +1465,8 @@ int AP_UnixApp::main(const char * szAppName, int argc, const char ** argv)
 		{
 			UT_DEBUGMSG(("DOM: not parsing command line or showing app\n"));
 		}
-	}		
+	}
+
 	// Step 4: Destroy the App.  It should take care of deleting all frames.
 	pMyUnixApp->shutdown();
 	delete pMyUnixApp;
@@ -1388,7 +1484,7 @@ XAP_Frame * AP_UnixApp::newFrame(AP_App * app)
 
 void AP_UnixApp::errorMsgBadArg(AP_Args * Args, int nextopt)
 {
-  printf ("Error on option %s: %s.\nRun '%s --help' to see a full list of available command line options.\n",
+  fprintf (stderr, "Error on option %s: %s.\nRun '%s --help' to see a full list of available command line options.\n",
 	  poptBadOption (Args->poptcon, 0),
 	  poptStrerror (nextopt),
 	  Args->XArgs->m_argv[0]);
@@ -1429,6 +1525,11 @@ void AP_UnixApp::initPopt (AP_Args * Args)
 #endif
 }
 
+
+#define PMSCALE 0.5
+#define PMW (PMSCALE * P_WIDTH)
+#define PMH (PMSCALE * P_HEIGHT)
+
 /*!
  * A callback for AP_Args's doWindowlessArgs call which handles
  * platform-specific windowless args.
@@ -1478,11 +1579,23 @@ bool AP_UnixApp::doWindowlessArgs(const AP_Args *Args)
 				conv.setImpProps (Args->m_impProps);
 			if (Args->m_expProps)
 				conv.setExpProps (Args->m_expProps);
-
-			PS_GraphicsFactory printFactory (Args->m_sPrintTo);
-
+#if 0
+			PS_Graphics pGraphics ((Args->m_sPrintTo[0] == '|' ? Args->m_sPrintTo+1 : Args->m_sPrintTo),
+								   Args->m_sPrintTo, 
+								   pMyUnixApp->getApplicationName(), pMyUnixApp->getFontManager(),
+								   (Args->m_sPrintTo[0] != '|'), pMyUnixApp);
+#endif
+			PS_GraphicsAllocInfo ai((Args->m_sPrintTo[0] == '|' ? Args->m_sPrintTo+1 : Args->m_sPrintTo),
+								 Args->m_sPrintTo, 
+								 pMyUnixApp->getApplicationName(), pMyUnixApp->getFontManager(),
+								 (Args->m_sPrintTo[0] != '|'), pMyUnixApp);
+			
+			PS_Graphics * pGraphics = (PS_Graphics*) XAP_App::getApp()->newGraphics(ai);
+			
 			conv.setVerbose(Args->m_iVerbose);
-			conv.print (Args->m_sFile, printFactory);
+			conv.print (Args->m_sFile, pGraphics);
+
+			delete pGraphics;
 	    }
 		else
 	    {
@@ -1492,6 +1605,67 @@ bool AP_UnixApp::doWindowlessArgs(const AP_Args *Args)
 
 		return false;
 	}
+	if (Args->m_iToThumb > 0) 
+	{
+
+#ifdef HAVE_GNOME
+
+		if ((Args->m_sFile = poptGetArg (Args->poptcon)) != NULL)
+	    {
+#if 0 // work out how to do this later
+			AP_Convert conv ;
+			if (Args->m_impProps)
+				conv.setImpProps (Args->m_impProps);
+			if (Args->m_expProps)
+				conv.setExpProps (Args->m_expProps);
+			UT_String sdimXY = Args->m_sThumbXY;
+			UT_uint32 loc = UT_String_findCh(sdimXY,'x');
+			if(loc>(size_t)-10)
+			{
+				return false;
+			}
+			UT_String sX = sdimXY.substr(0,loc);
+			UT_String sY = sdimXY.substr(loc+1,sdimXY.size());
+			UT_sint32 iX = atoi(sX.c_str());
+			UT_sint32 iY = atoi(sY.c_str());
+			guchar * buf;
+			gdouble p2b[6];
+			gint bpp = 3;
+			art_affine_scale (p2b, 1.0, -1.0);
+			p2b[5] = iY; // number of pixels in height
+			buf = g_new (guchar, iX * iY * bpp); 
+
+			GnomePrintContext * pc = gnome_print_rbuf_new (buf, iX, iY, bpp * iX, p2b, FALSE);
+
+			PD_Document *pDoc = new PD_Document(this);
+			pDoc->readFromFile(Args->m_sFile, IEFT_Unknown, Args->m_impProps);
+			double inWidth = pDoc->m_docPageSize.Width(DIM_IN);
+			double inHeight = pDoc->m_docPageSize.Height(DIM_IN);
+			XAP_UnixGnomePrintGraphics * pGraphics = new XAP_UnixGnomePrintGraphics(pc, inWidth,inHeight);
+			conv.setVerbose(Args->m_iVerbose);
+			conv.printFirstPage (pGraphics,pDoc);
+			UNREFP(pDoc);
+			gnome_print_context_close (pc);
+			GdkPixbuf* pb = gdk_pixbuf_new_from_data (buf, GDK_COLORSPACE_RGB, 
+													  false,
+													  8, iX, iY, bpp * iX, 
+													  NULL, NULL);
+			GError * err;
+			gdk_pixbuf_save(pb,Args->m_sThumb,"png",&err);
+#endif
+			return true;
+	    }
+		else
+	    {
+			// couldn't load document
+			fprintf(stderr, "Error: no file to print!\n");
+	    }
+		
+		return false;
+#else
+		fprintf(stderr,"Only works in GNOME build \n");
+#endif
+	}
 
 	if(Args->m_sPlugin)
 	{
@@ -1500,19 +1674,21 @@ bool AP_UnixApp::doWindowlessArgs(const AP_Args *Args)
 //
 	    const char * szName = NULL;
 		XAP_Module * pModule = NULL;
-		Args->m_sPlugin = poptGetArg(Args->poptcon);
+		const char * szRequest = NULL;
+		//		szRequest = poptGetArg(Args->poptcon);
+		szRequest = Args->m_sPlugin;
 		bool bFound = false;	
-		printf(" Looking for plugin name %s \n",Args->m_sPlugin);
+		printf(" Looking for plugin name %s \n",szRequest);
 		if(Args->m_sPlugin != NULL)
 		{
-			const UT_Vector * pVec = XAP_ModuleManager::instance().enumModules ();
+			const UT_GenericVector<XAP_Module*> * pVec = XAP_ModuleManager::instance().enumModules ();
 			printf(" %d plugins loaded \n",pVec->getItemCount());
 			for (UT_uint32 i = 0; (i < pVec->size()) && !bFound; i++)
 			{
-				pModule = static_cast<XAP_Module *>(pVec->getNthItem (i));
+				pModule = pVec->getNthItem (i);
 				szName = pModule->getModuleInfo()->name;
 				printf("Plugin %s loaded \n",szName);
-				if(UT_strcmp(szName,Args->m_sPlugin) == 0)
+				if(UT_strcmp(szName,szRequest) == 0)
 				{
 					printf("plugin %s found sending control there! \n",szName);
 					bFound = true;
@@ -1521,7 +1697,7 @@ bool AP_UnixApp::doWindowlessArgs(const AP_Args *Args)
 		}
 		if(!bFound)
 		{
-			printf("Plugin %s not found or loaded \n",Args->m_sPlugin);
+			printf("Plugin %s not found or loaded \n",szRequest);
 			return false;
 		}
 //
@@ -1553,10 +1729,10 @@ bool AP_UnixApp::doWindowlessArgs(const AP_Args *Args)
 	    const char * szName = NULL;
 		XAP_Module * pModule = NULL;
 		bool bFound = false;	
-		const UT_Vector * pVec = XAP_ModuleManager::instance().enumModules ();
+		const UT_GenericVector<XAP_Module*> * pVec = XAP_ModuleManager::instance().enumModules ();
 		for (UT_uint32 i = 0; (i < pVec->size()) && !bFound; i++)
 		{
-			pModule = static_cast<XAP_Module *>(pVec->getNthItem (i));
+			pModule = pVec->getNthItem (i);
 			szName = pModule->getModuleInfo()->name;
 			printf("Plugin %s loaded \n",szName);
 			if(UT_strcmp(szName,"AbiControl") == 0)
@@ -1648,14 +1824,15 @@ void AP_UnixApp::catchSignals(int sig_num)
 	}
 #endif
     UT_uint32 i = 0;
+	IEFileType abiType = IE_Imp::fileTypeForSuffix("abw");
     for(;i<m_vecFrames.getItemCount();i++)
     {
 		AP_UnixFrame * curFrame = const_cast<AP_UnixFrame*>(static_cast<const AP_UnixFrame*>(m_vecFrames[i]));
 		UT_ASSERT(curFrame);
 		if (NULL == curFrame->getFilename())
-		  curFrame->backup(".abw.CRASHED");
+		  curFrame->backup(".abw.SAVED",abiType);
 		else
-		  curFrame->backup(".CRASHED");
+		  curFrame->backup(".SAVED",abiType);
     }
     
     fflush(stdout);
@@ -1751,6 +1928,7 @@ static void set_prop (BonoboPropertyBag 	*bag,
 }
 
 
+#if 0
 /****************************************************************
  Whole bunch of bonobo UI verb implementations to enable useful things
  to be done from a merged menuBar.
@@ -1810,7 +1988,6 @@ verb_generic_cb (BonoboUIComponent *uic, gpointer data, const char *name)
 	}
 }
 
-#if 0
 static BonoboUIVerb abi_nautilus_verbs[] = {
 	BONOBO_UI_VERB ("fileSave",            verb_generic_cb),
 	BONOBO_UI_VERB ("fileSaveAs",          verb_generic_cb),
@@ -1995,7 +2172,7 @@ load_document_from_stream (BonoboPersistStream *ps,
 {
 	AbiWidget *abiwidget;
 	Bonobo_Stream_iobuf *buffer;
-	CORBA_long len_read;
+	size_t len_read;
 	FILE * tmpfile;
 #ifdef LOGFILE
 	fprintf(logfile,"Load file from stream \n");
@@ -2244,7 +2421,7 @@ static void zoom_level_func(GObject * z, float lvl, gpointer data)
   UT_return_if_fail ( pFrame != NULL ) ;
 
   pFrame->setZoomType (XAP_Frame::z_PERCENT);
-  pFrame->setZoomPercentage (static_cast<UT_uint32>(lvl));
+  pFrame->quickZoom (static_cast<UT_uint32>(lvl));
 }
 
 static void zoom_in_func(GObject * z, gpointer data)
@@ -2261,7 +2438,7 @@ static void zoom_in_func(GObject * z, gpointer data)
   zoom_lvl += ZOOM_PCTG ;
 
   pFrame->setZoomType (XAP_Frame::z_PERCENT);
-  pFrame->setZoomPercentage (zoom_lvl);  
+  pFrame->quickZoom (zoom_lvl);  
 }
 
 static void zoom_out_func(GObject * z, gpointer data)
@@ -2281,7 +2458,7 @@ static void zoom_out_func(GObject * z, gpointer data)
     return ;
 
   pFrame->setZoomType (XAP_Frame::z_PERCENT);
-  pFrame->setZoomPercentage (zoom_lvl);  
+  pFrame->quickZoom (zoom_lvl);  
 }
 
 static void zoom_to_fit_func(GObject * z, gpointer data)
@@ -2299,7 +2476,7 @@ static void zoom_to_fit_func(GObject * z, gpointer data)
 
   UT_uint32 newZoom = pView->calculateZoomPercentForWholePage();
   pFrame->setZoomType( XAP_Frame::z_WHOLEPAGE );
-  pFrame->setZoomPercentage(newZoom);
+  pFrame->quickZoom(newZoom);
 }
 
 static void zoom_to_default_func(GObject * z, gpointer data)
@@ -2313,7 +2490,7 @@ static void zoom_to_default_func(GObject * z, gpointer data)
   UT_return_if_fail ( pFrame != NULL ) ;
 
   pFrame->setZoomType (XAP_Frame::z_100);
-  pFrame->setZoomPercentage (100);  
+  pFrame->quickZoom (100);  
 }
 
 /*****************************************************************/
