@@ -106,13 +106,11 @@ static void
 focus_in_event (GtkWidget * drawing_area, GdkEventCrossing *event, AP_UnixFrameImpl * me)
 {
   gtk_widget_grab_focus (drawing_area);
-  me->focusIMIn ();
 }
 
 static void
 focus_out_event (GtkWidget * drawing_area, GdkEventCrossing * event, AP_UnixFrameImpl * me)
 {
-  me->focusIMOut ();
 }
 
 GtkWidget * AP_UnixFrameImpl::_createDocumentWindow()
