@@ -70,33 +70,33 @@ AP_Win32Dialog_MergeCells::~AP_Win32Dialog_MergeCells(void)
 void AP_Win32Dialog_MergeCells::runModeless(XAP_Frame * pFrame)
 {
 	// raise the dialog
- 	int iResult;
+	int iResult;
 	XAP_Win32App * pWin32App = static_cast<XAP_Win32App *>(m_pApp);
 
 	LPCTSTR lpTemplate = NULL;
 
-	UT_ASSERT(m_id == AP_DIALOG_ID_MERGE_CELLS);
+	UT_return_if_fail (m_id == AP_DIALOG_ID_MERGE_CELLS);
 
 	lpTemplate = MAKEINTRESOURCE(AP_RID_DIALOG_MERGECELLS);
 
- 	HWND hResult = CreateDialogParam(pWin32App->getInstance(),lpTemplate,
- 							static_cast<XAP_Win32FrameImpl*>(pFrame->getFrameImpl())->getTopLevelWindow(),
- 							(DLGPROC)s_dlgProc,(LPARAM)this);
- 	UT_ASSERT_HARMLESS((hResult != NULL));
- 
- 	m_hwndDlg = hResult;
- 
- 	// Save dialog the ID number and pointer to the widget
- 	UT_sint32 sid =(UT_sint32)  getDialogId();
- 	m_pApp->rememberModelessId( sid, (XAP_Dialog_Modeless *) m_pDialog);
- 
- 	iResult = ShowWindow( m_hwndDlg, SW_SHOW );
- 
- 	iResult = BringWindowToTop( m_hwndDlg );
- 
- 	startUpdater();
- 
- 	UT_ASSERT_HARMLESS((iResult != 0));
+	HWND hResult = CreateDialogParam(pWin32App->getInstance(),lpTemplate,
+							static_cast<XAP_Win32FrameImpl*>(pFrame->getFrameImpl())->getTopLevelWindow(),
+							(DLGPROC)s_dlgProc,(LPARAM)this);
+	UT_ASSERT_HARMLESS((hResult != NULL));
+
+	m_hwndDlg = hResult;
+
+	// Save dialog the ID number and pointer to the widget
+	UT_sint32 sid =(UT_sint32)  getDialogId();
+	m_pApp->rememberModelessId( sid, (XAP_Dialog_Modeless *) m_pDialog);
+
+	iResult = ShowWindow( m_hwndDlg, SW_SHOW );
+
+	iResult = BringWindowToTop( m_hwndDlg );
+
+	startUpdater();
+
+	UT_ASSERT_HARMLESS((iResult != 0));
 
 }
 
