@@ -41,18 +41,18 @@
 /*
 	Helpers
 */
-UT_String 	sRslt;
-UT_UTF8String  sRsltUTF8;
+static UT_String 	sRslt;
+static UT_UTF8String  sRsltUTF8;
 
-const char* fromUTF8toAnsi(const char* szIn)
+const char* fromUTF8toWinLocale(const char* szIn)
 {
-	sRslt =	AP_Win32App::s_fromUTF8ToAnsi(szIn);
+	sRslt =	AP_Win32App::s_fromUTF8ToWinLocale(szIn);
 	return sRslt.c_str();
 }
 
-const char* fromAnsitoUTF8(const char* szIn)
+const char* fromWinLocaletoUTF8(const char* szIn)
 {
-	sRsltUTF8 = AP_Win32App::s_fromAnsiToUTF8(szIn);				
+	sRsltUTF8 = AP_Win32App::s_fromWinLocaleToUTF8(szIn);				
 	return sRsltUTF8.utf8_str();
 }
 
@@ -79,7 +79,7 @@ AP_Win32Dialog_MetaData::~AP_Win32Dialog_MetaData(void)
 
 void AP_Win32Dialog_MetaData::runModal(XAP_Frame * pFrame)
 {
-	UT_ASSERT(pFrame);	
+	UT_return_if_fail (pFrame);	
 	XAP_Win32App * pWin32App = static_cast<XAP_Win32App *>(pFrame->getApp());	
 		
 	XAP_Win32PropertySheet				sheet;
@@ -151,11 +151,11 @@ void AP_Win32Dialog_MetaData_General::_onInitDialog()
 		SetDlgItemText(getHandle(), rgMapping[i].controlId, pSS->getValue(rgMapping[i].stringId));						
 	
 	// Setup previous text	
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_GENERAL_EDIT_TITLE,			fromUTF8toAnsi(getContainer()->getTitle().c_str()));									
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_GENERAL_EDIT_SUBJECT,		fromUTF8toAnsi(getContainer()->getSubject().c_str()));									
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_GENERAL_EDIT_AUTHOR,			fromUTF8toAnsi(getContainer()->getAuthor().c_str()));									
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_GENERAL_EDIT_PUBLISHER,		fromUTF8toAnsi(getContainer()->getPublisher().c_str()));									
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_GENERAL_EDIT_CONTRIBUTOR,	fromUTF8toAnsi(getContainer()->getCoAuthor().c_str()));									
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_GENERAL_EDIT_TITLE,			fromUTF8toWinLocale(getContainer()->getTitle().utf8_str()));									
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_GENERAL_EDIT_SUBJECT,		fromUTF8toWinLocale(getContainer()->getSubject().utf8_str()));									
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_GENERAL_EDIT_AUTHOR,			fromUTF8toWinLocale(getContainer()->getAuthor().utf8_str()));									
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_GENERAL_EDIT_PUBLISHER,		fromUTF8toWinLocale(getContainer()->getPublisher().utf8_str()));									
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_GENERAL_EDIT_CONTRIBUTOR,	fromUTF8toWinLocale(getContainer()->getCoAuthor().utf8_str()));									
 	
 	
 	HWND hParent = GetParent(getHandle());						
@@ -169,7 +169,7 @@ char* AP_Win32Dialog_MetaData_General::_get_text(XAP_String_Id nID, char *szBuff
 {
 	*szBuff=0;
 	 GetDlgItemText(getHandle(), nID, szBuff, nSize); 
-	 strcpy (szBuff, (AP_Win32App::s_fromAnsiToUTF8(szBuff)).utf8_str());
+	 strcpy (szBuff, (AP_Win32App::s_fromWinLocaleToUTF8(szBuff)).utf8_str());
 	 return szBuff;
 }
 
@@ -238,10 +238,10 @@ void AP_Win32Dialog_MetaData_Summary::_onInitDialog()
 		SetDlgItemText(getHandle(), rgMapping[i].controlId, pSS->getValue(rgMapping[i].stringId));				
 
 	// Setup previous text	
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_SUMMARY_EDIT_CATEGORY,		fromUTF8toAnsi(getContainer()->getCategory().c_str()));									
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_SUMMARY_EDIT_KEYWORDS,		fromUTF8toAnsi(getContainer()->getKeywords().c_str()));									
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_SUMMARY_EDIT_LANGUAGE,		fromUTF8toAnsi(getContainer()->getLanguages().c_str()));									
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_SUMMARY_EDIT_DESCRIPTION,	fromUTF8toAnsi(getContainer()->getDescription().c_str()));									
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_SUMMARY_EDIT_CATEGORY,		fromUTF8toWinLocale(getContainer()->getCategory().utf8_str()));									
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_SUMMARY_EDIT_KEYWORDS,		fromUTF8toWinLocale(getContainer()->getKeywords().utf8_str()));									
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_SUMMARY_EDIT_LANGUAGE,		fromUTF8toWinLocale(getContainer()->getLanguages().utf8_str()));									
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_SUMMARY_EDIT_DESCRIPTION,	fromUTF8toWinLocale(getContainer()->getDescription().utf8_str()));									
 
 }
 
@@ -249,7 +249,7 @@ char* AP_Win32Dialog_MetaData_Summary::_get_text(XAP_String_Id nID, char *szBuff
 {
 	*szBuff=0;
 	 GetDlgItemText(getHandle(), nID, szBuff, nSize); 
-	 strcpy (szBuff, (AP_Win32App::s_fromAnsiToUTF8(szBuff)).utf8_str());
+	 strcpy (szBuff, (AP_Win32App::s_fromWinLocaleToUTF8(szBuff)).utf8_str());
 	 return szBuff;
 }
 
@@ -316,10 +316,10 @@ void AP_Win32Dialog_MetaData_Permissions::_onInitDialog()
 		SetDlgItemText(getHandle(), rgMapping[i].controlId, pSS->getValue(rgMapping[i].stringId));				
 
 	// Setup previous text	
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_PERMISSIONS_EDIT_SOURCE,		fromUTF8toAnsi(getContainer()->getSource().c_str()));											
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_PERMISSIONS_EDIT_RELATION,	fromUTF8toAnsi(getContainer()->getRelation().c_str()));											
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_PERMISSIONS_EDIT_COVERAGE,	fromUTF8toAnsi(getContainer()->getCoverage().c_str()));											
-	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_PERMISSIONS_EDIT_RIGHTS,		fromUTF8toAnsi(getContainer()->getRights().c_str()));									
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_PERMISSIONS_EDIT_SOURCE,		fromUTF8toWinLocale(getContainer()->getSource().utf8_str()));											
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_PERMISSIONS_EDIT_RELATION,	fromUTF8toWinLocale(getContainer()->getRelation().utf8_str()));											
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_PERMISSIONS_EDIT_COVERAGE,	fromUTF8toWinLocale(getContainer()->getCoverage().utf8_str()));											
+	SetDlgItemText(getHandle(), AP_RID_DIALOG_META_PERMISSIONS_EDIT_RIGHTS,		fromUTF8toWinLocale(getContainer()->getRights().utf8_str()));									
 
 }												
 
@@ -354,6 +354,6 @@ char* AP_Win32Dialog_MetaData_Permissions::_get_text(XAP_String_Id nID, char *sz
 {
 	 *szBuff=0;
 	 GetDlgItemText(getHandle(), nID, szBuff, nSize); 
-	 strcpy (szBuff, (AP_Win32App::s_fromAnsiToUTF8(szBuff)).utf8_str());
+	 strcpy (szBuff, (AP_Win32App::s_fromWinLocaleToUTF8(szBuff)).utf8_str());
 	 return szBuff;
 }
