@@ -1,5 +1,5 @@
 /* AbiWord
- * Copyright (C) 2002 Tomas Frydrych <tomas@frydrych.uklinux.net>
+ * Copyright (C) 2002, 2003 Tomas Frydrych <tomas@frydrych.uklinux.net>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,8 +26,6 @@
 #include "ap_Dialog_ListRevisions.h"
 #include "ap_Strings.h"
 
-#include <fribidi.h>
-
 AP_Dialog_ListRevisions::AP_Dialog_ListRevisions(XAP_DialogFactory * pDlgFactory,
 					   XAP_Dialog_Id id)
   : XAP_Dialog_NonPersistent(pDlgFactory,id, "interface/dialogselectrevision"), m_answer(a_CANCEL),
@@ -69,6 +67,12 @@ const char * AP_Dialog_ListRevisions::getColumn2Label() const
 	return m_pSS->getValue(AP_STRING_ID_DLG_ListRevisions_Column2Label);
 }
 
+const char * AP_Dialog_ListRevisions::getColumn3Label() const
+{
+	UT_return_val_if_fail(m_pSS,NULL);
+	return m_pSS->getValue(AP_STRING_ID_DLG_ListRevisions_Column3Label);
+}
+
 UT_uint32 AP_Dialog_ListRevisions::getItemCount() const
 {
 	UT_return_val_if_fail(m_pDoc,0);
@@ -82,7 +86,35 @@ UT_uint32 AP_Dialog_ListRevisions::getNthItemId(UT_uint32 n) const
 	if(n == 0)
 		return 0;
 	
-	return ((PD_Revision *)(m_pDoc->getRevisions()).getNthItem(n-1))->getId();
+	return ((AD_Revision *)(m_pDoc->getRevisions()).getNthItem(n-1))->getId();
+}
+
+const char * AP_Dialog_ListRevisions::getNthItemTime(UT_uint32 n) const
+{
+	UT_return_val_if_fail(m_pDoc,0);
+
+	if(n == 0)
+		return NULL;
+
+	// TODO the date should be properly localised
+	static char s[30];
+	time_t tT = ((AD_Revision *)(m_pDoc->getRevisions()).getNthItem(n-1))->getStartTime();
+
+	if(tT != 0)
+	{
+		struct tm * tM = localtime(&tT);
+		strftime(s,30,"%c",tM);
+	}
+	else
+	{
+		// we use 0 to indicate unknown time
+		s[0] = '?';
+		s[1] = '?';
+		s[2] = '?';
+		s[3] = 0;
+	}
+	
+	return s;
 }
 
 char * AP_Dialog_ListRevisions::getNthItemText(UT_uint32 n) const
@@ -102,7 +134,7 @@ char * AP_Dialog_ListRevisions::getNthItemText(UT_uint32 n) const
 	{
 		
 		const UT_UCS4Char * pC
-			= ((PD_Revision *)(m_pDoc->getRevisions()).getNthItem(n-1))->getDescription();
+			= ((AD_Revision *)(m_pDoc->getRevisions()).getNthItem(n-1))->getDescription();
 	
 		if(!pC)
 			return NULL;
@@ -110,26 +142,16 @@ char * AP_Dialog_ListRevisions::getNthItemText(UT_uint32 n) const
 		// now we run this string through fribidi
 		if(XAP_App::getApp()->theOSHasBidiSupport() == XAP_App::BIDI_SUPPORT_NONE)
 		{
-			FriBidiChar *fbdStr = (FriBidiChar *)pC;
-			FriBidiChar *fbdStr2 = 0;
+			UT_UCS4Char * pStr2 = 0;
 			UT_uint32 iLen = UT_UCS4_strlen(pC);
 
-			fbdStr2  = (FriBidiChar *)UT_calloc( iLen + 1, sizeof(FriBidiChar));
-			UT_return_val_if_fail(fbdStr2,NULL);
+			pStr2  = (UT_UCS4Char *)UT_calloc( iLen + 1, sizeof(UT_UCS4Char));
+			UT_return_val_if_fail(pStr2,NULL);
 			bFree = true;
 
-			FriBidiCharType fbdDomDir = fribidi_get_type(fbdStr[0]);
-
-			fribidi_log2vis (		/* input */
-							 fbdStr,
-							 iLen,
-							 &fbdDomDir,
-							 /* output */
-							 fbdStr2,
-							 NULL,
-							 NULL,
-							 NULL);
-			pC = (const UT_UCS4Char *) fbdStr2;
+			UT_BidiCharType iDomDir = UT_bidiGetCharType(pC[0]);
+			UT_bidiReorderString(pC, iLen, iDomDir, pStr2);
+			pC = (const UT_UCS4Char *) pStr2;
 
 		}
 
