@@ -20,6 +20,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <ctype.h>
 #include "ut_assert.h"
 #include "ut_string.h"
 #include "ut_units.h"
@@ -71,11 +72,10 @@ AP_Dialog_Tab::tAnswer AP_Dialog_Tab::getAnswer(void) const
 
 void AP_Dialog_Tab::_storeWindowData()
 {
-	UT_ASSERT(m_pFrame); // needs to be set from runModal for some of the event_'s to work
+	UT_return_if_fail (m_pFrame); // needs to be set from runModal for some of the event_'s to work
 
 	FV_View *pView = (FV_View *)m_pFrame->getCurrentView();
-
-	UT_ASSERT(m_pCallbackFn);
+	UT_return_if_fail (pView && m_pCallbackFn);
 
 	(*m_pCallbackFn)(this, pView, m_pszTabStops, _gatherDefaultTabStop(), m_closure);
 }
@@ -88,10 +88,10 @@ void AP_Dialog_Tab::_populateWindowData(void)
 	else
 		m_dim = DIM_IN;
 
-	UT_ASSERT(m_pFrame); // needs to be set from runModal for some of the event_'s to work
+	UT_return_if_fail (m_pFrame); // needs to be set from runModal for some of the event_'s to work
 
 	FV_View *pView = (FV_View *)m_pFrame->getCurrentView();
-
+	UT_return_if_fail (pView);
 
 	// get the info used in the top ruler
 	AP_TopRulerInfo rulerInfo;
@@ -115,7 +115,7 @@ void AP_Dialog_Tab::_populateWindowData(void)
 	
 		// create new tab info
 		pTabInfo = new fl_TabStop();
-		UT_ASSERT(pTabInfo);
+		UT_return_if_fail (pTabInfo);
 
 
 		(*rulerInfo.m_pfnEnumTabStops)( rulerInfo.m_pVoidEnumTabStopsData,
@@ -184,7 +184,7 @@ void AP_Dialog_Tab::_event_TabSelected( UT_sint32 index )
 
 	if(index >= 0)
 	{
-		UT_ASSERT(index < (UT_sint32)m_tabInfo.getItemCount());
+		UT_return_if_fail (index < (UT_sint32)m_tabInfo.getItemCount());
 
 		fl_TabStop *pTabInfo = (fl_TabStop *)m_tabInfo.getNthItem(index);
 
@@ -207,7 +207,13 @@ void AP_Dialog_Tab::_event_Set(void)
 {
 	UT_String buffer;
 
-	buildTab(buffer);
+	// check the validity of the input
+	bool res = buildTab(buffer);
+	if (!res)
+	{
+		// TODO: add a message box here to inform our user - MARCM
+		return;
+	}
 
 	UT_DEBUGMSG(("DOM: %s\n", buffer.c_str()));
 
@@ -230,7 +236,7 @@ void AP_Dialog_Tab::_event_Set(void)
 	for (  i = 0; i < m_tabInfo.getItemCount(); i++ )
 	{
 		fl_TabStop *pTabInfo = (fl_TabStop *)m_tabInfo.getNthItem(i);
-		UT_ASSERT(pTabInfo);
+		UT_return_if_fail (pTabInfo);
 
 		// if we have a tab at that unit
 		if ( memcmp(cbuffer, _getTabString(pTabInfo), Dimension_size) == 0 )
@@ -256,20 +262,21 @@ void AP_Dialog_Tab::_event_Set(void)
 	delete [] m_pszTabStops;
 	m_pszTabStops = p_temp;
 
-	UT_ASSERT(m_pFrame); // needs to be set from runModal for some of the event_'s to work
+	UT_return_if_fail (m_pFrame); // needs to be set from runModal for some of the event_'s to work
 
 	FV_View *pView = static_cast<FV_View *>(m_pFrame->getCurrentView());
-
+	UT_return_if_fail(pView);
+	
 	buildTabStops(pView->getGraphics(), m_pszTabStops, m_tabInfo);
 
 	_setTabList(m_tabInfo.getItemCount());
 
-	// Select the new or changed tab in the lis.
+	// Select the new or changed tab in the list.
 
 	for (i = 0; i < m_tabInfo.getItemCount(); i++ )
 	{
-		fl_TabStop *pTabInfo = static_cast<fl_TabStop *>(m_tabInfo.getNthItem(i));
-		UT_ASSERT(pTabInfo);
+		fl_TabStop *pTabInfo = m_tabInfo.getNthItem(i);
+		UT_return_if_fail (pTabInfo);
 
 		// if we have a tab at that unit
 		if ( memcmp(cbuffer, _getTabString(pTabInfo), Dimension_size) == 0 )
@@ -295,7 +302,7 @@ void AP_Dialog_Tab::_event_Clear(void)
 	{
 		UT_return_if_fail(index < static_cast<UT_sint32>(m_tabInfo.getItemCount()));
 
-		_deleteTabFromTabString(static_cast<fl_TabStop *>(m_tabInfo.getNthItem(index)));
+		_deleteTabFromTabString(m_tabInfo.getNthItem(index));
 
 		UT_return_if_fail(m_pFrame); // needs to be set from runModal for some of the event_'s to work
 
@@ -366,7 +373,7 @@ void AP_Dialog_Tab::_event_ClearAll(void)
 		break;
 
 	default:
-		UT_ASSERT(UT_NOT_IMPLEMENTED);
+		UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED);
 		ch = 'L';
 		break;
 	}
@@ -400,7 +407,7 @@ void AP_Dialog_Tab::_event_ClearAll(void)
 		break;
 
 	default:
-		UT_ASSERT(UT_NOT_IMPLEMENTED);
+		UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED);
 		a = FL_TAB_LEFT;
 	}
 	return a;
@@ -414,14 +421,20 @@ void AP_Dialog_Tab::clearList()
 }
 
 
-void AP_Dialog_Tab::buildTab( UT_String & buffer )
+bool AP_Dialog_Tab::buildTab( UT_String & buffer )
 {
 	// get current value from member
 	const XML_Char* szOld = _gatherTabEdit();
-	const XML_Char* szNew = UT_reformatDimensionString(m_dim, szOld); 
+	bool res = UT_isValidDimensionString(szOld, MAX_TAB_LENGTH);
+	if (res)
+	{
+		const XML_Char* szNew = UT_reformatDimensionString(m_dim, szOld); 
 
-	UT_String_sprintf( buffer, "%s/%c%c", szNew, AlignmentToChar(_gatherAlignment()),
-		 				(static_cast<char>(_gatherLeader()))+'0');
+		UT_String_sprintf( buffer, "%s/%c%c", szNew, AlignmentToChar(_gatherAlignment()),
+		 					(static_cast<char>(_gatherLeader()))+'0');
+	}
+	
+	return res;
 }
 
 void AP_Dialog_Tab::_event_somethingChanged()
@@ -447,8 +460,8 @@ void AP_Dialog_Tab::_event_somethingChanged()
 
 	for ( UT_uint32 i = 0; i < m_tabInfo.getItemCount(); i++ )
 	{
-		fl_TabStop *pTabInfo = static_cast<fl_TabStop *>(m_tabInfo.getNthItem(i));
-		UT_ASSERT(pTabInfo);
+		fl_TabStop *pTabInfo = m_tabInfo.getNthItem(i);
+		UT_return_if_fail (pTabInfo);
 
 		// if we have a tab at that unit
 		if ( !strcmp(cbuffer, _getTabString(pTabInfo)) )
@@ -476,9 +489,9 @@ void AP_Dialog_Tab::_event_somethingChanged()
 char *AP_Dialog_Tab::_getTabDimensionString(UT_uint32 tabIndex)
 {
 
-	UT_ASSERT(tabIndex < m_tabInfo.getItemCount());
+	UT_return_val_if_fail (tabIndex < m_tabInfo.getItemCount(), NULL);
 
-	fl_TabStop *pTabInfo = static_cast<fl_TabStop *>(m_tabInfo.getNthItem(tabIndex));
+	fl_TabStop *pTabInfo = m_tabInfo.getNthItem(tabIndex);
 
 	const char* pStart = &m_pszTabStops[pTabInfo->getOffset()];
 	const char* pEnd = pStart;
@@ -488,7 +501,7 @@ char *AP_Dialog_Tab::_getTabDimensionString(UT_uint32 tabIndex)
 	}
 
 	UT_uint32 iLen = pEnd - pStart;
-	UT_ASSERT(iLen<20);
+	UT_return_val_if_fail (iLen<20, NULL);
 
 	strncpy(buf, pStart, iLen);
 	buf[iLen]=0;
@@ -506,7 +519,6 @@ char *AP_Dialog_Tab::_getTabString(fl_TabStop *pTabInfo)
 	}
 
 	UT_uint32 iLen = pEnd - pStart;
-	UT_ASSERT(iLen<20);
 
 	strncpy(buf, pStart, iLen);
 	buf[iLen]=0;
@@ -566,7 +578,7 @@ void AP_Dialog_Tab::_deleteTabFromTabString(fl_TabStop *pTabInfo)
 void AP_Dialog_Tab::_doSpin(tControl id, UT_sint32 amt)
 {
   //	UT_ASSERT(amt); // zero makes no sense
-	UT_ASSERT(id == id_SPIN_DEFAULT_TAB_STOP);
+	UT_return_if_fail (id == id_SPIN_DEFAULT_TAB_STOP);
 	if(amt == 0 )
 	{
 	        UT_DEBUGMSG(("AMOUNT = 0 amt = %d \n",amt));
@@ -607,7 +619,7 @@ void AP_Dialog_Tab::_doSpin(tControl id, UT_sint32 amt)
 		break;
 	default:
 
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -642,7 +654,7 @@ void AP_Dialog_Tab::_doSpin(tControl id, UT_sint32 amt)
 void AP_Dialog_Tab::_doSpinValue(tControl id, double value)
 {
   //	UT_ASSERT(amt); // zero makes no sense
-	UT_ASSERT(id == id_SPIN_DEFAULT_TAB_STOP);
+	UT_return_if_fail (id == id_SPIN_DEFAULT_TAB_STOP);
 
 	double d = value;
 
@@ -678,7 +690,7 @@ void AP_Dialog_Tab::_doSpinValue(tControl id, double value)
 		break;
 	default:
 
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -707,4 +719,3 @@ void AP_Dialog_Tab::_doSpinValue(tControl id, double value)
 
 	_setDefaultTabStop(szNew);
 }
-
