@@ -1,5 +1,6 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
+ * Copyright (C) 2004 Hubert Figuiere
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,12 +37,18 @@
 #include "ap_Prefs.h"
 #include "fv_View.h"
 #include "fp_TableContainer.h"
-
+#include "fp_FrameContainer.h"
 #include "pd_Document.h"
 
+#include "pp_AttrProp.h"
+#include "gr_Painter.h"
+
 /*****************************************************************/
 
 AP_LeftRuler::AP_LeftRuler(XAP_Frame * pFrame)
+#if XAP_DONTUSE_XOR
+	: m_guideCache(NULL)
+#endif
 {
 	m_pFrame = pFrame;
 	m_pView = NULL;
@@ -119,6 +126,15 @@ void AP_LeftRuler::setView(AV_View* pView, UT_uint32 iZoom)
 	static_cast<FV_View *>(pView)->setLeftRuler(this);
 }
 
+void AP_LeftRuler::setZoom(UT_uint32 iZoom)
+{
+	m_pG->setZoomPercentage(iZoom);
+    // TODO this dimension shouldn't be hard coded.
+	// in fact, it shouldn't need to be recomputed at all anymore.
+	m_minPageLength = UT_convertToLogicalUnits("0.5in");
+
+}
+
 void AP_LeftRuler::setView(AV_View * pView)
 {
 	if (m_pView && (m_pView != pView))
@@ -210,11 +226,16 @@ void AP_LeftRuler::mousePress(EV_EditModifierState /* ems */, EV_EditMouseButton
 
 	if(m_pView == NULL || m_pView->getPoint() == 0)
 		return;
+	FV_View * pView = static_cast<FV_View *>(m_pView);
+    if(pView->getDocument()->isPieceTableChanging())
+	{
+		return;
+	}
 	xxx_UT_DEBUGMSG(("LeftRuler: In mouse press x %d y %d\n",x,y));
 	m_bValidMouseClick = false;
 	m_draggingWhat = DW_NOTHING;
 	m_bEventIgnored = false;
-	FV_View * pView = static_cast<FV_View *>(m_pView);
+
 	GR_Graphics * pG = pView->getGraphics();
 	pView->getLeftRulerInfo(&m_infoCache);
 	UT_ASSERT(m_infoCache.m_yTopMargin >= 0);
@@ -230,6 +251,8 @@ void AP_LeftRuler::mousePress(EV_EditModifierState /* ems */, EV_EditMouseButton
 	
   	UT_Rect rTopMargin, rBottomMargin;
   	_getMarginMarkerRects(&m_infoCache,rTopMargin,rBottomMargin);
+	rTopMargin.width = getWidth();
+	rBottomMargin.width = getWidth();
  	if (rTopMargin.containsPoint(x,y))
  	{
  		m_bValidMouseClick = true;
@@ -244,17 +267,6 @@ void AP_LeftRuler::mousePress(EV_EditModifierState /* ems */, EV_EditMouseButton
 
  	if (rBottomMargin.containsPoint(x,y))
  	{
-		FV_View *pView = static_cast<FV_View *>(m_pView);
-
-		// it makes no sense to click on the bottom margin for the footer.
-		if (pView->isHdrFtrEdit())
-		{
-			fl_HdrFtrShadow * pShadow = pView->getEditShadow();
-
-			if (pShadow->getHdrFtrSectionLayout()->getHFType() 
-				                == FL_HDRFTR_FOOTER)
-				return;
-		}
 
  		m_bValidMouseClick = true;
  		m_draggingWhat = DW_BOTTOMMARGIN;
@@ -308,6 +320,15 @@ void AP_LeftRuler::mouseRelease(EV_EditModifierState ems, EV_EditMouseButton emb
 	{
 		return;
 	}
+	if(pView->getDocument() == NULL)
+	{
+		return;
+	}
+	if(pView->getDocument()->isPieceTableChanging())
+	{
+		return;
+	}
+
 	if (!m_bValidMouseClick || m_bEventIgnored)
 	{
 		m_draggingWhat = DW_NOTHING;
@@ -369,7 +390,20 @@ void AP_LeftRuler::mouseRelease(EV_EditModifierState ems, EV_EditMouseButton emb
 	fl_HdrFtrShadow * pShadow = pView->getEditShadow();
 
 	bool hdr = (hdrftr && 
-				pShadow->getHdrFtrSectionLayout()->getHFType() == FL_HDRFTR_HEADER);
+				pShadow->getHdrFtrSectionLayout()->getHFType() < FL_HDRFTR_FOOTER);
+	HdrFtrType hfType = FL_HDRFTR_HEADER;
+	if(pShadow)
+	{
+		hfType = pShadow->getHdrFtrSectionLayout()->getHFType();
+	}
+    PT_DocPosition insPos = pView->getPoint();
+	UT_sint32 iPage = -1;
+	fl_DocSectionLayout * pDSL = NULL;
+	if(pShadow)
+	{
+		pDSL = pShadow->getHdrFtrSectionLayout()->getDocSectionLayout();
+		iPage = pDSL->getDocLayout()->findPage(pShadow->getPage());
+	}
 
 	double dyrel = 0.0;
 	UT_sint32 yOrigin = m_infoCache.m_yPageStart + 
@@ -385,45 +419,216 @@ void AP_LeftRuler::mouseRelease(EV_EditModifierState ems, EV_EditMouseButton emb
 		
 	case DW_TOPMARGIN:
 		{
-			dyrel = tick.scalePixelDistanceToUnits(m_draggingCenter - yAbsTop);
-			if (!hdrftr)
-				properties[0] = "page-margin-top";
-			else
+			UT_String sHeights;
+			UT_DEBUGMSG(("Release TOP Margin 1 \n"));
+			if(!(m_infoCache.m_mode == AP_LeftRulerInfo::TRI_MODE_FRAME) && !pView->isInFrame(pView->getPoint()))
 			{
-				if (hdr)
-					properties[0] = "page-margin-header";
+				dyrel = tick.scalePixelDistanceToUnits(m_draggingCenter - yAbsTop);
+				UT_DEBUGMSG(("Release TOP Margin 1 Column \n"));
+
+				if (!hdrftr)
+					properties[0] = "page-margin-top";
 				else
 				{
-					properties[0] = "page-margin-footer";
+					if (hdr)
+						properties[0] = "page-margin-header";
+					else
+					{
+						properties[0] = "page-margin-bottom";
 
-					dyrel = tick.scalePixelDistanceToUnits
-						(pShadow->getHdrFtrSectionLayout()->
-						          getDocSectionLayout()->getBottomMargin() + 
-						 (m_draggingCenter + m_yScrollOffset -
-						(m_infoCache.m_yPageStart + m_infoCache.m_yPageSize)));
+						dyrel = tick.scalePixelDistanceToUnits(yEnd - m_draggingCenter);
+					}
+				}
+				sHeights = pG->invertDimension(tick.dimType,dyrel);
+				properties[1] = sHeights.c_str();
+				properties[2] = NULL;
+				pView->setSectionFormat(properties);
+			}
+			else
+			{
+				if(m_pView == NULL)
+				{
+					return;
+				}
+				FV_View * pView = static_cast<FV_View *>(m_pView);
+				fl_FrameLayout * pFrame = pView->getFrameLayout();
+				if(pFrame)
+				{
+					const PP_AttrProp* pSectionAP = NULL;
+					pFrame->getAP(pSectionAP);
+					const XML_Char * pszYpos = NULL;
+					UT_sint32 iYpos;
+					const XML_Char * pszHeight = NULL;
+					UT_sint32 iHeight;
+					if(!pSectionAP || !pSectionAP->getProperty("ypos",pszYpos))
+					{
+						UT_DEBUGMSG(("No ypos defined for Frame !\n"));
+						UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+						return;
+					}
+					else
+					{
+						iYpos = UT_convertToLogicalUnits(pszYpos);
+					}
+					if(!pSectionAP || !pSectionAP->getProperty("frame-height",pszHeight))
+					{
+						UT_DEBUGMSG(("No Height defined for Frame !\n"));
+						UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+						return;
+					}
+					else
+					{
+						iHeight = UT_convertToLogicalUnits(pszHeight);
+					}
+					UT_sint32 diff = ygrid - m_oldY;
+					iHeight -= diff;
+					iYpos += diff;
+					if(iHeight < 0)
+					{
+						iHeight = -iHeight;
+					}
+					UT_String sYpos("");
+					double dYpos = static_cast<double>(iYpos)/static_cast<double>(UT_LAYOUT_RESOLUTION);
+					sYpos = UT_formatDimensionedValue(dYpos,"in", NULL);
+					UT_String sHeight("");
+					double dHeight = static_cast<double>(iHeight)/static_cast<double>(UT_LAYOUT_RESOLUTION);
+					sHeight = UT_formatDimensionedValue(dHeight,"in", NULL);
+					const XML_Char * props[6] = {"ypos",sYpos.c_str(),
+						"frame-height",sHeight.c_str(),
+						NULL,NULL};
+					pView->setFrameFormat(props);
+				}
+				else
+				{
+					return;
 				}
 			}
+			_xorGuide(true);
+			m_draggingWhat = DW_NOTHING;
+			notify(pView, AV_CHG_HDRFTR);
+			pView->setPoint(insPos);
+			pView->notifyListeners(AV_CHG_MOTION | AV_CHG_HDRFTR );
+			pView->setPoint(insPos);
+			pView->ensureInsertionPointOnScreen();
+//
+// Put the point at the right point in the header/footer on the right page.
+//
+			if(iPage >= 0)
+			{
+				fp_Page * pPage = pDSL->getDocLayout()->getNthPage(iPage);
+				if(pPage)
+				{
+					fp_ShadowContainer* pShadowC = pPage->getHdrFtrP(hfType);
+					UT_return_if_fail(pShadowC);
+					pShadow = pShadowC->getShadow();
+					UT_return_if_fail(pShadow);
+					pView->setHdrFtrEdit(pShadow);
+				}
+				pView->setPoint(insPos);
+				pView->notifyListeners(AV_CHG_MOTION | AV_CHG_HDRFTR );
+				pView->setPoint(insPos);
+				pView->ensureInsertionPointOnScreen();
+			}
+
+			return;
 		}
-		break;
+		return;
 
 	case DW_BOTTOMMARGIN:
 		{
-			dyrel = tick.scalePixelDistanceToUnits(yEnd - m_draggingCenter);
-			if (!hdrftr)
-				properties[0] = "page-margin-bottom";
+			if(!(m_infoCache.m_mode == AP_LeftRulerInfo::TRI_MODE_FRAME) && !pView->isInFrame(pView->getPoint()))
+			{
+
+				dyrel = tick.scalePixelDistanceToUnits(yEnd - m_draggingCenter);
+				if (!hdrftr)
+					properties[0] = "page-margin-bottom";
+				else
+				{
+					if (hdr)
+					{
+						properties[0] = "page-margin-top";
+						dyrel = tick.scalePixelDistanceToUnits
+							(m_draggingCenter - yAbsTop);
+					}
+					else
+					{
+						properties[0] = "page-margin-footer";
+					}
+				}
+				UT_String sHeights = pG->invertDimension(tick.dimType,dyrel);
+				properties[1] = sHeights.c_str();
+				properties[2] = NULL;
+				pView->setSectionFormat(properties);
+			}
 			else
 			{
-				if (hdr)
+				if(m_pView == NULL)
 				{
-					properties[0] = "page-margin-top";
-					dyrel = tick.scalePixelDistanceToUnits
-						(m_draggingCenter - yAbsTop);
+					return;
+				}
+				FV_View * pView = static_cast<FV_View *>(m_pView);
+				fl_FrameLayout * pFrame = pView->getFrameLayout();
+				if(pFrame)
+				{
+					const PP_AttrProp* pSectionAP = NULL;
+					pFrame->getAP(pSectionAP);
+					const XML_Char * pszHeight = NULL;
+					UT_sint32 iHeight;
+					if(!pSectionAP || !pSectionAP->getProperty("frame-height",pszHeight))
+					{
+						UT_DEBUGMSG(("No Height defined for Frame !\n"));
+						UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+						return;
+					}
+					else
+					{
+						iHeight = UT_convertToLogicalUnits(pszHeight);
+					}
+					UT_sint32 diff = ygrid - m_oldY;
+					iHeight += diff;
+					if(iHeight < 0)
+					{
+						iHeight = -iHeight;
+					}
+					UT_String sHeight("");
+					double dHeight = static_cast<double>(iHeight)/static_cast<double>(UT_LAYOUT_RESOLUTION);
+					sHeight = UT_formatDimensionedValue(dHeight,"in", NULL);
+					const XML_Char * props[4] = {"frame-height",sHeight.c_str(),
+								NULL,NULL};
+					pView->setFrameFormat(props);
 				}
 				else
-					UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+				{
+					return;
+				}
 			}
+			_xorGuide(true);
+			m_draggingWhat = DW_NOTHING;
+			notify(pView, AV_CHG_HDRFTR);
+			pView->setPoint(insPos);
+			pView->notifyListeners(AV_CHG_MOTION | AV_CHG_HDRFTR );
+			pView->setPoint(insPos);
+			pView->ensureInsertionPointOnScreen();
+//
+// Put the point at the right point in the header/footer on the right page.
+//
+			if(iPage >= 0)
+			{
+				fp_Page * pPage = pDSL->getDocLayout()->getNthPage(iPage);
+				if(pPage)
+				{
+					fp_ShadowContainer* pShadowC = pPage->getHdrFtrP(hfType);
+					pShadow = pShadowC->getShadow();
+					pView->setHdrFtrEdit(pShadow);
+				}
+				pView->setPoint(insPos);
+				pView->notifyListeners(AV_CHG_MOTION | AV_CHG_HDRFTR );
+				pView->setPoint(insPos);
+				pView->ensureInsertionPointOnScreen();
+			}
+			return;
 		}
-		break;
+		return;
 	case DW_CELLMARK:
 	    {
 			xxx_UT_DEBUGMSG(("Handling CellMark \n"));
@@ -550,49 +755,9 @@ void AP_LeftRuler::mouseRelease(EV_EditModifierState ems, EV_EditMouseButton emb
 			return;
 		}
 	}
-
-	properties[1] = pG->invertDimension(tick.dimType,dyrel);
-	properties[2] = 0;
-
-	xxx_UT_DEBUGMSG(("LeftRuler: %s [%s]\n",properties[0], properties[1]));
-
-	_xorGuide(true);
-	m_draggingWhat = DW_NOTHING;
-
-	if(hdrftr)
-	{
-		pView->rememberCurrentPosition();
-
-		pView->clearHdrFtrEdit();
-  		pView->warpInsPtToXY(0,0,false);
-	}
-
-	//
-	// Finally we've got it all in place, Make the change!
-	//
-
-	pView->setSectionFormat(properties);
-
-    // todo: make this warp back to the last saved editing position
-    // todo: instead of just the header/footer in general
-	if (hdrftr)
-	{
-		if (hdr)
-			pView->cmdEditHeader();
-		else
-			pView->cmdEditFooter();
-
-		xxx_UT_DEBUGMSG(("DOM: %d\n", pView->getSavedPosition()));
-
-		pView->setPoint(pView->getSavedPosition());
-
-		pView->moveInsPtTo(pView->getSavedPosition());
-		pView->clearSavedPosition();
-	}
 //
 // Redraw the left ruler.
 //
-	notify(pView,AV_CHG_MOTION);
 	return;
 }
 
@@ -618,6 +783,14 @@ UT_sint32 AP_LeftRuler::setTableLineDrag(PT_DocPosition pos, UT_sint32 & iFixed,
 	{
 		return 0;
 	}
+	if(pView->getDocument() == NULL)
+	{
+		return 0;
+	}
+	if(pView->getDocument()->isPieceTableChanging())
+	{
+		return 0;
+	}
 	pView->getLeftRulerInfo(pos,&m_infoCache);
 	UT_ASSERT(m_infoCache.m_yTopMargin >= 0);
 
@@ -684,7 +857,18 @@ void AP_LeftRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32
 		}
 		return;
 	}
-	pView->getLeftRulerInfo(&m_infoCache);
+	if(pView->getDocument() == NULL)
+	{
+		return;
+	}
+	if(pView->getDocument()->isPieceTableChanging())
+	{
+		return;
+	}
+	if(!m_bValidMouseClick)
+	{
+		pView->getLeftRulerInfo(&m_infoCache);
+	}
 	UT_ASSERT(m_infoCache.m_yTopMargin >= 0);
 
 	// if they drag vertically off the ruler, we ignore the whole thing.
@@ -709,8 +893,11 @@ void AP_LeftRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32
 	
 		UT_Rect rTopMargin, rBottomMargin;
 		_getMarginMarkerRects(&m_infoCache,rTopMargin,rBottomMargin);
+		rTopMargin.width = getWidth();
+		rBottomMargin.width = getWidth();
 		if (rTopMargin.containsPoint(x,y))
 		{
+			UT_DEBUGMSG(("In Top Margin box \n"));
 			if(m_pG)
 			{
 				m_pG->setCursor( GR_Graphics::GR_CURSOR_UPDOWN);
@@ -718,6 +905,7 @@ void AP_LeftRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32
 		}
 		else if (rBottomMargin.containsPoint(x,y))
 		{
+			UT_DEBUGMSG(("In Bottom Margin box \n"));
 			if(m_pG)
 			{
 				m_pG->setCursor( GR_Graphics::GR_CURSOR_UPDOWN);
@@ -802,7 +990,7 @@ void AP_LeftRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32
 		fl_HdrFtrShadow * pShadow = pView->getEditShadow();
 
 		bool hdr = (hdrftr && 
-					pShadow->getHdrFtrSectionLayout()->getHFType() == FL_HDRFTR_HEADER);
+					pShadow->getHdrFtrSectionLayout()->getHFType() < FL_HDRFTR_FOOTER);
 
 		UT_sint32 oldDragCenter = m_draggingCenter;
 
@@ -848,6 +1036,16 @@ void AP_LeftRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32
 			return;
 		}
 
+		if (m_draggingWhat == DW_TOPMARGIN)
+		{
+			m_infoCache.m_yTopMargin += m_draggingCenter - oldDragCenter;
+		}
+		if (m_draggingWhat == DW_BOTTOMMARGIN)
+		{
+			m_infoCache.m_yBottomMargin -= m_draggingCenter - oldDragCenter;
+			UT_DEBUGMSG(("Dragging bottom margin new value %d \n",	m_infoCache.m_yBottomMargin));
+		}
+				
 		draw(NULL, &m_infoCache);
 		_xorGuide();
 		m_bBeforeFirstMotion = false;
@@ -1011,6 +1209,12 @@ bool AP_LeftRuler::notify(AV_View * pView, const AV_ChangeMask mask)
 	// Handle AV_Listener events on the view.
 
 	UT_ASSERT(pView==m_pView);
+	FV_View * pVView = static_cast<FV_View *>(m_pView);
+
+	if(pVView->getDocument() == NULL)
+	{
+		return false;
+	}
 
 	// If the caret has moved to a different page or any of the properties
 	// on the page (such as the margins) have changed, we force a redraw.
@@ -1052,7 +1256,11 @@ void AP_LeftRuler::scrollRuler(UT_sint32 yoff, UT_sint32 ylimit)
 {
 	UT_Rect rClip;
 	UT_Rect * prClip;
-
+	FV_View * pView = static_cast<FV_View *>(m_pView);
+	if(pView->getDocument() == NULL)
+	{
+		return;
+	}
 	if (ylimit > 0)
 		m_yScrollLimit = ylimit;
 
@@ -1128,51 +1336,54 @@ void AP_LeftRuler::_getMarginMarkerRects(AP_LeftRulerInfo * pInfo, UT_Rect &rTop
 void AP_LeftRuler::_drawMarginProperties(const UT_Rect * /* pClipRect */,
 										AP_LeftRulerInfo * pInfo, GR_Graphics::GR_Color3D clr)
 {
-	FV_View *pView = static_cast<FV_View *>(m_pView);
-	bool hdrftr = pView->isHdrFtrEdit();
+	//FV_View *pView = static_cast<FV_View *>(m_pView);
+	//bool hdrftr = pView->isHdrFtrEdit();
 	if(m_pG == NULL)
 	{
 		return;
 	}
-	fl_HdrFtrShadow * pShadow = pView->getEditShadow();
+	//fl_HdrFtrShadow * pShadow = pView->getEditShadow();
+
+	//bool hdr = (hdrftr && 
+	//			pShadow->getHdrFtrSectionLayout()->getHFType() < FL_HDRFTR_FOOTER);
 
-	bool hdr = (hdrftr && 
-				pShadow->getHdrFtrSectionLayout()->getHFType() == FL_HDRFTR_HEADER);
 
 	UT_Rect rTop, rBottom;
 	UT_uint32 onePX = m_pG->tlu(1);
 
 	_getMarginMarkerRects(pInfo,rTop,rBottom);
 	
-	m_pG->fillRect(GR_Graphics::CLR3D_Background, rTop);
+	GR_Painter painter(m_pG);
+
+	painter.fillRect(GR_Graphics::CLR3D_Background, rTop);
 
 	m_pG->setColor3D(GR_Graphics::CLR3D_Foreground);
-	m_pG->drawLine( rTop.left,  rTop.top, rTop.left + rTop.width, rTop.top);
-	m_pG->drawLine( rTop.left + rTop.width,  rTop.top, rTop.left + rTop.width, rTop.top + rTop.height);
-	m_pG->drawLine( rTop.left + rTop.width,  rTop.top + rTop.height, rTop.left, rTop.top + rTop.height);
-	m_pG->drawLine( rTop.left,  rTop.top + rTop.height, rTop.left, rTop.top);
+	painter.drawLine( rTop.left,  rTop.top, rTop.left + rTop.width, rTop.top);
+	painter.drawLine( rTop.left + rTop.width,  rTop.top, rTop.left + rTop.width, rTop.top + rTop.height);
+	painter.drawLine( rTop.left + rTop.width,  rTop.top + rTop.height, rTop.left, rTop.top + rTop.height);
+	painter.drawLine( rTop.left,  rTop.top + rTop.height, rTop.left, rTop.top);
 	m_pG->setColor3D(GR_Graphics::CLR3D_BevelUp);
-	m_pG->drawLine( rTop.left + onePX,  rTop.top + onePX, rTop.left + rTop.width - onePX, rTop.top + onePX);
-	m_pG->drawLine( rTop.left + onePX,  rTop.top + rTop.height - m_pG->tlu(2), rTop.left + onePX, rTop.top + onePX);
+	painter.drawLine( rTop.left + onePX,  rTop.top + onePX, rTop.left + rTop.width - onePX, rTop.top + onePX);
+	painter.drawLine( rTop.left + onePX,  rTop.top + rTop.height - m_pG->tlu(2), rTop.left + onePX, rTop.top + onePX);
 	
 	// TODO: this isn't the right place for this logic. But it works.
-	if (hdrftr && !hdr)
-		return;
+//	if (hdrftr && !hdr)
+//		return;
 
-	m_pG->fillRect(GR_Graphics::CLR3D_Background, rBottom);
+	painter.fillRect(GR_Graphics::CLR3D_Background, rBottom);
 
 	m_pG->setColor3D(GR_Graphics::CLR3D_Foreground);
-	m_pG->drawLine( rBottom.left,  rBottom.top, rBottom.left + rBottom.width, rBottom.top);
-	m_pG->drawLine( rBottom.left + rBottom.width,  rBottom.top, rBottom.left + rBottom.width, rBottom.top + rBottom.height);
-	m_pG->drawLine( rBottom.left + rBottom.width,  rBottom.top + rBottom.height, rBottom.left, rBottom.top + rBottom.height);
-	m_pG->drawLine( rBottom.left,  rBottom.top + rBottom.height, rBottom.left, rBottom.top);
+	painter.drawLine( rBottom.left,  rBottom.top, rBottom.left + rBottom.width, rBottom.top);
+	painter.drawLine( rBottom.left + rBottom.width,  rBottom.top, rBottom.left + rBottom.width, rBottom.top + rBottom.height);
+	painter.drawLine( rBottom.left + rBottom.width,  rBottom.top + rBottom.height, rBottom.left, rBottom.top + rBottom.height);
+	painter.drawLine( rBottom.left,  rBottom.top + rBottom.height, rBottom.left, rBottom.top);
 	m_pG->setColor3D(GR_Graphics::CLR3D_BevelUp);
-	m_pG->drawLine( rBottom.left + onePX,  rBottom.top + onePX, rBottom.left + rBottom.width - onePX, rBottom.top + onePX);
-	m_pG->drawLine( rBottom.left + onePX,  rBottom.top + rBottom.height - m_pG->tlu(2), rBottom.left + onePX, rBottom.top + onePX);
+	painter.drawLine( rBottom.left + onePX,  rBottom.top + onePX, rBottom.left + rBottom.width - onePX, rBottom.top + onePX);
+	painter.drawLine( rBottom.left + onePX,  rBottom.top + rBottom.height - m_pG->tlu(2), rBottom.left + onePX, rBottom.top + onePX);
 #if 0
     m_pG->setColor3D(GR_Graphics::CLR3D_BevelDown);
-	m_pG->drawLine( rBottom.left + rBottom.width - onePX,  rBottom.top + onePX, rBottom.left + rBottom.width - onePX, rBottom.top + rBottom.height - onePX);
-	m_pG->drawLine( rBottom.left + rBottom.width - onePX,  rBottom.top + rBottom.height - onePX, rBottom.left + onePX, rBottom.top + rBottom.height - onePX);
+	painter.drawLine( rBottom.left + rBottom.width - onePX,  rBottom.top + onePX, rBottom.left + rBottom.width - onePX, rBottom.top + rBottom.height - onePX);
+	painter.drawLine( rBottom.left + rBottom.width - onePX,  rBottom.top + rBottom.height - onePX, rBottom.left + onePX, rBottom.top + rBottom.height - onePX);
 #endif
 }
 
@@ -1212,6 +1423,7 @@ void AP_LeftRuler::_getCellMarkerRects(AP_LeftRulerInfo * pInfo, UT_sint32 iCell
 	UT_sint32 yOrigin = pInfo->m_yPageStart - m_yScrollOffset;
 	UT_sint32 pos =0;
 	fp_TableContainer * pTab = static_cast<fp_TableContainer *>(pLInfo->m_pCell->getContainer());
+	UT_return_if_fail(pTab);
 	fp_Page * pPage = NULL;
 	if(pBroke == NULL)
 	{
@@ -1242,9 +1454,17 @@ void AP_LeftRuler::_getCellMarkerRects(AP_LeftRulerInfo * pInfo, UT_sint32 iCell
 		rCell.set(0,0,0,0);
 		return;
 	}
-	fp_Column * pCol = static_cast<fp_Column *>(pBroke->getColumn());
-	UT_sint32 iColOffset = pCol->getY();
-	yOrigin += iColOffset;
+	if(!pView->isInFrame(pView->getPoint()))
+	{
+		fp_Column * pCol = static_cast<fp_Column *>(pBroke->getColumn());
+		UT_sint32 iColOffset = pCol->getY();
+		yOrigin += iColOffset;
+	}
+	else
+	{
+		fp_FrameContainer * pFC = static_cast<fp_FrameContainer *>(pView->getFrameLayout()->getFirstContainer());
+		yOrigin += pFC->getY();
+	}
 	UT_sint32 yoff = pBroke->getYBreak();
 	UT_sint32 yTab = 0;
 	if(pBroke->getYBreak() == 0)
@@ -1330,7 +1550,7 @@ void AP_LeftRuler::_drawCellProperties(AP_LeftRulerInfo * pInfo)
 	UT_sint32 nrows = pInfo->m_iNumRows;
 	UT_sint32 i = 0;
 	UT_Rect rCell;
-	UT_DEBUGMSG(("ap_LeftRuler: Draw Cell Marks start \n"));
+	xxx_UT_DEBUGMSG(("ap_LeftRuler: Draw Cell Marks start \n"));
 	fp_Page * pCurPage =  static_cast<FV_View *>(m_pView)->getCurrentPage();
 	PT_DocPosition pos = static_cast<FV_View *>(m_pView)->getPoint();
 	bool bStop = false;
@@ -1372,7 +1592,7 @@ void AP_LeftRuler::_drawCellProperties(AP_LeftRulerInfo * pInfo)
 			bStop = true;
 		}
 	}
-	UT_DEBUGMSG(("ap_LeftRuler: Draw Cell Marks end \n"));
+	xxx_UT_DEBUGMSG(("ap_LeftRuler: Draw Cell Marks end \n"));
 }
 
 void AP_LeftRuler::_drawCellMark(UT_Rect *prDrag, bool bUp)
@@ -1384,35 +1604,49 @@ void AP_LeftRuler::_drawCellMark(UT_Rect *prDrag, bool bUp)
 	{
 		return;
 	}
+
+	GR_Painter painter(m_pG);
+
 	UT_sint32 left = prDrag->left;
 	UT_sint32 right = left + prDrag->width - m_pG->tlu(1);
 	UT_sint32 top = prDrag->top;
 	UT_sint32 bot = top + prDrag->height - m_pG->tlu(1); // For the clever people: this gives the rect a height of 5 pixels (eg. top:10, bot:14 is 5 pixels)!
 	
-	m_pG->fillRect(GR_Graphics::CLR3D_Background, left, top, prDrag->width, prDrag->height);
+	painter.fillRect(GR_Graphics::CLR3D_Background, left, top, prDrag->width, prDrag->height);
 	
 	m_pG->setColor3D(GR_Graphics::CLR3D_Foreground);
-	m_pG->drawLine(left,top,right,top);
-	m_pG->drawLine(left,top,left,bot);
-	m_pG->drawLine(left,bot,right,bot);
-	m_pG->drawLine(right,top,right,bot);
+	painter.drawLine(left,top,right,top);
+	painter.drawLine(left,top,left,bot);
+	painter.drawLine(left,bot,right,bot);
+	painter.drawLine(right,top,right,bot);
 	
 	m_pG->setColor3D(GR_Graphics::CLR3D_BevelUp);
-	m_pG->drawLine( left + m_pG->tlu(1), top + m_pG->tlu(1), right - m_pG->tlu(1), top + m_pG->tlu(1));
-	m_pG->drawLine( left + m_pG->tlu(1), top + m_pG->tlu(1), left + m_pG->tlu(1), bot - m_pG->tlu(1));
+	painter.drawLine( left + m_pG->tlu(1), top + m_pG->tlu(1), right - m_pG->tlu(1), top + m_pG->tlu(1));
+	painter.drawLine( left + m_pG->tlu(1), top + m_pG->tlu(1), left + m_pG->tlu(1), bot - m_pG->tlu(1));
 }
 
 /*****************************************************************/
 
 void AP_LeftRuler::draw(const UT_Rect * pClipRect)
 {
-	if (!m_pView)
+
+	FV_View * pView = static_cast<FV_View *>(m_pView);
+	if (!pView)
 		return;
-	if(m_pView->getPoint() == 0)
+	if(pView->getPoint() == 0)
 	{
 		return;
 	}
-	(static_cast<FV_View *>(m_pView))->getLeftRulerInfo(&m_tempInfo);
+
+	if(pView->getDocument() == NULL)
+	{
+		return;
+	}
+	if(pView->getDocument()->isPieceTableChanging())
+	{
+		return;
+	}
+	pView->getLeftRulerInfo(&m_tempInfo);
 	
 // lfi will be reference to m_lfi which will be deleted before taking this.
 // value. This prevents a memory leak.
@@ -1424,6 +1658,9 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 {
 	if (!m_pG)
 		return;
+
+	GR_Painter painter(m_pG);
+
 	UT_ASSERT(lfi->m_yTopMargin >= 0);
 	UT_Rect r;
 	UT_Rect * pClipRect = NULL;
@@ -1449,10 +1686,8 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 	UT_ASSERT(m_iWidth);
 	// draw the background
 	
-	m_pG->fillRect(GR_Graphics::CLR3D_Background,0,0,
-		       getWidth(),getHeight());
-
-	// draw a dark-gray and white bar lined up with the paper
+	painter.fillRect(GR_Graphics::CLR3D_Background,0,0,
+					 getWidth(),getHeight());
 
 	UT_uint32 xLeft = m_pG->tlu(s_iFixedWidth)/4;
 	UT_uint32 xBar  = m_pG->tlu(s_iFixedWidth)/2;
@@ -1471,7 +1706,7 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 
 		y = yScrolledOrigin;
 		h = lfi->m_yTopMargin - m_pG->tlu(1);
-		m_pG->fillRect(GR_Graphics::CLR3D_BevelDown,xLeft,y,xBar,h);
+		painter.fillRect(GR_Graphics::CLR3D_BevelDown,xLeft,y,xBar,h);
 	}
 
 	yScrolledOrigin += lfi->m_yTopMargin + m_pG->tlu(1);
@@ -1482,7 +1717,7 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 
 		y = yScrolledOrigin;
 		h = docWithinMarginHeight - m_pG->tlu(1);
-		m_pG->fillRect(GR_Graphics::CLR3D_Highlight,xLeft,y,xBar,h);
+		painter.fillRect(GR_Graphics::CLR3D_Highlight,xLeft,y,xBar,h);
 	}
 
 	yScrolledOrigin += docWithinMarginHeight + m_pG->tlu(1);
@@ -1494,7 +1729,7 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 
 		y = yScrolledOrigin;
 		h = lfi->m_yBottomMargin - m_pG->tlu(1);
-		m_pG->fillRect(GR_Graphics::CLR3D_BevelDown,xLeft,y,xBar,h);
+		painter.fillRect(GR_Graphics::CLR3D_BevelDown,xLeft,y,xBar,h);
 	}
 
 	// draw 3D frame around top margin + document + bottom margin rects
@@ -1512,7 +1747,7 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 	if (pFont)
 	{
 		m_pG->setFont(pFont);
-		iFontHeight = m_pG->getFontHeight();
+		iFontHeight = m_pG->getFontHeight() * 100 / m_pG->getZoomPercentage();
 	}
 
 	// first draw the top margin
@@ -1527,7 +1762,7 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 				UT_uint32 w = ((k % tick.tickLong) 
 							   ? m_pG->tlu(2) : m_pG->tlu(6));
 				UT_uint32 x = xLeft + (xBar-w)/2;
-				m_pG->drawLine(x,y,x+w,y);
+				painter.drawLine(x,y,x+w,y);
 			}
 			else if (pFont)
 			{
@@ -1543,10 +1778,10 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 				UT_UCS4_strcpy_char(span, buf);
 				UT_uint32 len = strlen(buf);
 
-				UT_uint32 w = m_pG->measureString(span, 0, len, charWidths);
+				UT_uint32 w = m_pG->measureString(span, 0, len, charWidths) * 100 / m_pG->getZoomPercentage();
 				UT_uint32 x = xLeft + (xBar-w)/2;
 
-				m_pG->drawChars(span, 0, len, x, y - iFontHeight/2);
+				painter.drawChars(span, 0, len, x, y - iFontHeight/2);
 			}
 		}
 	}
@@ -1554,7 +1789,7 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 	m_pG->setColor3D(GR_Graphics::CLR3D_Foreground);	
 	
 	// draw everything below the top margin
-	for (k=1; (k*tick.tickUnit/tick.tickUnitScale < (lfi->m_yPageSize - lfi->m_yTopMargin)); k++)
+	for (k=1; (static_cast<UT_sint32>(k*tick.tickUnit/tick.tickUnitScale) < (static_cast<UT_sint32>(lfi->m_yPageSize) - static_cast<UT_sint32>(lfi->m_yTopMargin))); k++)
 	{
 		y = yOrigin + lfi->m_yTopMargin + k*tick.tickUnit/tick.tickUnitScale - m_yScrollOffset;
 		if (y >= 0)
@@ -1565,7 +1800,7 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 				UT_uint32 w = ((k % tick.tickLong) ? 
 							   m_pG->tlu(2) : m_pG->tlu(6));
 				UT_uint32 x = xLeft + (xBar-w)/2;
-				m_pG->drawLine(x,y,x+w,y);
+				painter.drawLine(x,y,x+w,y);
 			}
 			else if (pFont)
 			{
@@ -1581,14 +1816,14 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 				UT_UCS4_strcpy_char(span, buf);
 				UT_uint32 len = strlen(buf);
 
-				UT_uint32 w = m_pG->measureString(span, 0, len, charWidths);
+				UT_uint32 w = m_pG->measureString(span, 0, len, charWidths) * 100 / m_pG->getZoomPercentage();
 				UT_uint32 x = xLeft + (xBar-w)/2;
 
-				m_pG->drawChars(span, 0, len, x, y - iFontHeight/2);
+				painter.drawChars(span, 0, len, x, y - iFontHeight/2);
 			}
 		}
 	}
-	
+
 	//
 	// draw the various widgets for the left ruler
 	// 
@@ -1605,6 +1840,7 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 		m_pG->setClipRect(NULL);
 	}
 	m_lfi = lfi;
+
 }
 
 /*****************************************************************/
@@ -1612,16 +1848,22 @@ void AP_LeftRuler::draw(const UT_Rect * pCR, AP_LeftRulerInfo * lfi)
 void AP_LeftRuler::_xorGuide(bool bClear)
 {
 	UT_sint32 y = m_draggingCenter;
-
 	GR_Graphics * pG = (static_cast<FV_View *>(m_pView))->getGraphics();
 	UT_ASSERT(pG);
 
+	GR_Painter painter(pG);
+
 	// TODO we need to query the document window to see what the actual
 	// TODO background color is so that we can compose the proper color so
 	// TODO that we can XOR on it and be guaranteed that it will show up.
 
+#if XAP_DONTUSE_XOR
+	UT_RGBColor clrBlack(0,0,0);
+	pG->setColor(clrBlack);
+#else
 	UT_RGBColor clrWhite(255,255,255);
 	pG->setColor(clrWhite);
+#endif
 
 	UT_sint32 w = m_pView->getWindowWidth();
 	
@@ -1631,15 +1873,33 @@ void AP_LeftRuler::_xorGuide(bool bClear)
 			return;		// avoid flicker
 
 		// erase old guide
-		pG->xorLine(0, m_yGuide, w, m_yGuide);
+#if XAP_DONTUSE_XOR
+		if (m_guideCache) {
+			painter.drawImage(m_guideCache, m_guideCacheRect.left, m_guideCacheRect.top);
+			DELETEP(m_guideCache);
+		}
+#else
+		painter.xorLine(0, m_yGuide, w, m_yGuide);
+#endif
 		m_bGuide = false;
 	}
 
 	if (!bClear)
 	{
 		UT_ASSERT(m_bValidMouseClick);
-		pG->xorLine(0, y, w, y);
 
+#if XAP_DONTUSE_XOR
+		m_guideCacheRect.left = 0;
+		m_guideCacheRect.top = y - pG->tlu(1);
+		m_guideCacheRect.width = w;
+		m_guideCacheRect.height = pG->tlu(3);
+		DELETEP(m_guideCache);		// make sure it is deleted. we could leak it here
+		m_guideCache = painter.genImageFromRectangle(m_guideCacheRect);
+
+		painter.drawLine(0, y, w, y);
+#else
+		painter.xorLine(0, y, w, y);
+#endif
 		// remember this for next time
 		m_yGuide = y;
 		m_bGuide = true;
@@ -1652,7 +1912,7 @@ void AP_LeftRuler::_xorGuide(bool bClear)
 	UT_ASSERT( data && pPrefs );
 
 	const XML_Char *pszBuffer;
-	pPrefs->getPrefsValue(static_cast<XML_Char*>(AP_PREF_KEY_RulerUnits), &pszBuffer );
+	pPrefs->getPrefsValue(static_cast<const XML_Char *>(AP_PREF_KEY_RulerUnits), &pszBuffer );
 
 	// or should I just default to inches or something?
 	UT_Dimension dim = UT_determineDimension( pszBuffer, DIM_none );
