@@ -42,6 +42,8 @@
 #include "ut_Script.h"
 #include "spell_manager.h"
 #include "ie_mailmerge.h"
+#include "fp_TableContainer.h"
+#include "fl_BlockLayout.h"
 
 #ifdef _WIN32
 #include "ap_Win32App.h" 
@@ -54,113 +56,112 @@
 
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Autotext)
 {
-	UT_ASSERT(pFrame);
-	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
-	UT_ASSERT(pLabel);
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp && pLabel, NULL);
 
 	const char * c = NULL;
 
 	const XAP_StringSet * pss = pApp->getStringSet();
-	c = pss->getValueUTF8(AP_STRING_ID_DLG_Spell_NoSuggestions).utf8_str();
+	static UT_UTF8String s;
+	pss->getValueUTF8(AP_STRING_ID_DLG_Spell_NoSuggestions,s);
 
 	switch (id)
 	  {
 	  case AP_MENU_ID_AUTOTEXT_ATTN_1:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_ATTN_1).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_ATTN_1, s); break;
 	  case AP_MENU_ID_AUTOTEXT_ATTN_2:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_ATTN_2).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_ATTN_2, s); break;
 
 	  case AP_MENU_ID_AUTOTEXT_CLOSING_1:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_1).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_1, s); break;
 	  case AP_MENU_ID_AUTOTEXT_CLOSING_2:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_2).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_2, s); break;
 	  case AP_MENU_ID_AUTOTEXT_CLOSING_3:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_3).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_3, s); break;
 	  case AP_MENU_ID_AUTOTEXT_CLOSING_4:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_4).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_4, s); break;
 	  case AP_MENU_ID_AUTOTEXT_CLOSING_5:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_5).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_5, s); break;
 	  case AP_MENU_ID_AUTOTEXT_CLOSING_6:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_6).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_6, s); break;
 	  case AP_MENU_ID_AUTOTEXT_CLOSING_7:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_7).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_7, s); break;
 	  case AP_MENU_ID_AUTOTEXT_CLOSING_8:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_8).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_8, s); break;
 	  case AP_MENU_ID_AUTOTEXT_CLOSING_9:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_9).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_9, s); break;
 	  case AP_MENU_ID_AUTOTEXT_CLOSING_10:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_10).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_10, s); break;
 	  case AP_MENU_ID_AUTOTEXT_CLOSING_11:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_11).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_11, s); break;
 	  case AP_MENU_ID_AUTOTEXT_CLOSING_12:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_12).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_CLOSING_12, s); break;
 
 	  case AP_MENU_ID_AUTOTEXT_MAIL_1:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_1).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_1, s); break;
 	  case AP_MENU_ID_AUTOTEXT_MAIL_2:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_2).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_2, s); break;
 	  case AP_MENU_ID_AUTOTEXT_MAIL_3:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_3).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_3, s); break;
 	  case AP_MENU_ID_AUTOTEXT_MAIL_4:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_4).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_4, s); break;
 	  case AP_MENU_ID_AUTOTEXT_MAIL_5:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_5).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_5, s); break;
 	  case AP_MENU_ID_AUTOTEXT_MAIL_6:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_6).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_6, s); break;
 	  case AP_MENU_ID_AUTOTEXT_MAIL_7:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_7).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_7, s); break;
 	  case AP_MENU_ID_AUTOTEXT_MAIL_8:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_8).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_MAIL_8, s); break;
 
 	  case AP_MENU_ID_AUTOTEXT_REFERENCE_1:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_REFERENCE_1).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_REFERENCE_1, s); break;
 	  case AP_MENU_ID_AUTOTEXT_REFERENCE_2:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_REFERENCE_2).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_REFERENCE_2, s); break;
 	  case AP_MENU_ID_AUTOTEXT_REFERENCE_3:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_REFERENCE_3).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_REFERENCE_3, s); break;
 
 	  case AP_MENU_ID_AUTOTEXT_SALUTATION_1:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_SALUTATION_1).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_SALUTATION_1, s); break;
 	  case AP_MENU_ID_AUTOTEXT_SALUTATION_2:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_SALUTATION_2).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_SALUTATION_2, s); break;
 	  case AP_MENU_ID_AUTOTEXT_SALUTATION_3:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_SALUTATION_3).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_SALUTATION_3, s); break;
 	  case AP_MENU_ID_AUTOTEXT_SALUTATION_4:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_SALUTATION_4).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_SALUTATION_4, s); break;
 
 	  case AP_MENU_ID_AUTOTEXT_SUBJECT_1:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_SUBJECT_1).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_SUBJECT_1, s); break;
 
 	  case AP_MENU_ID_AUTOTEXT_EMAIL_1:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_EMAIL_1).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_EMAIL_1, s); break;
 	  case AP_MENU_ID_AUTOTEXT_EMAIL_2:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_EMAIL_2).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_EMAIL_2, s); break;
 	  case AP_MENU_ID_AUTOTEXT_EMAIL_3:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_EMAIL_3).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_EMAIL_3, s); break;
 	  case AP_MENU_ID_AUTOTEXT_EMAIL_4:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_EMAIL_4).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_EMAIL_4, s); break;
 	  case AP_MENU_ID_AUTOTEXT_EMAIL_5:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_EMAIL_5).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_EMAIL_5, s); break;
 	  case AP_MENU_ID_AUTOTEXT_EMAIL_6:
-	    c = pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_EMAIL_6).utf8_str(); break;
+	    pss->getValueUTF8(AP_STRING_ID_AUTOTEXT_EMAIL_6, s); break;
 
 	  default:
 	    c = "No clue"; break;
 	  }
 
+	if(!c) 	c = s.utf8_str();
+
 	return c;
 }
 
 Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Toolbar)
 {
-	UT_ASSERT(pFrame);
-	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
-	UT_ASSERT(pLabel);
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp && pLabel, NULL);
 
-	UT_ASSERT(id >= AP_MENU_ID_VIEW_TB_1);
-	UT_ASSERT(id <= AP_MENU_ID_VIEW_TB_4);
+	UT_ASSERT_HARMLESS(id >= AP_MENU_ID_VIEW_TB_1);
+	UT_ASSERT_HARMLESS(id <= AP_MENU_ID_VIEW_TB_4);
 
 	UT_uint32 ndx = (id - AP_MENU_ID_VIEW_TB_1);
 	const UT_Vector & vec = pApp->getToolbarFactory()->getToolbarNames();
@@ -187,18 +188,16 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Recent)
 	// We return a pointer to a static string (which will be overwritten
 	// on the next call).
 
-	UT_ASSERT(pFrame);
-	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
-	UT_ASSERT(pLabel);
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp && pLabel, NULL);
 
-	UT_ASSERT(id >= AP_MENU_ID_FILE_RECENT_1);
-	UT_ASSERT(id <= AP_MENU_ID_FILE_RECENT_9);
+	UT_ASSERT_HARMLESS(id >= AP_MENU_ID_FILE_RECENT_1);
+	UT_ASSERT_HARMLESS(id <= AP_MENU_ID_FILE_RECENT_9);
 
 	UT_uint32 ndx = (id - AP_MENU_ID_FILE_RECENT_1 + 1);
 
 	XAP_Prefs * pPrefs = pApp->getPrefs();
-	UT_ASSERT(pPrefs);
+	UT_return_val_if_fail (pPrefs, NULL);
 
 	if (ndx <= pPrefs->getRecentCount())
 	{
@@ -221,12 +220,10 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_About)
 {
 	// Compute the menu label for the _help_about item.
 
-	UT_ASSERT(pFrame);
-	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
-	UT_ASSERT(pLabel);
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp && pLabel, NULL);
 
-	UT_ASSERT(id == AP_MENU_ID_HELP_ABOUT);
+	UT_ASSERT_HARMLESS(id == AP_MENU_ID_HELP_ABOUT);
 
 	const char * szFormat = pLabel->getMenuLabel();
 	static char buf[128];
@@ -241,12 +238,10 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Contents)
 {
 	// Compute the menu label for the _help_contents item.
 
-	UT_ASSERT(pFrame);
-	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
-	UT_ASSERT(pLabel);
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp && pLabel, NULL);
 
-	UT_ASSERT(id == AP_MENU_ID_HELP_CONTENTS);
+	UT_ASSERT_HARMLESS(id == AP_MENU_ID_HELP_CONTENTS);
 
 	const char * szFormat = pLabel->getMenuLabel();
 	static char buf[128];
@@ -261,12 +256,10 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Index)
 {
 	// Compute the menu label for the _help_index item.
 
-	UT_ASSERT(pFrame);
-	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
-	UT_ASSERT(pLabel);
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp && pLabel, NULL);
 
-	UT_ASSERT(id == AP_MENU_ID_HELP_INDEX);
+	UT_ASSERT_HARMLESS(id == AP_MENU_ID_HELP_INDEX);
 
 	const char * szFormat = pLabel->getMenuLabel();
 	static char buf[128];
@@ -281,12 +274,10 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Search)
 {
 	// Compute the menu label for the _help_search item.
 
-	UT_ASSERT(pFrame);
-	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
-	UT_ASSERT(pLabel);
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp && pLabel, NULL);
 
-	UT_ASSERT(id == AP_MENU_ID_HELP_SEARCH);
+	UT_ASSERT_HARMLESS(id == AP_MENU_ID_HELP_SEARCH);
 
 	const char * szFormat = pLabel->getMenuLabel();
 	static char buf[128];
@@ -301,12 +292,10 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_AboutOS)
 {
 	// Compute the menu label for the about OS help item.
 
-	UT_ASSERT(pFrame);
-	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
-	UT_ASSERT(pLabel);
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp && pLabel, NULL);
 
-	UT_ASSERT(id == AP_MENU_ID_HELP_ABOUTOS);
+	UT_ASSERT_HARMLESS(id == AP_MENU_ID_HELP_ABOUTOS);
 
 	const char * szFormat = pLabel->getMenuLabel();
 	static char buf[128];
@@ -321,12 +310,10 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Checkver)
 {
 	// Compute the menu label for the about the check version item.
 
-	UT_ASSERT(pFrame);
-	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
-	UT_ASSERT(pLabel);
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp && pLabel, NULL);
 
-	UT_ASSERT(id == AP_MENU_ID_HELP_CHECKVER);
+	UT_ASSERT_HARMLESS(id == AP_MENU_ID_HELP_CHECKVER);
 
 	const char * szFormat = pLabel->getMenuLabel();
 	static char buf[128];
@@ -341,19 +328,19 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Checkver)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Window)
 {
-	UT_ASSERT(pAV_View);
+	UT_return_val_if_fail (pAV_View, EV_MIS_Gray);
 
-	UT_ASSERT(id >= AP_MENU_ID_WINDOW_1);
-	UT_ASSERT(id <= AP_MENU_ID_WINDOW_9);
+	UT_ASSERT_HARMLESS(id >= AP_MENU_ID_WINDOW_1);
+	UT_ASSERT_HARMLESS(id <= AP_MENU_ID_WINDOW_9);
 
 	UT_uint32 ndx = (id - AP_MENU_ID_WINDOW_1);
 
 	EV_Menu_ItemState s = EV_MIS_ZERO;
 
 	XAP_Frame * pFrame = static_cast<XAP_Frame *>(pAV_View->getParentData());
-	UT_ASSERT(pFrame);
+	UT_return_val_if_fail (pFrame, EV_MIS_Gray);
 	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
+	UT_return_val_if_fail (pApp, EV_MIS_Gray);
 
 	if (pFrame == pApp->getFrame(ndx))
 		s = EV_MIS_Toggled;
@@ -367,14 +354,11 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Window)
 	// We return a pointer to a static string (which will be overwritten
 	// on the next call).
 
-	UT_ASSERT(pFrame);
-	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
-	UT_ASSERT(pLabel);
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp && pLabel, NULL);
 
-
-	UT_ASSERT(id >= AP_MENU_ID_WINDOW_1);
-	UT_ASSERT(id <= AP_MENU_ID_WINDOW_9);
+	UT_ASSERT_HARMLESS(id >= AP_MENU_ID_WINDOW_1);
+	UT_ASSERT_HARMLESS(id <= AP_MENU_ID_WINDOW_9);
 
 	UT_uint32 ndx = (id - AP_MENU_ID_WINDOW_1);
 
@@ -388,7 +372,7 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Window)
 		static char buf[128];
 
 		XAP_Frame * pFrame = pApp->getFrame(ndx);
-		UT_ASSERT(pFrame);
+		UT_return_val_if_fail (pFrame, NULL);
 
 		const char * szTitle = pFrame->getTitle(128 - strlen(szFormat));
 
@@ -406,11 +390,9 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_WindowMore)
 {
 	// Compute the menu label for the _window_more ("More Windows...") item.
 
-	UT_ASSERT(pFrame);
-	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
-	UT_ASSERT(pLabel);
-	UT_ASSERT(id == AP_MENU_ID_WINDOW_MORE);
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp && pLabel, NULL);
+	UT_ASSERT_HARMLESS(id == AP_MENU_ID_WINDOW_MORE);
 
 	// if we have more than 9 windows in our window list,
 	// we return the static menu label.  if not, we return
@@ -430,10 +412,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Spelling)
   EV_Menu_ItemState s = EV_MIS_ZERO ;
 
   XAP_Prefs *pPrefs = XAP_App::getApp()->getPrefs();
-  UT_ASSERT( pPrefs );
+  UT_return_val_if_fail (pPrefs, EV_MIS_Gray);
 
   bool b = true ;
-  pPrefs->getPrefsValueBool(static_cast<XML_Char*>(AP_PREF_KEY_AutoSpellCheck),&b) ;
+  pPrefs->getPrefsValueBool(static_cast<const XML_Char *>(AP_PREF_KEY_AutoSpellCheck),&b) ;
 
   // if there are no loaded dictionaries and we are spell checking
   // as we type
@@ -448,20 +430,122 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Spelling)
 Defun_EV_GetMenuItemState_Fn(ap_GetState_ColumnsActive)
 {
   ABIWORD_VIEW ;
-  UT_ASSERT(pView) ;
+  UT_return_val_if_fail (pView, EV_MIS_Gray);
 
   EV_Menu_ItemState s = EV_MIS_ZERO ;
 
-  if(pView->isHdrFtrEdit())
+  if(pView->isHdrFtrEdit()  || pView->isInHdrFtr(pView->getPoint()))
     s = EV_MIS_Gray;
 
   return s;
 }
 
+Defun_EV_GetMenuItemState_Fn(ap_GetState_BookmarkOK)
+{
+  ABIWORD_VIEW ;
+  UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	EV_Menu_ItemState s = EV_MIS_ZERO ;
+
+	if(pView->isTOCSelected())
+	{
+	    s = EV_MIS_Gray ;
+		return s;
+	}
+	PT_DocPosition posStart = pView->getPoint();
+	PT_DocPosition posEnd = pView->getSelectionAnchor();
+	fl_BlockLayout * pBL1 = pView->getBlockAtPosition(posStart);
+	fl_BlockLayout * pBL2 = pView->getBlockAtPosition(posEnd);
+	if((pBL1 == NULL) || (pBL2 == NULL)) // make sure we get valid blocks from selection beginning and end
+	{
+		s = EV_MIS_Gray;
+		return s;
+	}
+	if(pBL1 != pBL2) // don't allow Insert bookmark if selection spans multiple blocks
+	{
+	    s = EV_MIS_Gray ;
+		return s;
+	}
+
+	return s ;
+}
+
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_TOCOK)
+{
+  ABIWORD_VIEW ;
+  UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+  EV_Menu_ItemState s = EV_MIS_ZERO ;
+
+  if(pView->isHdrFtrEdit()  || pView->isInHdrFtr(pView->getPoint()))
+  {
+    s = EV_MIS_Gray;
+  }
+  if(pView->isInHdrFtr(pView->getPoint()))
+  {
+    s = EV_MIS_Gray;
+  }
+
+  else if(pView->isInTable()) // isintable includes first
+  {
+    s = EV_MIS_Gray;
+  }
+  else if(pView->isInTable(pView->getSelectionAnchor())) // isintable includes first
+  {
+    s = EV_MIS_Gray;
+  }
+  else if(pView->isInFrame(pView->getPoint()))
+  {
+    s = EV_MIS_Gray;
+  }
+  else if(pView->getFrameEdit()->isActive())
+  {
+    s = EV_MIS_Gray;
+  }
+  else if(pView->isInFrame(pView->getSelectionAnchor()))
+  {
+    s = EV_MIS_Gray;
+  }
+  else if(pView->isInFootnote())
+  {
+    s = EV_MIS_Gray;
+  }
+  else if(pView->isInFootnote(pView->getSelectionAnchor()))
+  {
+    s = EV_MIS_Gray;
+  }
+  else if(pView->isInEndnote())
+  {
+    s = EV_MIS_Gray;
+  }
+  else if(pView->isInEndnote(pView->getSelectionAnchor()))
+  {
+    s = EV_MIS_Gray;
+  }
+  else if(pView->isInTable() && (pView->getPoint() > 3) && pView->isInFootnote(pView->getPoint()-2))
+  {
+    s = EV_MIS_Gray;
+  }
+  else if(pView->isInTable()  && (pView->getPoint() > 3) && pView->isInEndnote(pView->getPoint()-2))
+  {
+    s = EV_MIS_Gray;
+  }
+  else if(pView->getSelectionMode() >= FV_SelectionMode_Multiple)
+  {
+	  return EV_MIS_Gray;
+  }
+  else if(pView->getHyperLinkRun(pView->getPoint()) != NULL)
+  {
+	  return EV_MIS_Gray;
+  }
+  return s;
+}
+
 Defun_EV_GetMenuItemState_Fn(ap_GetState_SomethingSelected)
 {
 	ABIWORD_VIEW ;
-	UT_ASSERT(pView) ;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
 	EV_Menu_ItemState s = EV_MIS_ZERO ;
 
@@ -473,13 +557,70 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_SomethingSelected)
 	return s ;
 }
 
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_TextToTableOK)
+{
+	ABIWORD_VIEW ;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	EV_Menu_ItemState s = EV_MIS_ZERO ;
+
+	if ( pView->isSelectionEmpty () || pView->isInTable() || pView->isInHdrFtr(pView->getPoint()))
+	  {
+	    s = EV_MIS_Gray ;
+	  }
+
+	return s ;
+}
+
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_HyperlinkOK)
+{
+	ABIWORD_VIEW ;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	EV_Menu_ItemState s = EV_MIS_ZERO ;
+
+	if ( pView->isSelectionEmpty())
+	  {
+		  if(pView->getHyperLinkRun(pView->getPoint()) == NULL)
+		  {
+			  s = EV_MIS_Gray ;
+			  return s;
+		  }
+		  return s;
+	  }
+
+	if(pView->isTOCSelected())
+	{
+	    s = EV_MIS_Gray ;
+		return s;
+	}
+	PT_DocPosition posStart = pView->getPoint();
+	PT_DocPosition posEnd = pView->getSelectionAnchor();
+	fl_BlockLayout * pBL1 = pView->getBlockAtPosition(posStart);
+	fl_BlockLayout * pBL2 = pView->getBlockAtPosition(posEnd);
+	if((pBL1 == NULL) || (pBL2 == NULL)) // make sure we get valid blocks from selection beginning and end
+	{
+		s = EV_MIS_Gray;
+		return s;
+	}
+	if(pBL1 != pBL2) // don't allow Insert Hyperlink if selection spans multiple blocks
+	{
+	    s = EV_MIS_Gray ;
+		return s;
+	}
+
+	return s ;
+}
+
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Suggest)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
-	UT_ASSERT(id >= AP_MENU_ID_SPELL_SUGGEST_1);
-	UT_ASSERT(id <= AP_MENU_ID_SPELL_SUGGEST_9);
+	UT_ASSERT_HARMLESS(id >= AP_MENU_ID_SPELL_SUGGEST_1);
+	UT_ASSERT_HARMLESS(id <= AP_MENU_ID_SPELL_SUGGEST_9);
 
 	UT_uint32 ndx = (id - AP_MENU_ID_SPELL_SUGGEST_1 + 1);
 
@@ -503,22 +644,24 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Suggest)
 	// We return a pointer to a static string (which will be overwritten
 	// on the next call).
 
-	UT_ASSERT(pFrame);
-	XAP_App * pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
-	UT_ASSERT(pLabel);
+	XAP_App * pApp = XAP_App::getApp();
+	UT_return_val_if_fail (pApp && pLabel, NULL);
+	XAP_Frame * frame = pApp->getLastFocussedFrame();
 
-	AV_View * pAV_View = pFrame->getCurrentView();
+	AV_View * pAV_View = frame->getCurrentView();
 	ABIWORD_VIEW;
 
-	UT_ASSERT(id >= AP_MENU_ID_SPELL_SUGGEST_1);
-	UT_ASSERT(id <= AP_MENU_ID_SPELL_SUGGEST_9);
+	UT_ASSERT_HARMLESS(id >= AP_MENU_ID_SPELL_SUGGEST_1);
+	UT_ASSERT_HARMLESS(id <= AP_MENU_ID_SPELL_SUGGEST_9);
+
+	UT_return_val_if_fail(pView != NULL, NULL);
 
 	UT_uint32 ndx = (id - AP_MENU_ID_SPELL_SUGGEST_1 + 1);
 
 	const char * c = NULL;
 	const UT_UCSChar *p;
 	static char cBuf[128];		// BUGBUG: possible buffer overflow
+	UT_UTF8String s;
 	UT_uint32 len = 0;
 
 	p = pView->getContextSuggest(ndx);
@@ -538,9 +681,10 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Suggest)
 	}
 	else if (ndx == 1)
 	{
-		// placeholder when no suggestions
+		// placeholder when no suggestions		
 		const XAP_StringSet * pSS = pApp->getStringSet();
-		c = pSS->getValueUTF8(AP_STRING_ID_DLG_Spell_NoSuggestions).utf8_str();
+		pSS->getValueUTF8(AP_STRING_ID_DLG_Spell_NoSuggestions,s);
+		c = s.utf8_str();
 	}
 
 	FREEP(p);
@@ -567,20 +711,20 @@ Defun_EV_GetMenuItemComputedLabel_Fn(ap_GetLabel_Suggest)
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Changes)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
 	EV_Menu_ItemState s = EV_MIS_ZERO;
 
 	XAP_Frame * pFrame = static_cast<XAP_Frame *> (pAV_View->getParentData());
-	UT_ASSERT(pFrame);
+	UT_return_val_if_fail (pFrame, EV_MIS_Gray);
 
 	AP_FrameData *pFrameData = static_cast<AP_FrameData *> (pFrame->getFrameData());
-	UT_ASSERT(pFrameData);
+	UT_return_val_if_fail (pFrameData, EV_MIS_Gray);
 
 	switch(id)
 	{
 	case AP_MENU_ID_FILE_SAVE:
-	  if (!pView->getDocument()->isDirty() || !pView->canDo(true) || pView->getDocument()->getFileName() == NULL)
+	  if (!pView->getDocument()->isDirty() || !pView->canDo(true))
 	    s = EV_MIS_Gray;
 	  break;
 	case AP_MENU_ID_FILE_REVERT:
@@ -607,12 +751,12 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Changes)
 			s = EV_MIS_Gray;
 	  break;
 
-	case AP_MENU_ID_INSERT_INSERTHEADER:
+	case AP_MENU_ID_INSERT_HEADER:
 		if (pView->isHeaderOnPage())
 			s = EV_MIS_Gray;
 	  break;
 
-	case AP_MENU_ID_INSERT_INSERTFOOTER:
+	case AP_MENU_ID_INSERT_FOOTER:
 		if (pView->isFooterOnPage())
 			s = EV_MIS_Gray;
 	  break;
@@ -628,22 +772,22 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Changes)
 		break;
 
 	case AP_MENU_ID_TABLE_INSERT:
-		if (pView->isHdrFtrEdit())
+		if (pView->isHdrFtrEdit() || pView->isInHdrFtr(pView->getPoint()))
 			s = EV_MIS_Gray;
 		break;
 
 	case AP_MENU_ID_TABLE_INSERT_TABLE:
-		if (pView->isHdrFtrEdit())
+		if (pView->isHdrFtrEdit() || pView->isInHdrFtr(pView->getPoint()))
 			s = EV_MIS_Gray;
 		break;
 
 	case AP_MENU_ID_TABLE_INSERTTABLE:
-		if (pView->isHdrFtrEdit())
+		if (pView->isHdrFtrEdit()  || pView->isInHdrFtr(pView->getPoint()))
 			s = EV_MIS_Gray;
 		break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -667,10 +811,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Selection)
 {
         ABIWORD_VIEW;
 	XAP_App * pApp = XAP_App::getApp();
-	UT_ASSERT(pApp);
+	UT_return_val_if_fail (pApp, EV_MIS_Gray);
 
 	XAP_Prefs * pPrefs = pApp->getPrefs();
-	UT_ASSERT(pPrefs);
+	UT_return_val_if_fail (pPrefs, EV_MIS_Gray);
 
 	EV_Menu_ItemState s = EV_MIS_ZERO;
 
@@ -684,7 +828,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Selection)
 		break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -694,7 +838,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Selection)
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Clipboard)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
 	EV_Menu_ItemState s = EV_MIS_ZERO;
 
@@ -706,7 +850,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Clipboard)
 		break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -715,14 +859,14 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Clipboard)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Prefs)
 {
-        ABIWORD_VIEW;
-	UT_ASSERT(pView);
+    ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
 	XAP_App *pApp = XAP_App::getApp();
-	UT_ASSERT(pApp);
+	UT_return_val_if_fail (pApp, EV_MIS_Gray);
 
 	XAP_Prefs * pPrefs = pApp->getPrefs();
-	UT_ASSERT(pPrefs);
+	UT_return_val_if_fail (pPrefs, EV_MIS_Gray);
 
 	EV_Menu_ItemState s = EV_MIS_ZERO;
 
@@ -731,22 +875,54 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Prefs)
 	switch (id)
 	  {
 	  case AP_MENU_ID_TOOLS_AUTOSPELL:
-	    pPrefs->getPrefsValueBool(static_cast<XML_Char*>(AP_PREF_KEY_AutoSpellCheck), &b);
+	    pPrefs->getPrefsValueBool(static_cast<const XML_Char *>(AP_PREF_KEY_AutoSpellCheck), &b);
 	    s = (b ? EV_MIS_Toggled : EV_MIS_ZERO);
 	    break;
 
 	  default:
-	    UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	    UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 	    break;
 	  }
 
 	return s;
 }
 
+Defun_EV_GetMenuItemState_Fn(ap_GetState_FmtHdrFtr)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+	EV_Menu_ItemState s = EV_MIS_ZERO;
+	if(pView->getPoint() == 0)
+	{
+		return EV_MIS_Gray;
+	}
+	fp_Page * pPage = pView->getCurrentPage();
+	if(pPage == NULL)
+	{
+		return EV_MIS_Gray;
+	}
+	fl_DocSectionLayout * pDSLP = pPage->getOwningSection();
+	if(pDSLP == NULL)
+	{
+		return EV_MIS_Gray;
+	}
+	fl_BlockLayout * pBL = pView->getCurrentBlock();
+	if(pBL == NULL)
+	{
+		return EV_MIS_Gray;
+	}
+	fl_DocSectionLayout	* pDSL = pBL->getDocSectionLayout();
+	if(pDSL != pDSLP)
+	{
+		return EV_MIS_Gray;
+	}
+	return s;
+}
+
 Defun_EV_GetMenuItemState_Fn(ap_GetState_CharFmt)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 	bool bMultiple = false;
 
 	EV_Menu_ItemState s = EV_MIS_ZERO;
@@ -820,7 +996,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_CharFmt)
 		break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -859,7 +1035,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_CharFmt)
 Defun_EV_GetMenuItemState_Fn(ap_GetState_BlockFmt)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
 	EV_Menu_ItemState s = EV_MIS_ZERO;
 
@@ -894,7 +1070,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_BlockFmt)
 		break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -920,16 +1096,16 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_BlockFmt)
 Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
 	XAP_Frame *pFrame = static_cast<XAP_Frame *> (pAV_View->getParentData());
-	UT_ASSERT(pFrame);
+	UT_return_val_if_fail (pFrame, EV_MIS_Gray);
 
 	AP_FrameData *pFrameData = static_cast<AP_FrameData *> (pFrame->getFrameData());
-	UT_ASSERT(pFrameData);
+	UT_return_val_if_fail (pFrameData, EV_MIS_Gray);
 
 	XAP_App *pApp = pFrame->getApp();
-	UT_ASSERT(pApp);
+	UT_return_val_if_fail (pApp, EV_MIS_Gray);
 
 	EV_Menu_ItemState s = EV_MIS_ZERO;
 
@@ -938,6 +1114,8 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 	case AP_MENU_ID_VIEW_RULER:
 		if ( pFrameData->m_bShowRuler && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
+		else if( pFrameData->m_bIsFullScreen )
+			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
@@ -956,24 +1134,32 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 	case AP_MENU_ID_VIEW_TB_1:
 		if ( pFrameData->m_bShowBar[0] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
+		else if( pFrameData->m_bIsFullScreen )
+			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
 	case AP_MENU_ID_VIEW_TB_2:	
 		if ( pFrameData->m_bShowBar[1] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
+		else if( pFrameData->m_bIsFullScreen )
+			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
 	case AP_MENU_ID_VIEW_TB_3:	
 		if ( pFrameData->m_bShowBar[2] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
+		else if( pFrameData->m_bIsFullScreen )
+			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
 	case AP_MENU_ID_VIEW_TB_4:	
 		if ( pFrameData->m_bShowBar[3] && !pFrameData->m_bIsFullScreen)
 			s = EV_MIS_Toggled;
+		else if( pFrameData->m_bIsFullScreen )
+			s = EV_MIS_Gray;
 		else
 			s = EV_MIS_ZERO;
 		break;
@@ -991,6 +1177,8 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
               if ( pFrameData->m_bShowStatusBar &&
 				   !pFrameData->m_bIsFullScreen )
 				  s = EV_MIS_Toggled;
+			  else if( pFrameData->m_bIsFullScreen )
+				  s = EV_MIS_Gray;
 			  else
 				  s = EV_MIS_ZERO;
 			  break;
@@ -1024,7 +1212,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 	  break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -1034,7 +1222,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_View)
 Defun_EV_GetMenuItemState_Fn(ap_GetState_StylesLocked)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
         if(pView->getDocument()->areStylesLocked()) {
             return EV_MIS_Gray;
@@ -1043,26 +1231,185 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_StylesLocked)
         return EV_MIS_ZERO;
 }
 
+Defun_EV_GetMenuItemState_Fn(ap_GetState_History)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+	PD_Document * pDoc = pView->getDocument();
+	UT_return_val_if_fail( pDoc, EV_MIS_Gray );
+
+	// disable for documents that have not been saved yet
+	if(!pDoc->getFilename())
+		return EV_MIS_Gray;
+	
+    return EV_MIS_ZERO;
+}
+
 Defun_EV_GetMenuItemState_Fn(ap_GetState_MarkRevisions)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
-        if(pView->isMarkRevisions()) {
-            return EV_MIS_Toggled;
-        }
+	if(pView->getDocument()->isAutoRevisioning())
+	{
+		return EV_MIS_Gray;
+	}
 
-        return EV_MIS_ZERO;
+	if(pView->isMarkRevisions())
+	{
+		return EV_MIS_Toggled;
+	}
+
+    return EV_MIS_ZERO;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionsSelectLevel)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	if(pView->getDocument()->isAutoRevisioning() || pView->isMarkRevisions())
+	{
+		return EV_MIS_Gray;
+	}
+
+    return EV_MIS_ZERO;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_HasRevisions)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	if(pView->getDocument()->getHighestRevisionId() == 0)
+		return EV_MIS_Gray;
+	
+	return EV_MIS_ZERO;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_AutoRevision)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	if(pView->getDocument()->isAutoRevisioning())
+	{
+		return (EV_Menu_ItemState) (EV_MIS_Toggled);
+	}
+
+	return EV_MIS_ZERO;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisions)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	if(pView->getDocument()->isAutoRevisioning())
+	{
+		return EV_MIS_Gray;
+	}
+	   
+	if(pView->getDocument()->getHighestRevisionId() == 0)
+		return EV_MIS_Gray;
+	
+	if(pView->isShowRevisions())
+	{
+		return (EV_Menu_ItemState) (EV_MIS_Toggled | EV_MIS_Gray);
+	}
+
+	return EV_MIS_ZERO;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsAfter)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	if(pView->getDocument()->isAutoRevisioning())
+	{
+		return EV_MIS_Gray;
+	}
+
+	if(pView->getDocument()->getHighestRevisionId() == 0)
+		return EV_MIS_Gray;
+	
+	if(pView->isMarkRevisions())
+	{
+		if(pView->getRevisionLevel() == PD_MAX_REVISION)
+			return EV_MIS_Toggled;
+		else
+			return EV_MIS_ZERO;
+	}
+	else if(!pView->isShowRevisions() && pView->getRevisionLevel() == PD_MAX_REVISION)
+	{
+		return (EV_Menu_ItemState) (EV_MIS_Toggled | EV_MIS_Gray);
+	}
+
+	return EV_MIS_ZERO;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsAfterPrev)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	if(pView->getDocument()->isAutoRevisioning())
+	{
+		return EV_MIS_Gray;
+	}
+
+	if(pView->getDocument()->getHighestRevisionId() == 0)
+		return EV_MIS_Gray;
+	
+	if(pView->isMarkRevisions())
+	{
+		if(pView->getDocument()->getHighestRevisionId() == pView->getRevisionLevel() + 1)
+			return EV_MIS_Toggled;
+		else
+			return EV_MIS_ZERO;
+	}
+	else
+		return EV_MIS_Gray;
+	
+	return EV_MIS_ZERO;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_ShowRevisionsBefore)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	if(pView->getDocument()->isAutoRevisioning())
+	{
+		return EV_MIS_Gray;
+	}
+
+	if(pView->getDocument()->getHighestRevisionId() == 0)
+		return EV_MIS_Gray;
+	
+	if(pView->isMarkRevisions())
+	{
+		// cannot hide revisions when in revisions mode
+		return EV_MIS_Gray;
+	}
+
+	if(!pView->isShowRevisions() && pView->getRevisionLevel() == 0)
+	{
+		return (EV_Menu_ItemState) (EV_MIS_Toggled | EV_MIS_Gray);
+	}
+
+	return EV_MIS_ZERO;
 }
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionPresent)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
 	if(pView->isMarkRevisions())
 		return EV_MIS_Gray;
-    else if(pView->getInsertionPointContext(NULL,NULL) != EV_EMC_REVISION)
+    else if(!pView->doesSelectionContainRevision())
         return EV_MIS_Gray;
 
 	return EV_MIS_ZERO;
@@ -1071,7 +1418,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionPresent)
 Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionPresentContext)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
 	if(pView->isMarkRevisions())
 		return EV_MIS_Gray;
@@ -1082,7 +1429,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_RevisionPresentContext)
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InTable)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
 	if(pView->isInTable())
 		return EV_MIS_ZERO;
@@ -1096,7 +1443,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_TableOK)
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
 
-	if(pView->isInTable(pView->getPoint()-1) && pView->isInTable() && pView->isHdrFtrEdit())
+	if(pView->isInTable() && (pView->isHdrFtrEdit() || pView->isInHdrFtr(pView->getPoint())))
 	{
 		return EV_MIS_Gray;
 	}
@@ -1108,39 +1455,136 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_TableOK)
 	{
 		return EV_MIS_Gray;
 	}
+	else if(pView->getHyperLinkRun(pView->getPoint()) != NULL)
+	{
+		return EV_MIS_Gray;
+	}
     return EV_MIS_ZERO;
 }
 
 
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InTOC)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+	if(pView->isTOCSelected())
+	{
+		return EV_MIS_ZERO;
+	}
+	return EV_MIS_Gray;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InTableMerged)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+
+	if(pView->isInTable())
+	{
+		return EV_MIS_ZERO;
+	}
+    return EV_MIS_Gray;
+}
+
+
 Defun_EV_GetMenuItemState_Fn(ap_GetState_InFootnote)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
-
-	if(!pView->isInFootnote() && !pView->isHdrFtrEdit())
-	{
-		return EV_MIS_ZERO;
-	}
-	else
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+	if(pView->getHyperLinkRun(pView->getPoint()) != NULL)
 	{
 		return EV_MIS_Gray;
 	}
+	else if(pView->getEmbedDepth(pView->getPoint()) > 0)
+	{
+		return EV_MIS_Gray;
+	}
+	else if(pView->getFrameEdit()->isActive())
+	{
+	        return EV_MIS_Gray;
+	}
+	if(!pView->isInFootnote() && !pView->isHdrFtrEdit() && !pView->isInHdrFtr(pView->getPoint()) && !pView->isInFrame(pView->getPoint()) 
+		&& !pView->isTOCSelected())
+	{
+		return EV_MIS_ZERO;
+	}
+	return EV_MIS_Gray;
+
 }
 
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InImage)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+	if(pView->isImageSelected())
+	{
+		return EV_MIS_ZERO;
+	}
+	return EV_MIS_Gray;
+}
+
+Defun_EV_GetMenuItemState_Fn(ap_GetState_InFrame)
+{
+	ABIWORD_VIEW;
+	UT_return_val_if_fail (pView, EV_MIS_Gray);
+	if(pView->isInFrame(pView->getPoint()))
+	{
+		return EV_MIS_ZERO;
+	}
+	return EV_MIS_Gray;
+}
+
+
 Defun_EV_GetMenuItemState_Fn(ap_GetState_BreakOK)
 {
 	ABIWORD_VIEW;
 	UT_return_val_if_fail (pView, EV_MIS_Gray);
-
+	
 	if(pView->isInFootnote())
 	{
 		return EV_MIS_Gray;
 	}
+	if(pView->isInFootnote(pView->getSelectionAnchor()))
+	{
+		return EV_MIS_Gray;
+	}
 	if(pView->isInEndnote())
 	{
 		return EV_MIS_Gray;
 	}
-	else if(pView->isInTable(pView->getPoint()-1) && pView->isInTable())
+	if(pView->isInEndnote(pView->getSelectionAnchor()))
+	{
+		return EV_MIS_Gray;
+	}
+	else if(pView->isInFrame(pView->getPoint()))
+	{
+		return EV_MIS_Gray;
+	}
+	else if(pView->isInFrame(pView->getSelectionAnchor()))
+	{
+		return EV_MIS_Gray;
+	}
+	else if(pView->isInTable())
+	{
+		return EV_MIS_Gray;
+	}
+	else if(pView->getFrameEdit()->isActive())
+	{
+	        return EV_MIS_Gray;
+	}
+	else if(pView->isInTable(pView->getSelectionAnchor()))
+	{
+		return EV_MIS_Gray;
+	}
+	else if(pView->isHdrFtrEdit() || pView->isInHdrFtr(pView->getPoint()))
+	{
+		return EV_MIS_Gray;
+	}
+	else if(pView->getSelectionMode() >= FV_SelectionMode_Multiple)
+	{
+		return EV_MIS_Gray;
+	}
+	else if(pView->getHyperLinkRun(pView->getPoint()) != NULL)
 	{
 		return EV_MIS_Gray;
 	}
@@ -1158,8 +1602,8 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_AlwaysDisabled)
 
 Defun_EV_GetMenuItemState_Fn(ap_GetState_Recent)
 {
-	ABIWORD_VIEW;
-	UT_return_val_if_fail(pView, EV_MIS_ZERO);
+  // ABIWORD_VIEW;
+  // UT_return_val_if_fail(pView, EV_MIS_ZERO);
 
 	XAP_Prefs *pPrefs = XAP_App::getApp()->getPrefs();
 	UT_return_val_if_fail(pPrefs, EV_MIS_ZERO);
@@ -1176,7 +1620,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Zoom)
 	UT_return_val_if_fail(pView, EV_MIS_ZERO);
 
 	XAP_Frame * pFrame = static_cast<XAP_Frame *> (pAV_View->getParentData());
-	UT_ASSERT(pFrame);
+	UT_return_val_if_fail (pFrame, EV_MIS_Gray);
 
 	EV_Menu_ItemState s = EV_MIS_ZERO;
 
@@ -1220,7 +1664,7 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_Lists)
 	ABIWORD_VIEW;
 	UT_return_val_if_fail(pView, EV_MIS_ZERO);
 
-	if(pView->getDocument()->areStylesLocked() || pView->isHdrFtrEdit())
+	if(pView->getDocument()->areStylesLocked() || pView->isHdrFtrEdit()  || pView->isInHdrFtr(pView->getPoint()))
 	{
 		return EV_MIS_Gray;
 	}
