@@ -40,6 +40,7 @@
 
 menuitem(FILE)
 menuitem(FILE_NEW)
+menuitem(FILE_NEW_USING_TEMPLATE)
 menuitem(FILE_OPEN)
 menuitem(FILE_SAVE)
 menuitem(FILE_SAVEAS)
@@ -84,6 +85,10 @@ menuitem(EDIT_EDITHEADER)
 menuitem(EDIT_EDITFOOTER)
 menuitem(EDIT_REMOVEHEADER)
 menuitem(EDIT_REMOVEFOOTER)
+menuitem(EDIT_DELETEFRAME)
+menuitem(EDIT_CUT_FRAME)
+menuitem(EDIT_COPY_FRAME)
+menuitem(EDIT_SELECT_FRAME)
 
 menuitem(VIEW)
 menuitem(VIEW_TOOLBARS)
@@ -117,6 +122,7 @@ menuitem(INSERT_BREAK)
 menuitem(INSERT_PAGENO)
 menuitem(INSERT_DATETIME)
 menuitem(INSERT_FIELD)
+menuitem(INSERT_TEXTBOX)
 menuitem(INSERT_MAILMERGE)
 menuitem(INSERT_FILE)
 menuitem(INSERT_SYMBOL)
@@ -126,10 +132,13 @@ menuitem(INSERT_PICTURE)
 menuitem(INSERT_CLIPART)
 menuitem(INSERT_GRAPHIC)
 menuitem(INSERT_BOOKMARK)
+menuitem(INSERT_TABLEOFCONTENTS)
 menuitem(INSERT_HYPERLINK)
+menuitem(INSERT_GOTO_HYPERLINK)
+menuitem(INSERT_EDIT_HYPERLINK)
 menuitem(INSERT_DELETE_HYPERLINK)
-menuitem(INSERT_INSERTHEADER)
-menuitem(INSERT_INSERTFOOTER)
+menuitem(INSERT_HEADER)
+menuitem(INSERT_FOOTER)
 menuitem(INSERT_DIRECTIONMARKER)	
 menuitem(INSERT_DIRECTIONMARKER_LRM)	
 menuitem(INSERT_DIRECTIONMARKER_RLM)	
@@ -143,15 +152,19 @@ menuitem(FMT_BORDERS)
 menuitem(FMT_COLUMNS)
 menuitem(FMT_TOGGLECASE)
 menuitem(FMT_BACKGROUND)
+menuitem(FMT_BACKGROUND_PAGE_IMAGE)
+menuitem(FMT_BACKGROUND_PAGE_COLOR)
 menuitem(FMT_HDRFTR)
 menuitem(FMT_FOOTNOTES)
+menuitem(FMT_TABLEOFCONTENTS)
 menuitem(FMT_TABS)
 menuitem(FMT_LANGUAGE)
 menuitem(FMT_IMAGE)
 menuitem(FMT_STYLE)
 menuitem(FMT_STYLE_DEFINE)
 menuitem(FMT_IMPORTSTYLES)
-
+menuitem(FMT_STYLIST)
+menuitem(FMT_FRAME)
 menuitem(FMT)
 menuitem(FMT_BOLD)
 menuitem(FMT_ITALIC)
@@ -180,11 +193,24 @@ menuitem(TOOLS_OPTIONS)
 menuitem(TOOLS_LANGUAGE)
 menuitem(TOOLS_REVISIONS)
 menuitem(TOOLS_REVISIONS_MARK)
+menuitem(TOOLS_REVISIONS_SHOW)
+menuitem(TOOLS_REVISIONS_SHOW_AFTER)
+menuitem(TOOLS_REVISIONS_SHOW_AFTERPREV)
+menuitem(TOOLS_REVISIONS_SHOW_BEFORE)
 menuitem(TOOLS_REVISIONS_SET_VIEW_LEVEL)
 menuitem(TOOLS_REVISIONS_ACCEPT_REVISION)
 menuitem(TOOLS_REVISIONS_REJECT_REVISION)
+menuitem(TOOLS_REVISIONS_COMPARE_DOCUMENTS)
+menuitem(TOOLS_REVISIONS_MERGE_DOCUMENTS)
+menuitem(TOOLS_REVISIONS_FIND_NEXT)
+menuitem(TOOLS_REVISIONS_FIND_PREV)
+menuitem(TOOLS_REVISIONS_AUTO)
+menuitem(TOOLS_HISTORY)
+menuitem(TOOLS_HISTORY_SHOW)
 menuitem(CONTEXT_REVISIONS_ACCEPT_REVISION)
 menuitem(CONTEXT_REVISIONS_REJECT_REVISION)
+menuitem(CONTEXT_REVISIONS_FIND_NEXT)
+menuitem(CONTEXT_REVISIONS_FIND_PREV)
 menuitem(ALIGN)
 menuitem(ALIGN_LEFT)
 menuitem(ALIGN_CENTER)
@@ -203,6 +229,8 @@ menuitem(TABLE_INSERT_ROWS_BEFORE)
 menuitem(TABLE_INSERT_ROWS_AFTER)
 menuitem(TABLE_INSERTROW)
 menuitem(TABLE_INSERT_CELLS)
+menuitem(TABLE_INSERT_SUMCOLS)
+menuitem(TABLE_INSERT_SUMROWS)
 
 menuitem(TABLE_DELETE)
 menuitem(TABLE_DELETE_TABLE)
@@ -223,6 +251,11 @@ menuitem(TABLE_MERGE_CELLS)
 menuitem(TABLE_SPLIT_CELLS)
 menuitem(TABLE_SPLIT_TABLE)
 menuitem(TABLE_FORMAT)
+menuitem(TABLE_TEXTTOTABLE)
+menuitem(TABLE_SORTROWSASCEND)
+menuitem(TABLE_SORTROWSDESCEND)
+menuitem(TABLE_SORTCOLSASCEND)
+menuitem(TABLE_SORTCOLSDESCEND)
 menuitem(TABLE_AUTOFIT)
 menuitem(TABLE_HEADING_ROWS_REPEAT)
 menuitem(TABLE_SORT)
@@ -321,5 +354,3 @@ menuitem(AUTOTEXT_EMAIL_5)
 menuitem(AUTOTEXT_EMAIL_6)
 
 /**** Add Menu Items here ****/
-
-
