@@ -23,6 +23,7 @@
 #include "ut_types.h"
 #include "ut_debugmsg.h"
 #include "ut_string.h"
+#include "ap_Args.h"
 
 #include "xap_EncodingManager.h"
 /*****************************************************************/
@@ -142,14 +143,26 @@ Failed:
 void AP_Prefs::overlaySystemPrefs(void)
 {
 	// read system prefs file and overlay builtin values.
-
-	const char * szSystemDefaultPrefsDir = m_pApp->getAbiSuiteAppDir();
-	char buf[1024];
-	sprintf(buf,"%s/%s",szSystemDefaultPrefsDir,"system.profile");
-	const char** items = localeinfo_combinations(buf,"","-",0);
+	const char** items = localeinfo_combinations("system.profile","","-",0);
+	UT_String path;
 	while(*items) {
-	    loadSystemDefaultPrefsFile(*items++);
+	    const char * item = *items++;
+	    if (m_pApp->findAbiSuiteAppFile(path,item))
+		loadSystemDefaultPrefsFile(path.c_str());
 	};
 }
 
 
+const char * AP_Prefs::getPrefsPathname(void) const
+{
+	if(AP_Args::m_sUserProfile)
+	{
+		UT_DEBUGMSG(("AP_Prefs::getPrefsPathname: using cmd line value [%s]\n",
+					 AP_Args::m_sUserProfile));
+		
+		return AP_Args::m_sUserProfile;
+	}
+	else
+		return _getPrefsPathname();
+}
+
