@@ -31,6 +31,7 @@
 #include "ev_Toolbar_Actions.h"
 #include "ev_Toolbar_Labels.h"
 #include "xap_App.h"
+#include "xap_Prefs.h"
 #include "xap_Clipboard.h"
 #include "xap_Frame.h"
 #include "fv_View.h"
@@ -43,7 +44,7 @@
 #include "ut_Script.h"
 #include "spell_manager.h"
 #include "ap_EditMethods.h"
-
+#include "fp_TableContainer.h"
 
 #define ABIWORD_VIEW  	FV_View * pView = static_cast<FV_View *>(pAV_View)
 
@@ -84,6 +85,74 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_ScriptsActive)
   return s;
 }
 
+Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_BookmarkOK)
+{
+	ABIWORD_VIEW;
+	CHECK_INC_LOAD;
+	EV_Toolbar_ItemState s = EV_TIS_ZERO;
+
+	if(pView->isTOCSelected())
+	{
+	    s = EV_TIS_Gray ;
+		return s;
+	}
+	PT_DocPosition posStart = pView->getPoint();
+	PT_DocPosition posEnd = pView->getSelectionAnchor();
+	fl_BlockLayout * pBL1 = pView->getBlockAtPosition(posStart);
+	fl_BlockLayout * pBL2 = pView->getBlockAtPosition(posEnd);
+	if((pBL1 == NULL) || (pBL2 == NULL)) // make sure we get valid blocks from selection beginning and end
+	{
+		s = EV_TIS_Gray;
+		return s;
+	}
+	if(pBL1 != pBL2) // don't allow Insert bookmark if selection spans multiple blocks
+	{
+	    s = EV_TIS_Gray ;
+		return s;
+	}
+
+	return s ;
+}
+
+Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_HyperlinkOK)
+{
+	ABIWORD_VIEW ;
+	CHECK_INC_LOAD;
+	UT_return_val_if_fail (pView, EV_TIS_Gray);
+
+	EV_Toolbar_ItemState s = EV_TIS_ZERO;
+	if ( pView->isSelectionEmpty())
+	  {
+		  if(pView->getHyperLinkRun(pView->getPoint()) == NULL)
+		  {
+			  s = EV_TIS_Gray ;
+			  return s;
+		  }
+		  return s;
+	  }
+	if(pView->isTOCSelected())
+	{
+	    s = EV_TIS_Gray ;
+		return s;
+	}
+	PT_DocPosition posStart = pView->getPoint();
+	PT_DocPosition posEnd = pView->getSelectionAnchor();
+	fl_BlockLayout * pBL1 = pView->getBlockAtPosition(posStart);
+	fl_BlockLayout * pBL2 = pView->getBlockAtPosition(posEnd);
+	if((pBL1 == NULL) || (pBL2 == NULL)) // make sure we get valid blocks from selection beginning and end
+	{
+		s = EV_TIS_Gray;
+		return s;
+	}
+	if(pBL1 != pBL2) // don't allow Insert Hyperlink if selection spans multiple blocks
+	{
+	    s = EV_TIS_Gray ;
+		return s;
+	}
+
+	return s ;
+}
+
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Spelling)
 {
 	//ABIWORD_VIEW;
@@ -92,10 +161,10 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Spelling)
   EV_Toolbar_ItemState s = EV_TIS_ZERO;
 
   XAP_Prefs *pPrefs = XAP_App::getApp()->getPrefs();
-  UT_ASSERT( pPrefs );
+  UT_return_val_if_fail ( pPrefs, EV_TIS_Gray );
 
   bool b = true ;
-  pPrefs->getPrefsValueBool(static_cast<XML_Char*>(AP_PREF_KEY_AutoSpellCheck),&b) ;
+  pPrefs->getPrefsValueBool(static_cast<const XML_Char *>(AP_PREF_KEY_AutoSpellCheck),&b) ;
 
   // if there are no loaded dictionaries and we are spell checking
   // as we type
@@ -124,7 +193,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Changes)
 	switch (id)
 	{
 	case AP_TOOLBAR_ID_FILE_SAVE:
-	  if (!pView->getDocument()->isDirty() || !pView->canDo(true) || pView->getDocument()->getFileName() == NULL)
+	  if (!pView->getDocument()->isDirty() || !pView->canDo(true) || pView->getDocument()->getFilename() == NULL)
 	    s = EV_TIS_Gray;
 	  break;
 	case AP_TOOLBAR_ID_EDIT_UNDO:
@@ -138,7 +207,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Changes)
 		break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -165,7 +234,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Selection)
 		break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -196,7 +265,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Clipboard)
 	  break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -207,7 +276,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Clipboard)
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_HdrFtr)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_TIS_Gray);
 
 	if (pszState)
 		*pszState = NULL;
@@ -227,7 +296,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_HdrFtr)
 		  s = EV_TIS_Gray;
 		break;
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 	return s;
@@ -244,7 +313,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Style)
 	{
 	case AP_TOOLBAR_ID_FMT_STYLE:
 		{
-			const XML_Char * sz;
+			const XML_Char * sz = NULL;
 			if (!pView->getStyle(&sz))
 			{
 				static const char * sz2 ="None";
@@ -254,7 +323,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Style)
 
 			if (sz)
 			{
-				static const char * sz2;
+				static const char * sz2 = "None";
 				sz2 = sz;
 				*pszState = sz2;
 				s = EV_TIS_UseString;
@@ -269,7 +338,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Style)
 		}
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 	return s;
@@ -283,13 +352,13 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Bullets)
 
 	EV_Toolbar_ItemState s = EV_TIS_ZERO;
 
-	if(pView->isHdrFtrEdit())
+	if(pView->isHdrFtrEdit()  || pView->isInHdrFtr(pView->getPoint()))
 	{
 		return EV_TIS_Gray;
 	}
 
 	fl_BlockLayout * pBlock = pView->getCurrentBlock();
-	UT_ASSERT(pBlock);
+	UT_return_val_if_fail (pBlock, EV_TIS_Gray);
 	if(pBlock->isListItem() == false)
 		return s;
 	if(pBlock->getListType() == BULLETED_LIST)
@@ -304,13 +373,13 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Numbers)
 	CHECK_INC_LOAD;
 
 	EV_Toolbar_ItemState s = EV_TIS_ZERO;
-	if(pView->isHdrFtrEdit())
+	if(pView->isHdrFtrEdit()  || pView->isInHdrFtr(pView->getPoint()))
 	{
 		return EV_TIS_Gray;
 	}
 
 	fl_BlockLayout * pBlock = pView->getCurrentBlock();
-        UT_ASSERT(pBlock);
+    UT_return_val_if_fail (pBlock, EV_TIS_Gray);
 	if(pBlock->isListItem() == false)
 	        return s;
 	if(pBlock->getListType() == NUMBERED_LIST)
@@ -330,7 +399,9 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Indents)
 	s_getPageMargins(pView, margin_left, margin_right,
 					 page_margin_left, page_margin_right);
 
-	FriBidiCharType iBlockDir = pView->getCurrentBlock()->getDominantDirection();
+	UT_BidiCharType iBlockDir = UT_BIDI_LTR;
+	if(pView->getCurrentBlock())
+		iBlockDir = pView->getCurrentBlock()->getDominantDirection();
 
 	switch(id)
 	{
@@ -340,7 +411,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Indents)
 				s = EV_TIS_Gray;
 			break;
 		case AP_TOOLBAR_ID_UNINDENT:
-			allowed = iBlockDir == FRIBIDI_TYPE_LTR ? margin_left : margin_right;
+			allowed = iBlockDir == UT_BIDI_LTR ? margin_left : margin_right;
 
 			if (allowed <= 0.)
 				s = EV_TIS_Gray;
@@ -448,7 +519,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_CharFmt)
 		break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -513,7 +584,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_SectionFmt)
 
 	EV_Toolbar_ItemState s = EV_TIS_ZERO;
 
-	if(pView->isHdrFtrEdit())
+	if(pView->isHdrFtrEdit()  || pView->isInHdrFtr(pView->getPoint()))
 	  {
 	    switch(id)
 	      {
@@ -547,7 +618,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_SectionFmt)
 		val = "3";
 		break;
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 	s = EV_TIS_ZERO;
@@ -657,7 +728,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_BlockFmt)
 
 	default:
 		UT_DEBUGMSG(("id=%d", id));
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -697,32 +768,32 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_BlockFmt)
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Zoom)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_TIS_Gray);
 
 	XAP_Frame * pFrame = static_cast<XAP_Frame *> ( pView->getParentData());
 	const XAP_StringSet * pSS = XAP_App::getApp()->getStringSet();
 
 	EV_Toolbar_ItemState s = EV_TIS_UseString;
 
-	static char buf[10];
 	UT_uint32 iZoom = 0 ;
-
+	static UT_UTF8String str;
+	
 	switch(pFrame->getZoomType())
 	{
 	// special cases
 	case XAP_Frame::z_PAGEWIDTH:
-		sprintf(buf, "%s", pSS->getValue(XAP_STRING_ID_TB_Zoom_PageWidth, pFrame->getApp()->getDefaultEncoding()).c_str());
+		pSS->getValueUTF8(XAP_STRING_ID_TB_Zoom_PageWidth, str);
 		break;
 	case XAP_Frame::z_WHOLEPAGE:
-		sprintf(buf, "%s", pSS->getValue(XAP_STRING_ID_TB_Zoom_WholePage, pFrame->getApp()->getDefaultEncoding()).c_str());
+		pSS->getValueUTF8(XAP_STRING_ID_TB_Zoom_WholePage, str);
 		break;
 	default:
 	  iZoom = pView->getGraphics()->getZoomPercentage();
-	  sprintf(buf, "%d%%", iZoom);
+	  UT_UTF8String_sprintf(str, "%d%%", iZoom);
 	  break;
 	}
 
-	*pszState = buf;
+	*pszState = str.utf8_str();
 
 	return s;
 }
@@ -730,13 +801,13 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Zoom)
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_View)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_TIS_Gray);
 
 	XAP_Frame * pFrame = static_cast<XAP_Frame *> (pAV_View->getParentData());
-	UT_ASSERT(pFrame);
+	UT_return_val_if_fail (pFrame, EV_TIS_Gray);
 
 	AP_FrameData *pFrameData = static_cast<AP_FrameData *> (pFrame->getFrameData());
-	UT_ASSERT(pFrameData);
+	UT_return_val_if_fail (pFrameData, EV_TIS_Gray);
 
 	EV_Toolbar_ItemState s = EV_TIS_ZERO;
 
@@ -748,7 +819,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_View)
         break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -758,7 +829,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_View)
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_StylesLocked)
 {
         ABIWORD_VIEW;
-        UT_ASSERT(pView);
+        UT_return_val_if_fail (pView, EV_TIS_Gray);
 
         if(pView->getDocument()->areStylesLocked()) {
             return EV_TIS_Gray;
@@ -770,9 +841,9 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_StylesLocked)
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Table)
 {
   ABIWORD_VIEW;
-  UT_ASSERT(pView);
+  UT_return_val_if_fail (pView, EV_TIS_Gray);
   
-  if(pView->isInTable(pView->getPoint()-1) && pView->isInTable() )
+  if(pView->isInTable() )
   {
 	  return EV_TIS_ZERO;
   }
@@ -785,7 +856,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_TableOK)
   ABIWORD_VIEW;
   UT_return_val_if_fail (pView, EV_TIS_Gray);
   
-  if(pView->isInTable(pView->getPoint()-1) && pView->isInTable() && pView->isHdrFtrEdit())
+  if(pView->isInTable() && (pView->isHdrFtrEdit() || pView->isInHdrFtr(pView->getPoint())))
   {
 	  return EV_TIS_Gray;
   }
@@ -797,10 +868,25 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_TableOK)
   {
 	  return EV_TIS_Gray;
   } 
+  else if(pView->getHyperLinkRun(pView->getPoint()) != NULL)
+  {
+	  return EV_TIS_Gray;
+  }
 
   return EV_TIS_ZERO;
 }
 
+Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_TableMerged)
+{
+  ABIWORD_VIEW;
+  UT_return_val_if_fail (pView, EV_TIS_Gray);
+  
+  if(pView->isInTable())
+  {
+	  return EV_TIS_ZERO;
+  }
+  return EV_TIS_Gray;
+}
 
 // HACK TO ALWAYS DISABLE A TOOLBAR ITEM... DELETE ME
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_AlwaysDisabled)
