@@ -66,8 +66,12 @@ public:
 
 	const UT_UTF8String & name () const { return m_name; }
 
+	bool getCanCopy () const {
+		return m_bCanCopy;
+	}
+	
 protected:
-	IE_ExpSniffer(const char * name);
+	IE_ExpSniffer(const char * name, bool canCopy = false);
 
 private:
 	const UT_UTF8String	m_name;
@@ -75,6 +79,8 @@ private:
 	// only IE_Exp ever calls this
 	inline void setFileType (IEFileType type) {m_type = type;}
 	IEFileType m_type;
+
+	bool m_bCanCopy;
 };
 
 class ABI_EXPORT IE_Exp
@@ -127,6 +133,10 @@ public:
 
 	void setProps (const char * props);
 
+	bool isCopying () const {
+		return getDocRange () != NULL;
+	}
+
 protected:
 	// todo: remove the = 0 for 2.2
 	IE_Exp(PD_Document * pDocument, UT_Confidence_t fidelity = 0);
