@@ -19,9 +19,9 @@
  * 02111-1307, USA.
  */
 
-#include <locale.h>
 #include <time.h>
 
+#include "ut_locale.h"
 #include "ut_string.h"
 #include "ut_types.h"
 #include "ut_bytebuf.h"
@@ -29,6 +29,7 @@
 #include "ut_debugmsg.h"
 #include "ut_set.h"
 #include "ut_string_class.h"
+#include "ut_uuid.h"
 
 #ifdef ENABLE_RESOURCE_MANAGER
 #include "xap_ResourceManager.h"
@@ -172,12 +173,12 @@ protected:
 	void				_outputData(const UT_UCSChar * p, UT_uint32 length);
 	void				_outputXMLChar(const XML_Char * data, UT_uint32 length);
 	void				_handleStyles(void);
-	void				_handleIgnoredWords(void);
 	void				_handleLists(void);
 	void				_handlePageSize(void);
 	void				_handleDataItems(void);
     void                _handleMetaData(void);
 	void                _handleRevisions(void);
+	void                _handleHistory(void);
 
 	PD_Document *		m_pDocument;
 	IE_Exp_AbiWord_1 *	m_pie;
@@ -299,7 +300,7 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 								   bool bIgnoreProperties)
 {
 #ifdef ENABLE_RESOURCE_MANAGER
-	UT_ASSERT (!m_bOpenChar);
+	UT_ASSERT_HARMLESS (!m_bOpenChar);
 
 	UT_UTF8String tag("<");
 	tag += szPrefix;
@@ -432,7 +433,7 @@ void s_AbiWord_1_Listener::_outputXMLChar(const XML_Char * data, UT_uint32 lengt
 	UT_String sBuf;
 	const XML_Char * pData;
 
-	UT_ASSERT(sizeof(UT_Byte) == sizeof(char));
+	UT_return_if_fail(sizeof(UT_Byte) == sizeof(char));
 
 	for (pData=data; (pData<data+length); /**/)
 	{
@@ -467,7 +468,7 @@ void s_AbiWord_1_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length
 	UT_UTF8String sBuf;
 	const UT_UCSChar * pData;
 
-	UT_ASSERT(sizeof(UT_Byte) == sizeof(char));
+	UT_return_if_fail(sizeof(UT_Byte) == sizeof(char));
 
 	for (pData=data; (pData<data+length); /**/)
 	{
@@ -574,9 +575,9 @@ s_AbiWord_1_Listener::s_AbiWord_1_Listener(PD_Document * pDocument,
 	// now we begin the actual document.
 
 	_handleMetaData();
+	_handleHistory();
 	_handleRevisions();
 	_handleStyles();
-	_handleIgnoredWords();
 	_handleLists();
 	_handlePageSize();
 }
@@ -656,7 +657,7 @@ bool s_AbiWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
                     _closeField();
                     _openTag("field","",false,api);
                     m_pCurrentField = pcro->getField();
-                    UT_ASSERT(m_pCurrentField);
+                    UT_ASSERT_HARMLESS(m_pCurrentField);
                     return true;
                 }
    			case PTO_Bookmark:
@@ -709,10 +710,14 @@ bool s_AbiWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 		}
 
 	case PX_ChangeRecord::PXT_InsertFmtMark:
+#if 0
+		// fmt marks are temporary placeholders for props and
+		// attributes and should not be saved
 		if(m_bOpenChar)
 			_closeTag();
 		_openTag("c","",false,pcr->getIndexAP());
 		_closeTag();
+#endif
 		return true;
 
 	default:
@@ -728,6 +733,12 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 	UT_return_val_if_fail(pcr->getType() == PX_ChangeRecord::PXT_InsertStrux, false);
 	const PX_ChangeRecord_Strux * pcrx = static_cast<const PX_ChangeRecord_Strux *> (pcr);
 	*psfh = 0;							// we don't need it.
+#ifndef ENABLE_RESOURCE_MANAGER
+	PT_AttrPropIndex api = pcr->getIndexAP();
+	const XML_Char* image_name = getObjectKey(api, static_cast<const XML_Char*>(PT_STRUX_IMAGE_DATAID));
+	if (image_name)
+		m_pUsedImages.insert(image_name);
+#endif
 
 	switch (pcrx->getStruxType())
 	{
@@ -781,6 +792,16 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			_openTag("endnote","",true,pcr->getIndexAP());
 			return true;
 		}
+	case PTX_SectionTOC:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			m_bInBlock = false;
+			_openTag("toc","",true,pcr->getIndexAP());
+			return true;
+		}
 	case PTX_SectionMarginnote:
 		{
 			_closeSpan();
@@ -837,6 +858,15 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 			m_bInBlock = true;
 			return true;
 		}
+	case PTX_EndTOC:
+		{
+			_closeSpan();
+            _closeField();
+            _closeHyperlink();
+			_closeBlock();
+			m_pie->write("</toc>");
+			return true;
+		}
 	case PTX_EndMarginnote:
 		{
 			_closeSpan();
@@ -851,6 +881,7 @@ bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
             _closeField();
             _closeHyperlink();
 			_closeBlock();
+			m_pie->write("</frame>");
 			return true;
 		}
 	case PTX_Block:
@@ -979,12 +1010,12 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 
 	const char * szName = NULL;
 	const PD_Style * pStyle=NULL;
-	UT_Vector vecStyles;
+	UT_GenericVector<PD_Style *> vecStyles;
 	m_pDocument->getAllUsedStyles(&vecStyles);
 	UT_uint32 k = 0;
 	for (k=0; k < vecStyles.getItemCount(); k++)
 	{
-		pStyle = static_cast<PD_Style *>(vecStyles.getNthItem(k));
+		pStyle = vecStyles.getNthItem(k);
 		if (!bWroteOpenStyleSection)
 		{
 			m_pie->write("<styles>\n");
@@ -997,7 +1028,7 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 
 	for (k=0; m_pDocument->enumStyles(k, &szName, &pStyle); k++)
 	{
-		if (!pStyle->isUserDefined() || (vecStyles.findItem(const_cast<void *>(static_cast<const void *>(pStyle))) >= 0))
+		if (!pStyle->isUserDefined() || (vecStyles.findItem(const_cast<PD_Style*>(pStyle))) >= 0)
 			continue;
 
 		if (!bWroteOpenStyleSection)
@@ -1016,45 +1047,6 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 	return;
 }
 
-void s_AbiWord_1_Listener::_handleIgnoredWords(void)
-{
-	UT_return_if_fail(m_pDocument);
-
-	XAP_App *pApp = m_pDocument->getApp();
-	UT_return_if_fail(pApp);
-
-	XAP_Prefs *pPrefs = pApp->getPrefs();
-	UT_return_if_fail(pPrefs);
-
-	bool saveIgnores = false;
-	pPrefs->getPrefsValueBool(static_cast<XML_Char *>(AP_PREF_KEY_SpellCheckIgnoredWordsSave), &saveIgnores);
-
-	bool bWroteIgnoredWords = false;
-
-	if (!saveIgnores)
-		return;  // don't bother
-
-	const UT_UCSChar *word;
-	for (UT_uint32 i = 0; m_pDocument->enumIgnores(i, &word); i++)
-	{
-
-		if (!bWroteIgnoredWords)
-		{
-			bWroteIgnoredWords = true;
-			m_pie->write("<ignoredwords>\n");
-		}
-
-		m_pie->write("<iw>");
-		_outputData (word, UT_UCS4_strlen (word));
-		m_pie->write("</iw>\n");
-	}
-
-	if (bWroteIgnoredWords)
-		m_pie->write("</ignoredwords>\n");
-
-	return;
-}
-
 void s_AbiWord_1_Listener::_handleLists(void)
 {
 	bool bWroteOpenListSection = false;
@@ -1106,22 +1098,25 @@ void s_AbiWord_1_Listener::_handleLists(void)
 
 void s_AbiWord_1_Listener::_handleMetaData(void)
 {
-  // TODO: make me UCS4 aware!!
+	if (m_pie->isCopying ())
+		return;
 
   // set all of the important meta-data props
 
   m_pDocument->setMetaDataProp ( PD_META_KEY_GENERATOR, UT_UTF8String("AbiWord") ) ;
   m_pDocument->setMetaDataProp ( PD_META_KEY_FORMAT,    UT_UTF8String(IE_MIME_AbiWord) ) ;
 
+#if 0
   // get the saved time, remove trailing newline
   time_t now = time ( NULL ) ;
   UT_String now_str ( ctime(&now) ) ;
   now_str = now_str.substr ( 0, now_str.size() -1 ) ;
-  m_pDocument->setMetaDataProp ( PD_META_KEY_DATE_LAST_CHANGED, now_str ) ;
+  m_pDocument->setMetaDataProp ( PD_META_KEY_DATE_LAST_CHANGED, UT_UTF8String(now_str.c_str()) ) ;
+#endif
 
   // TODO: set dc.date and abiword.date_created if document is new (i.e. first save)
 
-  UT_StringPtrMap & ref = m_pDocument->getMetaData() ;
+  UT_GenericStringMap<UT_UTF8String*> & ref = m_pDocument->getMetaData() ;
 
   // don't print out a thing
   if ( ref.size () == 0 )
@@ -1129,14 +1124,14 @@ void s_AbiWord_1_Listener::_handleMetaData(void)
 
   m_pie->write("<metadata>\n");
 
-  UT_StringPtrMap::UT_Cursor cursor ( &ref ) ;
+  UT_GenericStringMap<UT_UTF8String*>::UT_Cursor cursor ( &ref ) ;
 
-  const void * val = NULL ;
+  const UT_UTF8String * val = NULL ;
   for ( val = cursor.first(); cursor.is_valid(); val = cursor.next () )
     {
       if ( val )
 	{
-	  const UT_UTF8String *stringval = static_cast<const UT_UTF8String*>(val);
+	  const UT_UTF8String *stringval = val;
 	  if( stringval->size () > 0 )
 	    {
 	      m_pie->write( "<m key=\"" ) ;
@@ -1156,10 +1151,7 @@ void s_AbiWord_1_Listener::_handlePageSize(void)
   //
   // Code to write out the PageSize Definitions to disk
   //
-	char *old_locale;
-
-	old_locale = setlocale (LC_NUMERIC, "C");
-        m_pie->write("<pagesize pagetype=\"");
+	m_pie->write("<pagesize pagetype=\"");
 	m_pie->write(m_pDocument->m_docPageSize.getPredefinedName());
 	m_pie->write("\"");
 
@@ -1169,14 +1161,14 @@ void s_AbiWord_1_Listener::_handlePageSize(void)
 	else
 	        m_pie->write("landscape\"");
 	UT_Dimension docUnit = m_pDocument->m_docPageSize.getDims();
+
+	UT_LocaleTransactor t(LC_NUMERIC, "C");
 	m_pie->write( UT_String_sprintf(" width=\"%f\"", m_pDocument->m_docPageSize.Width(docUnit)).c_str() );
 	m_pie->write( UT_String_sprintf(" height=\"%f\"",m_pDocument->m_docPageSize.Height(docUnit)).c_str() );
 	m_pie->write(" units=\"");
 	m_pie->write(UT_dimensionName(docUnit));
 	m_pie->write("\"");
 	m_pie->write( UT_String_sprintf(" page-scale=\"%f\"/>\n",m_pDocument->m_docPageSize.getScale()).c_str() );
-	setlocale (LC_NUMERIC, old_locale);
-	return;
 }
 
 void s_AbiWord_1_Listener::_handleDataItems(void)
@@ -1295,26 +1287,42 @@ void s_AbiWord_1_Listener::_handleRevisions(void)
 {
 	bool bWroteOpenRevisionsSection = false;
 
-	const PD_Revision * pRev=NULL;
-	char buf[35];
+	const AD_Revision * pRev=NULL;
 
-	UT_Vector & vRevisions = m_pDocument->getRevisions();
+	const UT_GenericVector<AD_Revision*> & vRevisions = m_pDocument->getRevisions();
 
 	UT_uint32 k = 0;
 	for (k=0; k < vRevisions.getItemCount(); k++)
 	{
-		pRev = static_cast<PD_Revision *>(vRevisions.getNthItem(k));
+		pRev = vRevisions.getNthItem(k);
+		UT_return_if_fail(pRev);
+		
+		UT_String s;
+		
 		if (!bWroteOpenRevisionsSection)
 		{
-			m_pie->write("<revisions>\n");
+			UT_String_sprintf(s, "<revisions show=\"%d\" mark=\"%d\" show-level=\"%d\" auto=\"%d\">\n",
+							  m_pDocument->isShowRevisions(),
+							  m_pDocument->isMarkRevisions(),
+							  m_pDocument->getShowRevisionId(),
+							  m_pDocument->isAutoRevisioning());
+			
+			m_pie->write(s.c_str());
 			bWroteOpenRevisionsSection = true;
 		}
-		UT_String s = "<r id=\"";
-		sprintf(buf, "%d\">", pRev->getId());
-		s += buf;
+
+		UT_String_sprintf(s, "<r id=\"%d\" time-started=\"%d\" version=\"%d\">",
+						  pRev->getId(),
+						  pRev->getStartTime(),
+						  pRev->getVersion());
+		
 		m_pie->write(s.c_str());
 
-		_outputData(pRev->getDescription(), UT_UCS4_strlen(pRev->getDescription()));
+		if(pRev->getDescription())
+		{
+			_outputData(pRev->getDescription(), UT_UCS4_strlen(pRev->getDescription()));
+		}
+		
 
 		m_pie->write("</r>\n");
 	}
@@ -1324,3 +1332,46 @@ void s_AbiWord_1_Listener::_handleRevisions(void)
 
 	return;
 }
+
+void s_AbiWord_1_Listener::_handleHistory(void)
+{
+	bool bWroteOpenSection = false;
+
+	UT_uint32 k = 0;
+	const UT_uint32 iCount = m_pDocument->getHistoryCount();
+	
+	for (k=0; k < iCount; k++)
+	{
+		UT_uint32 iVersion  =  m_pDocument->getHistoryNthId(k);
+		const UT_UUID& UID  =  m_pDocument->getHistoryNthUID(k);
+		time_t tStarted     =  m_pDocument->getHistoryNthTimeStarted(k);
+		bool bAuto          =  m_pDocument->getHistoryNthAutoRevisioned(k);
+		
+		UT_UTF8String s, hUid;
+		UID.toString(hUid);
+		
+		
+		if (!bWroteOpenSection)
+		{
+			UT_UTF8String_sprintf(s, "<history version=\"%d\" edit-time=\"%d\" last-saved=\"%d\" "
+							     "uid=\"%s\">\n",
+							  m_pDocument->getDocVersion(),
+							  m_pDocument->getEditTime(),
+							  m_pDocument->getLastSavedTime(),
+							  m_pDocument->getDocUUIDString());
+			
+			m_pie->write(s.utf8_str());
+			bWroteOpenSection = true;
+		}
+
+		UT_UTF8String_sprintf(s, "<version id=\"%d\" started=\"%d\" uid=\"%s\" auto=\"%d\"/>\n",
+						  iVersion, tStarted, hUid.utf8_str(),bAuto);
+		
+		m_pie->write(s.utf8_str());
+	}
+
+	if (bWroteOpenSection)
+		m_pie->write("</history>\n");
+
+	return;
+}
