@@ -63,9 +63,9 @@ bool IE_Exp_GZipAbiWord_Sniffer::getDlgLabels(const char ** pszDesc,
 	*pszDesc = "GZipped AbiWord (.zabw)";
 
 #ifdef WIN32  
-	*pszSuffixList = ".zabw" ;  
+	*pszSuffixList = "*.zabw" ;  
 #else  
-	*pszSuffixList = ".abw.gz; *.zabw";
+	*pszSuffixList = "*.abw.gz; *.zabw";
 #endif
 
 	*ft = getFileType();
