@@ -23,7 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <locale.h>
+#include "ut_locale.h"
 
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
@@ -37,6 +37,7 @@
 #include "ut_string_class.h"
 
 #include "fg_GraphicRaster.h"
+#include "ie_imp_PasteListener.h"
 
 #include "pd_Document.h"
 
@@ -97,12 +98,16 @@ UT_Confidence_t IE_Imp_XHTML_Sniffer::recognizeContents(const char * szBuf,
 	p = szBuf ;
 	while( iLinesToRead-- )
 	{
-		magic = "<html " ;
+		magic = "<html" ;
 		if ( (iNumbytes - iBytesScanned) < strlen(magic) ) return(UT_CONFIDENCE_ZILCH);
-		if ( strncmp(p, magic, strlen(magic)) == 0 ) return(UT_CONFIDENCE_PERFECT);
+		if ( strncmp(p, magic, strlen(magic)) == 0 ) return(UT_CONFIDENCE_GOOD);
 		magic = "<!DOCTYPE html" ;
 		if ( (iNumbytes - iBytesScanned) < strlen(magic) ) return(UT_CONFIDENCE_ZILCH);
-		if ( strncmp(p, magic, strlen(magic)) == 0 ) return(UT_CONFIDENCE_PERFECT);
+		if ( strncmp(p, magic, strlen(magic)) == 0 ) return(UT_CONFIDENCE_GOOD);
+
+		magic = "<!DOCTYPE HTML" ;
+		if ( (iNumbytes - iBytesScanned) < strlen(magic) ) return(UT_CONFIDENCE_ZILCH);
+		if ( strncmp(p, magic, strlen(magic)) == 0 ) return(UT_CONFIDENCE_GOOD);
 		/*  Seek to the next newline:  */
 		while ( *p != '\n' && *p != '\r' )
 		{
@@ -119,15 +124,16 @@ UT_Confidence_t IE_Imp_XHTML_Sniffer::recognizeContents(const char * szBuf,
 			}
 		}
 	}
-	return(UT_CONFIDENCE_ZILCH);
+	return UT_CONFIDENCE_ZILCH;
 }
 
 UT_Confidence_t IE_Imp_XHTML_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-  if (!(UT_stricmp(szSuffix,".html")) || !(UT_stricmp(szSuffix,".xhtml"))
-      || !(UT_stricmp(szSuffix,".htm")))
-    return UT_CONFIDENCE_PERFECT;
-  return UT_CONFIDENCE_ZILCH;    
+	if (UT_stricmp(szSuffix,".xhtml") == 0)
+		return UT_CONFIDENCE_PERFECT;
+	if ((UT_stricmp(szSuffix,".html") == 0) || (UT_stricmp(szSuffix,".htm") == 0))
+		return UT_CONFIDENCE_GOOD;
+	return UT_CONFIDENCE_ZILCH;    
 }
 
 UT_Error IE_Imp_XHTML_Sniffer::constructImporter(PD_Document * pDocument,
@@ -208,7 +214,7 @@ bool	IE_Imp_XHTML_Sniffer::getDlgLabels(const char ** pszDesc,
 IE_Imp_XHTML::IE_Imp_XHTML(PD_Document * pDocument) :
 	IE_Imp_XML(pDocument,false),
 #ifdef USE_IE_IMP_TABLEHELPER
-	m_TableHelperStack(new IE_Imp_TableHelperStack(pDocument)),
+	m_TableHelperStack(new IE_Imp_TableHelperStack()),
 #else
 	m_TableHelperStack(0),
 #endif
@@ -220,7 +226,8 @@ IE_Imp_XHTML::IE_Imp_XHTML(PD_Document * pDocument) :
 	m_iNewImage(0),
 	m_szBookMarkName(NULL),
 	m_addedPTXSection(false),
-	m_iPreCount(0)
+	m_iPreCount(0),
+	m_bFirstBlock(false)
 {
 }
 
@@ -590,25 +597,56 @@ UT_Error IE_Imp_XHTML::importFile(const char * szFilename)
 		m_dirname = "";
 
 	UT_Error e = IE_Imp_XML::importFile(szFilename);
+	// m_parseState = _PS_Sec; // no point having another sections the end
  	if (!requireBlock ()) e = UT_IE_BOGUSDOCUMENT;
 	return e;
 }
 
-void IE_Imp_XHTML::pasteFromBuffer(PD_DocumentRange * pDocRange,
+bool IE_Imp_XHTML::pasteFromBuffer(PD_DocumentRange * pDocRange,
 								   const unsigned char * pData, 
 								   UT_uint32 lenData, 
 								   const char * szEncoding)
 {
-	UT_return_if_fail(getDoc() == pDocRange->m_pDoc);
-	UT_return_if_fail(pDocRange->m_pos1 == pDocRange->m_pos2);
-
-	setClipboard (pDocRange->m_pos1);
-
-	UT_XML xml;
-	xml.setListener (this);
+	UT_return_val_if_fail(getDoc() == pDocRange->m_pDoc,false);
+	UT_return_val_if_fail(pDocRange->m_pos1 == pDocRange->m_pos2,false);
+	
+	PD_Document * newDoc = new PD_Document(getDoc()->getApp());
+	newDoc->createRawDocument();
+	UT_XML * newXML = new UT_XML;
+	IE_Imp_XHTML * p = new IE_Imp_XHTML(newDoc);
+	newXML->setListener(p);
 	UT_ByteBuf buf (lenData);
 	buf.append (pData, lenData);
-	xml.parse (&buf);
+	UT_Error e = newXML->parse (&buf);
+	if(e != UT_OK)
+	{
+		UT_UTF8String sData;
+		sData.clear();
+		sData.append(reinterpret_cast<const char *>(pData), lenData);
+		UT_DEBUGMSG(("Error pasting HTML.... \n"));
+		if(lenData < 10000)
+		{
+			UT_DEBUGMSG(("Data is %s Length of buffer is %d \n",sData.utf8_str(),lenData));
+		}
+		delete p;
+		delete newXML;
+		UNREFP( newDoc);
+		return false;
+	}
+	newDoc->finishRawCreation();
+	//
+	// OK Broadcast from the just filled source document into our current
+	// doc via the paste listener
+	//
+	IE_Imp_PasteListener * pPasteListen = new  IE_Imp_PasteListener(getDoc(),pDocRange->m_pos1,newDoc);
+	newDoc->tellListener(static_cast<PL_Listener *>(pPasteListen));
+	delete pPasteListen;
+	delete p;
+	delete newXML;
+	UNREFP( newDoc);
+	return true;
+	//	setClipboard (pDocRange->m_pos1);
+
 }
 
 /*****************************************************************/
@@ -620,7 +658,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 	int failLine;
 	failLine = 0;
 	UT_DEBUGMSG(("startElement: %s, parseState: %u, listType: %u\n", name, m_parseState, m_listType));
-
+	UT_ASSERT(m_error == 0);
 	X_EatIfAlreadyError();				// xml parser keeps running until buffer consumed
 	                                                // this just avoids all the processing if there is an error
 #define NEW_ATTR_SZ 3
@@ -671,7 +709,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			UT_UTF8String * prev = 0;
 			if (m_divStyles.getItemCount ())
 				{
-					prev = reinterpret_cast<UT_UTF8String *>(m_divStyles.getLastItem ());
+					prev = m_divStyles.getLastItem ();
 				}
 			UT_UTF8String * style = 0;
 			if (prev)
@@ -851,13 +889,18 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 	  	else 
 			m_listType = L_UL;
 
-		if(m_parseState != _PS_Sec)
+		if(m_parseState == _PS_Block)
 		{
 			endElement("li");
 			m_parseState = _PS_Sec;
 			m_bWasSpace = false;
 			/* this sort of tag shuffling can mess up the space tracking */
 		}
+		else if(m_parseState != _PS_Sec)
+		{
+			requireSection ();
+			m_bWasSpace = false;
+		}
 
 		parentID = new UT_uint16(m_iListID);
 		m_utsParents.push(parentID);
@@ -887,8 +930,9 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 	case TT_DT:
 	case TT_DD:
 	{
+		if (m_parseState == _PS_Block)
+			m_parseState =  _PS_Sec;
 		X_CheckError (requireSection ());
-		m_parseState = _PS_Block;
 
 		XML_Char *sz;
 
@@ -907,10 +951,12 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			UT_String_sprintf(szListID, "%u", thisID);
 			UT_String_sprintf(szParentID, "%u", *parentID);
 			UT_String_sprintf(szLevel, "%u", m_utsParents.getDepth());
-			char * old_locale = setlocale (LC_NUMERIC, "C");
-			UT_String_sprintf(szMarginLeft, " margin-left: %.2fin", 
-					  m_utsParents.getDepth() * 0.5);
-			setlocale (LC_NUMERIC, old_locale);
+
+			{
+				UT_LocaleTransactor t(LC_NUMERIC, "C");
+				UT_String_sprintf(szMarginLeft, " margin-left: %.2fin", 
+								  m_utsParents.getDepth() * 0.5);
+			}
 
 			const int LevelPos = 1;
 			const int IDpos = 3;
@@ -928,7 +974,8 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			listAtts[propsPos] = props.c_str();
 
 			X_CheckError(appendStrux(PTX_Block, listAtts));
-
+			m_parseState = _PS_Block;
+			m_bFirstBlock = true;
 			listAtts[propsPos] = temp;
 
 			// append a field
@@ -1056,6 +1103,8 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 
 		if ( szSrc == 0) break;
 		if (*szSrc == 0) break;
+		UT_UTF8String sWidth;
+		UT_UTF8String sHeight;
 
 		FG_Graphic * pfg = 0;
 
@@ -1071,6 +1120,25 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 
 		char * mimetype = UT_strdup ("image/png");
 		X_CheckError(mimetype);
+		if(szWidth)
+			{
+				UT_Dimension units = UT_determineDimension (szWidth);
+				if(units == DIM_PERCENT)
+					{
+						getDoc()->convertPercentToInches(szWidth,sWidth);
+						szWidth = sWidth.utf8_str();
+					}
+			}
+		if(szHeight)
+			{
+				UT_Dimension units = UT_determineDimension (szHeight);
+				if(units == DIM_PERCENT)
+					{
+						getDoc()->convertPercentToInches(szWidth,sHeight);
+						szHeight = sHeight.utf8_str();
+					}
+			}
+
 
 		UT_UTF8String utf8val;
 		if (szStyle)
@@ -1085,9 +1153,12 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 				double d = UT_convertDimensionless (szWidth);
 				float width = static_cast<float>(UT_convertDimensions (d, units, DIM_IN));
 				UT_String tmp;
-				char * old_locale = setlocale (LC_NUMERIC, "C");
-				UT_String_sprintf (tmp, "%gin", width);
-				setlocale (LC_NUMERIC, old_locale);
+
+				{
+					UT_LocaleTransactor t(LC_NUMERIC, "C");
+					UT_String_sprintf (tmp, "%gin", width);
+				}
+
 				if (!tmp.empty ())
 					{
 						if (utf8val.byteLength ()) utf8val += "; ";
@@ -1101,9 +1172,12 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 				double d = UT_convertDimensionless (szHeight);
 				float height = static_cast<float>(UT_convertDimensions (d, units, DIM_IN));
 				UT_String tmp;
-				char * old_locale = setlocale (LC_NUMERIC, "C");
-				UT_String_sprintf (tmp, "%gin", height);
-				setlocale (LC_NUMERIC, old_locale);
+
+				{
+					UT_LocaleTransactor t(LC_NUMERIC, "C");
+					UT_String_sprintf (tmp, "%gin", height);
+				}
+
 				if (!tmp.empty ())
 					{
 						if (utf8val.byteLength ()) utf8val += "; ";
@@ -1113,28 +1187,54 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			}
 		if ((strstr (utf8val.utf8_str (), "width")  == 0) ||
 			(strstr (utf8val.utf8_str (), "height") == 0))
+		{
+			float width  = static_cast<float>(pfg->getWidth ());
+			float height = static_cast<float>(pfg->getHeight ());
+			if ((width > 0) && (height > 0))
+	   		{
+				UT_DEBUGMSG(("missing width or height; reverting to image defaults\n"));
+#if 0
+				if(strstr (utf8val.utf8_str (), "width")  != 0)
+				{
+					float rat = height/width;
+					float fwidth = UT_convertToInches(szWidth);
+					height = rat*fwidth;
+					UT_String tmp;
+					{
+						UT_LocaleTransactor t(LC_NUMERIC, "C");
+						UT_String_sprintf (tmp, "%gin", height);
+					}
+					if (utf8val.byteLength ()) 
+						utf8val += "; ";
+					utf8val += "height:";
+					utf8val += tmp.c_str ();
+					goto got_string;
+				}
+#endif
+			}
+			else
 			{
-				float width  = static_cast<float>(pfg->getWidth ());
-				float height = static_cast<float>(pfg->getHeight ());
-
-				if ((width > 0) && (height > 0))
-					{
-						UT_DEBUGMSG(("missing width or height; reverting to image defaults\n"));
-					}
-				else
-					{
-						UT_DEBUGMSG(("missing width or height; setting these to 100x100\n"));
-						width  = static_cast<float>(100);
-						height = static_cast<float>(100);
-					}
-				const char * old_locale = setlocale (LC_NUMERIC, "C");
-				UT_String tmp;
+				UT_DEBUGMSG(("missing width or height; setting these to 100x100\n"));
+				width  = static_cast<float>(100);
+				height = static_cast<float>(100);
+			}
+			width = width/96.0;
+			height = height/96.0;
+			if(height > 8.0)
+				{
+					float rat = 8.0/height;
+					width = width * rat;
+					height = 8.0;
+				}
+			UT_String tmp;
+			{
+				UT_LocaleTransactor t(LC_NUMERIC, "C");
 				UT_String_sprintf (tmp, "width:%gin; height:%gin", width, height);
-				setlocale (LC_NUMERIC, old_locale);
-
-				utf8val = tmp.c_str ();
 			}
 
+			utf8val = tmp.c_str ();
+		}
+		got_string:
 		const XML_Char * api_atts[5];
 
 		UT_String dataid;
@@ -1162,7 +1262,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 			{
 				X_CheckError(requireBlock ());
 			}
-		UT_DEBUGMSG(("inserting `%s' as `%s' [%s]\n",szSrc,dataid.c_str(),utf8val.utf8_str()));
+		xxx_UT_DEBUGMSG(("inserting `%s' as `%s' [%s]\n",szSrc,dataid.c_str(),utf8val.utf8_str()));
 
 		X_CheckError(appendObject (PTO_Image, api_atts));
 		X_CheckError(getDoc()->createDataItem (dataid.c_str(), false, pBB, static_cast<void*>(mimetype), NULL));
@@ -1172,19 +1272,28 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 		return;
 #ifdef USE_IE_IMP_TABLEHELPER
 	case TT_CAPTION:
+		{
+			UT_DEBUGMSG(("Found a caption \n"));
+			m_TableHelperStack->setCaptionOn();
+			m_parseState = _PS_Block;
+			break;
+		}
+		break;
 	case TT_COLGROUP:
 	case TT_COL:
 		// TODO
 		break;
 	case TT_TABLE:
 		{
+			m_parseState = _PS_Table;
 			const XML_Char * szStyle = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
 
-			X_CheckError(m_TableHelperStack->tableStart (static_cast<const char *>(szStyle)));
+			X_CheckError(m_TableHelperStack->tableStart (getDoc(),static_cast<const char *>(szStyle)));
 		}
 		break;
 	case TT_THEAD:
 		{
+			m_parseState = _PS_Table;
 			const XML_Char * szStyle = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
 
 			m_TableHelperStack->theadStart (static_cast<const char *>(szStyle));
@@ -1192,6 +1301,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 		break;
 	case TT_TFOOT:
 		{
+			m_parseState = _PS_Table;
 			const XML_Char * szStyle = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
 
 			m_TableHelperStack->tfootStart (static_cast<const char *>(szStyle));
@@ -1199,6 +1309,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 		break;
 	case TT_TBODY:
 		{
+			m_parseState = _PS_Table;
 			const XML_Char * szStyle = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
 
 			m_TableHelperStack->tbodyStart (static_cast<const char *>(szStyle));
@@ -1206,21 +1317,24 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 		break;
 	case TT_TR:
 		{
+			m_parseState = _PS_Cell;
 			const XML_Char * szStyle = _getXMLPropValue (static_cast<const XML_Char *>("style"), atts);
 
 			m_TableHelperStack->trStart (static_cast<const char *>(szStyle));
+			UT_DEBUGMSG(("Finished TR process \n"));
 		}
 		break;
 	case TT_TH:
 	case TT_TD:
 		{
+			UT_DEBUGMSG(("Doing TD \n"));
+			m_parseState = _PS_Block;
 			const XML_Char * szStyle   = _getXMLPropValue (static_cast<const XML_Char *>("style"),   atts);
 			const XML_Char * szColSpan = _getXMLPropValue (static_cast<const XML_Char *>("colspan"), atts);
 			const XML_Char * szRowSpan = _getXMLPropValue (static_cast<const XML_Char *>("rowspan"), atts);
 
 			UT_uint32 colspan = 1;
 			UT_uint32 rowspan = 1;
-
 			if (szColSpan)
 				{
 					int span = atoi (static_cast<const char *>(szColSpan));
@@ -1233,7 +1347,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 					if (span > 1)
 						rowspan = static_cast<UT_uint32>(span);
 				}
-			m_TableHelperStack->tdStart (colspan, rowspan, static_cast<const char *>(szStyle));
+			m_TableHelperStack->tdStart (rowspan,colspan, static_cast<const char *>(szStyle));
 		}
 		break;
 #endif /* USE_IE_IMP_TABLEHELPER */
@@ -1264,7 +1378,7 @@ void IE_Imp_XHTML::startElement(const XML_Char *name, const XML_Char **atts)
 
 		return;
 	}
-
+	UT_ASSERT(m_error == 0);
 	return;
 X_Fail:
 	UT_DEBUGMSG (("X_Fail at %d\n", failLine));
@@ -1291,7 +1405,10 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 	case TT_BODY:
 		/* add two empty blocks at the end...
 		 */
+		requireSection ();
+		// m_parseState = _PS_Sec; // no point having two sections at the end
 		newBlock ("Normal", 0, 0);
+		m_parseState = _PS_Sec; // no point having two sections at the end
 		newBlock ("Normal", 0, 0);
 
 		m_parseState = _PS_Init;
@@ -1306,7 +1423,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 			if (m_divStyles.getItemCount ())
 				{
 					UT_UTF8String * prev = 0;
-					prev = reinterpret_cast<UT_UTF8String *>(m_divStyles.getLastItem ());
+					prev = m_divStyles.getLastItem ();
 					DELETEP(prev);
 				}
 			m_divStyles.pop_back ();
@@ -1332,7 +1449,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 	case TT_LI:
 	case TT_DT:
 	case TT_DD:
-		UT_ASSERT(m_lenCharDataSeen==0);
+		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
 		m_parseState = _PS_Sec;
 		while (_getInlineDepth ()) _popInlineFmt ();
 		return;
@@ -1345,7 +1462,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 	case TT_H5:
 	case TT_H6:
 	case TT_BLOCKQUOTE:
-		UT_ASSERT(m_lenCharDataSeen==0);
+		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
 		if (m_listType == L_NONE)
 			{
 				m_parseState = _PS_Sec;
@@ -1374,7 +1491,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 	case TT_CITE:
 	case TT_EM:
 	case TT_I:
-		UT_ASSERT(m_lenCharDataSeen==0);
+		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
 		_popInlineFmt ();
 		if (m_parseState == _PS_Block)
 			{
@@ -1383,7 +1500,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 		return;
 
 	case TT_SPAN:
-		UT_ASSERT(m_lenCharDataSeen==0);
+		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
 		//UT_DEBUGMSG(("B %d\n", m_parseState));
 		X_VerifyParseState(_PS_Block);
 		X_CheckDocument(_getInlineDepth()>0);
@@ -1392,12 +1509,18 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 		return;
 
 	case TT_BR:						// not a container, so we don't pop stack
-		UT_ASSERT(m_lenCharDataSeen==0);
+		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
 		//UT_DEBUGMSG(("B %d\n", m_parseState));
 //		X_VerifyParseState(_PS_Block);
 		return;
 #ifdef USE_IE_IMP_TABLEHELPER
 	case TT_CAPTION:
+		{
+			UT_DEBUGMSG(("End Caption \n"));
+			m_TableHelperStack->setCaptionOff();
+			m_parseState = _PS_Table;
+			break;
+		}
 	case TT_COLGROUP:
 	case TT_COL:
 		// TODO
@@ -1405,22 +1528,30 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 	case TT_TABLE:
 		{
 			m_TableHelperStack->tableEnd ();
+			m_parseState = _PS_Sec;
 		}
 		break;
 	case TT_THEAD:
 	case TT_TFOOT:
 	case TT_TBODY:
 		{
-			m_TableHelperStack->tbodyStart ();
+			//			m_TableHelperStack->tbodyStart ();
+			m_parseState = _PS_Table;
 		}
 		break;
 	case TT_TR:
+		{
+			m_parseState = _PS_Table;
 		// 
-		break;
+			break;
+		}
 	case TT_TH:
 	case TT_TD:
-		// 
-		break;
+		{
+			m_TableHelperStack->tdEnd();
+			m_parseState = _PS_Cell;
+			break;
+		}
 #endif /* USE_IE_IMP_TABLEHELPER */
 	case TT_HEAD:
 	case TT_TITLE:
@@ -1461,7 +1592,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 		return;
 
 	case TT_PRE:
-		UT_ASSERT(m_lenCharDataSeen==0);
+		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
 		if (m_parseState == _PS_Block) m_parseState = _PS_Sec;
 		m_iPreCount--;
 		m_bWhiteSignificant = (m_iPreCount > 0);
@@ -1474,7 +1605,7 @@ void IE_Imp_XHTML::endElement(const XML_Char *name)
 		// Eventually we want to render the <rt> text above the
 		// <ruby> text.  The <rp> text will not be rendered but should
 		// be retained so it can be exported.
-		UT_ASSERT(m_lenCharDataSeen==0);
+		UT_ASSERT_HARMLESS(m_lenCharDataSeen==0);
 		//UT_DEBUGMSG(("B %d\n", m_parseState));
 		X_VerifyParseState(_PS_Block);
 		X_CheckDocument(_getInlineDepth()==0);
@@ -1664,14 +1795,18 @@ bool IE_Imp_XHTML::pushInline (const char * props)
 
 	UT_XML_cloneString (sz, PT_PROPS_ATTRIBUTE_NAME);
 	if (sz == NULL)
-		return false;
+		{
+			UT_return_val_if_fail(0,false);
+		}
 	api_atts[0] = sz;
 
 	sz = NULL;
 
 	UT_XML_cloneString (sz, props);
 	if (sz == NULL)
-		return false;
+		{
+			UT_return_val_if_fail(0,false);
+		}
 	api_atts[1] = sz;
 
 	api_atts[2] = NULL;
@@ -1682,11 +1817,15 @@ bool IE_Imp_XHTML::pushInline (const char * props)
 
 bool IE_Imp_XHTML::newBlock (const char * style_name, const char * css_style, const char * align)
 {
-	if (!requireSection ()) return false;
+	if (!requireSection ()) 
+		{
+			UT_return_val_if_fail(0,false);
+		}
+
 
 	UT_UTF8String * div_style = 0;
 	if (m_divStyles.getItemCount ())
-		div_style = reinterpret_cast<UT_UTF8String *>(m_divStyles.getLastItem ());
+		div_style = m_divStyles.getLastItem ();
 
 	UT_UTF8String style;
 	if (div_style)
@@ -1717,14 +1856,18 @@ bool IE_Imp_XHTML::newBlock (const char * style_name, const char * css_style, co
 
 	UT_XML_cloneString (sz, PT_STYLE_ATTRIBUTE_NAME);
 	if (sz == NULL)
-		return false;
+		{
+			UT_return_val_if_fail(0,false);
+		}
 	api_atts[0] = sz;
 
 	sz = NULL;
 
 	UT_XML_cloneString (sz, style_name);
 	if (sz == NULL)
-		return false;
+		{
+			UT_return_val_if_fail(0,false);
+		}
 	api_atts[1] = sz;
 
 	if (utf8val.byteLength ())
@@ -1733,20 +1876,25 @@ bool IE_Imp_XHTML::newBlock (const char * style_name, const char * css_style, co
 
 			UT_XML_cloneString (sz, PT_PROPS_ATTRIBUTE_NAME);
 			if (sz == NULL)
-				return false;
+				{
+					UT_return_val_if_fail(0,false);
+				}
 			api_atts[2] = sz;
 
 			sz = NULL;
 
 			UT_XML_cloneString (sz, utf8val.utf8_str ());
 			if (sz == NULL)
-				return false;
+				{
+					UT_return_val_if_fail(0,false);
+				}
 			api_atts[3] = sz;
 		}
 	if (!appendStrux (PTX_Block, api_atts))
 		{
-			return false;
+			UT_return_val_if_fail(0,false);
 		}
+	m_bFirstBlock = true;
 	m_parseState = _PS_Block;
 
 	_data_NewBlock (); // warn XML charData() handler that a new block is beginning
@@ -1776,14 +1924,144 @@ bool IE_Imp_XHTML::requireSection ()
 
 	if (!appendStrux (PTX_Section,NULL))
 		{
-			return false;
+			UT_return_val_if_fail(0,false);
 		}
 	m_parseState = _PS_Sec;
-
+	m_bFirstBlock = false;
 	m_addedPTXSection = true;
 	return true;
 }
 
+bool IE_Imp_XHTML::appendStrux(PTStruxType pts, const XML_Char ** attributes)
+{
+	UT_DEBUGMSG(("XHTML Import - appendStruxStrux type %d document %x \n",pts,getDoc()));
+	if(pts == PTX_Section)
+	{
+		m_bFirstBlock = false;
+		m_addedPTXSection = true;
+	}
+	else if(pts == PTX_Block)
+	{
+		m_bFirstBlock = true;
+	}
+	if(!bInTable())
+		{
+			return getDoc()->appendStrux(pts,attributes);
+		}
+	else
+		{
+			return m_TableHelperStack->Block(pts,attributes);
+		}
+	return true;
+}
+
+bool IE_Imp_XHTML::appendFmt( const XML_Char ** attributes)
+{
+	if(!m_addedPTXSection)
+		{
+			appendStrux(PTX_Section,NULL);
+		}
+	if(!m_bFirstBlock)
+		{
+			appendStrux(PTX_Block,NULL);
+		}
+	if(!bInTable())
+		{
+			return getDoc()->appendFmt(attributes);
+		}
+	else
+		{
+			return m_TableHelperStack->InlineFormat(attributes);
+		}
+	return true;
+}
+
+bool IE_Imp_XHTML::appendFmt(const UT_GenericVector<XML_Char*>* pVecAttributes)
+{
+	if(!m_addedPTXSection)
+		{
+			appendStrux(PTX_Section,NULL);
+		}
+	if(!m_bFirstBlock)
+		{
+			appendStrux(PTX_Block,NULL);
+		}
+	if(!bInTable())
+		{
+			return getDoc()->appendFmt(pVecAttributes);
+		}
+	else
+		{
+			const XML_Char * attributes[3] = {"props",NULL,NULL};
+			UT_String sPropString("");
+			UT_uint32 i = 0;
+			for(i=0; i< pVecAttributes->getItemCount(); i +=2)
+				{
+					UT_String sProp = static_cast<char *>(pVecAttributes->getNthItem(i));
+					UT_String sVal = static_cast<char *>(pVecAttributes->getNthItem(i));
+					UT_String_setProperty(sPropString,sProp,sVal);
+				}
+			attributes[1] = sPropString.c_str();
+			return m_TableHelperStack->InlineFormat(attributes);
+		}
+	return true;
+}
+
+bool IE_Imp_XHTML::appendSpan(const UT_UCSChar * p, UT_uint32 length)
+{
+	if(!m_addedPTXSection)
+		{
+			appendStrux(PTX_Section,NULL);
+		}
+	if(!m_bFirstBlock)
+		{
+			appendStrux(PTX_Block,NULL);
+		}
+
+	if(!bInTable())
+		{
+			return getDoc()->appendSpan(p,length);
+		}
+	else
+		{
+			UT_DEBUGMSG(("Doing Inline length %d \n",length));
+			return m_TableHelperStack->Inline(p, static_cast<UT_sint32>(length));
+		}
+	return true;
+}
+
+
+bool IE_Imp_XHTML::appendObject(PTObjectType pto, const XML_Char ** attributes)
+{
+	if(!m_addedPTXSection)
+		{
+			appendStrux(PTX_Section,NULL);
+		}
+	if(!m_bFirstBlock)
+		{
+			appendStrux(PTX_Block,NULL);
+		}
+	if(!bInTable())
+		{
+			return getDoc()->appendObject(pto,attributes);
+		}
+	else
+		{
+			return m_TableHelperStack->Object(pto,attributes);
+		}
+	return true;
+}
+
+
+
+/*!
+ * Returns true if we're in a table
+ */
+bool IE_Imp_XHTML::bInTable(void)
+{
+	return !(m_TableHelperStack->top() == NULL);
+}
+
 /* returns true if child of a <div> with a recognized "class" attribute
  */
 bool IE_Imp_XHTML::childOfSection ()
@@ -2069,12 +2347,17 @@ static void s_props_append (UT_UTF8String & props, UT_uint32 css_mask,
 					/* AbiWord uses "text-position" for CSS's "vertical-align" in the case
 					 * of super-/subscripts.
 					 */
-					if ((UT_strcmp (value, "superscript") == 0) ||
-						(UT_strcmp (value, "subscript") == 0))
+					if (UT_strcmp (value, "super") == 0)
 						{
 							static const char * text_position = "text-position";
 							name = text_position;
-							verbatim = value;
+							verbatim = "superscript";
+						}
+					if (UT_strcmp (value, "sub") == 0)
+						{
+							static const char * text_position = "text-position";
+							name = text_position;
+							verbatim = "subscript";
 						}
 				}
 			else if ((UT_strcmp (name, "color") == 0) || (UT_strcmp (name, "background") == 0))
@@ -2178,9 +2461,12 @@ static void s_props_append (UT_UTF8String & props, UT_uint32 css_mask,
 					double d = UT_convertDimensionless (value);
 					float dim = static_cast<float>(UT_convertDimensions (d, units, DIM_IN));
 					UT_String tmp;
-					char * old_locale = setlocale (LC_NUMERIC, "C");
-					UT_String_sprintf (tmp, "%gin", dim);
-					setlocale (LC_NUMERIC, old_locale);
+
+					{
+						UT_LocaleTransactor t(LC_NUMERIC, "C");
+						UT_String_sprintf (tmp, "%gin", dim);
+					}
+
 					if (!tmp.empty ())
 						{
 							if (props.byteLength ()) props += "; ";
