@@ -1,5 +1,6 @@
 /* AbiSource Program Utilities
  * Copyright (C) 2002-2003 Dom Lachowicz <cinamod@hotmail.com>
+ * Copyright (C) 2005 Robert Staudinger <robsta@stereolyzer.net>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -95,6 +96,8 @@ public:
   */
   const XML_Char ** getAbiPageAtts(const XML_Char * masterName)
   {
+    UT_return_val_if_fail(masterName != NULL, (const XML_Char **)m_pageAtts);
+
     if (UT_strcmp (m_name.c_str(), masterName))
       // FIXME can there be more than one master-page?
       UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
@@ -213,18 +216,18 @@ class OO_Style
 {
 public:
   
-  OO_Style (const XML_Char ** props, const PD_Style * pParentStyle)
-    : m_bColBreakBefore (false), m_bPageBreakBefore (false)
+  OO_Style (const XML_Char ** props, const PD_Style * pParentStyle, const bool bOpenDocument)
+    : m_bColBreakBefore (false), m_bPageBreakBefore (false), m_pParentStyle (pParentStyle), m_bOpenDocument (bOpenDocument)
   {
-    parse (props, pParentStyle);
+    parse (props);
   }
 
-  OO_Style (const OO_Style * other, const XML_Char ** props, PD_Style * pParentStyle)
-    : m_bColBreakBefore (false), m_bPageBreakBefore (false)
+  OO_Style (const OO_Style * other, const XML_Char ** props, PD_Style * pParentStyle, const bool bOpenDocument)
+    : m_bColBreakBefore (false), m_bPageBreakBefore (false), m_pParentStyle (pParentStyle), m_bOpenDocument (bOpenDocument)
   {
     if (other)
       *this = *other;
-    parse (props, pParentStyle);
+    parse (props);
   }
   
   ~OO_Style ()
@@ -250,11 +253,8 @@ public:
     return m_bPageBreakBefore;
   }
 
-private:
-
-  void parse (const XML_Char ** props, const PD_Style * pParentStyle)
+  void parse (const XML_Char ** props)
   {
-    const XML_Char** p = props;   
     const XML_Char * val = NULL;
 
     val = UT_getAttribute ("fo:text-align", props);
@@ -284,7 +284,10 @@ private:
     if (val)
       m_color = UT_String_sprintf ("color: %s;", val);      
     
-    val = UT_getAttribute ("style:text-background-color", props);
+	if (m_bOpenDocument)
+	    val = UT_getAttribute ("fo:background-color", props);
+	else
+	    val = UT_getAttribute ("style:text-background-color", props);
     if(val)
       m_bgcolor = UT_String_sprintf ("bgcolor: %s;", val);      
     
@@ -294,12 +297,21 @@ private:
 
     val = UT_getAttribute("fo:font-size", props);
     if(val) {
-      if ((val[strlen(val)-1] == '%') && pParentStyle) {
+	  UT_Dimension dim = UT_determineDimension(val, DIM_none);
+	  if (dim == DIM_PERCENT && !m_pParentStyle) {
+		UT_DEBUGMSG(("*** [open-writer] no parent style to resolve '%s'\n", val));
+	  }
+      else if (dim == DIM_PERCENT && m_pParentStyle) {
         // calculate font-size based on parent's 
-	const XML_Char * parentFontSize = NULL;
-        pParentStyle->getProperty("font-size", parentFontSize);
-        double fontSize = atoi(parentFontSize) * atoi(val) / 100.0;
-        m_fontSize = UT_String_sprintf ("font-size: %gpt;", rint(fontSize));
+        const XML_Char * parentFontSize = NULL;
+        double fontSize = 12;
+		if (m_pParentStyle->getProperty("font-size", parentFontSize)) {
+          fontSize = atoi(parentFontSize) * atoi(val) / 100.0;
+        }
+		else {
+		  UT_DEBUGMSG(("*** [open-writer] using fallback font-size '%f'\n", fontSize));
+		}
+		m_fontSize = UT_String_sprintf ("font-size: %gpt;", rint(fontSize));
       }
       else
         m_fontSize = UT_String_sprintf ("font-size: %s;", val);
@@ -320,27 +332,33 @@ private:
 	m_textPos += "normal;";
     }
 
-    if (UT_getAttribute("style:text-underline", props) || UT_getAttribute("style:text-crossing-out", props))
-      {
-	const XML_Char * undr = UT_getAttribute("style:text-underline", props);
-	const XML_Char * strk = UT_getAttribute("style:text-crossing-out", props);
-	
-	m_textDecoration = "text-decoration: ";
-	
-	if(undr)
-	  if (UT_strcmp(undr, "none") != 0)
-	    m_textDecoration += "underline";
-	
-	if (undr && strk)
-	  m_textDecoration += ",";
-	
-	if(strk)
-	  if (UT_strcmp(strk, "none") != 0)
-	    m_textDecoration += "line-through";
-	
-	  m_textDecoration += ";";
-      }
-
+	const XML_Char * undr = NULL;
+	const XML_Char * strk = NULL;
+	if (m_bOpenDocument) {
+		undr = const_cast<const XML_Char *>(UT_getAttribute("style:text-underline-style", props));
+		strk = const_cast<const XML_Char *>(UT_getAttribute("style:text-line-through-style", props));
+	}
+	else {
+		undr = const_cast<const XML_Char *>(UT_getAttribute("style:text-underline", props));
+		strk = const_cast<const XML_Char *>(UT_getAttribute("style:text-crossing-out", props));
+	}
+	if (undr || strk) {
+		m_textDecoration = "text-decoration: ";
+		
+		if(undr)
+		  if (UT_strcmp(undr, "none") != 0)
+		    m_textDecoration += "underline";
+		
+		if (undr && strk)
+		  m_textDecoration += ",";
+		
+		if(strk)
+		  if (UT_strcmp(strk, "none") != 0)
+		    m_textDecoration += "line-through";
+		
+		  m_textDecoration += ";";
+	}
+ 
     val = UT_getAttribute ("fo:margin-left", props);
     if(val)
       m_marginLeft = UT_String_sprintf ("margin-left: %s;", val);
@@ -412,7 +430,9 @@ private:
 	m_bPageBreakBefore = true;
     }
 
-#define APPEND_STYLE(sty) if (sty.size()) m_styleProps += sty;
+	m_styleProps = "";
+
+#define APPEND_STYLE(sty) if (sty.size()) { m_styleProps += sty; }
     
     APPEND_STYLE(m_align);
     APPEND_STYLE(m_fontWeight);
@@ -441,6 +461,8 @@ private:
       m_styleProps [m_styleProps.size()-1] = 0; 
   }
 
+private:
+
   OO_Style (); // no impl
 
   UT_String m_align;
@@ -468,6 +490,8 @@ private:
 
   bool m_bColBreakBefore;
   bool m_bPageBreakBefore;
+  const PD_Style * m_pParentStyle;
+  bool m_bOpenDocument;
 }; // class OO_Style
 
 /*****************************************************************************/
@@ -494,6 +518,7 @@ public:
 
 private:
 
+  UT_Error _handleMimetype ();
   UT_Error _handleMetaStream ();
   UT_Error _handleSettingsStream ();
   UT_Error _handleStylesStream ();
@@ -501,6 +526,8 @@ private:
 
   GsfInfile * m_oo;
   UT_GenericStringMap<OO_Style *> m_styleBucket;
+  //! TRUE ... OASIS OpenDocument, FALSE ... OpenOffice.org 1.0 OpenWriter
+  bool m_bOpenDocument;
 };
 
 /*****************************************************************************/
@@ -520,7 +547,8 @@ IE_Imp_OpenWriter_Sniffer::~IE_Imp_OpenWriter_Sniffer ()
  */
 UT_Confidence_t IE_Imp_OpenWriter_Sniffer::recognizeSuffix (const char * szSuffix)
 {
-  if (!UT_stricmp(szSuffix, ".sxw"))
+  if (!UT_stricmp(szSuffix, ".sxw") || 
+	  !UT_stricmp(szSuffix, ".odt"))
     return UT_CONFIDENCE_PERFECT;
   return UT_CONFIDENCE_ZILCH;
 }
@@ -561,8 +589,8 @@ bool IE_Imp_OpenWriter_Sniffer::getDlgLabels (const char ** szDesc,
 					      const char ** szSuffixList,
 					      IEFileType * ft)
 {
-  *szDesc = "OpenOffice Writer (.sxw)";
-  *szSuffixList = "*.sxw";
+  *szDesc = "OpenOffice Writer (.sxw, .odt)";
+  *szSuffixList = "*.sxw; *.odt";
   *ft = getFileType();
   return true;
 }
@@ -572,7 +600,7 @@ void IE_Imp_OpenWriter::defineSimpleStyle (const UT_UTF8String & name, const XML
   if (!name.size() || !props)
     return;
   
-  OO_Style * style = new OO_Style (props, NULL);
+  OO_Style * style = new OO_Style (props, NULL, m_bOpenDocument);
   m_styleBucket.insert (name.utf8_str(), style);
 }
 
@@ -586,6 +614,8 @@ const XML_Char* IE_Imp_OpenWriter::mapStyle (const XML_Char * name) const
 
 const OO_Style * IE_Imp_OpenWriter::mapStyleObj (const XML_Char * name) const
 {
+  if (!name)
+    return NULL;
   return m_styleBucket.pick((const char *)name);
 }
 
@@ -629,6 +659,8 @@ UT_Error IE_Imp_OpenWriter::importFile(const char * szFilename)
   
   UT_Error err = UT_OK;
 
+  if ( UT_OK != (err = _handleMimetype ()))
+    return err;
   if ( UT_OK != (err = _handleMetaStream ()))
     return err;
   if ( UT_OK != (err = _handleStylesStream ()))
@@ -765,10 +797,16 @@ static UT_Error handleStream ( GsfInfile * oo,
 class ABI_EXPORT OpenWriter_MetaStream_Listener : public OpenWriter_Stream_Listener
 {
 public:
-  OpenWriter_MetaStream_Listener ( IE_Imp_OpenWriter * importer )
-    : OpenWriter_Stream_Listener ( importer )
+  OpenWriter_MetaStream_Listener ( IE_Imp_OpenWriter * importer, bool bOpenDocument )
+    : OpenWriter_Stream_Listener ( importer ), 
+      m_bOpenDocument ( bOpenDocument )
   {
-    getDocument()->setMetaDataProp(PD_META_KEY_FORMAT, UT_UTF8String ("OpenWriter::SXW"));
+    if (m_bOpenDocument) {  
+      getDocument()->setMetaDataProp(PD_META_KEY_FORMAT, UT_UTF8String ("OpenWriter::ODT"));
+    }
+	else {
+      getDocument()->setMetaDataProp(PD_META_KEY_FORMAT, UT_UTF8String ("OpenWriter::SXW"));
+    }
   }
   
   virtual ~OpenWriter_MetaStream_Listener ()
@@ -813,14 +851,61 @@ private:
 
   UT_UTF8String mCharData;
   UT_UTF8String mAttrib;
+  const bool m_bOpenDocument;
 };
 
+/*!
+ * Determine mimetype (odt or sxw)
+ */
+UT_Error IE_Imp_OpenWriter::_handleMimetype ()
+{
+  guint8 const *data = NULL;
+  size_t len = 0;
+
+  GsfInput * input = gsf_infile_child_by_name(m_oo, "mimetype");
+
+  if (!input) {
+    UT_DEBUGMSG(("Error: didn't get a mimetype. Assuming that it's a application/vnd.sun.xml.writer document\n"));
+    UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+
+    m_bOpenDocument = FALSE;
+    return UT_OK;
+  }
+
+  UT_UTF8String mimetype = "";
+  if (gsf_input_size (input) > 0) {
+    while ((len = gsf_input_remaining (input)) > 0) {
+      // FIXME: we want to pass the stream in chunks, but libXML2 finds this disagreeable.
+      // we probably need to pass some magic to our XML parser? 
+      // len = UT_MIN (len, BUF_SZ);
+      if (NULL == (data = gsf_input_read (input, len, NULL))) {
+        g_object_unref (G_OBJECT (input));
+        return UT_ERROR;
+      }
+      mimetype += (const char*)data;
+	}
+  }
+  
+  if (!UT_strcmp("application/vnd.oasis.opendocument.text", mimetype.utf8_str())) {
+    m_bOpenDocument = TRUE;
+  }
+  else if (!UT_strcmp("application/vnd.sun.xml.writer", mimetype.utf8_str())) {
+    m_bOpenDocument = FALSE;
+  }
+  else {
+	UT_DEBUGMSG(("*** Unknown mimetype '%s'\n", mimetype.utf8_str()));  
+  }
+	  
+  g_object_unref (G_OBJECT (input));
+  return UT_OK;
+}
+
 /*!
  * Handle the meta-stream
  */
 UT_Error IE_Imp_OpenWriter::_handleMetaStream ()
 {
-  OpenWriter_MetaStream_Listener listener ( this );
+  OpenWriter_MetaStream_Listener listener ( this, m_bOpenDocument );
   return handleStream (m_oo, "meta.xml", listener);
 }
 
@@ -833,8 +918,9 @@ UT_Error IE_Imp_OpenWriter::_handleMetaStream ()
 class ABI_EXPORT OpenWriter_SettingsStream_Listener : public OpenWriter_Stream_Listener
 {
 public:
-  OpenWriter_SettingsStream_Listener ( IE_Imp_OpenWriter * importer )
-    : OpenWriter_Stream_Listener ( importer )
+  OpenWriter_SettingsStream_Listener ( IE_Imp_OpenWriter * importer, bool bOpenDocument )
+    : OpenWriter_Stream_Listener ( importer ), 
+      m_bOpenDocument ( bOpenDocument )
   {
   }
 
@@ -853,6 +939,9 @@ public:
   virtual void charData (const XML_Char * buffer, int length)
   {
   }
+  
+private:
+  const bool m_bOpenDocument;
 };
 
 /*!
@@ -860,7 +949,7 @@ public:
  */
 UT_Error IE_Imp_OpenWriter::_handleSettingsStream ()
 {
-  OpenWriter_SettingsStream_Listener listener ( this );
+  OpenWriter_SettingsStream_Listener listener ( this, m_bOpenDocument );
   return handleStream (m_oo, "settings.xml", listener);
 }
 
@@ -873,18 +962,29 @@ UT_Error IE_Imp_OpenWriter::_handleSettingsStream ()
 class ABI_EXPORT OpenWriter_StylesStream_Listener : public OpenWriter_Stream_Listener
 {
 public:
-  OpenWriter_StylesStream_Listener ( IE_Imp_OpenWriter * importer )
-    : OpenWriter_Stream_Listener ( importer ), m_ooStyle (0), m_pageMaster(0)
+  OpenWriter_StylesStream_Listener ( IE_Imp_OpenWriter * importer, bool bOpenDocument )
+    : OpenWriter_Stream_Listener ( importer ), m_ooStyle (0), m_pageMaster(0), m_bOpenDocument(bOpenDocument)
   {
   }
   
   virtual ~OpenWriter_StylesStream_Listener ()
   {
+    m_styleNameMap.purgeData();
+  }
+
+  UT_UTF8String getStyleName(const UT_UTF8String & in) const
+  {
+    UT_UTF8String * name = m_styleNameMap.pick(in.utf8_str());
+    if (!name)
+      return UT_UTF8String(in);
+    else
+      return *name;
   }
   
   virtual void startElement (const XML_Char * name, const XML_Char ** atts) 
   {
-    if (!UT_strcmp (name, "style:page-master")) {
+	/* SXW || ODT */
+    if (!UT_strcmp (name, "style:page-master") || !UT_strcmp (name, "style:page-layout")) {
       m_pageMaster = UT_getAttribute("style:name", atts);
     }
     else if (!UT_strcmp (name, "style:master-page")) {
@@ -899,15 +999,29 @@ public:
       if (attr)
 	m_name = attr;
 
+      attr = UT_getAttribute("style:display-name",atts);
+      if (attr)
+	m_displayName = attr;
+
       if (m_name != "Standard") {
 	attr = UT_getAttribute("style:parent-style-name",atts);
-	if (attr)
-	  m_parent = attr;
+	if (attr) {
+	  if (!UT_strcmp(attr, "Standard")) {
+	    m_parent = "Normal";
+	  } else {
+	    m_parent = attr;
+	  }
+	}
 	
 	attr = UT_getAttribute("style:next-style-name",atts);
-	if (attr)
-	  m_next = attr;
-	
+	if (attr) {
+	  if (!UT_strcmp(attr, "Standard")) {
+	    m_next = "Normal";
+	  } else {
+	    m_next = attr;
+	  }
+	}
+
 	attr = UT_getAttribute("style:family",atts);
 	if (attr && strcmp (attr, "paragraph") != 0)
 	  m_type = CHARACTER;
@@ -921,14 +1035,25 @@ public:
 	m_type = PARAGRAPH;
       }
       DELETEP(m_ooStyle);
+      m_ooStyle = NULL;
     }
-    else if (!strcmp (name, "style:properties") && m_pageMaster) {
-      // page setup
+    else if (/* SXW || ODT */
+	     (!strcmp (name, "style:properties") || !strcmp (name, "style:page-layout-properties")) && 
+	     m_pageMaster) {
+      // page setup, because we are in page-master subtree
       m_ooPageStyle.appendPageMaster(m_pageMaster, atts);
     }
-    else if (!strcmp (name, "style:properties")) {
-      getDocument()->getStyle (m_parent.utf8_str(), &m_pParentStyle);
-      m_ooStyle = new OO_Style (atts, m_pParentStyle);
+    else if (!strcmp (name, "style:properties") || 				/* SXW */
+	     !strcmp (name, "style:text-properties") || 		/* ODT */
+	     !strcmp (name, "style:paragraph-properties")) {
+      
+      if (m_ooStyle == NULL) {
+	getDocument()->getStyle (m_parent.utf8_str(), &m_pParentStyle);
+	m_ooStyle = new OO_Style (atts, m_pParentStyle, m_bOpenDocument);
+      }
+      else {
+	m_ooStyle->parse(atts);
+      }
     }
   } 
 
@@ -944,8 +1069,16 @@ public:
 	const XML_Char * atts[11];
 	atts[propCtr++] = "type";
 	atts[propCtr++] = (m_type == PARAGRAPH ? "P" : "C");
+
 	atts[propCtr++] = "name";
-	atts[propCtr++] = m_name.utf8_str();
+	if (m_displayName.size()) {	 
+	  atts[propCtr++] = m_displayName.utf8_str();
+	  m_styleNameMap.insert (m_name.utf8_str(), new UT_UTF8String(m_displayName));
+	}
+	else {
+	  atts[propCtr++] = m_name.utf8_str();
+	  m_styleNameMap.insert (m_name.utf8_str(), new UT_UTF8String(m_name));
+	}
 
 	if (m_ooStyle) {
 	  atts[propCtr++] = "props";
@@ -968,9 +1101,11 @@ public:
       }
 
       m_name.clear ();
+      m_displayName.clear ();
       m_parent.clear ();
       m_next.clear ();
       DELETEP(m_ooStyle);
+      m_ooStyle = NULL;
     }
   }
   
@@ -991,6 +1126,7 @@ public:
 private:
 
   UT_UTF8String m_name;
+  UT_UTF8String m_displayName;
   UT_UTF8String m_parent;
   UT_UTF8String m_next;
   enum { CHARACTER, PARAGRAPH } m_type;
@@ -998,6 +1134,9 @@ private:
   PD_Style *m_pParentStyle;
   OO_PageStyle m_ooPageStyle;
   const XML_Char * m_pageMaster;
+  const bool m_bOpenDocument;
+
+  UT_GenericStringMap<UT_UTF8String *> m_styleNameMap;
 };
 
 /*!
@@ -1005,7 +1144,7 @@ private:
  */
 UT_Error IE_Imp_OpenWriter::_handleStylesStream ()
 {
-  m_pSSListener = new OpenWriter_StylesStream_Listener(this);
+  m_pSSListener = new OpenWriter_StylesStream_Listener(this, m_bOpenDocument);
   return handleStream (m_oo, "styles.xml", *m_pSSListener);
 }
 
@@ -1022,6 +1161,7 @@ private:
   bool m_bAcceptingText;
   bool m_bInSection;  
   bool m_bSectionHasPara;
+  bool m_bInTOC;
 
   UT_UTF8String m_curStyleName;
 
@@ -1035,10 +1175,12 @@ private:
   int m_col;
   int m_cel;
 
+  const bool m_bOpenDocument;
+
 public:
-  OpenWriter_ContentStream_Listener ( IE_Imp_OpenWriter * importer, OpenWriter_StylesStream_Listener * pSSListener )
-    : OpenWriter_Stream_Listener ( importer ), m_bAcceptingText(false), m_bInSection(false), m_bSectionHasPara(false), 
-      m_pSSListener(pSSListener), m_imgCnt(0), m_row(0), m_col(0), m_cel(0)
+  OpenWriter_ContentStream_Listener ( IE_Imp_OpenWriter * importer, OpenWriter_StylesStream_Listener * pSSListener, bool bOpenDocument )
+    : OpenWriter_Stream_Listener ( importer ), m_bAcceptingText(false), m_bInSection(false), m_bSectionHasPara(false), m_bInTOC(false), 
+      m_pSSListener(pSSListener), m_imgCnt(0), m_row(0), m_col(0), m_cel(0), m_bOpenDocument(bOpenDocument)
   {
   }
 
@@ -1048,6 +1190,10 @@ public:
 
   virtual void startElement (const XML_Char * name, const XML_Char ** atts) 
   {
+    // if we're inside a TOC, ignore the contents since SXW/ODT include a copy of the TOC
+    if (m_bInTOC)
+      return;
+
     if (!UT_strcmp(name, "office:body"))
       {
       }
@@ -1059,6 +1205,7 @@ public:
 	_appendSection(abi_sty);
 	m_bInSection = true;
 	m_bSectionHasPara = false;
+	m_bAcceptingText = false;
       }
     else if (!UT_strcmp(name, "text:p" ) || !UT_strcmp(name, "text:h" ))
       {
@@ -1068,8 +1215,8 @@ public:
 	      m_bInSection = true;
 	      m_bSectionHasPara = false;
 	  }
-	const XML_Char * oo_sty = UT_getAttribute ("text:style-name", atts);
-	const OO_Style * abi_sty = _mapStyleObj (oo_sty);
+	UT_UTF8String oo_sty;
+	const OO_Style * abi_sty = _mapStyleObj (UT_getAttribute ("text:style-name", atts), oo_sty);
 
 	if (abi_sty) {
 	  // append an empty paragraph with this one char
@@ -1086,10 +1233,10 @@ public:
 	  
 	  getDocument()->appendStrux(PTX_Block, (const XML_Char **)props);
 	}
-	else if (oo_sty) {
+	else if (oo_sty.size()) {
 	  const XML_Char * props[3];
 	  props[0] = "style";
-	  props[1] = oo_sty;
+	  props[1] = oo_sty.utf8_str();
 	  props[2] = 0;
 
 	  getDocument()->appendStrux(PTX_Block, (const XML_Char **)props);
@@ -1124,7 +1271,11 @@ public:
 	m_curStyleName.clear ();
 	m_curStyleName = UT_getAttribute ("style:name", atts);
       }
-    else if (!UT_strcmp(name, "style:properties") || !UT_strcmp(name, "style:columns"))
+    else if (/* SXW */
+			!UT_strcmp(name, "style:properties") || 
+			!UT_strcmp(name, "style:columns") ||
+			/* ODT */
+			!UT_strcmp(name, "style:text-properties"))
       {
 	_defineSimpleStyle (atts);
       }
@@ -1143,6 +1294,21 @@ public:
       _insertBookmark (name, "start");
       _insertBookmark (name, "end");
     }
+    else if (!UT_strcmp(name, "text:table-of-content"))
+    {
+      _flush ();
+
+      if(!m_bAcceptingText)
+	{
+	  getDocument()->appendStrux(PTX_Block, NULL);
+	  m_bAcceptingText = true ;
+	}
+      
+      getDocument()->appendStrux(PTX_SectionTOC, NULL);
+      getDocument()->appendStrux(PTX_EndTOC, NULL);
+
+      m_bInTOC = true;
+    }
     else if (!UT_strcmp(name, "draw:image"))
       {
 	_flush ();
@@ -1239,7 +1405,7 @@ public:
     else if (!UT_strcmp(name, "text:p" ) || !UT_strcmp(name, "text:h" ))
       {
 	_flush ();
-	m_bAcceptingText= false;
+	m_bAcceptingText = false;
       }
     else if (!UT_strcmp(name, "text:span"))
       {
@@ -1252,6 +1418,9 @@ public:
 	_flush ();
 	getDocument()->appendObject(PTO_Hyperlink, NULL);
       }
+    else if (!UT_strcmp(name, "text:table-of-content")) {
+      m_bInTOC = false;
+    }
     else if (!UT_strcmp(name, "table:table"))
       {
 	_closeTable ();
@@ -1289,7 +1458,7 @@ public:
 
   virtual void charData (const XML_Char * buffer, int length)
   {
-    if (buffer && length && m_bAcceptingText)
+    if (buffer && length && m_bAcceptingText && !m_bInTOC)
       m_charData += UT_UCS4String (buffer, length, true);
   }
 
@@ -1308,9 +1477,16 @@ private:
 
     GsfInfile * pictures_dir = GSF_INFILE(gsf_infile_child_by_name(getImporter()->getOO(), "Pictures"));
 
-    // 10 == strlen("#Pictures/");
-    error = loadStream(pictures_dir, href+10, img_buf);
-    g_object_unref (G_OBJECT (pictures_dir));
+	if (m_bOpenDocument) {
+	    // 9 == strlen("Pictures/");
+	    error = loadStream(pictures_dir, href+9, img_buf);
+	    g_object_unref (G_OBJECT (pictures_dir));
+	}
+	else {
+	    // 10 == strlen("#Pictures/");
+	    error = loadStream(pictures_dir, href+10, img_buf);
+	    g_object_unref (G_OBJECT (pictures_dir));
+	}
 
     if (error != UT_OK)
       return;
@@ -1446,6 +1622,8 @@ private:
   void _closeTable ()
   {
     getDocument()->appendStrux(PTX_EndTable, NULL);
+
+    m_row = m_cel = m_col = 0;
   }
 
   void _closeColumn ()
@@ -1470,12 +1648,14 @@ private:
 
   const XML_Char* _mapStyle (const XML_Char * name) const
   {
-    return getImporter ()->mapStyle(name);
+    UT_UTF8String styleName = m_pSSListener->getStyleName(name);
+    return getImporter ()->mapStyle(styleName.utf8_str());
   }
 
-  const OO_Style * _mapStyleObj (const XML_Char * name) const
+  const OO_Style * _mapStyleObj (const XML_Char * name, UT_UTF8String & oo_sty) const
   {
-    return getImporter ()->mapStyleObj (name);
+    oo_sty = m_pSSListener->getStyleName(name);
+    return getImporter ()->mapStyleObj (oo_sty.utf8_str());
   }
 
   bool _pushInlineFmt(const XML_Char ** atts)
@@ -1519,6 +1699,6 @@ private:
  */
 UT_Error IE_Imp_OpenWriter::_handleContentStream ()
 {
-  OpenWriter_ContentStream_Listener listener (this, m_pSSListener);
+  OpenWriter_ContentStream_Listener listener (this, m_pSSListener, m_bOpenDocument);
   return handleStream (m_oo, "content.xml", listener);
 }
