@@ -24,6 +24,8 @@
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
 #include "ut_string.h"
+#include "xap_App.h"
+#include "xap_Frame.h"
 #include "ie_imp_DocBook.h"
 #include "ie_types.h"
 #include "pd_Document.h"
@@ -58,7 +60,7 @@ UT_Confidence_t IE_Imp_DocBook_Sniffer::recognizeContents(const char * szBuf,
 
 UT_Confidence_t IE_Imp_DocBook_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-	if (UT_stricmp(szSuffix,".dbk") == 0)
+	if (!UT_stricmp(szSuffix,".dbk") || !UT_stricmp(szSuffix,".xml"))
 	  return UT_CONFIDENCE_PERFECT;
 	return UT_CONFIDENCE_ZILCH;
 }
@@ -75,8 +77,8 @@ bool	IE_Imp_DocBook_Sniffer::getDlgLabels(const char ** pszDesc,
 											 const char ** pszSuffixList,
 											 IEFileType * ft)
 {
-	*pszDesc = "DocBook (.dbk)";
-	*pszSuffixList = "*.dbk";
+	*pszDesc = "DocBook (.dbk, .xml)";
+	*pszSuffixList = "*.dbk; *.xml";
 	*ft = getFileType();
 	return true;
 }
@@ -761,16 +763,16 @@ void IE_Imp_DocBook :: createList (void)
 		}
 	}
 
-	
 	/* creates the new list */
 	fl_AutoNum *an = new fl_AutoNum (
 			m_iCurListID,
 			pid,
 			NUMBERED_LIST,
 			1,
-			(m_iInSection == 1) ? "Chapter %L." : ((m_iInSection == 2) ? "Section %L." : "%L."),
-			"",
-			getDoc ()
+			(m_iInSection == 1) ? (const XML_Char *)"Chapter %L." : ((m_iInSection == 2) ?(const XML_Char *) "Section %L." : (const XML_Char *)"%L."),
+			(const XML_Char *)"",
+			getDoc (),
+			NULL
 		);
 	getDoc () -> addList (an);
 
