@@ -1,5 +1,6 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
+ * Copyright (C) 2004 Marc Maurer (j.m.maurer@student.utwente.nl)
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,6 +44,7 @@
 #include "px_CR_Strux.h"
 #include "xap_EncodingManager.h"
 #include "fd_Field.h"
+#include "ie_Table.h"
 
 #include "ut_string_class.h"
 
@@ -251,6 +253,10 @@ protected:
 
 	UT_uint16		m_iBlockType;	// BT_*
 	UT_Wctomb		m_wctomb;
+	
+	// Table utility members
+	
+	ie_Table *			m_pTableHelper;
 };
 
 void s_LaTeX_Listener::_closeParagraph(void)
@@ -347,8 +353,16 @@ void s_LaTeX_Listener::_openCell(PT_AttrPropIndex api)
 	if (bHaveProp && pAP)
 	{
 		pAP->getProperty("left-attach", szValue);
-		if (!UT_strcmp("0",szValue)) m_pie->write("\\\\\n\\hline\n");
-		else m_pie->write("&");
+		if (!UT_strcmp("0",szValue))
+		{
+			pAP->getProperty("top-attach", szValue);
+			if (!UT_strcmp("0",szValue))
+				m_pie->write("\n\\hline\n");
+			else
+				m_pie->write("\\\\\n\\hline\n");
+		}
+		else
+			m_pie->write("&");
 	}
 }
 
@@ -510,7 +524,7 @@ void s_LaTeX_Listener::_openParagraph(PT_AttrPropIndex api)
 				if (height < 0.9 || height > 1.1)
 				{
 					m_pie->write("\\begin{spacing}{");
-					printf("m_bLineHeight = true\n");
+					UT_DEBUGMSG(("m_bLineHeight = true\n"));
 					m_bLineHeight = true;
 				}
 				if (height > 1.4 && height < 1.6)
@@ -814,25 +828,15 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 
 void s_LaTeX_Listener::_openTable(PT_AttrPropIndex api)
 {
-	// Table size (number of columns) appears only if
-	// they are of custom size...
-	int i,iColumns = 0;
-	const XML_Char * szValue;
-	const PP_AttrProp * pAP = NULL;
-	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-
-	if (bHaveProp && pAP)
-	{
-		if (pAP->getProperty("table-column-props",szValue))
-		{
-			for(i = 0;szValue[i];i++) if(szValue[i] == '/') iColumns++;
-		}
-		m_pie->write("\n% table begins");
-		m_pie->write("\n% ");
-		m_pie->write("\n\\begin{tabular}{|");
-		for(i = 0;i < iColumns;i++) m_pie->write("c|");
-		m_pie->write("}%");
-	}
+	UT_sint32 i = 0;
+	
+	m_pie->write("\n\\\\");
+	m_pie->write("\n%");
+	m_pie->write("\n% Table begins");
+	m_pie->write("\n% ");
+	m_pie->write("\n\\begin{table}[h]\\begin{tabular}{|");
+	for(i = 0; i < m_pTableHelper->getNumCols(); i++) m_pie->write("l|");
+	m_pie->write("}");
 }
 
 void s_LaTeX_Listener::_closeCell(void)
@@ -991,7 +995,7 @@ void s_LaTeX_Listener::_closeSpan(void)
 
 void s_LaTeX_Listener::_closeTable(void)
 {
-		m_pie->write("\\\\\n\\hline\n\\end{tabular}\n");
+		m_pie->write("\\\\\n\\hline\n\\end{tabular}\n\\end{table}\n");
 }
 
 void s_LaTeX_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
@@ -1211,7 +1215,7 @@ s_LaTeX_Listener::s_LaTeX_Listener(PD_Document * pDocument,
 	m_pie->write("%% ================================================================================\n");
 	m_pie->write("%% This LaTeX file was created by AbiWord.                                         \n");
 	m_pie->write("%% AbiWord is a free, Open Source word processor.                                  \n");
-	m_pie->write("%% You may obtain more information about AbiWord at www.abisource.com              \n");
+	m_pie->write("%% More information about AbiWord is available at http://www.abisource.com/        \n");
 	m_pie->write("%% ================================================================================\n");
 	m_pie->write("\n");
 
@@ -1248,6 +1252,8 @@ s_LaTeX_Listener::s_LaTeX_Listener(PD_Document * pDocument,
 	m_bInScript = false;
 	//	m_pie->write("\\begin{document}\n");  // I've to leave this step to the openSection, and that implies
 	//	m_pie->write("\n");                   // future problems when we will support several sections in the same doc...
+	
+	m_pTableHelper = new ie_Table(pDocument);
 }
 
 s_LaTeX_Listener::~s_LaTeX_Listener()
@@ -1295,7 +1301,6 @@ bool s_LaTeX_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 			bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
 			const XML_Char* szValue = NULL;
-			XML_Char * szImageName;
 
 			fd_Field* field = NULL;
 
@@ -1316,12 +1321,7 @@ bool s_LaTeX_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 				m_pie->write(szValue);
 				m_pie->write("]{");
 				pAP->getAttribute("dataid", szValue);
-				// szImageName is something like foo.png_0
-				szImageName = UT_strdup(szValue);
-				for (i = strlen(szValue); szValue[i] != '.'; i--) {
-					szImageName[i - 1] = '\0';
-				}
-				m_pie->write(szImageName);
+				m_pie->write(szValue);
 				m_pie->write("}");
 
 				return true;
@@ -1385,7 +1385,7 @@ bool s_LaTeX_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 	}
 }
 
-bool s_LaTeX_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
+bool s_LaTeX_Listener::populateStrux(PL_StruxDocHandle sdh,
 										   const PX_ChangeRecord * pcr,
 										   PL_StruxFmtHandle * psfh)
 {
@@ -1473,6 +1473,7 @@ bool s_LaTeX_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 
 	case PTX_SectionTable:
 	{
+		m_pTableHelper->OpenTable(sdh, pcr->getIndexAP());
 		_openTable(pcr->getIndexAP());
 		return true;
 	}
@@ -1480,6 +1481,7 @@ bool s_LaTeX_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 	case PTX_EndTable:
 	{
 		_closeTable();
+		m_pTableHelper->CloseTable();
 		return true;
 	}
 
@@ -1515,8 +1517,8 @@ bool s_LaTeX_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 
 	case PTX_EndEndnote:
 	default:
-		UT_ASSERT_NOT_REACHED();
-		return false;
+		UT_ASSERT(UT_TODO);
+		return true;
 	}
 }
 
