@@ -155,8 +155,8 @@ protected:
 //										 const gchar *fontName, const float fontSize, const float lineSpacing);
     UT_Error							_updateDocumentOrderedListDefinition(ABI_ListDefinition *pListDefinition, 
 												     int iLevel, const char listType, 
-												     const WPXString &sTextBeforeNumber, 
-												     const WPXString &sTextAfterNumber,
+												     const UT_UTF8String &sTextBeforeNumber, 
+												     const UT_UTF8String &sTextAfterNumber,
 												     int iStartingNumber);    
     UT_Error							_updateDocumentUnorderedListDefinition(ABI_ListDefinition *pListDefinition, 
 												       int level);    
